/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Gael Harbonnier.

====================================================================*/

package org.objectweb.benchmark.ejb.latency;

//Package dependencies.
import java.rmi.RemoteException;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import org.objectweb.benchmark.util.ResultPrinter;


/**
 * This client performs EJB latency Bencmark by calling the ping method.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class Client
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================
    
    /** Number of steps*/
    private int nb_step;

    /** Number of invocations per step*/
    private int nb_step_invocation;
    
    /** Beginning of the first step*/
    private int step_start;

    /** Results file name */
    private String result_file;
    
    /** Ping interface reference */
    private PingRemoteInterface pingBean = null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     * 
     * @param n - Number of invocations to do.
     * @param name - Results file name.
     */
    public Client(String name)
    {
        result_file = name;
        java.util.Properties properties = new java.util.Properties();


        try {
            java.io.InputStream propStream = null;
        
            propStream = getClass().getClassLoader().getResourceAsStream("benchmark.properties");
            properties.load(propStream);
        
            nb_step = Integer.parseInt( properties.getProperty("step.number") );
            nb_step_invocation = Integer.parseInt( properties.getProperty("step.invocation.number") );
            step_start = Integer.parseInt( properties.getProperty("step.start") );
        }catch (Exception e) {
            e.printStackTrace();
        }

    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
   /**
     * The bootstrap method.
     * 
     * @param args - Application arguments
     */
    public static void main(String[] args)
                throws RemoteException
    {
        if (args.length != 1)
        {
            System.out.println("Usage: Client <dest_file>");
            System.exit(0);
        }

        Client c = new Client(args[0]);
        c.run();
    }

   /**
     * Runs n invocations on the EJB client.
     */
    public void run()
         throws RemoteException
    {
        PingRemoteHome pingHome = null;
        
        // Obtains the Ping Itf reference
        try {
            InitialContext ctx = new InitialContext() ;
            Object obj= ctx.lookup("MyPingBean") ;
            
            pingHome = (PingRemoteHome) PortableRemoteObject.narrow(obj, PingRemoteHome.class) ;
            pingBean = pingHome.create() ;
        }catch(Exception ex){
            ex.printStackTrace();
        }
        
        // Runs benchmark
        test();
    }
    
    /**
     * Runs the latency benchmark.
     */
    public void test()
    throws RemoteException
    {
        // Don't include the first step_start invocations
        for (int i=0; i<step_start; i++)
           pingBean.ping();

        // Runs test
        System.out.println("Running EJB Benchmark with " + nb_step + " step(s) of " + nb_step_invocation +" method calls.");
        try{
            for(int j=0; j<nb_step; j++)
            {
                long timer = System.currentTimeMillis();
                for (int i=0; i<nb_step_invocation; i++)
                    pingBean.ping();
                timer = (long) ( ( System.currentTimeMillis() - timer ) * 1000 / (float)nb_step_invocation );
                
                // write results
                ResultPrinter p = new ResultPrinter(result_file);
                p.write(timer);
                p.close();
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
}
