/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.benchmark.util;

//Package dependencies.
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;


/**
 * This class is used to easily write benchmark results in a file.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class ResultPrinter
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The file to store results in. */
    private PrintStream file = null;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     * 
     * @param filename - The name of the results file.
     */
    public ResultPrinter(String filename)
    {
       file = openFile( new File(filename) );
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Create the file if it does'nt exist, else open it.
     * 
     * @param f - The file to open.
     * 
     * @return A data output stream.
     */
    private PrintStream
    openFile(File f)
    {
        FileOutputStream fos = null;
        
        try
        {
            // Create the file if it doesn't exists
            f.createNewFile();
            // Open the file for append
            fos = new FileOutputStream(f, true);
        } catch (IOException e) {
            e.printStackTrace();
        }
        
        return new PrintStream(fos);
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Prints a long to file.
     * 
     * @param l - The long to write.
     */
    public void write(long l)
    {
        file.println(l);
    }
    
    /**
     * Close the file.
     */
    public void close()
    {
        file.close();
    }
    
}
