/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java;

import java.util.Collection;

import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;

/**
 * This action clears all the elements of a <code>java.util.Collection</code> object.
 * A <code>UnsupportedOperationException</code> may be thrown if the implementation 
 * of the collection doesn't supported the requested operation. 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class CollectionClearAction 
    implements MenuItem {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for MenuItem.
    //
    //==================================================================

    /**
     * Indicates if the menu item must be enabled, disabled or not visible.
     * <br>Possible values:
     * <ul>
     * <li>ENABLED_STATUS: The menu item is enabled.</li>
     * <li>DISABLED_STATUS: The menu item is disabled.</li>
     * <li>NOT_VISIBLE_STATUS: The menu item is not visible.</li>
     * </ul>
     *  
     * @param treeView A view to obtain the node to check.
     * 
     * @return The status associated to the menu item.
     */
    public int 
    getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;    
    }

    /**
     * Invoked when an action occurs on the menu item.
     *
     * @param treeView The contextual MenuItemTreeView.
     * 
     * @exception Exception The invocation may thrown every kind of exception.
     */
    public void 
    actionPerformed(MenuItemTreeView treeView) 
    throws Exception {
        Collection collection = (Collection)treeView.getSelectedObject();
        collection.clear();    
    }

}