/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.lang.reflect.Modifier;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.objectweb.util.browser.api.IconProvider;
import org.objectweb.util.browser.core.common.ClassResolver;

/**
 * Provides an icon to represent <code>java.lang.Class</code> object.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class ClassIconProvider 
    implements IconProvider {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The icons to use for interface. */
    protected static Icon interface_; 

    /** The icons to use for public class. */
    protected static Icon publicClass_, publicAbstractClass_, publicStaticClass_, publicFinalClass_, publicStaticFinalClass_; 
    
    /** The icons to use for protected class. */
    protected static Icon protectedClass_, protectedAbstractClass_, protectedStaticClass_, protectedFinalClass_, protectedStaticFinalClass_;

    /** The icons to use for package class. */
    protected static Icon packageClass_, packageAbstractClass_, packageStaticClass_, packageFinalClass_, packageStaticFinalClass_; 
    
    /** The icons to use for private class. */
    protected static Icon privateClass_, privateFinalClass_;


    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    /**
     * Default constructor
     */
    public ClassIconProvider(){
        interface_ = new ImageIcon(ClassResolver.getResource("interface.png"));
        publicClass_ = new ImageIcon(ClassResolver.getResource("publicClass.png")); 
        publicAbstractClass_ = new ImageIcon(ClassResolver.getResource("publicAbstractClass.png"));
        publicStaticClass_ = new ImageIcon(ClassResolver.getResource("publicStaticClass.png"));
        publicFinalClass_ = new ImageIcon(ClassResolver.getResource("publicFinalClass.png"));
        publicStaticFinalClass_ = new ImageIcon(ClassResolver.getResource("publicStaticFinalClass.png"));
        protectedClass_ = new ImageIcon(ClassResolver.getResource("protectedClass.png"));
        protectedAbstractClass_ = new ImageIcon(ClassResolver.getResource("protectedAbstractClass.png"));
        protectedStaticClass_ = new ImageIcon(ClassResolver.getResource("protectedStaticClass.png"));
        protectedFinalClass_ = new ImageIcon(ClassResolver.getResource("protectedFinalClass.png"));
        protectedStaticFinalClass_ = new ImageIcon(ClassResolver.getResource("protectedStaticFinalClass.png"));
        privateClass_ = new ImageIcon(ClassResolver.getResource("privateClass.png"));
        privateFinalClass_ =  new ImageIcon(ClassResolver.getResource("privateFinalClass.png"));
        packageClass_ = new ImageIcon(ClassResolver.getResource("packageClass.png"));
        packageAbstractClass_ = new ImageIcon(ClassResolver.getResource("packageAbstractClass.png")); 
        packageStaticClass_ = new ImageIcon(ClassResolver.getResource("packageStaticClass.png"));
        packageFinalClass_ = new ImageIcon(ClassResolver.getResource("packageFinalClass.png"));
        packageStaticFinalClass_ = new ImageIcon(ClassResolver.getResource("packageStaticFinalClass.png"));
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for IconProvider interface.
    //
    //==================================================================

    /**
     * Return an <code>javax.swing.Icon</code> according to the given object.
     *
     * @param object The object to compute the associated icon.
     * 
     * @return The icon according to the given object.
     */
    public Icon 
    newIcon(Object object){
        Class theClass = (Class)object;
        int modifier = theClass.getModifiers();
        if (Modifier.isInterface(modifier)) {
            return interface_;                
        } else if (Modifier.isPrivate(modifier)){
            if (Modifier.isFinal(modifier))
                return privateFinalClass_;
            else
                return privateClass_;                
        } else if (Modifier.isProtected(modifier)){
            if (Modifier.isAbstract(modifier)) 
                return protectedAbstractClass_;
            else if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return protectedStaticFinalClass_;
            else if (Modifier.isStatic(modifier))
                return protectedStaticClass_;
            else if (Modifier.isFinal(modifier))
                return protectedFinalClass_;
            else
                return protectedClass_;                
        } else if (Modifier.isPublic(modifier)){
            if (Modifier.isAbstract(modifier)) 
                return publicAbstractClass_;
            else if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return publicStaticFinalClass_;
            else if (Modifier.isStatic(modifier))
                return publicStaticClass_;
            else if (Modifier.isFinal(modifier))
                return publicFinalClass_;
            else
                return publicClass_;                
        } else if (!Modifier.isPrivate(modifier) && !Modifier.isProtected(modifier) && !Modifier.isPublic(modifier)){
            if (Modifier.isAbstract(modifier)) 
                return packageAbstractClass_;
            else if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier))
                return packageStaticFinalClass_;
            else if (Modifier.isStatic(modifier))
                return packageStaticClass_;
            else if (Modifier.isFinal(modifier))
                return packageFinalClass_;
            else
                return packageClass_;                
        } else 
            return null;
        
    }
    
}