/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.plugin.java.reflect;

import java.lang.reflect.Modifier;
import java.util.HashMap;
import java.util.Map;

/**
 * 
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class ClassVisibility {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    /** The configuration map */
    public static Map config_ = initMap();
    
    /** The <code>String</code> representing the default configuration */
    public static final String DEFAULT_CONFIG = "_default_config_";
        
    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public static methods.
    //
    //==================================================================

    /**
     * Indicates if an element (<code>Field</code>, <code>Method</code> or <code>Class</code>)
     * must be visible depending on the user configuration.
     */
    public static boolean hasToBeAdded(int modifier, ClassVisibilityConfig config, String type){
        boolean addElement = false;
            
        if (((Modifier.isPrivate(modifier) && config.get(type + "." + ClassVisibilityConfig.PRIVATE)) || (Modifier.isProtected(modifier) && config.get(type + "." + ClassVisibilityConfig.PROTECTED)) || (Modifier.isPublic(modifier) && config.get(type + "." + ClassVisibilityConfig.PUBLIC)))||(!Modifier.isPrivate(modifier) && !Modifier.isProtected(modifier) && !Modifier.isPublic(modifier) && config.get(type + "." + ClassVisibilityConfig.PACKAGE))) {
            if (Modifier.isStatic(modifier) && Modifier.isFinal(modifier) && config.get(type + "." + ClassVisibilityConfig.STATIC) && config.get(type + "." + ClassVisibilityConfig.FINAL))
                addElement = true;
            else if (Modifier.isStatic(modifier) && !Modifier.isFinal(modifier) && config.get(type + "." + ClassVisibilityConfig.STATIC))
                addElement = true;
            else if (Modifier.isFinal(modifier) && !Modifier.isStatic(modifier) && config.get(type + "." + ClassVisibilityConfig.FINAL))
                addElement = true;
            else if (Modifier.isAbstract(modifier) && config.get(type + "." + ClassVisibilityConfig.ABSTRACT))
                addElement = true;
            else if (!Modifier.isStatic(modifier) && !Modifier.isFinal(modifier) && !Modifier.isAbstract(modifier)) 
                addElement = true;
        }
        
        return addElement;
    }

    /**
     * Provides a displayable name representing the name of a class
     * Example:
     * <ul>
     * <li><code>java.lang.String<code> returns <code>String</code></li>
     * <li><code>byte<code> returns <code>byte</code></li>
     * <li><code>[Ljava.lang.Object;<code> returns <code>Object[]</code></li>
     * <li><code>[[I<code> returns <code>int[][]</code></li>
     * </ul> 
     * @param name The class
     * @return The displayable name
     * @see java.lang.Class#getName()
     */
    public static String getTypeName(Class theClass){
        if(theClass!=null){
            StringBuffer name = new StringBuffer();
            int dimension = 0;
            if(theClass.isArray()){
                while (theClass.isArray()) {
                    dimension++;
                    theClass = theClass.getComponentType();
                }
            }
            if(theClass.isPrimitive()) {
                name.append(theClass.getName());
            } else {
                if (theClass.getName().lastIndexOf('.')!=-1)
                    name.append(theClass.getName().substring(theClass.getName().lastIndexOf('.') + 1));
                else
                    name.append(theClass.getName());
            }
            for(int i=0 ; i<dimension ; i++)
                name.append("[]");
            return name.toString();
        }
        return "";
    }

    /**
     * Provides a Map with one entry: the default configuration.
     * @return A new <code>Map</code>.
     */
    public static Map initMap(){
            config_ = new HashMap();
            config_.put(DEFAULT_CONFIG, createNewDefaultConfig());
            return config_;
        }

    /**
     * Creates a new default configuration. It displays:
     * <ul>
     *   <li>Public attributes</li>
     *   <li>Public methods</li>
     *   <li>Public inner classes</li>
     * </ul>
     * @return
     */
    public static ClassVisibilityConfig createNewDefaultConfig(){
        ClassVisibilityConfig defaultConfig = new ClassVisibilityConfig();
        defaultConfig.put(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.PUBLIC, true);
        defaultConfig.put(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.STATIC, true);
        defaultConfig.put(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.FINAL, true);
        defaultConfig.put(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.TYPE, true);
        defaultConfig.put(ClassVisibilityConfig.ATTRIBUTE + "." + ClassVisibilityConfig.DECLARING_CLASS, true);
        defaultConfig.put(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.PUBLIC, true);
        defaultConfig.put(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.STATIC, true);
        defaultConfig.put(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.ABSTRACT, true);
        defaultConfig.put(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.PARAMS, true);
        defaultConfig.put(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.RETURN_TYPE, true);
        defaultConfig.put(ClassVisibilityConfig.METHOD + "." + ClassVisibilityConfig.DECLARING_CLASS, true);
        defaultConfig.put(ClassVisibilityConfig.INNER_CLASS + "." + ClassVisibilityConfig.PUBLIC, true);
        defaultConfig.put(ClassVisibilityConfig.INNER_CLASS + "." + ClassVisibilityConfig.STATIC, true);
        defaultConfig.put(ClassVisibilityConfig.INNER_CLASS + "." + ClassVisibilityConfig.FINAL, true);
        defaultConfig.put(ClassVisibilityConfig.INNER_CLASS + "." + ClassVisibilityConfig.DECLARING_CLASS, true);
        return defaultConfig;        
    }

}