/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.api;

/** The Java API's imports */
import javax.swing.Icon;

/**
 * <p>
 * Standard interface for <code>javax.swing.Icon</code> provider. <br>
 * In order to define an icon provider, you can have a look to the corresponding
 * <a href="../../../../../../resources/browser_dtd.html#icon">DTD element</a> or to 
 * an <a href="package-summary.html#xml">XML example.</a>
 * </p>
 *
 * <p>
 * Example:
 * <pre>
 *      public Icon 
 *      newIcon(Object object) 
 *      {
 *          Properties properties = (Properties)object;
 *          return new NumberIcon(properties.size());
 *      }
 * </pre>
 * This example is specify for <code>java.util.Properties</code> objet 
 * and displays an icon containing the number of properties containing in its.
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.2
 */
public interface IconProvider 
{

    /**
     * Return an <code>javax.swing.Icon</code> according to the given object.
     *
     * @param object The object to compute the associated icon.
     * 
     * @return The icon according to the given object.
     */
    public Icon 
    newIcon(Object object);

}