/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.api;

import org.objectweb.util.browser.api.Entry;

/**
 * Entries are non-mutable structures representing entries in contexts.
 *
 * An Entry is composed of a <value,name,context>,
 * where value is an Object, name is a Name, and
 * context is the owner Context.
 *
 * Extended entry allows to fix this value after instanciation.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 *
 */
public interface ExtendedEntry 
    extends Entry {

    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String EXTENDED_ENTRY = ExtendedEntry.class.getName();

    /**
     * Changes the value of the Entry.
     *
     * @param value The value to set.
     */
    public void setOWValue(Object value);

    /**
     * Returns the wrapped object. The returned value is the same as the getOWObject method if no Context is defined for this object.
     *
     * @return The wrapped object
     */
    public Object getWrappedObject();

    // Example : 
    // NameService -> Corba Object -> Decodage IOR -> New interface -> New search into wrapper table -> create new context.
    /**
     * Sets the wrapped object. It's necessary when the object crosses more than one factory, as every CORBA Object does.
     */
    public void setWrappedObject(Object wrapped);

}
