/*====================================================================

Objectweb Browser framework
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.dnd;

import java.awt.dnd.DnDConstants;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.util.browser.api.DropAction;
import org.objectweb.util.browser.core.api.DropActionFactory;

/**
 * Basic implementation for DropAction factories
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class DefaultDropActionFactory 
    implements DropActionFactory {

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The descriptions of the actions */
    protected Map labels_ = null;

    /** The class to instantiate */
    protected Map classes_ = null;

    //==================================================================
    //
    // Constructor.
    // 
    //==================================================================

    public DefaultDropActionFactory(){
        labels_ = new HashMap();
        classes_ = new HashMap();
    }
    
    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for DropActionFactory interface.
    //
    //==================================================================

    /* 
     * @see org.objectweb.util.browser.api.DropActionFactory#newDropAction(java.lang.Object)
     */
    public DropAction newDropAction(Object object, int dropAction) {
        Class theClass = (Class)classes_.get(new Integer(dropAction));
        if (theClass != null) {
            try {
                return (DropAction) theClass.newInstance();
            } catch (java.lang.InstantiationException e) {
                System.out.println(theClass.getName() + " : Instanciation exception");
            } catch (java.lang.IllegalAccessException e) {
                System.out.println(theClass.getName() + " : Illegal access exception");
            }
        }
        return null;
    }

    /* 
     * @see org.objectweb.util.browser.api.DropActionFactory#setClassName(java.lang.Class)
     */
    public void setClassName(Class theClass, int dropAction) {
        classes_.put(new Integer(dropAction), theClass);
    }

    /*
     * @see org.objectweb.util.browser.api.DropActionFactory#setLabel(java.lang.String)
     */
    public void setLabel(String label, int dropAction){
        labels_.put(new Integer(dropAction), label);    
    }
    
	/* 
	 * @see org.objectweb.util.browser.api.DropActionFactory#getLabel()
	 */
    /**
     * @param dropAction The code of the dropAction (copy/move/link). 
     *        -1 to display all available drag and drop labels.
     */
	public String getLabel(int dropAction) {
        if(dropAction==-1) {
            StringBuffer sb =  new StringBuffer();
            String copyLabel = (String)labels_.get(new Integer(DnDConstants.ACTION_COPY));
            if(copyLabel!=null && !copyLabel.equals(""))
                sb.append("<br> - Drag-copy: " + copyLabel);
            String moveLabel = (String)labels_.get(new Integer(DnDConstants.ACTION_MOVE));
            if(moveLabel!=null && !moveLabel.equals(""))
                sb.append("<br> - Drag-move: " + moveLabel);
            String linkLabel = (String)labels_.get(new Integer(DnDConstants.ACTION_LINK));
            if(linkLabel!=null && !linkLabel.equals(""))
                sb.append("<br> - Drag-link: " + linkLabel);
            return sb.toString();
        } else
            return (String)labels_.get(new Integer(dropAction));
	}

}