/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.icon;

import org.objectweb.util.browser.core.api.AssociatedIcon;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.common.AssociatedElementImpl;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.IconProvider;
import javax.swing.Icon;

/**
 * Implementation of AssociatedIcon API with a cache.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class AssociatedIconImpl 
     extends AssociatedElementImpl 
  implements AssociatedIcon 
{

    /** 
     * Default constructor
     * @param loader The propertyLoader to use to load new configuration
     */
    public AssociatedIconImpl(BrowserProperty loader) {
        super();
        context_ = new IconPropertyContainer(loader);
    }

    /** 
     * Returns the icon associates to the node
     * Equivalent to a getIcon(node,false) call
     *
     * @param node The node 
     * @return The associated icon or null if no context is found
     */
    public Icon getIcon(Object node) {
        return getIcon(node, false);
    }

    /** 
     * Returns the icon associates to the node
     *
     * @param node The node 
     * @param disabledJavaLangObject If true, then java.lang.Object is not consider as a valid entry
     * @return The associated icon or null if no context is found
     */
    public Icon getIcon(Object node, boolean disabledJavaLangObject) {
        if (context_ != null) {
            Entry entry = context_.getRecursiveProperty(node);
            if (entry != null && !(entry.getName().toString().equals("java.lang.Object") && disabledJavaLangObject)) {
                IconProvider iconFactory = (IconProvider) entry.getValue();
                return iconFactory.newIcon(node);
            }
        }
        return null;
    }
}
