/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.popup;

import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.core.common.AssociatedElementImpl;
import org.objectweb.util.browser.core.common.DynamicTree;
import org.objectweb.util.browser.core.api.AssociatedMenu;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.MenuFactory;

/**
 * Implementation of AssociatedMenu API without any cache 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class AssociatedMenuImpl 
     extends AssociatedElementImpl 
  implements AssociatedMenu {

    /** The tree */
    protected DynamicTree tree_ = null;

    /**
     * Constructs a AssociatedMenuImpl and initializes cache
     */
    public AssociatedMenuImpl(BrowserProperty loader, DynamicTree tree) {
        super();
        context_ = new MenuPropertyContainer(loader);
        tree_ = tree;
    }

    /** 
     * Returns the menu associates to the node
     * Equivalent to a getIcon(node,false) call
     *
     * @param node The node 
     * @return The associated menu or null if no context is found
     */
    public AdvancedJPopupMenu getMenu(Object node, MenuItemTreeView treeView) {
        return getMenu(node, false, treeView);
    }

    /** 
     * Returns the menu associates to the node
     *
     * @param node The node 
     * @param disabledJavaLangObject If true, then java.lang.Object is not consider as a valid entry
     * @return The associated menu or null if no context is found
     */
    public AdvancedJPopupMenu getMenu(Object node, boolean disabledJavaLangObject, MenuItemTreeView treeView) {
        if (context_ != null) {
            Entry entry = context_.getRecursiveProperty(node);
            if (entry != null && !(entry.getName().toString().equals("java.lang.Object") && disabledJavaLangObject)) {
                MenuFactory menuFactory = (MenuFactory) entry.getValue();
                return menuFactory.newMenu(node, tree_, treeView);
            }
        }
        return null;
    }

    /**
     * Returns the menu for childrens of the given node at the tree level
     * @param parent The object containing the parent menu
     * @param node The node on which applying the actions
     * @return The associated menu or null if no context is found
     */
    public AdvancedJPopupMenu getParentMenu(Object parent) {
        if (context_ != null) {
            Entry parentFactoryEntry = context_.getRecursiveProperty(parent);
            if (parentFactoryEntry != null) {
                MenuFactory parentFactory = (MenuFactory) parentFactoryEntry.getValue();
                return parentFactory.newParentMenu(tree_);
            }
        }
        return null;
    }
}
