/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.wrapper;

/** The console's imports */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.api.BrowserProperty;
import org.objectweb.util.browser.core.api.ContextFactory;
import org.objectweb.util.browser.core.api.Role;
import org.objectweb.util.browser.core.common.DefaultPropertyContainer;
import org.objectweb.util.browser.core.common.ExtendedBoolean;

/**
 * This class is used to manipulate the wrapper property
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * @version 0.1
 */
public class WrapperPropertyContainer extends DefaultPropertyContainer {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default contstructor
     * @param loader The object to use to load new configuration
     */
    public WrapperPropertyContainer(BrowserProperty loader) {
        super(loader);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Tries to load an element for the associated class name.
     * @param className The name of the class 
     * @return The founded entry or null
     */
    protected Entry getElement(String className, Role role, ExtendedBoolean nodeFound) {
        return loader_.getWrapper(className, role, nodeFound);
    }

    /**
     * Adds the contextFactory to the container.
     *
     * @param id The name of the entry to be retrieved.
     * @param object The contextFactory to add
     * 
     */
    protected void addEntry(String id, ContextFactory object) {
        super.addEntry(id, object);
    }

    // ==================================================================
    //
    // Surcharging public methods for DefaultContextContainer class.
    //
    // ==================================================================

    /**
     * Adds the object to the container. It really adds the factory of the param object
     *
     * @param id The name of the entry to be retrieved.
     * @param object The object to add
     * 
     */
    public void addEntry(String id, Object object) {
        if (object != null) {
            addEntry(id, (ContextFactory) object);
        }
    }

}
