/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.Components;


import org.omg.Components.CCMObject;
import org.omg.Components.ComponentPortDescription;
import org.omg.Components.FacetDescription;
import org.omg.Components.ReceptacleDescription;
import org.omg.Components.ConsumerDescription;
import org.omg.Components.EmitterDescription;
import org.omg.Components.PublisherDescription;
import org.objectweb.openccm.explorer.CORBA.CORBAContext;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;

/**
 * Basic implementation of the Context interface
 * for managing Components.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.2
 */
public class ComponentContext 
     extends CORBAContext {

    // ==================================================================
    //
    // No Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for Context interface.
    //
    // ==================================================================

    public Entry[] getEntries(Object object) {
        CCMObject component = (CCMObject)object;
        
        ComponentPortDescription cpd = ((CCMObject)component).get_all_ports();
        java.util.List v = new java.util.Vector();

        Entry[] attributes = super.getEntries(object);
        for (int i = 0; i < attributes.length; i++)
            v.add(attributes[i]);

        FacetDescription[] fd = cpd.facets;
        for (int i = 0; i < fd.length; i++)
            v.add(new DefaultEntry(fd[i].name, fd[i]));

        ConsumerDescription[] cd = cpd.consumers;
        for (int i = 0; i < cd.length; i++)
            v.add(new DefaultEntry(cd[i].name, cd[i]));

        ReceptacleDescription[] rd = cpd.receptacles;
        for (int i = 0; i < rd.length; i++) {
            ReceptacleContainer rc = new ReceptacleContainer();
            rc.setComponent((CCMObject)component);
            rc.setReceptacle(rd[i]);
            v.add(new DefaultEntry(rd[i].name, rc));
        }

        EmitterDescription[] ed = cpd.emitters;
        for (int i = 0; i < ed.length; i++) {
            EmitterContainer ec = new EmitterContainer();
            ec.setComponent((CCMObject)component);
            ec.setEmitter(ed[i]);
            v.add(new DefaultEntry(ed[i].name, ec));
        }

        PublisherDescription[] pd = cpd.publishers;
        for (int i = 0; i < pd.length; i++) {
            PublisherContainer pc = new PublisherContainer();
            pc.setComponent((CCMObject)component);
            pc.setPublisher(pd[i]);
            v.add(new DefaultEntry(pd[i].name, pc));
        }

        return (Entry[]) v.toArray(new Entry[0]);
    }

}
