/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.Components;

import org.objectweb.util.explorer.api.DropAction;
import org.objectweb.util.explorer.api.DropTreeView;
import org.omg.Components.CCMObject;
import org.omg.Components.FacetDescription;
import org.omg.Components.ReceptacleDescription;

/**
 * This action allows a receptacle to connect to a facet using the Drag&Drop mechanism.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class ConnectOnDropAction 
    implements DropAction {

    //==================================================================
    //
    // No internal state.
    //
    //==================================================================

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for DropAction interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.DropAction#execute(org.objectweb.util.explorer.api.DropTreeView)
     */
    public void execute(DropTreeView dropTreeView) throws Exception {
        if(dropTreeView!=null){
            Object object = dropTreeView.getDragSourceObject();

            org.omg.CORBA.Object obj_ref = null;
            if (object instanceof org.omg.CORBA.Object) {
                obj_ref = (org.omg.CORBA.Object) object;
            } else if (object instanceof FacetDescription) {
                obj_ref = ((FacetDescription)object).facet_ref;
            } else {
                throw new Exception("CORBA object expected !");
            }

            ReceptacleContainer rc = (ReceptacleContainer) dropTreeView.getSelectedObject();
            ReceptacleDescription receptacle = rc.getReceptacle();
            CCMObject component = rc.getComponent();
            if (component != null && receptacle != null && obj_ref != null) {
                component.connect(receptacle.name, obj_ref);
            }
        } 
    }

}
