/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.CosNaming;

import org.objectweb.util.explorer.api.DropAction;
import org.objectweb.util.explorer.api.DropTreeView;
import org.objectweb.util.explorer.swing.gui.api.DialogAction;
import org.objectweb.util.explorer.swing.gui.api.DialogBox;
import org.objectweb.util.explorer.swing.gui.lib.DefaultDialogBox;
import org.objectweb.util.explorer.swing.gui.lib.LabelBox;
import org.omg.CosNaming.NamingContextExt;

/**
 * This action binds an object into the NameService using the Drag&Drop mechanism.
 * If the name is already bound in the context, it replaces it.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class BindObjectOnDropAction 
    implements DropAction, DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    protected NamingContextExt namingContext_;

    protected LabelBox name_;

    protected org.omg.CORBA.Object object_ = null;

    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for DropAction interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.DropAction#execute(org.objectweb.util.explorer.api.DropTreeView)
     */
    public void execute(DropTreeView dropTreeView) throws Exception {
        namingContext_ = (NamingContextExt)dropTreeView.getSelectedObject();
        String name = dropTreeView.getDragSourceEntry().getName().toString();
        Object dragObject = dropTreeView.getDragSourceObject();
        
        if(org.omg.CORBA.Object.class.isAssignableFrom(dragObject.getClass())){
            object_ = (org.omg.CORBA.Object)dragObject;            
        } else {
            throw new Exception("You must select a org.omg.CORBA.Object \n Found : " + dragObject.getClass().getName());
        }
        if(object_ != null) {
            DialogBox dialog = new DefaultDialogBox("Binds a new object into the NameService");      
            name_ = new LabelBox("Object name",name);
            dialog.addElementBox(name_);
            dialog.setValidateAction(this);
            dialog.show();
        }
       
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.gui.api.DialogAction#executeAction()
     */
    public void executeAction() throws Exception {
        namingContext_.rebind(namingContext_.to_name(name_.getLabel()),object_);
    }

}