/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading;

/** The OMG Trading Service imports. */
import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.omg.CosTrading.Lookup;
import org.omg.CosTradingRepos.ServiceTypeRepository;
import org.omg.CosTradingRepos.ServiceTypeRepositoryHelper;


/**
 * Basic implementation of the Context interface
 * for managing org.omg.CosTrading.Lookup references. 
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class LookupContext 
  implements Context
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** 
     * States if the Trading Service must be manipulated by an admin
     * or a user. Default value is the user view.
     */
    private static boolean admin_view_=false;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Toogles the view of the trading service. If the previous view
     * was an admin view, it becomes a user view and vice-versa. <br/>
     *
     * <b>Warning:</b> toogling the view will impact all Lookup
     * references in the administration console.
     */
    public static void toogleAdminUserView() 
    {
        LookupContext.admin_view_ = ! LookupContext.admin_view_;
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Context#getEntries(java.lang.Object)
     */
    public Entry[] getEntries(Object object) {
        Lookup lookup = (Lookup)object;
        ServiceTypeRepository str = ServiceTypeRepositoryHelper.narrow(lookup.type_repos());
        Entry[] values = null;
        
        // The admin view consider all CosTrading management interfaces.
        if (LookupContext.admin_view_) {
            values = new Entry[6];
            values[0] = new DefaultEntry("Service Type Repository", str);
            values[1] = new DefaultEntry("Queries", new Queries(lookup));
            values[2] = new DefaultEntry("Registered Offers", new Offers(lookup.admin_if()));
            values[3] = new DefaultEntry("Proxy Offers", new ProxyOffers(lookup.admin_if()));
            values[4] = new DefaultEntry("Trader Config", lookup.admin_if());
            values[5] = new DefaultEntry("Trader Links", lookup.link_if());
        // The user view only consider CosTrading query an service type repos. interfaces.
        } else {
            values = new Entry[2];
            values[0] = new DefaultEntry("Service Type Repository", str);
            values[1] = new DefaultEntry("Queries", new Queries(lookup));
        }
        return values;
    }

}
