/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading.gui;

/** The java API's imports */
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.Dimension;

/**
 * A Box to display available service types in a combo box. This
 * class also provides a way to register a listener in order to be
 * notified of any change in the combo box.
 * 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class AvailableServiceTypesBox extends Box {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The combo box. */
    protected JComboBox types_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** 
     * Default constructor.
     *
     * @param label The label placed behind the combo box.
     * @param available_types Available service types IDs.
     */
    public AvailableServiceTypesBox(String label, String[] available_types) {
        super(BoxLayout.X_AXIS);
        add(Box.createHorizontalGlue());
        JLabel fieldLabel = new JLabel(label, SwingConstants.RIGHT);
        fieldLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        fieldLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(fieldLabel);
        add(Box.createHorizontalStrut(5));        
        types_ = new JComboBox(available_types);
        types_.setPreferredSize(new Dimension(225, 20));
        types_.setMaximumSize(new Dimension(225, 20));
        add(types_);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Returns the specified service type.
     *
     * @return The service type selected in the combo box.
     */
    public String getTypeName() {
        return (String)types_.getSelectedItem();
    }

    /**
     * Adds a listener on the combo box.
     *
     * @param listener The listener to add in the combo box.
     */
    public void addItemListener(java.awt.event.ItemListener listener) {
        types_.addItemListener(listener);
    }
}
