/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading.gui;

/** The Java API's imports */
import javax.swing.JComboBox;
import javax.swing.SwingConstants;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.JLabel;

import java.awt.Component;
import java.awt.Dimension;

/**
 * A Box to display a boolean value in a combo box. This class also
 * provides a way to register a listener in order to be notified of
 * any change in the combo box.
 * 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class BooleanBox extends Box {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The combo box */
    protected JComboBox bool_combo_;

    /** The label of the entry */
    protected JLabel fieldLabel_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** 
     * Default constructor.
     *
     * @param label The label puts behind the combo box.
     * @param init The initial value.
     */
    public BooleanBox(String label, boolean init) {
        super(BoxLayout.X_AXIS);
        add(Box.createHorizontalGlue());
        fieldLabel_ = new JLabel(label, SwingConstants.RIGHT);
        fieldLabel_.setAlignmentX(Component.RIGHT_ALIGNMENT);
        fieldLabel_.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(fieldLabel_);
        add(Box.createHorizontalStrut(5));
        bool_combo_ = new JComboBox(new String[] {"true", "false"});
        bool_combo_.setSelectedIndex(init?0:1);
        bool_combo_.setPreferredSize(new Dimension(225, 20));
        bool_combo_.setMaximumSize(new Dimension(225, 20));
        add(bool_combo_);
    }

    /** 
     * Default constructor with initial value at <code>true</code>.
     *
     * @param label The label puts behind the combo box.
     */
    public BooleanBox(String label) {
        this(label, true);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Adds a listener on the combo box.
     *
     * @param listener The listener to add in the combo box.
     */
    public void addItemListener(java.awt.event.ItemListener listener) {
        bool_combo_.addItemListener(listener);
    }

    /** 
     * Returns the specified boolean value.
     *
     * @return The boolean value selected in the combo box.
     */
    public boolean getBoolean() {
        String bool = (String)bool_combo_.getSelectedItem();
        if (bool.equals("true")) return true;
        return false;
    }

    // ==================================================================
    //
    // Public methods of Box (since j2sdk-1.4.1 only).
    //
    // ==================================================================

    /**
     * Sets the tool tip text on each JComponent of the box.
     *
     * @param text The tool tip text to set.
     */
    public void setToolTipText(String text) {
        bool_combo_.setToolTipText(text);
        fieldLabel_.setToolTipText(text);
    }

    /**
     * Gets the tool tip text set on each JComponent of the box.
     *
     * @return The tool tip text used.
     */
    public String getToolTipText() {
        return bool_combo_.getToolTipText();
    }
}

