/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.explorer.CosTrading.gui;

/** The Java API's imports */
import javax.swing.JPanel;
import java.awt.Color;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.TitledBorder;
import javax.swing.JLabel;

/** OMG Trading Service imports. */
import org.omg.CosTrading.LinkPackage.LinkInfo;
import org.omg.CosTrading.FollowOption;

/**
 * OMG CosTradingRepos TypeStruct Panel. 
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class LinkInfoPanel extends DefaultNodeViewPanel {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The link to display. */
    protected LinkInfo linkInfo_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor.
     */
    public LinkInfoPanel() {
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Returns the string image of a follow option.
     *
     * @return The string image of a follow option.
     */
    protected String optionToString(FollowOption o) {
        if (o.value() == FollowOption._local_only) return "local_only";
        if (o.value() == FollowOption._if_no_local) return "if_no_local";
        return "always";
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Creates the content of the Panel.
     */
    public void onSetTreeView() {
        if (getTreeView() == null) return;
        linkInfo_ = (LinkInfo)getTreeView().getSelectedObject();
        String name = getTreeView().getSelectedEntry().getName().toString();
        if (linkInfo_ != null) {

            JPanel infoPanel = new JPanel();
            infoPanel.setBackground(Color.white);
            infoPanel.setBorder(new TitledBorder(null," " + name + " ",TitledBorder.CENTER,TitledBorder.TOP));
            
            Box box = Box.createVerticalBox();

            box.add(Box.createVerticalGlue());

            JLabel def = new JLabel();
            def.setText("Default Follow Link Rule: " + optionToString(linkInfo_.def_pass_on_follow_rule) + ".");
            box.add(def);

            box.add(Box.createVerticalStrut(10));

            JLabel limit = new JLabel();
            limit.setText("Limiting Follow Link Rule: " + optionToString(linkInfo_.limiting_follow_rule) + ".");
            box.add(limit);

            box.add(Box.createVerticalGlue());
            infoPanel.add(box);

            add(infoPanel);
        }
    }
}














