/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.explorer.Deployment;

import java.net.URL;

import org.objectweb.openccm.explorer.menu.JFileChooserSingleton;
import org.objectweb.openccm.explorer.menu.OpenCCMBrowserConstants;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.gui.api.DialogAction;
import org.objectweb.util.explorer.swing.gui.api.DialogBox;
import org.objectweb.util.explorer.swing.gui.lib.DefaultDialogBox;
import org.objectweb.util.explorer.swing.gui.lib.FileChooserBox;
import org.objectweb.util.explorer.swing.gui.lib.LabelBox;
import org.omg.Components.Deployment.ComponentInstallation;

/**
 * The install operation installs a component on the particular host
 * on which the ComponentInstallation object resides.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class InstallOnComponentInstallation 
    implements MenuItem, DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================
    
    /** The componentInstallation for installing selected archive. */
    protected ComponentInstallation componentInstallation_;

    /** The UUID value */
    protected LabelBox uuid_;

    /** The location value */
    protected FileChooserBox location_;

    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    /**
     * Creates a box containing all the box to specify all the params
     */
    protected void createBox(DialogBox dialogBox) {
        uuid_ = new LabelBox("UUID");
        dialogBox.addElementBox(uuid_);
        location_ = new FileChooserBox("Location",JFileChooserSingleton.getInstance(OpenCCMBrowserConstants.JAR_FILE_CHOOSER));
        dialogBox.addElementBox(location_);
    }

    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#getStatus(org.objectweb.util.explorer.api.TreeView)
     */
    public int getStatus(TreeView arg0){
        return MenuItem.ENABLED_STATUS;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#actionPerformed(org.objectweb.util.explorer.api.MenuItemTreeView)
     */
    public void actionPerformed(MenuItemTreeView e) throws Exception {
        componentInstallation_ = (ComponentInstallation) e.getSelectedObject();
        DialogBox dialogBox = new DefaultDialogBox("Archive installation");      
        createBox(dialogBox);
        dialogBox.setValidateAction(this);
        dialogBox.setValidateLabel("Create");
        dialogBox.setCancelLabel("Cancel");
        dialogBox.show();
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.gui.api.DialogAction#executeAction()
     */
    public void executeAction() throws Exception {
        URL theURL = location_.getURL();
        if(theURL!=null){
            componentInstallation_.install(uuid_.getLabel(), theURL.toString());    
        } else {
            throw new Exception("Location: Malformed URL !");
        }
    }

}
