/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.actor.componentBrowser;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import org.coach.idltree.*;
import org.coach.tracing.api.pi.TracingService;

public class InvokePanel extends JPanel implements ModelChangeListener {
    TracingService tracingService;
    
    FlowLayout flowLayout1 = new FlowLayout(FlowLayout.LEFT);
    JLabel targetLabel = new JLabel();
    JButton invokeButton = new JButton();
    IdlOperation method;
    IdlModel model;
    IdlModel replyModel;
    private String[] methods;
    private IdlOperation[] types;
    org.omg.CORBA.Object ref;
    JComboBox methodSelector = new JComboBox();
    JCheckBox inheritenceCheckBox = new JCheckBox();
    String id;
    JFrame frm = new JFrame();
    String facetName;
    org.omg.CORBA.ORB orb;
    MethodPanel methodPanel;

    public InvokePanel() {
        try  {
            jbInit();
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }

    public InvokePanel(MethodPanel mp, org.omg.CORBA.ORB orb) {
        this();
        this.methodPanel = mp;
        this.orb = orb;
//        replyModel = new IdlModel();
//        methodPanel.setReply(replyModel);
    }

    public void setOperation(String id, String facetName, org.omg.CORBA.Object target) {
        this.id = id;
        this.facetName = facetName;
        ref = target;
        initMethods();
    }
        
    private void initMethods() {
        method = null;
        methodSelector.removeAllItems();
        try {
            IdlInterface itf = new IdlInterface(id, false);
            methods = itf.getOperations();
            // create an array with all operations.
            // could also create a cache table and fill it as needed
            types = new IdlOperation[methods.length];
            for (int i = 0; i < methods.length; i++) {
                types[i] = new IdlOperation(id, methods[i]);
            }
            for (int i = 0; i < methods.length; i++) {
                methodSelector.addItem(methods[i]);
            }
            if (methods.length > 0) {
                method = types[0];
                model = new IdlModel(method);
            }
            replyModel = null;
        } catch (Exception e) {
        }
    }

    private void jbInit() throws Exception {
        targetLabel.setText("Methods");
        this.setLayout(flowLayout1);

        invokeButton.setText("invoke");
        invokeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                invokeButton_actionPerformed(e);
            }
        });

        methodSelector.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                methodSelector_actionPerformed(e);
            }
        });

        inheritenceCheckBox.setText("show inherited");
        inheritenceCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                inheritenceCheckBox_actionPerformed(e);
            }
        });

        this.add(targetLabel, null);
        this.add(methodSelector, null);
        this.add(invokeButton, null);
    }

    public IdlModel getModel() {
        return model;
    }

    public void modelChanged(ModelChangeEvent evt) {
        model = evt.getModel();
        method = (IdlOperation)model.getRoot();
    }
                
    void invokeButton_actionPerformed(ActionEvent e) {
        if (tracingService == null) {
            try {
                tracingService = org.coach.tracing.api.pi.TracingServiceHelper.narrow(org.objectweb.openccm.corba.TheORB.getORB().resolve_initial_references("TracingService"));     
                tracingService.start();
                System.err.println("************ starting tracing service " + Thread.currentThread().getName());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
        if (ref == null) {
            JOptionPane.showMessageDialog(null, "target not set!");
        } else {
            try {
                IdlReply reply = method.invoke(orb, ref);
                if (reply != null) {
                    if (replyModel == null) {
                        replyModel = new IdlModel(reply);
                        methodPanel.setReply(replyModel);
                    } else {
                        replyModel.setRoot(reply);
                        replyModel.reload();
                    }
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                JOptionPane.showMessageDialog(null, ex.toString());
            }
        }
    }

    void methodSelector_actionPerformed(ActionEvent e) {
        if (method != null) {
            int idx = methodSelector.getSelectedIndex();
            method = types[idx];
            model.setRoot(method);
            model.reload();
            methodPanel.setOperation(model);
            replyModel = null;
        }
    }

    void inheritenceCheckBox_actionPerformed(ActionEvent e) {
        initMethods();
    }
}
