/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.actor.namingBrowser;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.io.*;

public class NameModel extends DefaultTreeModel {
    private TreePath selection;
    private NameNode nodeRoot;

    public NameModel(ORB orb) {
        super(null);
        try {
            NamingContext rootCtx = null;
            String iorFile = System.getProperty("ior.file");
            if (iorFile != null) {
                try {
                    BufferedReader file = new BufferedReader(new FileReader(iorFile));
                    rootCtx = NamingContextExtHelper.narrow(orb.string_to_object(file.readLine()));
                } catch (Exception e) {
                }
            }
            if (rootCtx == null) {
                rootCtx = NamingContextExtHelper.narrow(orb.resolve_initial_references("NameService"));
            }
            NameComponent ncmp = new NameComponent("root", "context");
            NameComponent[] path = {ncmp};
            Binding bi = new Binding(path, BindingType.ncontext);

            NameNode.setOrb(orb);
            nodeRoot = new NameNode(bi, rootCtx);
            refresh();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public NameModel(NameNode root) {
        super(root);
        nodeRoot = root;
        refresh();
    }

    public TreePath getSelection() {
        return selection;
    }

    public void setSelection(TreePath s) {
        selection = s;
    }

    public void refresh() {
        NameNode newRoot = (NameNode)nodeRoot.clone();
        newRoot.explore();
        setRoot(newRoot);
    }
}