/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import java.io.*;
import java.util.*;
import java.util.jar.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.lang.reflect.*;

public class IdList {
    private static FNFilter filter;
    private static DirFilter dirFilter;
    private static JTree tree;
    
    private IdList() {
    }
        
    static {
        try {
            List list = new LinkedList();
            try {
                filter = new FNFilter("Helper.class");
                dirFilter = new DirFilter();
                String path[] = IdlNode.splitName(System.getProperty("java.class.path"), File.pathSeparator);               
                for (int i = 0; i < path.length; i++) {
                    if(path[i].endsWith(".jar")) {
                        addIds(list, path[i]);
                    } else {
                        addIds(list, new File(path[i]), path[i]);             
                    }
                }
            } catch (Throwable t) {
            } 
            IdNode n = new IdNode();
            n.add("octet");
            n.add("short");
            n.add("ushort");
            n.add("long");
            n.add("ulong");
            n.add("longlong");
            n.add("ulonglong");
            n.add("float");
            n.add("double");
            n.add("longdouble");
            n.add("fixed");
            n.add("char");
            n.add("wchar");
            n.add("string");
            n.add("wstring");
            n.add("fixed");
            n.add("any");
    
            Collections.sort(list);
            Iterator it = list.iterator();
            while(it.hasNext()) {
                n.add((String)it.next());
            }
            
            tree = new JTree(new DefaultTreeModel(n));
        } catch (Throwable tt) {
            tt.printStackTrace();
        }  
    }
        
    public static JTree getTree() {
        return tree;
    }

    private static void addIds(List list, File dir, String root) {
        File[] files  = dir.listFiles(filter);
        for (int i = 0; i < files.length; i++) {
            String name = files[i].getPath();
            name = name.substring(root.length() + 1, name.lastIndexOf("."));
            name = name.replace('/', '.');
            name = name.replace('\\', '.');
            if (!name.endsWith(".Helper")) {
                try {
                    Class hc = Class.forName(name);
                    Method hm = hc.getMethod("id", null);
                    list.add((String)hm.invoke(null, null));
                } catch (Exception e) {
                    System.err.println("Can't create class for: " + name);
                }
            }
        }

        File[] dirs = dir.listFiles(dirFilter);
        for (int i = 0; i < dirs.length; i++) {
            addIds(list, dirs[i], root);
        }
    }        

    private static void addIds(List list, String jarName) {
        File jarFile = new File(jarName);
        JarEntry jarEntry = null;
        try {
            JarInputStream input = new JarInputStream(new BufferedInputStream(new FileInputStream(jarFile)));
            while ((jarEntry = input.getNextJarEntry()) != null) {
                String name = jarEntry.getName();
                if (name.endsWith("Helper.class")) {
                    Class hc = Class.forName(name.substring(0, name.lastIndexOf(".")).replace('/', '.').replace('\\', '.'));
                    Method hm = hc.getMethod("id", null);
                    list.add((String)hm.invoke(null, null));
                }                
            }
        } catch (Exception e) {
        }
    }
            
/*                                       
    public String[] getList() {
    }
*/

    private static class FNFilter implements FilenameFilter {
        private String ext;

        public FNFilter(String ext) {
            this.ext = ext;
        }
    
        public boolean accept(File dir, String name) {
            File file = new File(dir, name);
            if (file.isFile() && name.endsWith(ext)) {
                return true;
            }
            return false;
        }
    }

    private static class DirFilter implements FilenameFilter {
        public boolean accept(File dir, String name) {
            File file = new File(dir, name);
            if (file.isDirectory()) {
                return true;
            }
            return false;
        }
    }
    
    private static class IdNode extends DefaultMutableTreeNode {
        private boolean isLeaf;
        
        public IdNode() {
            this(false);
        }

        public IdNode(boolean leaf) {
            isLeaf = leaf;
            setUserObject("");
        }
    
        public void add(String id) {
            int idx = id.indexOf("/");
            if (idx >= 0) {
                String n1 = id.substring(0, idx);
                String n2 = id.substring(idx + 1);
                IdNode c = getChild(n1);
                c.add(n2);
            } else {
                add(getLeaf(id));
            }
        }

        private IdNode getLeaf(String n) {
            IdNode c = new IdNode(true);
            c.setUserObject(n);
            return c;
        }
            
        private IdNode getChild(String n) {
            IdNode c = null;
            for (int i = 0; i < getChildCount(); i++) {
                c = (IdNode)getChildAt(i);
                if (((String)c.getUserObject()).equals(n)) {
                    return c;
                }
            }
            c = new IdNode();
            c.setUserObject(n);
            add(c);
            return c;
        }
    }                
}