package org.coach.idltree;

import org.w3c.dom.Node;
import javax.swing.tree.*;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;
import org.omg.CORBA.ORB;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;
import org.coach.util.IorPrinter;

public class IdlComponent extends IdlInterface implements IdlWritable {    
    protected String[] facets;

    protected IdlComponent() {
        setUserObject(this);
        type = "Component";
        value = "null";
        tc = orb.get_primitive_tc(TCKind.tk_objref);
    }

    protected IdlComponent(Any any) {
        this();
        setNode(any.type());
        setNode(any);
    }
        
    protected IdlComponent(TypeCode tc) {
        this();
        setNode(tc);
    }

    /**
     * Assigns the node value from a string.
     *
     * @param v The string from which to convert the value.
     */
    public void setValue(String v) {
        if (v.toUpperCase().equals("NULL")) {
            value = "null";
            id = "";
        } else {
            int idx = v.indexOf("IOR:");
            if (idx < 0) {
                throw new RuntimeException("value is not an IOR: " + v);
            }
            value = v.substring(idx);
            try {
                IorPrinter iorPrinter = new IorPrinter(value);
                id = iorPrinter.getTypeId();
            } catch (Exception e) {
                id = "";
            }
        }
    }
       
    // XML section

    /**
     * Create an IdlInterface node from an Xml representation.
     *
     * XML format example:
     * <pre>
     *   &lt;component&gt;id="IDL:examples/MyInterface:1.0"&gt;
     *       IOR:000....
     *   &lt;/component&gt;
     * </pre>
     *
     * @param xml The XML string from which to create an IdlInterface instance.
     */
    public IdlComponent(String xml) {
        this(XmlNode.getNode(xml));
    }
      
    /**
     * Construct an XmlNode from an Xml node.
     * Package access only. Called from the Xml parser to construct
     * an Xml type tree from an Xml representation.
     */
    IdlComponent(Node n) {
        this();
        try {
            if (n == null || !n.getNodeName().toUpperCase().equals("COMPONENT")) {
                throw new RuntimeException("component expected");
            }
            setNode(XmlNode.type(XmlNode.getId(n)));   
            setValue(XmlNode.getText(n));
            operations = XmlNode.getOperations(id);
        } catch (Exception e) {
            throw new RuntimeException(e.toString());
        }
    }

    public IdlComponent(String id, boolean dummy) {
        this();
        try {
            this.id = id;
            setNode(XmlNode.type(id));
            operations = XmlNode.getOperations(id);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public String[] getFacets() {
        return facets;
    }

   
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        w.write_component(id, value);
    }
}