/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import org.w3c.dom.Node;
import org.omg.CORBA.TypeCode;
import org.omg.CORBA.Any;
import org.omg.CORBA.TCKind;

/**
 * The class IdlUlonglong represents an CORBA IDL unsigned long long value. Instances are created through one
 * of the create() factory methods in IdlNode.
 * An IdlUlonglong object has no child nodes.
 *
 * @author <a href="mailto:batteram@lucent.com">Harold Batteram</a> <b>Lucent Technologies</b>
 */
public class IdlUlonglong extends IdlNode implements IdlWritable {
    protected IdlUlonglong() {
        setUserObject(this);
        type = "ulonglong";
        value = "0";
        tc = orb.get_primitive_tc(TCKind.tk_ulonglong);
    }

    protected IdlUlonglong(Any any) {
        this();
        setNode(any);
    }
        
    protected IdlUlonglong(TypeCode tc) {
        this();
        setNode(tc);
    }
    
    protected void setNode(Any any) {
        try {         
            setNode(any.type());     
            value = "" + any.create_input_stream().read_ulonglong();
        } catch (Exception e) {
            e.printStackTrace();
        }      
    }

    /**
     * Returns the current value as a CORBA Any value.
     *
     * @return The current value as a CORBA Any.
     */
    public Any toAny() {
        try {
            Any any = orb.create_any();
            org.omg.CORBA.portable.OutputStream out = any.create_output_stream();
            any.type(tc);
            out.write_ulonglong(Long.parseLong(value));
            any.read_value(out.create_input_stream(), tc);
            return any;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Writes the value to a CORBA outputstream.
     *
     * @param is The outputstream to write to.
     */
    public void write(org.omg.CORBA.portable.OutputStream os) {
        try {
            os.write_ulonglong(Long.parseLong(value));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Reads the value from a CORBA inputstream.
     *
     * @param is The inputstream to read from.
     */
    public void read(org.omg.CORBA.portable.InputStream is) {
        value = "" + is.read_ulonglong();
    }
    
    // XML section

    /**
     * Create an IdlUlonglong node from an Xml representation.
     * XML format example:
     * <pre>
     *   &lt;ulonglong&gt;3&lt;/ulonglong&gt;
     * </pre>
     */
    public IdlUlonglong(String xml) {
        this(XmlNode.getNode(xml));
    }
    
    /**
     * Construct an XmlNode from an Xml node.
     * Package access only. Called from the Xml parser to contruct
     * an Xml type tree from an Xml representation.
     */
    IdlUlonglong(Node n) {
        this();
        if (n == null || !n.getNodeName().toUpperCase().equals("ULONGLONG")) {
            throw new RuntimeException("ulonglong expected");
        }
        setValue(XmlNode.getText(n));
    } 
 
    /**
     * Write the current value to an IdlWriter object.
     *
     * @param w The IdlWriter object to write the current value to.
     */
    public void write(IdlWriter w) {
        w.write_ulonglong(value);
    }
}