/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA & USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba;

// Package dependencies.
import org.objectweb.util.misc.api.ExceptionWrapper;

/**
 * This class encapsulates a CORBA::UserException exception
 * allowing us to avoid to throw them in every operation sypnopsis.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class UserExceptionWrapper
     extends ExceptionWrapper
  implements UserExceptionHolder
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    UserExceptionWrapper()
    {
        // Calls the ExceptionWrapper constructor.
        super();
    }

    /**
     * The constructor with the CORBA::UserException object to wrap.
     *
     * @param userException The CORBA::UserException object to wrap.
     */
    public
    UserExceptionWrapper(org.omg.CORBA.UserException userException)
    {
        // Calls the ExceptionWrapper constructor.
        super(userException);
    }

    /**
     * The constructor with the CORBA::UserException object to wrap
     * and a message.
     *
     * @param userException The CORBA::UserException object to wrap.
     * @param message The message describing the exception.
     */
    public
    UserExceptionWrapper(org.omg.CORBA.UserException userException,
                         String message)
    {
        // Calls the ExceptionWrapper constructor.
        super(userException, message);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.corba.util.api.UserExceptionHolder
    //
    // ==================================================================

    /**
     * Obtains the wrapped CORBA::UserException object.
     *
     * @return The wrapped CORBA::UserException object.
     */
    public org.omg.CORBA.UserException
    getUserException()
    {
        return (org.omg.CORBA.UserException)getException();
    }

    /**
     * Sets the wrapped CORBA::UserException object.
     *
     * @param userException The wrapped CORBA::UserException object.
     */
    public void
    setUserException(org.omg.CORBA.UserException userException)
    {
        setException(userException);
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
