/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): Philippe Merle

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.objectweb.openccm.corba.*;

/**
 * This is the implementation of the NodeInformation facet provide
 * by the NodeManager component.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class NodeInformationSegImpl 
    extends  org.objectweb.openccm
             .OpenCCM_DCI.NodeManagerSessionComposition
             .NodeInformationSeg
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The NPD reprenstation tree */
    private org.objectweb.openccm.descriptor.node.beans.NodeBean nodeTree;

    /** all the descriptors used by the node */
    private org.ist.coach.DCI.DCIDescriptor[] descriptors;

    /** Prefix for output message */
    private String prefix="NodeInformation >> ";

    // ==================================================================
    //
    // Internal methods..
    //
    // ==================================================================

    // ==================================================================
    //
    // constructor
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * TODO simplify the node properties reading
     * This method set the NodePropertiesDescriptor used to configure
     * the node.
     * @param npdFilePath the node properties file path
     */
    public void 
    setTree(String npdFilePath)
    {
        try
        {
            //open the node properties file
            java.io.File nodeFile= new java.io.File (npdFilePath);
            //unmarshall the node properties file
            nodeTree=
                org.objectweb.openccm
                .descriptor.node.beans
                .NodeBeanImpl.unmarshalBean(nodeFile);
            //set the node_name with the node_tree information or
            // the java property if no NPF was define
            if(System.getProperty("NodeName")!=null){                
                System.err.println(prefix+ "Used NodeName java property");
                System.err.println(prefix+ "No user NPD define");
                ((org.objectweb.openccm.OpenCCM_DCI.NodeManagerCCM)get_context())
                    .node_name(System.getProperty("NodeName"));     
            }
            else               
                ((org.objectweb.openccm.OpenCCM_DCI.NodeManagerCCM)get_context())
                    .node_name(nodeTree.getId());            

            //save the NodePropertiesDescriptor
            descriptors=new org.ist.coach.DCI.DCIDescriptor[1];
            descriptors[0]=new org.ist.coach.DCI.DCIDescriptor();
            descriptors[0].filename=npdFilePath;
            descriptors[0].type=org.ist.coach.DCI.DCIDescriptorType.NPD;
            
            java.io.BufferedReader br=
                new java.io.BufferedReader(new java.io.FileReader(nodeFile));
            //Read the descriptor and store it.
            while(br.ready())
                descriptors[0].descriptor= 
                    descriptors[0].descriptor+br.readLine()+"\n";
                       
            //Setup the componentinstallation directory
            
            if(System.getProperty("ComponentInstallationDir")!=null){    
                System.err.println(prefix+ "Used ComponentInstallationDir java property");
                System.err.println(prefix+ "No user NPD define");
                ((ExtComponentInstallationImpl)
                 get_main_segment().obtain_executor("component_installation"))
                    .setComponentInstallationDir(System.getProperty("ComponentInstallationDir"));
            }
            else    
                ((ExtComponentInstallationImpl)
                 get_main_segment().obtain_executor("component_installation"))
                    .setComponentInstallationDir(nodeTree.getProperties()
                                                 .getProperty().getValue()
                                                 .getValue());               
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
    }
    
    /**
     * Return a propety value which name is properties_name
     * @param properties_name The name of the property
     * @return A corba Any object containing the property value
     */
    public  org.omg.CORBA.Any
    get_property_value(String properties_name)
    {
        // TODO: Must be implemented!
        throw new org.omg.CORBA.NO_IMPLEMENT();            
    }
    
    /**
     * Return a node descriptor
     * @return The descriptors
     */
    public  org.ist.coach.DCI.DCIDescriptor[]
    get_node_properties()
    {        
        if(descriptors!=null)
            return descriptors;                    
            
        return new org.ist.coach.DCI.DCIDescriptor[0];
    }
    
    /**
     * Test if a property is set.
     * @param properties_name The name of the property to test
     * @return The result of the test
     */
    public boolean
    is_property_defined(java.lang.String propertyName)
    {
        // TODO: Must be implemented!
        throw new org.omg.CORBA.NO_IMPLEMENT();            
    }
}
