/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base;

/**
 * The ErrorManager interface specify all the method that a ErrorManager
 * must implement to be valide.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public interface ErrorManager {

    /**
     * The submitException method is in charge to evaluate the gravity of
     * a DeployerFailureException. According to the user configuration if
     * the exception is considered as a fatal error, the FatalDeploymentException
     * is thrown. In another case a simply log is done.
     * 
     * @param e The exception to evaluate
     * @throws FatalDeploymentException thrown if this exception is configure as a
     * fatal deployment error.
     */
    public void 
    submitException(DeployerFailureException e)
    throws FatalDeploymentException;
    
    /**
     * The submitException method is in charge to evaluate the gravity of
     * a InitializationError. According to the user configuration if
     * the exception is considered as a fatal error, the FatalDeploymentException
     * is thrown. In another case a simply log is done.
     * 
     * @param e The exception to evaluate
     * @throws FatalDeploymentException thrown if this exception is configure as a
     * fatal deployment error.
     */
    public void 
    submitException(InitializationError e)
    throws InitializationError;    
    
    /**
     * TODO: -->Must be deprecated
     */
    // public void connectObservedDeployer(DeployerContext deployer);
    
    public void 
    connectDeploymentLogger(DeploymentLogger deploymentLogger);       
    
}