/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base;

/**
 * The InitializationError is thrown during the building of the 
 * deployment machine if an error occured.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Frederic Briclet</a>
 *
 * @version 0.1
 */
public class InitializationError 
       extends DeployerFailureException 
    {
    
    //  ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private int gravity=0;
    // ==================================================================
    //
    // Constructors.
    //
    // ==============================s====================================
    /**
     * This constructor is used when the deployment machine detect a problem
     * and want to signal it.
     * @param deployer The deployer concern by the error
     * @param message the error explaination message 
     */
    public InitializationError(DeployerContext deployer, 
                               String message)
    {
        super(deployer,null,message);
    }
    /**
     * This constructor is used when an unwanted exception occur during the 
     * deployment machine building.
     * 
     * @param deployer The deployer concern by the exception
     * @param e The exception that occured
     * @param message An explaination message
     */
    public InitializationError(DeployerContext deployer,
                               Exception e, 
                               String message)
    {
        super(deployer,e,message);      
    }

    /**
     * This constructor is used when an unwanted exception occur during the 
     * deployment machine building.
     * 
     * @param deployer The deployer concern by the exception
     * @param e The exception that occured
     * @param message An explaination message
     */
    public InitializationError(DeployerContext deployer,
                               Exception e, 
                               String message,
                               int gravity)
    {
        super(deployer,e,message);
        this.gravity=gravity;
    }
    //  ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    //  ==================================================================
    //
    //  Public methods 
    //
    // ================================================================== 
  
    public void
    printStackTrace()
    { 
        
        
        System.err.println(getMessage());
        if(gravity==1&&getInitialException()!=null)
            getInitialException().printStackTrace();
        else 
            if(getInitialException()!=null)
                System.err.println(getInitialException().getMessage());
        //comment to compile on jdk1.3.1
        //System.err.println("\t at:"+super.getStackTrace()[0].toString());
        //System.err.println("\t at:"+super.getStackTrace()[1].toString());
        
    
    }
    
    public String 
    getMessage()
    {   if(message==null)
            message="";
            
        if(getDeployer()==null)
            return "The deployment machine building was aborted during "+
               " initialization due to\n"+message;
               
                           
        return "The deployment machine building was aborted when tried "+
               "to initialize "+getDeployer().getIdentifier()+" due to :\n"+message;
    }
}
