/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.openccm.descriptor.componentassembly.beans.ComponentinstantiationrefBeanImpl;
import org.objectweb.openccm.descriptor.componentassembly.beans.ComponentsupportedinterfaceBeanImpl;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.HandlerBase;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer.ComponentinstantiationDeployer;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.util.FindbyDeployer;

/**
 * This handler is used to produce componentsupportedinterface deployer from an
 * componentsupportedinterface bean.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ComponentsupportedinterfaceDeployerHandler 
       extends HandlerBase 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    //The current deployer
    private ComponentsupportedinterfaceDeployer _deploy;
     //  ==================================================================
     //
     // Constructors.
     //
     // ==================================================================
     // ==================================================================
     //
     // Internal methods.
     //
     // ==================================================================
     // ==================================================================
     //
     //  Public methods.
     //
     // ==================================================================
    /**
     * Produce the componentsupportedinterface deployer binded
     * to the givean @param cons.
     * 
     * @param cons The componentsupportedinterface bean 
     * @return the deployer produced in initialize state
     * @throws InitializationError thrown if a problem is detected
     */
    public Object 
    getInstance(ComponentsupportedinterfaceBeanImpl cons)
    throws InitializationError 
    {
        if (_deploy == null) {
            _deploy = new ComponentsupportedinterfaceDeployer();
            _deploy.connectComponentsupportedinterface(cons);
            applyCommonConfig(_deploy);
            if (cons.getFindby() != null) {
                _deploy.connectFindbyDeployer(
                    (FindbyDeployer) getDeployer(cons.getFindby()));
            }
            if (cons.getComponentinstantiationref() != null
                && cons.getComponentinstantiationref().getIdref() != null) {
                try {

                    _deploy.connectComponentinstantiationDeployer(
                        (ComponentinstantiationDeployer) getDeployer(
                            ((ComponentinstantiationrefBeanImpl) cons
                            .getComponentinstantiationref())
                            .pullIdref()));
                } catch (java.lang.ClassCastException e) {
                     String classFound=((ComponentinstantiationrefBeanImpl)cons.getComponentinstantiationref())
                                                   .pullIdref().getClass().getName();
                     throw new InitializationError(_deploy,"Similar identifiants was detected, it was expected that \""+
                                                    cons.getComponentinstantiationref().getIdref()+"\" id of <componentsupportedinterface> element"+
                                                  " denote a <componentinstantiation> element:\n"
                                                   +getStringifiedDescription(cons)
                                                   +"\n but a <"
                                                   +classFound.substring(classFound.lastIndexOf('.')+1,classFound.lastIndexOf("BeanImpl"))
                                                   +"> element was found:\n "
                                                   +getStringifiedDescription(((ComponentinstantiationrefBeanImpl)cons.getComponentinstantiationref()).pullIdref())
                                                   +"check your CAD file a identifiant is used for two different elements."+
                                                   " To achieve successfully the deployment use different names for those two identifiants");

                }
            }
            _deploy.initialize();
        }
    
        return (_deploy);
    }

}
