/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder;

// importation of the needed packages
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.DeployerActiveState;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.DeployerInactiveState;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.FatalDeploymentException;

/**
 * This deployer is in charge to build synchronous and asynchronous connections. 
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ConnectionsDeployer
       extends ConnectionsDeployerContext
    {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * establishes all the synchronous and asynchronous connections
     * @exception FatalDeploymentException is thrown if a connection
     * failed.
     */
    public void 
    deploy()
    throws FatalDeploymentException
    {     
        // browse the connectinterface list
        traverse(this,getConnectinterfaceDeployers());         
        //browse the connectevent list
        traverse(this,getConnecteventDeployers());
        getLifeCycleManager().stepEndedSuccessFully();
    }
    
    /**
     * The visitor method to establish event connection.
     * 
     * @param deployer The event deployer to proceed.
     * @param status The status of the current deployer.
     * @throws FatalDeploymentException is thrown if a fatal problem occured
     */
    public void
    visit(ConnecteventDeployer deployer, DeployerInactiveState status)
    throws FatalDeploymentException
    {
        try{
            deployer.deploy();
        }
        catch(ConnectEventEstablishmentFailureException e){
                getErrorManager().submitException(e);
        }
    }
    
    /**
     * The visitor method to establish interface connection.
     * 
     * @param deployer The event deployer to proceed.
     * @param status The status of the current deployer.
     * @throws FatalDeploymentException is thrown if a fatal problem occured
     */    
    public void 
    visit(ConnectinterfaceDeployer deployer,DeployerInactiveState status)
    throws FatalDeploymentException{
        try{
            deployer.deploy();
        }
        catch(ConnectInterfaceEstablishmentFailureException e){
                        getErrorManager().submitException(e);
        }

    }
    /**
     * Tear down all the connection.
     * @param FatalDeploymentException is thrown if a problem occured
     * during disconnecting.             
     */
    public void 
    tearDown()
    throws FatalDeploymentException
    {
       traverse(this,getConnectinterfaceDeployers());
       traverse(this,getConnecteventDeployers());
       getLifeCycleManager().stepEndedSuccessFully();            
    }
    
    /**
     * The visitor method to establish event connection.
     * 
     * @param deployer The event deployer to proceed.
     * @param status The status of the current deployer.
     * @throws FatalDeploymentException is thrown if a fatal problem occured
     */
    public void visit(ConnecteventDeployer deployer, DeployerActiveState status)
    throws FatalDeploymentException{
        try{
            deployer.tearDown();
        }
        catch(ConnectEventBreakingFailureException e){
            getErrorManager().submitException(e);
        }
    }
    
    /**
     * The visitor method to break interface connection.
     * 
     * @param deployer The event deployer to proceed.
     * @param status The status of the current deployer.
     * @throws FatalDeploymentException is thrown if a fatal problem occured
     */
    public void visit(ConnectinterfaceDeployer deployer,DeployerActiveState status)
    throws FatalDeploymentException{
        try{
            deployer.tearDown();
        }
        catch(ConnectInterfaceBreakingFailureException e){
              getErrorManager().submitException(e);
        }

    }
    
}
