/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.openccm.descriptor.componentassembly.Consumesport;
import org.objectweb.openccm.descriptor.componentassembly.beans.ConsumesportBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer.ComponentinstantiationDeployer;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.util.FindbyDeployer;

/**
 * The consumesport deployer context defines all the connection requires for
 * consumer resolution.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ConsumesportDeployerContext
       extends ChildDeployerContext 
    {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================  
    private ConsumesportBean consumesport;
    private FindbyDeployer findby;
    private ComponentinstantiationDeployer componentinstantiationDeployer;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessor methods.
    //
    // ==================================================================
  
    public void 
    connectConsumesport(ConsumesportBean consumesport)
    {
        this.consumesport=consumesport;
        connectDeployerDescription(consumesport);
    }
    
    public Consumesport 
    getConsumesport()
    {
        return consumesport;       
    }
    
    public void 
    connectFindbyDeployer(FindbyDeployer findby)
    {
        this.findby=findby;    
    }
    
    public FindbyDeployer
    getFindbyDeployer()
    {
        return findby;
    }
    /**
     * @return
     */
    public ComponentinstantiationDeployer 
    getComponentinstantiationDeployer()
    {
       return componentinstantiationDeployer;
    }

    /**
    * @param deployer
    */
    public void
    connectComponentinstantiationDeployer(ComponentinstantiationDeployer deployer) 
    {
       componentinstantiationDeployer = deployer;
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError
    {
        if(getConsumesport()==null)
            getErrorManager()
           .submitException(new InitializationError(this,"Connection on consumesport bean is required"+
                                              "check the handler code it must specify this connection"));
                                          
        if(getConsumesport().getComponentinstantiationref()!=null&&(
            getConsumesport().getComponentinstantiationref().getIdref()==null ||
            getConsumesport().getComponentinstantiationref().getIdref().length()==0)) 
                getErrorManager()
              .submitException(new InitializationError(this,"The idref attribute of the componentinstantiationref element"+
                                                   "must be fullfilled, check the consumesport element"+
                                                   "description\n."+getStringifiedDeployerDescription()));
        if(getConsumesport().getComponentinstantiationref()==null&&getFindbyDeployer()==null)
            getErrorManager()
          .submitException(new InitializationError(this,"One of componentinstantiationref/findby children must be"+
                                               "specified check the consumesport element"+
                                               "description\n."+getStringifiedDeployerDescription()));                                                              
        getLifeCycleManager().stepEndedSuccessFully();
    }
   

}
