/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.binder;

import org.objectweb.openccm.descriptor.componentassembly.Existinginterface;
import org.objectweb.openccm.descriptor.componentassembly.beans.ExistinginterfaceBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.util.FindbyDeployer;

/**
 * Theexistinginterface deployer context defines all the connection requires for
 * interface resolution
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ExistinginterfaceDeployerContext extends ChildDeployerContext {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private ExistinginterfaceBean existingInterface;
    private FindbyDeployer findbyDeployer;    

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessor methods.
    //
    // ==================================================================  
    
    /**
     * @return
     */
    public Existinginterface 
    getExistingInterface() 
    {
        return existingInterface;
    }

    /**
     * @return
     */
    public FindbyDeployer 
    getFindbyDeployer() 
    {
        return findbyDeployer;
    }

    /**
     * @param existinginterface
     */
    public void 
    connectExistingInterface(ExistinginterfaceBean existinginterface) 
    {
        existingInterface = existinginterface;
        connectDeployerDescription(existinginterface);
    }

    /**
     * @param deployer
     */
    public void 
    connectFindbyDeployer(FindbyDeployer deployer) 
    {
        findbyDeployer = deployer;
    }
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */    
    public void 
    initialize() 
    throws InitializationError
    {
        if(getExistingInterface()==null)
            getErrorManager()
           .submitException(new InitializationError(this,"Connection on existinginterface bean is required"+
                                              "check the handler code it must specify this connection"));
    
        if(getFindbyDeployer()==null)
            getErrorManager()
          .submitException(new InitializationError(this,"Findby child must be"+
                                                "specified check the existinginterface element"+
                                                  "description\n."+getStringifiedDeployerDescription()));                                                              
        getLifeCycleManager().stepEndedSuccessFully();
    }
}
