/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

import java.util.Iterator;
import java.util.LinkedList;
import org.objectweb.openccm.descriptor.componentassembly.Componentfile;
import org.objectweb.openccm.descriptor.componentassembly.beans.ComponentfileBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.*;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.util.FileinarchiveDeployer;
import org.objectweb.openccm.descriptor.softpkg.beans.ImplementationBean;
import org.objectweb.openccm.descriptor.softpkg.beans.ImplementationBeanImpl;
import org.objectweb.openccm.descriptor.softpkg.beans.SoftpkgBean;
import org.objectweb.openccm.descriptor.softpkg.ccm.ImplementationDeployer;
import org.objectweb.openccm.descriptor.softpkg.ccm.SoftpkgDeployer;

/**
 * The componentfile deployer context defines all the connection requires for
 * softpkg tree initialization.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public abstract class ComponentfileDeployerContext
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private FileinarchiveDeployer fia;
    private ComponentfileBean comp;
    private SoftpkgBean softpkg;
    private SoftpkgDeployer softpkgDeployer;
    private LinkedList implementationBeans;
    private LinkedList implementationDeployers;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================    
    public ComponentfileDeployerContext() {
        implementationBeans = new LinkedList();
        implementationDeployers = new LinkedList();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================   
    public void 
    connectImplementation(ImplementationBean impl)
    {
        implementationBeans.addLast(impl);
    }

    public ImplementationBean[] 
    getImplementations() 
    {
        return (ImplementationBean[]) implementationBeans.toArray(
            new ImplementationBeanImpl[implementationDeployers.size()]);
    }

    public void 
    connectImplementationDeployer(ImplementationDeployer impl) 
    {
        implementationDeployers.addLast(impl);
    }

    public ImplementationDeployer[] 
    getImplementationDeployers() 
    {
        return (ImplementationDeployer[]) implementationDeployers.toArray(
            new ImplementationDeployer[implementationDeployers.size()]);
    }

    public ImplementationDeployer 
    getImplementationDeployer(String id)
    {
        for (Iterator it = implementationDeployers.iterator(); it.hasNext();) {
            ImplementationDeployer impl = (ImplementationDeployer) it.next();
            if (impl.getImplementation().getId().equals(id))
                return impl;
        }
        return null;
    }

    public void 
    connectFileinarchiveDeployer(FileinarchiveDeployer fia) 
    {
        this.fia = fia;
    }

    public FileinarchiveDeployer 
    getFileinarchiveDeployer() 
    {
        return fia;
    }

    public void 
    connectComponentfile(ComponentfileBean comp) 
    {
        this.comp = comp;
        connectDeployerDescription(comp);
    }

    public Componentfile 
    getComponentfile() 
    {
        return comp;
    }
    
    /**
     * @return
     */
    public SoftpkgBean 
    getSoftpkg() 
    {
        return softpkg;
    }

    /**
     * @param bean
     */
    public void 
    connectSoftpkg(SoftpkgBean bean) 
    {
        softpkg = bean;
    }

    /**
     * @return
     */
    public SoftpkgDeployer 
    getSoftpkgDeployer() 
    {
        return softpkgDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectSoftpkgDeployer(SoftpkgDeployer deployer) 
    {
        softpkgDeployer = deployer;
    }

    public abstract void 
    initializeSoftpkgTree() 
    throws InitializationError;
    
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError 
    {
        if (getComponentfile() == null)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "Connection on Componentfile is required"
                        + "check the handler code it must specify this connection"));

        if (getFileinarchiveDeployer() == null)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "A fileinarchive must be specified in componentfile element,"
                        + "check you componentfiles element description.\n"
                        + getStringifiedDeployerDescription()));
        initializeSoftpkgTree();
        getLifeCycleManager().stepEndedSuccessFully();
    }


}
