/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;


import java.util.LinkedList;
import org.objectweb.openccm.descriptor.componentassembly.Componentplacement;
import org.objectweb.openccm.descriptor.componentassembly.beans.ComponentplacementBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.*;

/**
 * The componentplacement deployer context defines all the connection requires for
 * install a component.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class ComponentplacementDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================    
    private ExistinghomeDeployer existinghome;
    private ComponentplacementBean componentPlacement;
    private LinkedList componentInstantiations;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    public ComponentplacementDeployerContext(){
        this.componentInstantiations=new LinkedList();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accesors methods.
    //
    // ==================================================================   

    
    
    public void 
    connectComponentplacement(ComponentplacementBean componentPlacement)
    {
        this.componentPlacement=componentPlacement;
        connectDeployerDescription(componentPlacement);
    }
    
    public Componentplacement 
    getComponentplacement()
    {
        return componentPlacement;
    }
    
    public void 
    connectExistinghomeDeployer(ExistinghomeDeployer existinghome)
    {
        this.existinghome=existinghome;
    }
    
    public ExistinghomeDeployer 
    getExistinghomeDeployer()
    {
        return existinghome;
    }
    
    public void 
    connectComponentinstantiationDeployer
    (ComponentinstantiationDeployer componentInstantiation)
    {
        this.componentInstantiations.addLast(componentInstantiation);    
    }
    
    public ComponentinstantiationDeployer[]
    getComponentinstantiationDeployer()
    {
        return  (ComponentinstantiationDeployer [])
                componentInstantiations.toArray
                (new ComponentinstantiationDeployer[componentInstantiations.size()]);
    }
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */   
    public void 
    initialize() 
    throws InitializationError
    {
        if(getComponentplacement()==null)
            getErrorManager()
           .submitException( new InitializationError(this,"Connection on componentplacement bean is required"+
                                          "check the handler code it must specify this connection"));
                                          
        if(getComponentinstantiationDeployer().length==0){    
            getErrorManager()
           .submitException(new InitializationError(this,"At least one componentinstantiation must be specified in"+
                                          "componentplacement element check you componentplacement element"+
                                          "description.\n"+getStringifiedDeployerDescription()));    
        }
          
        if(getExistinghomeDeployer()==null)
                getErrorManager()
               .submitException(new InitializationError(this,"The existinghome element must be specified in"+
                                          "componentplacement element check you componentplacement element"+
                                          "description."+getStringifiedDeployerDescription()));
        getLifeCycleManager().stepEndedSuccessFully();
    }
    
}

