/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

/**
 * The DestinationDeployerFactory interface defines the methods that a destination
 * deployer factory must implements to be used by the deployment machine.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public interface DestinationDeployerFactory {

    /**
     * getDestinationDeployer return the right DestinationDeployer able to
     * work with the destination type precised with @param destinationtype
     * 
     * @param destinationtype the destinationtype key
     * @throws UnknownDestinationDeployer if this target type is not supported 
     */ 
    public DestinationDeployerBase
    getDestinationDeployer(String destination)
    throws UnknownDestinationDeployer;
    
    /**
     * addDeployer is used to add a new kind of destination to support. 
     * The prefixIdentifier is the kind key used in Componentassembly 
     * descriptor to precise the destination kind.
     * 
     * @param prefixIdentifier the destination kind key.
     * @param className The full classname include package name
     * where the class can be found for being instantiate.
     */    
    public void 
    addDeployer(String prefixIdentifier,
                String className);                
                
}
