/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

import java.util.LinkedList;

import org.objectweb.openccm.descriptor.componentassembly.Hostcollocation;
import org.objectweb.openccm.descriptor.componentassembly.beans.HostcollocationBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;

/**
 * The hostcollocation deployer context defines all the connection requires for
 * deploy a homeplacement collocation.
 *
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class HostcollocationDeployerContext 
       extends ChildDeployerContext 
    {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private HostcollocationBean hostcollocation;
    private LinkedList homeplacementDeployers;
    private DestinationDeployer destinationDeployer;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    public HostcollocationDeployerContext(){
        this.homeplacementDeployers=new LinkedList();        
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================    

    public void connectHostcollocation(HostcollocationBean hostcollocation){
        this.hostcollocation=hostcollocation;    
        connectDeployerDescription(hostcollocation);
    }
    
    public Hostcollocation 
    getHostcollocation()
    {
        return hostcollocation;
    }
    
    public void 
    connectHomeplacementDeployer(HomeplacementDeployer home)
    {
        this.homeplacementDeployers.addLast(home);   
    }
    
    public HomeplacementDeployer[] 
    getHomeplacementDeployers()
    {
        return (HomeplacementDeployer[])
                homeplacementDeployers.toArray
                (new HomeplacementDeployer[homeplacementDeployers.size()]);
    }
    /**
     * @return
     */
    public DestinationDeployer 
    getDestinationDeployer() 
    {
        return destinationDeployer;
    }

    /**
     * @param deployer
     */
    public void 
    connectDestinationDeployer(DestinationDeployer deployer) 
    {
        destinationDeployer = deployer;
    }
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */
    public void 
    initialize() 
    throws InitializationError
    {
        if(getHostcollocation()==null)
             getErrorManager()
            .submitException(new InitializationError(this,"Connection on hostcollocation bean is required"+
                                          "check the handler code it must specify this connection"));
        if(getHostcollocation().getDestination()==null)
            getErrorManager()
            .submitException(new InitializationError(this,"The destination element must be precised into"+
                                                          "an hostcollocation\n"+getStringifiedDeployerDescription()));
        if(getHomeplacementDeployers().length==0)
             getErrorManager()
            .submitException(new InitializationError(this,"At least one homeplacement child must be specified "+
                                          "check the hostcollocation element"+
                                          "description\n."+getStringifiedDeployerDescription()));  
        getLifeCycleManager().stepEndedSuccessFully();
   }


}