/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frdric.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.installer;

import java.util.LinkedList;

import org.objectweb.openccm.descriptor.componentassembly.Partitioning;
import org.objectweb.openccm.descriptor.componentassembly.beans.PartitioningBean;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.ChildDeployerContext;
import org.objectweb.openccm.descriptor.componentassembly.ccm.deployer.base.InitializationError;

/**
 * 
 * The partitioningdeployer context defines all the connection requires for
 * processing a partitioning.
 * 
 * @author <a href="mailto:briclet@lifl.fr">Briclet Frdric</a>
 *
 * @version 0.1
 */
public class PartitioningDeployerContext extends ChildDeployerContext {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    private LinkedList homeplacement;
    private LinkedList componentplacement;
    private LinkedList hostcollocation;
    private LinkedList processcollocation;
    private PartitioningBean partitioning;
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================    
    public PartitioningDeployerContext() {
        homeplacement = new LinkedList();
        componentplacement = new LinkedList();
        hostcollocation = new LinkedList();
        processcollocation = new LinkedList();
    }
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    //  Public accessors methods.
    //
    // ==================================================================    
    public void 
    connectHomeplacementDeployer(HomeplacementDeployer homeplacement) 
    {
        this.homeplacement.addLast(homeplacement);
    }
    public HomeplacementDeployer[] 
    getHomeplacementDeployers() 
    {
        return (HomeplacementDeployer[]) this.homeplacement.toArray(
            new HomeplacementDeployer[homeplacement.size()]);
    }

    public void 
    connectComponentplacementDeployer(ComponentplacementDeployer componentplacement) 
    {
        this.componentplacement.addLast(componentplacement);
    }

    public ComponentplacementDeployer[] 
    getComponentplacementDeployers() 
    {
        return (ComponentplacementDeployer[]) this.componentplacement.toArray(
            new ComponentplacementDeployer[componentplacement.size()]);
    }

    public void 
    connectHostcollocationDeployer(HostcollocationDeployer hostcollocation)
    {
        this.hostcollocation.addLast(hostcollocation);
    }

    public HostcollocationDeployer[] 
    getHostcollocationDeployers() 
    {
        return (HostcollocationDeployer[]) this.hostcollocation.toArray(
            new HostcollocationDeployer[hostcollocation.size()]);
    }

    public void 
    connectProcesscollocationDeployer(ProcesscollocationDeployer processcollocation) 
    {
        this.processcollocation.addLast(processcollocation);
    }

    public ProcesscollocationDeployer[] 
    getProcesscollocationDeployers() 
    {
        return (ProcesscollocationDeployer[]) this.processcollocation.toArray(
            new ProcesscollocationDeployer[processcollocation.size()]);
    }
    /**
     * @return
     */
    public Partitioning 
    getPartitioning() 
    {
        return partitioning;
    }

    /**
     * @param partitioning
     */
    public void 
    connectPartitioning(PartitioningBean partitioning) 
    {
        this.partitioning = partitioning;
        connectDeployerDescription(partitioning);
    }
    /**
     * This initialization method is used to check that all the services and 
     * information required to deploy are presents. If all is OK the deployer 
     * is put in inactive deployer state waiting for the deployment processing.
     * 
     * @exception InitializationError is thrown if a problem is detected 
     */    
    public void 
    initialize() 
    throws InitializationError 
    {
        if (getPartitioning() == null)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "Connection on partitioning bean is required"
                        + "check the handler code it must specify this connection"));
        if (getHomeplacementDeployers().length == 0
            && getComponentplacementDeployers().length == 0
            && getHostcollocationDeployers().length == 0
            && getProcesscollocationDeployers().length == 0)
            getErrorManager().submitException(
                new InitializationError(
                    this,
                    "At least one of homeplacement/hostcollocation/processcollocation/"
                        + "componentplacement/hostcollation children must be specified "
                        + "check the partitioning element"
                        + "description\n."
                        + getStringifiedDeployerDescription()));
        getLifeCycleManager().stepEndedSuccessFully();
    }


}
