/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Fr?d?ric.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.descriptor.softpkg.ccm;

//import the needed packages
import org.objectweb.openccm.descriptor.softpkg.*;

/**
 * Process and traduce the component dependency to a 
 * configvalue array that the target container cannot use.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class DependencyDeployer 
       extends DependencyDeployerContext
    {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Resolve a depency converting it into a configvalue.
     *
     * @param server_inst The ComponentInstallation to use to install the 
     * dependency archive
     * @return A configuration value to use in the Container::install_home
     * method if the dependency action is "install" and type is "Java Class". 
     */
    public org.omg.Components.ConfigValue
    resolveDependency(org.omg.Components.Deployment.ComponentInstallation server_inst)
    throws DependencyResolutionFailureException
    {
        try{
        
            org.omg.Components.ConfigValue config = null;
            Dependency dep=getDependency();
            if ((dep.getAction().compareToIgnoreCase("install") == 0)
                && (dep.getFileinarchive()!=null)
                && (dep.getFileinarchive().getName()!=null||(dep.getFileinarchive().getLink()!=null))
                /*&& (dep.getName()!=null)
                && (dep.getName().getValue()!=null)*/
                )
                {                
                    String fileAdress =
                        getFileinarchiveDeployer()
                        .provideRessource();                
                    
                    server_inst.install
                        (/*dep.getName().getValue()*/
                         dep.getFileinarchive().getName()
                        ,fileAdress);
                    /*server_inst.install(dep.getName().getValue(),
                      absolutePath);                         */

                    // Set up the dependency as config value for home
                    // installation if the dependency is java classes
                    if (dep.getType().trim().
                        compareToIgnoreCase("Java Class") == 0)
                    {
                        config = new org.objectweb.openccm.Components.ConfigValueImpl();
                        config.name = "load_lib_dependency";
                        config.value = org.objectweb.openccm.corba.TheORB.getORB().create_any();
                        config.value.insert_string(dep.getFileinarchive().getName()/*dep.getName().getValue()*/);
                    }
                }
            return config;
        }
        catch(Exception e){
            e.printStackTrace();
            throw new DependencyResolutionFailureException(this,e);
        }
    }


}
