package ist.coach.coachEmfClientComponents.gui;


import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

public class MOTree extends JPanel implements Runnable {

    protected DefaultMutableTreeNode rootNode;
	private String rootName;
    protected DefaultTreeModel treeModel;
    protected JTree tree;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();

	private GuiClient gui_parent;
    public MOTree(GuiClient gui_parent, 
			String root_name) {

		rootName = root_name;
        rootNode = new DefaultMutableTreeNode(
				new MOTreeObject(rootName));
				//new MOTreeObject("coach.ist"));
				//new MOTreeObject("Root Node", "0.0"));
		this.gui_parent = gui_parent;

        treeModel = new DefaultTreeModel(rootNode);
        treeModel.addTreeModelListener(new MOTreeModelListener());
        tree = new JTree(treeModel);

        tree.getSelectionModel().setSelectionMode
               (TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setShowsRootHandles(true);
        tree.addTreeSelectionListener(new MOTreeSelectionListener());

				//customising tree display
		tree.putClientProperty("JTree.lineStyle", "Angled");
		ToolTipManager.sharedInstance().registerComponent(tree);

		tree.setCellRenderer(new MORenderer());

        JScrollPane scrollPane = new JScrollPane(tree);
		scrollPane.setPreferredSize(new Dimension(100, 200));
		scrollPane.setMinimumSize(new Dimension(10, 10));
        setLayout(new GridLayout(1,0));
        add(scrollPane);
    }

	public void update() {
	}
//HACK THREAD
	public void run() {}

    /** Remove all nodes except the root node. */
    public void clear() {
        rootNode.removeAllChildren();
        treeModel.reload();
    }

    /** Remove the currently selected node. */
    public void removeCurrentNode() {
        TreePath currentSelection = tree.getSelectionPath();
        if (currentSelection != null) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode)
                         (currentSelection.getLastPathComponent());
            MutableTreeNode parent = (MutableTreeNode)(currentNode.getParent());
            if (parent != null) {
                treeModel.removeNodeFromParent(currentNode);
                return;
            }
        }

        // Either there was no selection, or the root was selected.
        toolkit.beep();
    }

	public DefaultMutableTreeNode getCurrentTreeSelection() {

		DefaultMutableTreeNode parentNode = null;
        TreePath parentPath = tree.getSelectionPath();

        if (parentPath == null) {
            parentNode = rootNode;
        } else {
            parentNode = (DefaultMutableTreeNode)
                         (parentPath.getLastPathComponent());
        }

		return parentNode;
	}

    /** Add child to the currently selected node. */
    public DefaultMutableTreeNode addObject(Object child) {

        return addObject(getCurrentTreeSelection(), child, true);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent,
                                            Object child) {
        return addObject(parent, child, false);
    }

    public DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent,
                                            Object child,
                                            boolean shouldBeVisible) {
        DefaultMutableTreeNode childNode =
                new DefaultMutableTreeNode(child);

        if (parent == null) {
            parent = rootNode;
        }

        treeModel.insertNodeInto(childNode, parent,
                                 parent.getChildCount());

        // Make sure the user can see the lovely new node.
        if (shouldBeVisible) {
            tree.scrollPathToVisible(new TreePath(childNode.getPath()));
        }
        return childNode;
    }

    class MOTreeModelListener implements TreeModelListener {
        public void treeNodesChanged(TreeModelEvent e) {
            DefaultMutableTreeNode node;
            node = (DefaultMutableTreeNode)
                     (e.getTreePath().getLastPathComponent());

            /*
             * If the event lists children, then the changed
             * node is the child of the node we've already
             * gotten.  Otherwise, the changed node and the
             * specified node are the same.
             */
            try {
                int index = e.getChildIndices()[0];
                node = (DefaultMutableTreeNode)
                       (node.getChildAt(index));
            } catch (NullPointerException exc) {}

            System.out.println("The user has finished editing the node.");
            System.out.println("New value: " + node.getUserObject());
        }

        public void treeNodesInserted(TreeModelEvent e) {

			System.out.println("Tree node inserted!!!");
        	DefaultMutableTreeNode node;
        	node = (DefaultMutableTreeNode)
            	(e.getTreePath().getLastPathComponent());

         	int index = e.getChildIndices()[0];
               node = (DefaultMutableTreeNode)
                         (node.getChildAt(index));
        	System.out.println("New value: " + ((MOTreeObject) node.getUserObject()).toString());

        }
        public void treeNodesRemoved(TreeModelEvent e) {
        }

        public void treeStructureChanged(TreeModelEvent e) {
			System.out.println("Tree structure changed " + e);
        }
    }

    class MOTreeSelectionListener implements TreeSelectionListener {

		public void valueChanged(TreeSelectionEvent e) {

        	DefaultMutableTreeNode node = (DefaultMutableTreeNode)
               		  tree.getLastSelectedPathComponent();

       		if (node == null) return;

        	TreePath parentPath = tree.getSelectionPath();

        	MOTreeObject nodeInfo = (MOTreeObject) node.getUserObject();
		//GuiClient.println("Node class = " + nodeInfo.getClass().getName());
			//
			// possibly the tree needs to be updated
			//

		   boolean node_is_leaf = node.isLeaf();
			String[] children = gui_parent.getContained(nodeInfo.key());

			node.removeAllChildren();
			if (!node_is_leaf)
				treeModel.reload(node);

			if (children != null && children.length > 0) {
				for(int i = 0; i < children.length; i++)
					gui_parent.create(node, children[i]);
			}

    	}

	}

	class MORenderer extends DefaultTreeCellRenderer {

		public MORenderer() {
		}

		public Component getTreeCellRendererComponent(
			JTree tree,
			Object value,
			boolean sel,
			boolean expanded,
			boolean leaf,
			int row,
			boolean hasFocus) {

			super.getTreeCellRendererComponent(tree, value, sel,
				expanded, leaf, row,
				hasFocus);

			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			MOTreeObject mo = (MOTreeObject)(node.getUserObject());

			if (leaf) {
				setIcon(mo.leafIcon);
			}
			else {
				setIcon(mo.nodeIcon);
			}

			setToolTipText(mo.toolTipText); //no tool tip
			return this;

		}
	}
}
