/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.plugin.java.reflect;

import java.io.File;
import java.util.jar.JarFile;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.objectweb.fractal.api.Component;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.swing.gui.api.DialogAction;
import org.objectweb.util.explorer.swing.gui.api.DialogBox;
import org.objectweb.util.explorer.swing.gui.api.TreeChooserObserver;
import org.objectweb.util.explorer.swing.gui.api.TreeProvider;
import org.objectweb.util.explorer.swing.gui.lib.DefaultDialogBox;
import org.objectweb.util.explorer.swing.gui.lib.FileChooserBox;
import org.objectweb.util.explorer.swing.gui.lib.TreeChooserBox;

/**
 * Selects a jar and more precisly a Java class.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class SelectClassAction 
    implements MenuItem, DialogAction, TreeChooserObserver 
{

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The FileChooserBox used to load a jar resource. */
    protected FileChooserBox jarFile_;
    
    /** The TreeChooserBox used to browse the Jar file. */
    protected TreeChooserBox classFile_;
    
    /** The JFileChooser used to load a jar resource. */
    protected JFileChooser fileChooser_;
    
    /** The Tree used to choose a file resource. */
    protected Tree tree_;

    /** The associated TreeView. */
    protected TreeView treeView_;

    /** The TreeProvider used to create a DynamicTree */
    protected TreeProvider treeProvider_;

    /** The DialogBox to use. */
    protected DialogBox action_;
    
    //==================================================================
    //
    // No constructor.
    // 
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================
    
    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#getStatus(org.objectweb.util.explorer.api.TreeView)
     */
    public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.MenuItem#actionPerformed(org.objectweb.util.explorer.api.MenuItemTreeView)
     */
    public void actionPerformed(MenuItemTreeView treeView) throws Exception {
        treeView_ = treeView;
        
        action_ = new DefaultDialogBox("Choose a class");
        fileChooser_ = new JFileChooser();
        jarFile_ = new FileChooserBox("Jar file",fileChooser_,true,false);
        action_.addElementBox(jarFile_);
        
        Component treeCpt = treeView.getTree().duplicate(false);
        tree_ = (Tree)treeCpt.getFcInterface(Tree.TREE);
        
        classFile_ = new TreeChooserBox("Class File", treeCpt);
        classFile_.setTreeChooserObserver(this);
        action_.addElementBox(classFile_);

        action_.setValidateAction(this);
        action_.show(); 
    }

    //==================================================================
    //
    // Public methods for TreeChooserObserver interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.gui.api.TreeChooserObserver#refresh()
     */
    public Component refresh(){
        Component treeCpt = null;
        File f = jarFile_.getFile();
        if(f!=null) {
            treeCpt = treeView_.getTree().duplicate(false);
            try {
                Tree treeItf = (Tree)treeCpt.getFcInterface(Tree.TREE);
                JarFile jf = new JarFile(f);
                tree_.addEntry(f.getName(), new JarFileStructure(jf,f.toURL()));
            } catch (Exception e) {
                JOptionPane.showMessageDialog(null, "Jar file expected !", "Exception", JOptionPane.ERROR_MESSAGE);
            }
        }
        return treeCpt;
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.gui.api.DialogAction#executeAction()
     */
    public void executeAction() throws Exception {
        System.out.println("OK");
    }

}


    