/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.api;

/**
 * This interface defines the way to customize icons.<br>
 * In order to define an icon provider, you can have a look to the corresponding
 * <a href="explorer-dtd.html#icon">DTD element</a> or to 
 * an <a href="package-summary.html#xml">XML example.</a>
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * 
 * @version 0.3
 */
public interface IconProvider 
{

    /**
     * Returns a icon computed according to the given object.
     * The type of the returned object may be:
     * <ul>
     *   <li>A <code>java.lang.String</code> defining the path to an icon file located on the local machine</li>
     *   <li>An <code>java.net.UTL</code> pointing out an icon file.</li>
     *   <li>An icon representation dependending on the used graphical framework (more formaly, an <code>org.eclipse.swt.graphics.Image</code> for SWT 
     *   and a <code>javax.swing.Icon</code> for Swing application.</li>
     * </ul>
     *
     * @param object The object on which the icon must be computed.
     * 
     * @return A reference pointing out the icon to display or the icon itself.
     */
    public Object 
    newIcon(Object object);

}