/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

===================================================================
$Id: Tree.java,v 1.2 2005/07/06 15:36:01 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.api;

import org.objectweb.fractal.api.Component;
import org.objectweb.util.explorer.swing.gui.api.TreeProvider;

/**
 * Defines methods in order to populate and to manipulate the tree.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public interface Tree {

    /** The name of the fractal interface reference. */
    public static String TREE = "tree";        

    /** Fractal Interface name. */
    public static String TREE_TMPL = "tree-tmpl";
    
    /** Fractal Interface name. */
    public static String TREE_ITSELF = "tree-itself";
        
    /**
     * Refresh all the tree.
     */
    public void 
    refreshAll();

    /**
     * Closes the tree.
     * We suggest to call this method before closing the application based on this Tree.
     */
    public void
    close();

    /**
     * Adds an object into the initial context.
     *  
     * @param name Name of the object
     * @param value The object to add
     */
    public void 
    addEntry(Object name, Object value);

    /**
     * Adds an object into the initial context and expands the node until a specific level if this one is a context.
     *  
     * @param name Name of the object
     * @param value The object to add
     * @param level The level (>0 for a specific level or <0 for all levels)
     */
    public void 
    addEntry(Object name, Object value, int level);
   
    /**
     * Selects the given path representation if it exists.
     * Each node needs to be separated by the '/' character (i.e. elt1/elt2/elt3).
     * @param path The representation of the path
     */
    public void 
    selectPath(String path);
    
    /**
     * Refreshes the selected node name.
     * 
     * @param odlName The old name.
     * @param newName The new name.
     */
    public void 
    renameSelectedNode(Object odlName, Object newName);
    
    /**
     * Removes the entry identified by the given name. 
     * This entry must be found within the InitialContext. 
     * 
     * @param name The name
     */
    public void 
    removeEntry(Object name);    
    
    /**
     * Renames the entry identified by the given name. 
     * This entry must be found within the InitialContext. 
     * 
     * @param currentName The current name of the entry.
     * @param newName The new name.
     */
    public void 
    renameInitialEntry(Object currentName, Object newName); 
    
    /**
     * Removes all the entries contained within the InitialContext. 
     */
    public void 
    clear();    

    /**
     * Provides the number of entries contained within the InitialContext.
     * @return The number of entries.
     */
    public int
    getInitialContextSize();

    /**
     * Provides the selcted entry or null if no entry is selected.
     * @return The selected entry.
     */
    public Entry
    getSelectedEntry();
    
    /**
     * Creates a new Tree instance with a same configuration of one contains in the explorer component.
     * Equivalent to a <code>duplicate(true);</code> call.
     * @return The new Tree instance. 
     * @see org.objectweb.util.explorer.api.Tree
     * @see TreeProvider#duplicate(boolean)
     */
    public Component duplicate();

    /**
     * Creates a new Tree instance with a same configuration of one contains in the explorer component.
     * @param withData A boolean to specify if the data of the source tree have to be added into the new one. The default value is "true"
     * @return The new Tree instance.
     * @see org.objectweb.util.explorer.api.Tree
     */
    public Component duplicate(boolean withData);
    
}
