/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.context.lib;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.ContextContainer;
import org.objectweb.util.explorer.core.common.lib.DefaultContextContainer;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DecoderContextContainer
     extends DecoderContext
  implements ContextContainer
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    protected ContextContainer delegate_ = null;
  
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default Constructor
     */
    public DecoderContextContainer(){
        this(new DefaultContextContainer());
    }
    
    /**
     * Constructor with specific delegate
     */
    public DecoderContextContainer(ContextContainer delegate){
        if(delegate!=null)
            delegate_ = delegate;
        else 
            delegate_ = new DefaultContextContainer();
        super.delegate_= this.delegate_;
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for ContextContainer interface. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ContextContainer#getEntry(java.lang.Object)
     */
    public Entry getEntry(Object name) {
        Entry entry = delegate_.getEntry(name);
        if(entry!=null && decoder_!=null){
            entry.setValue(decoder_.decode(entry.getValue()));
        }
        return entry;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ContextContainer#addEntry(java.lang.Object, java.lang.Object)
     */
    public void addEntry(Object name, Object value) {
        delegate_.addEntry(name,value);
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ContextContainer#addEntry(org.objectweb.util.explorer.api.Entry)
     */
    public void addEntry(Entry entry) {
        delegate_.addEntry(entry);
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ContextContainer#removeEntry(java.lang.Object)
     */
    public void removeEntry(Object name) {
        delegate_.removeEntry(name);
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ContextContainer#clear()
     */
    public void clear() {
        delegate_.clear();
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.ContextContainer#getSize()
     */
    public int getSize() {
        return delegate_.getSize();
    }
    
}


