/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================
$Id: CodeDispatcher.java,v 1.2 2005/07/06 15:36:00 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.core.code.lib;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.core.code.api.Code;
import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.code.api.CodeProvider;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class CodeDispatcher 
     extends BindingFeature 
  implements CodeProvider 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal method.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for BindingFeature abstract class.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{CodeProvider.CLIENT_CODE_PROVIDER};
    }

    // ==================================================================
    //
    // Public methods for CodeProvider interface.
    //
    // ==================================================================
    
    /**
     * A syntaxic pattern has been defined in order to dispatch towards the appropriate CodeProvider component.
     * Indeed, the name of the binding between the dispatcher and the CodeProvider is the following:<br>
     * &#160;&#160;&#160;&#160;<code>CodeProvider.CLIENT_CODE_PROVIDER + "-" + codeDesc.getLanguage().toLowerCase()</code>
     * @see org.objectweb.util.explorer.core.code.api.CodeProvider#getCode(org.objectweb.util.explorer.core.code.api.CodeDescription)
     */
    public Code getCode(CodeDescription codeDesc) {
        try {
            CodeProvider codeProvider = (CodeProvider)lookupFc(CodeProvider.CLIENT_CODE_PROVIDER + "-" + codeDesc.getLanguage().toLowerCase());
            return codeProvider.getCode(codeDesc);
        } catch (NoSuchInterfaceException e) {
            e.printStackTrace();
        }        
        return null;
    }

}
