/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.common.api;

import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.Entry;

/**
 * ContextContainer interface.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public interface ContextContainer 
         extends Context 
{

    /**
     * Returns the value of the entry identify by the given name
     * @param name The key of the entry
     * @return The value of the associated entry or null.
     */
    //public Object getLocalValue(String name);

    /**
     * Returns the entry identify by the given name
     * @param name The key of the entry
     * @return The associated entry or null.
     */
    public Entry getEntry(Object name);
    
    /**
     * Adds an object into the container.
     * If an entry with the same id already exists in the container, 
     * this one is replaced by the new value. 
     *
     * This method does not modify the given name and the given value, 
     * but it modifies the target context.
     *
     * @param name The name of the entry to be retrieved.
     * @param value The object to add
     * 
     */
    public void addEntry(Object name, Object value);

    /**
     * Adds an entry into the container.
     * If an entry with the same id already exists in the container, 
     * this one is replaced by the new value. 
     *
     * This method does not modify the given name and the given value, 
     * but it modifies the target context.
     *
     * @param name The name of the entry to be retrieved.
     * @param value The object to add
     * 
     */
    public void addEntry(Entry entry);
    
    /**
     * Removes the object corresponding to the given name from the context
     * This method modifies the target context.
     * @param name The name of the entry to be removed.
     */
    public void removeEntry(Object name);
    
    /**
     * Removes all of the elements of this context.
     */
    public void clear();

    /**
     * Provides the number of entries contained into the given context
     * @return The size of the context.
     */
    public int getSize();    
    
}