/*===========================================================================

ObjectWeb Naming Context Framework 
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.explorer.core.common.lib;

/** The Browsr API's imports */
import java.io.Serializable;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;

/**
 * Offers a limited view on the tree
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultTreeView 
  implements /*ExtendedTreeView,*/TreeView, Serializable 
{

    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** The selected entry and his parent */
    protected Entry entry_, parent_;

    /** The tree instance. */
    protected Tree tree_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default Constuctor
     * @param tree The associated tree
     */
    public DefaultTreeView(Tree tree, Entry current, Entry parent) {
        tree_ = tree;
        entry_ = current;
        parent_ = parent;
    }

    // ==================================================================
    //
    // Protected methods.
    //
    // ==================================================================

    /**
     * @param entry The entry on which you want to obtain the value
     * @return The associated value of the entry
     */
    protected Object getValue(Entry entry) {
        return (entry != null)?entry.getValue():null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Returns the current selected Entry.
     */
    public Entry getSelectedEntry() {
        return entry_;
    }

    /**
     * Returns the parent entry
     */
    public Entry getParentEntry() {
        return parent_;
    }

    /**
     * Returns the wrapped object of value of the associated Entry
     */
    public Object getSelectedObject() {
        return getValue(entry_);
    }

    /**
     * Returns the parent object
     */
    public Object getParentObject() {
        return getValue(parent_);
    }

    /**
     * Returns an object to manipulate the Tree.
     */
   public Tree
   getTree(){
        return (Tree)tree_;        
    }
    
}
