/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.dnd.lib;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.dnd.api.DnDDescription;
import org.objectweb.util.explorer.core.dnd.api.DnDDescriptions;

/**
 * Basic implementation of the DnDDescription interface.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicDnDDescriptions 
  implements DnDDescriptions 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The map where storing the drop actions. */
    protected Map dropActions_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public BasicDnDDescriptions(){
        dropActions_ = new HashMap();
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add the description of the drop action into the given list if the given dndDesc is not null.
     */
    protected void addDescription(DnDDescription dndDesc, List l){
        if(dndDesc!=null && !"".equals(dndDesc.getLabel().trim())){
            l.add("[" + dndDesc.getType() + "] " + dndDesc.getLabel());
        }
    }

    protected String getDnDDescription(DnDDescription dndDesc){
        if(dndDesc!=null){
            return dndDesc.toString();
        }
        return "null";
    }
    
    // ==================================================================
    //
    // Public methods for Description interface. 
    //
    // ==================================================================       
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#getDescriptionType()
     */
    public String getDescriptionType() {
        return Description.DND_DESCRIPTION;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#isEmpty()
     */
    public boolean isEmpty() {
        return dropActions_.isEmpty();
    }
    
    // ==================================================================
    //
    // Public methods for DnDDescription interface. 
    //
    // ==================================================================       
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#addDropAction(int, org.objectweb.util.explorer.core.code.api.CodeDescription)
     */
    public void addDropAction(DnDDescription dndDesc) {
        if (dndDesc!=null && !dndDesc.isEmpty()
                && (DnDDescription.MOVE_ACTION.equals(dndDesc.getType())
                || DnDDescription.COPY_ACTION.equals(dndDesc.getType())
                || DnDDescription.LINK_ACTION.equals(dndDesc.getType()))){
            dropActions_.put(dndDesc.getType(), dndDesc);
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescription#getDropAction(int)
     */
    public DnDDescription getDropAction(String actionType) {
        return (DnDDescription)dropActions_.get(actionType);
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.dnd.api.DnDDescriptions#getActionsDescription()
     */
    public String[] getActionsDescription() {
        Vector l = new Vector();
        addDescription(getDropAction(DnDDescription.MOVE_ACTION),l);
        addDescription(getDropAction(DnDDescription.COPY_ACTION),l);
        addDescription(getDropAction(DnDDescription.LINK_ACTION),l);
        return (String[])l.toArray(new String[l.size()]);
    }

    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    
   
    /**
     * Surcharging the Object.equals method.
     * @see DefaultKey#equals(DefaultKey)
     */    
    public boolean equals(Object o){
        if(o!=null && o instanceof BasicDnDDescription)
            return equals((BasicDnDDescription)o);
        return false;
    }
    
    /**
     * Return a String representation of a BasicItemDescription  
     */ 
    public String toString(){
        StringBuffer buf = new StringBuffer();
        buf.append("BasicDnDDescriptions[dropActions=[");
        buf.append("move=" + getDnDDescription(getDropAction(DnDDescription.MOVE_ACTION)) + ", ");
        buf.append("copy=" + getDnDDescription(getDropAction(DnDDescription.COPY_ACTION)) + ", ");
        buf.append("link=" + getDnDDescription(getDropAction(DnDDescription.LINK_ACTION)));
        buf.append("]]");
        return buf.toString();
    }     
    
}
 
