/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.menu.api;

import org.objectweb.util.explorer.core.common.api.Description;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public interface MenuDescription          
         extends Description
{

    /**
     * To know if the description is used to generate popup menu or menu bar menu.
     * The associated values are respectively <code>popup</code> and <code>bar</code>
     * This method is only used in the SWT context.
     * @param descriptionType
     */
    public void setDescriptionType(String descriptionType);
    
    public void setInheritTreeMenu(boolean inheritTreeMenu);
    public boolean getInheritTreeMenu();
    
    public void setInheritTypeMenu(boolean inheritTypeMenu);
    public boolean getInheritTypeMenu();

    /**
     * Adds a MenuElement into the MenuDescription.
     * @param menuElement
     */
    public void addMenuElement(MenuElement menuElement);
    
    /**
     * Adds an array of MenuElement objects into the MenuDescription.
     * @param menuElement
     */
    public void addMenuElements(MenuElement[] menuElements);
        
    /**
     * Returns the MenuElement objects defined for the menu itself.
     * @return A list of menu elements.
     */
    public MenuElement[] getMenuElements();
    
    /**
     * Returns the MenuElement objects defined for the children of the menu.
     * @return A list of menu elements.
     */
    public MenuElement[] getChildrenMenuElements();
    
    /**
     * Returns the MenuElements of this menu. Is consists in a merge between the menu elements defined
     * for the menu itself and those defined for its children. A separator is included between these two
     * kinds of elements is necessary.
     * @return A list of menu elements.
     * @see MenuDescription#getMenuElements()
     * @see MenuDescription#getChildrenMenuElements()
     */
    public MenuElement[] getAllMenuElements();
    
}
