/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.menu.lib;

import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.menu.api.AcceleratorDescription;

/**
 * Basic implementation of the <code>AcceleratorDescription</code> interface.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicAcceleratorDescription
  implements AcceleratorDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected char char_=0;
    protected boolean ctrlKey_ = false;
    protected boolean altKey_ = false;
    protected boolean shiftKey_ = false;
    protected boolean metaKey_ = false;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected boolean equals(BasicAcceleratorDescription accDesc){
        if(accDesc!=null){
            return char_==accDesc.char_
                && ctrlKey_==accDesc.ctrlKey_
                && altKey_==accDesc.altKey_
                && shiftKey_==accDesc.shiftKey_
                && metaKey_==accDesc.metaKey_;
        }
        return false;
    }
        
    protected String getStringRepresentation(){
        StringBuffer sb = new StringBuffer();
        sb.append(ctrlKey_?"ctrl-":"");
        sb.append(altKey_?"alt-":"");
        sb.append(shiftKey_?"shift-":"");
        sb.append(metaKey_?"meta-":"");
        sb.append(Character.toUpperCase(char_));
        return sb.toString();
    }
    
    // ==================================================================
    //
    // Public methods for Description interface.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#getDescriptionType()
     */
    public String getDescriptionType() {
        return Description.ACCELERATOR_DESCRIPTION;
    }

    /* (non-Javadoc)
     * @see boolean#isNull()
     */
    public boolean isEmpty() {
        return char_==0;
    }
    
    // ==================================================================
    //
    // Public methods for AcceleratorDescription interface.
    //
    // ==================================================================
    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#setAcceleratorCharacter(char)
     */
    public void setAcceleratorCharacter(char car) {
        char_ = car;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#getAcceleratorCharacter()
     */
    public char getAcceleratorCharacter() {
        return char_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#setCtrlKey(boolean)
     */
    public void setCtrlKey(boolean ctrlKey) {
        ctrlKey_ = ctrlKey;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#getCtrlKey()
     */
    public boolean getCtrlKey() {
        return ctrlKey_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#setAltKey(boolean)
     */
    public void setAltKey(boolean altKey) {
        altKey_ = altKey;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#getAltKey()
     */
    public boolean getAltKey() {
        return altKey_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#setShiftKey(boolean)
     */
    public void setShiftKey(boolean shiftKey) {
        shiftKey_ = shiftKey;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#getShiftKey()
     */
    public boolean getShiftKey() {
        return shiftKey_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#setMetaKey(boolean)
     */
    public void setMetaKey(boolean metaKey) {
        metaKey_ = metaKey;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#getMetaKey()
     */
    public boolean getMetaKey() {
        return metaKey_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.menu.api.AcceleratorDescription#getName()
     */
    public String getName() {
        return getStringRepresentation();
    }

    // ==================================================================
    //
    // Public methods surcharging Object class.
    //
    // ==================================================================    

    /**
     * Surcharging the Object.equals method.
     * @see Object#equals(java.lang.Object)
     */
    public boolean equals(Object o){
        if(o!=null && o instanceof BasicAcceleratorDescription){
            return equals((BasicAcceleratorDescription)o);
        }
        return false;
    }
   
    /**
     * Return a String representation of a BasicAcceleratorDescription 
     * @see Object#toString() 
     */    
    public String toString(){
        return "BasicAcceleratorDescription[value=" + getStringRepresentation() + "]";
    } 
 
}


