/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.panel.lib;

import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.panel.api.PanelDescription;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public abstract class AbstractPanelDescription 
  implements PanelDescription 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The code description of this panel. */
    protected CodeDescription codeDescription_;

    /** This boolean indicates if a panel has to inherit panel from parents*/
    protected boolean inheritTypePanel_ = false;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for Description interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#getDescriptionType()
     */
    public String getDescriptionType() {
        return Description.PANEL_DESCRIPTION;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.api.Description#isEmpty()
     */
    public boolean isEmpty() {
        return codeDescription_ == null || codeDescription_.isEmpty();
    }
    
    // ==================================================================
    //
    // Public methods for PanelDescription interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.PanelDescription#getPanelType()
     */
    public abstract String getPanelType();

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.PanelDescription#getCodeDescription()
     */
    public CodeDescription getCodeDescription() {
        return codeDescription_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.PanelDescription#setCodeDescription(org.objectweb.util.explorer.core.code.api.CodeDescription)
     */
    public void setCodeDescription(CodeDescription codeDesc) {
        codeDescription_ = codeDesc;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.PanelDescription#setInheritTypePanel(boolean)
     */
    public void setInheritTypePanel(boolean inheritTypePanel) {
        inheritTypePanel_ = inheritTypePanel;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.panel.api.PanelDescription#inheritTypePanel()
     */
    public boolean inheritTypePanel() {
        return inheritTypePanel_;
    }

}
