/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.interpreter.lib;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DescriptionInterpreterDispatcher 
     extends BindingFeature 
  implements DescriptionInterpreter 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /**
     * Provides the interface compliant with the given interface name suffixe.
     * A syntaxic pattern has been defined in order to dispatch towards the appropriate 
     * DescriptionInterpreter component. The name of the binding between the dispatcher 
     * and the DescriptionInterpreter must be the following:<br>
     * <code>&#160;&#160;DescriptionInterpreter.DESCRIPTION_INTERPRETER + "-collection-" + itfSuffixe</code>
     * @return The associated PropertyResolver
     */
    protected DescriptionInterpreter getDescriptionInterpreter(String itfSuffixe){
        try {
            return (DescriptionInterpreter)lookupFc(DescriptionInterpreter.DESCRIPTION_INTERPRETER + "-collection-" + itfSuffixe);
        } catch (NoSuchInterfaceException e) {
            getTrace().info(DescriptionInterpreter.DESCRIPTION_INTERPRETER + "-collection-" + itfSuffixe + ": interface not found!");
            return null;
        }
    }
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{DescriptionInterpreter.DESCRIPTION_INTERPRETER + "-collection"};
    }

    // ==================================================================
    //
    // Public methods for DescriptionInterpreter interface. 
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter#interprete(org.objectweb.util.explorer.core.common.api.Description, java.lang.Object)
     */
    public Object interprete(Description description, Object params) {
        if(description!=null){
            DescriptionInterpreter descriptionInterpreter = getDescriptionInterpreter(description.getDescriptionType());
            if (descriptionInterpreter!=null)
                return descriptionInterpreter.interprete(description, params);
        }
        return null;
    }


}


