/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.interpreter.lib;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.RootContext;
import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.core.root.api.RootDescriptions;
import org.objectweb.util.explorer.core.root.lib.DefaultRootContext;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class RootInterpreter 
     extends BindingFeature
  implements DescriptionInterpreter
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    protected static final String CODE_INTERPRETER = "code-interpreter";
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected DescriptionInterpreter getCodeInterpreter(){
        try {
            return (DescriptionInterpreter)lookupFc(CODE_INTERPRETER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(CODE_INTERPRETER + ": interface not found!");
            return null;
        }
    }    
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================     
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{CODE_INTERPRETER};
    }    

    // ==================================================================
    //
    // Public methods for DescriptionInterpreter interface. 
    //
    // ==================================================================     

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter#interprete(org.objectweb.util.explorer.core.common.api.Description, java.lang.Object)
     */
    public Object interprete(Description description, Object params) {
        RootDescriptions rootDesc = (RootDescriptions)description;
        DefaultRootContext rootContext = new DefaultRootContext();
        CodeDescription[] codeDesc = rootDesc.getRootDescriptions();
        for (int i = 0; i < codeDesc.length; i++) {
            rootContext.addRootContext((RootContext)getCodeInterpreter().interprete(codeDesc[i], params));
        }
        return rootContext;
    }

}
