/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.interpreter.lib.swing;

import java.net.URL;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.IconProvider;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.core.code.api.Code;
import org.objectweb.util.explorer.core.code.api.CodeProvider;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.icon.api.IconDescription;
import org.objectweb.util.explorer.core.menu.api.ItemDescription;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.api.MenuElement;
import org.objectweb.util.explorer.core.menu.api.MenuSeparator;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.interpreter.lib.AbstractDescriptionInterpreter;
import org.objectweb.util.explorer.swing.icon.IconFileProvider;
import org.objectweb.util.explorer.swing.menu.GenericAction;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class MenuInterpreter 
     extends AbstractDescriptionInterpreter 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected final String ICON_INTERPRETER = "icon-interpreter";
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================
 
    /**
     * Provides the icon for the given icon description 
     * applied to the given object.
     */
    protected Icon getIcon(IconDescription iconDesc, TreeView treeView){
        if(iconDesc!=null){
            IconProvider iconProvider = (IconProvider) getIconInterpreter().interprete(iconDesc, treeView);
            if(iconProvider!=null){
                Object object = iconProvider.newIcon(treeView.getSelectedObject());
                if(object instanceof Icon){
                    return (Icon)object;
                } else if(object instanceof String) {
                    return (Icon)(new IconFileProvider((String)object)).newIcon(treeView.getSelectedObject());
                } else if (object instanceof URL) {
                    return (Icon)(new IconFileProvider((URL)object)).newIcon(treeView.getSelectedObject());
                }
            }
        }
        return null;
    }
    
    /**
     * Provides the Java/Swing action defined by the given item description.
     */
    protected Action getAction(ItemDescription itemDesc, MenuItemTreeView treeView){
        if(itemDesc!=null && !itemDesc.isEmpty()){
            Code actionCode = getCodeProvider().getCode(itemDesc.getCodeDescription());
            if(actionCode!=null && actionCode.isInstanceOf(MenuItem.class)){
                MenuItem action = (MenuItem)actionCode.createInstance();
                int actionStatus = action.getStatus(treeView);
                Icon icon = getIcon(itemDesc.getIconDescription(), treeView);
                if(actionStatus!=MenuItem.NOT_VISIBLE_STATUS){
                    return new GenericAction(itemDesc, action, treeView, icon, actionStatus==MenuItem.ENABLED_STATUS);
                }
            } else {
                getTrace().warn(itemDesc.getCodeDescription().getCode() + " is not a MenuItem instance!");
            }
        }
        return null;
    }
    
    /**
     * Provides a Java/Swing menu separator.
     */
    protected JSeparator getSeparator(MenuSeparator menuSeparator){
        return new JSeparator();
    }
    
    /**
     * Creates the Java/Swing popup menu defined by the given menu description.
     */
    protected JPopupMenu createMenu(MenuDescription menuDesc, MenuItemTreeView treeView){
        JPopupMenu menu = null;
        if(menuDesc!=null && !menuDesc.isEmpty()){
            menu = new JPopupMenu();
            MenuElement[] menuElement = menuDesc.getAllMenuElements();
            for (int i = 0; i < menuElement.length; i++) {
                MenuElement element = menuElement[i];
                if(element instanceof ItemDescription){
                    Action action = getAction((ItemDescription)element, treeView); 
                    if(action != null) {
                        menu.add(action);
                    }
                } else if(element instanceof MenuSeparator){
                    menu.add(getSeparator((MenuSeparator)element));
                }
            }
        }
        return menu;
    }

    protected DescriptionInterpreter getIconInterpreter(){
        try {
            return (DescriptionInterpreter)lookupFc(ICON_INTERPRETER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(ICON_INTERPRETER + ": interface not found!");
            return null;
        }
    }    
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================     
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{CodeProvider.CODE_PROVIDER, ICON_INTERPRETER};
    }
    
    // ==================================================================
    //
    // Public methods DescriptionInterpreter interface. 
    //
    // ==================================================================     
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter#interprete(org.objectweb.util.explorer.core.common.api.Description, java.lang.Object)
     */
    public Object interprete(Description description, Object params) {
        MenuDescription menuDesc = (MenuDescription)description;
        return createMenu(menuDesc, (MenuItemTreeView)params);
    }

}


   
