/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.interpreter.lib.swt;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.MessageBox;
import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.core.menu.api.MenuItemTreeViewConfiguration;
import org.objectweb.util.explorer.swt.lib.DynamicTree;
import org.objectweb.util.trace.TraceSystem;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class GenericAction 
  implements Listener 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    protected MenuItem delegate_;
    protected MenuItemTreeView treeView_;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public GenericAction(MenuItem action, MenuItemTreeView treeView){
        delegate_ = action;
        treeView_ = treeView;        
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for Listener interface. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.eclipse.swt.widgets.Listener#handleEvent(org.eclipse.swt.widgets.Event)
     */
    public void handleEvent(Event event) {
        try{
            ((MenuItemTreeViewConfiguration)treeView_).setEvent(event);
            delegate_.actionPerformed(treeView_);
        } catch (Exception e) {
            TraceSystem.get("explorer").warn(getClass().getName() + ", " + e.getClass().getName() + ": "+ e.getMessage());
            MessageBox mb = new MessageBox(DynamicTree.shell_, SWT.ICON_ERROR | SWT.OK);
            mb.setMessage(e.getClass().getName() + ":\n" + e.getMessage());
            mb.setText("Exception (" + event.widget + ")");
            mb.open();
        }
        treeView_.getTree().refreshAll(); 
    }

}
