/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.resolver.lib;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.BadParamException;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.common.lib.ClassesInheritance;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicMenuSeparator;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class MenuResolver 
     extends AbstractPropertyResolver   
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.lib.AbstractPropertyResolver#getPropertyName()
     */
    protected String getPropertyType() {
        return ExplorerConstants.MENU_PROPERTY;
    }    
        
    /**
     * Merges the two menu definition
     * This method adds the content of the target menu description into 
     * the content of the source menu description.
     * The target description is not modified whereas the source description is modified. 
     */
    protected MenuDescription mergeMenuDescription(MenuDescription source, MenuDescription target) {
        if(target!=null){
            source.addMenuElements(target.getAllMenuElements());
        }
        return source;
    }    
    
    /**
     * Tries to find a menu description for the given key.
     * @param key The storage key
     * @return The associated description (or null if no description is found).
     */
    protected MenuDescription getMenuDesc(Object key){
        return (MenuDescription)getPropertyProvider().getPropertyDescription(getPropertyType(), key);
    }
    
    /**
     * Provides a menu for the given entry
     * @param entry The entry for which a menu needs to be computed.
     * @return The associated menu description.
     */
    protected MenuDescription getMenuDescription(Entry entry, boolean childrenMenuElement){
        ExtendedMenuDescription menuDesc = new ExtendedMenuDescription();
        if(entry!=null && entry.getValue()!=null){
            Class[] javaTypes = ClassesInheritance.getInheritedClasses(entry.getValue().getClass());
            String[] roleIds = getRoleProvider().getInheritedRoleIds(getRoleManager().getCurrentRoleIds());
            boolean isFirst = true;
            for (int i = 0; i < javaTypes.length; i++) {
                for (int j = 0; j < roleIds.length; j++) {
                    Object key = null;
                    try {
                        key = getKeyProvider().computeKey(new String[]{getTypeSystemId(),javaTypes[i].getName(),roleIds[j]});
                    } catch (BadParamException e) {
                        getTrace().warn("Bad key param: " + e.getMessage());
                    }
                    if(key!=null){
                        MenuDescription newMenuDesc = getMenuDesc(key);
                        if(newMenuDesc!=null && !newMenuDesc.isEmpty()){
                            if(childrenMenuElement) {
                                menuDesc.addChildrenMenuElements(newMenuDesc);
                            } else {
                                if(isFirst){
                                    isFirst = false;
                                } else {
                                    menuDesc.addMenuElement(new BasicMenuSeparator());
                                }
                                menuDesc.addMenuElements(newMenuDesc,i==0);
                            }
                        }
                    }
                }
            }
        }
        return menuDesc;
    }
    
    /**
     * Computes the menu for the given entries.
     * @param currentEntry The current entry.
     * @param parentEntry The parent of the current entry (may be null);
     * @return The description of the computed menu.
     */
    protected MenuDescription getMenuDescription(Entry currentEntry, Entry parentEntry){
        MenuDescription currentMenu = getMenuDescription(currentEntry,false);
        if(parentEntry!=null){
            return mergeMenuDescription(currentMenu, getMenuDescription(parentEntry,true));
        } else {
            return currentMenu;
        }
    }

    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry, Entry parentEntry) {
        return getMenuDescription(currentEntry, parentEntry);
    }

}
