/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.resolver.lib;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.BadParamException;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.common.lib.ClassesInheritance;
import org.objectweb.util.explorer.core.panel.api.CompositePanelDescription;
import org.objectweb.util.explorer.core.panel.api.PanelDescription;
import org.objectweb.util.explorer.core.panel.lib.BasicCompositePanelDescription;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class PanelResolver 
     extends AbstractPropertyResolver 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.lib.AbstractPropertyResolver#getPropertyType()
     */
    protected String getPropertyType() {
        return ExplorerConstants.PANEL_PROPERTY;
    }

    /**
     * Tries to find a panel description for the given key.
     * @param key The storage key
     * @return The associated description (or null if no description is found).
     */
    protected PanelDescription getPanelDesc(Object key){
        return (PanelDescription)getPropertyProvider().getPropertyDescription(getPropertyType(), key);
    }    
    
    /**
     * Provides a panel for the given entry
     * @param entry The entry for which a panel needs to be computed.
     * @return The associated panel description.
     */    
    protected Description getPanelDescription(Entry currentEntry){
        CompositePanelDescription desc = null;
        if(currentEntry!=null && currentEntry.getValue()!=null){
            Class[] javaTypes = ClassesInheritance.getInheritedClasses(currentEntry.getValue().getClass());
            String[] roleIds = getRoleProvider().getInheritedRoleIds(getRoleManager().getCurrentRoleIds());
            for (int i = 0; i < javaTypes.length; i++) {
                for (int j = 0; j < roleIds.length; j++) {
                    Object key = null;
                    try {
                        key = getKeyProvider().computeKey(new String[]{getTypeSystemId(),javaTypes[i].getName(),roleIds[j]});
                    } catch (BadParamException e) {
                        getTrace().warn("Bad key param: " + e.getMessage());
                    }
                    if(key!=null){
                        PanelDescription newPanelDesc = getPanelDesc(key);
                        if(newPanelDesc!=null){
                            if(desc==null){
                                desc = new BasicCompositePanelDescription();
                            }
                            desc.addPanelDescription(newPanelDesc);
                        }
                        if(newPanelDesc!=null) break;
                    }
                }
                if(desc!=null && !desc.inheritTypePanel()) break;
            }
        }
        return desc;
    }
    
    // ==================================================================
    //
    // Public methods for PropertyResolver interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry, Entry parentEntry) {
        return getPanelDescription(currentEntry);
    }

}


