/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swing.api;

import javax.swing.JMenuBar;
import javax.swing.JToolBar;
import javax.swing.JTree;

/**
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */

public interface Explorer {

    /** The name of the fractal interface reference. */
    public static String EXPLORER = "explorer";

	/**
	 * Provides the DynamicTree.
	 * @return The Dynamic Tree.
	 */
	public JTree getTree();

    /**
     * Fixes the menu bar where added the "Actions" and the "Roles" menus.
     * @param menuBar The menu bar to use.
     */
    public void setMenuBar(JMenuBar menuBar);    
    
    /**
     * Fixes the tool bar to use.
     * @param menuBar The tool bar to use.
     */
    public void setToolBar(JToolBar toolBar);

    /**
     * Indicates if the application allows the selection of several roles at the same time (the default value is false).
     * @param multipleRolesEnabled True if the application allows the selection of several roles at the same time, false either.
     */
    public void setMultipleRoles(boolean multipleRolesEnabled);
    
    /**
     * Returns true if the application allows the selection of several roles at the same time.
     * @return true if the application allows the selection of several roles at the same time, false either.
     */
    public boolean isMultipleRoles();
    
    /** 
     * Fixes the current roles.
     * @param role_id The list role ids to define as current roles.
     */
    public void setCurrentRoles(String[] roleIds);    

    /**
     * Indicates if the popup are enabled or not. By default, the popup menus are enabled.
     * @param popupEnabled True if the popup menu have to be enabled, false either.
     */
    public void setPopupEnabled(boolean popupEnabled);
    
    /**
     * Returns true if the popup menus are enabled.
     * @return true if the popup menus are enabled.
     */
    public boolean isPopupEnabled();
     
    /**
     * Indicates if the Drag and Drop is enabled or not. By default, the Drag and Drop is available.
     * @param dndEnabled True if the popup menu have to be enabled, false either.
     */
    public void setDragAndDropEnabled(boolean dndEnabled);
        
    /**
     * Returns true if the Drag and Drop is enabled.
     * @return true if the Drag and Drop is enabled.
     */
    public boolean isDragAndDropEnabled();
        
}
