/*====================================================================

 Objectweb Explorer Framework
 Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
 Contact: openccm@objectweb.org

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA

 Initial developer(s): ________________________________.
 Contributor(s): ______________________________________.

 ====================================================================*/
package org.objectweb.util.explorer.swing.graph;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphContext;

/**
 * 
 * @version 0.2
 */
public class MyEdgeHandle extends EdgeView.EdgeHandle {

    public MyEdgeHandle(EdgeView edge, GraphContext ctx) {
        super(edge, ctx);
    }

    // Handle mouse pressed event.
    public void mousePressed(MouseEvent event) {
        System.out.println("mouse event");
        /* INV: currentPoint = null; source = target = label = false; */
        boolean bendable =
            graph.isBendable()
                && GraphConstants.isBendable(edge.getAllAttributes());
        boolean disconnectable =
            graph.isDisconnectable()
                && GraphConstants.isDisconnectable(orig.getAllAttributes());
        int x = event.getX();
        int y = event.getY();
        // Detect hit on control point
        int index = 0;
        for (index = 0; index < r.length; index++) {
            if (r[index].contains(x, y)) {
                currentPoint = edge.getPoint(index);
                source =
                    index == 0
                        && (edge.getSource() == null
                            || (disconnectable
                                && GraphConstants.isDisconnectable(
                                    edge
                                        .getSource()
                                        .getParentView()
                                        .getAllAttributes())));
                target =
                    index == r.length - 1
                        && (edge.getTarget() == null
                            || (disconnectable
                                && GraphConstants.isDisconnectable(
                                    edge
                                        .getTarget()
                                        .getParentView()
                                        .getAllAttributes())));
                break;
            }
        }
        // Detect hit on label
        if (!isEditing()
            && graph.isMoveable()
            && GraphConstants.isMoveable(edge.getAllAttributes())
            && loc != null
            && loc.contains(x, y)
            && !isAddPointEvent(event)
            && !isRemovePointEvent(event)) {
            if (event.getClickCount() == graph.getEditClickCount())
                graph.startEditingAtCell(edge);
            else
                label = true;
            // Remove Point
        }
        if (isRemovePointEvent(event)
            && currentPoint != null
            && !source && !target
            && bendable) {
            edge.removePoint(index);
            mouseReleased(event);
            // Add Point
        } else if (isAddPointEvent(event) && !isEditing() && bendable) {
            int s = graph.getHandleSize();
            Rectangle2D rect =
                graph.fromScreen(new Rectangle(x - s, y - s, 2 * s, 2 * s));
            if (edge.intersects(graph.getGraphics(), rect)) {
                Point2D point =
                    graph.fromScreen(
                        graph.snap(new Point(event.getPoint())));
                double min = Double.MAX_VALUE, dist = 0;
                for (int i = 0; i < edge.getPointCount() - 1; i++) {
                    Point2D p = edge.getPoint(i);
                    Point2D p1 = edge.getPoint(i + 1);
                    dist = new Line2D.Double(p, p1).ptLineDistSq(point);
                    if (dist < min) {
                        min = dist;
                        index = i + 1;
                    }
                }
                edge.addPoint(index, point);
                currentPoint = point;
                reloadPoints(edge);
                paint(graph.getGraphics());
            }
        }
        if (isEditing())
            event.consume();
    }

}