/*====================================================================

 Objectweb Explorer Framework
 Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
 Contact: openccm@objectweb.org

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA

 Initial developer(s): Latitia Lafeuille.
 Contributor(s): ______________________________________.

 ====================================================================*/

package org.objectweb.util.explorer.swing.graph;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.jgraph.JGraph;
import org.jgraph.graph.CellViewRenderer;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.Port;
import org.jgraph.graph.PortRenderer;
import org.jgraph.graph.PortView;
import org.jgraph.graph.VertexView;

public class MyPortView extends PortView {

	/** The interface where are defined the graphics method to draw a port */
    private PortGraphicsInterface pg;

	/** The interface where are defined the graphics method to draw a vertex */
    private VertexGraphicsInterface vg;

	/** The port renderer*/
    protected static MyPortRenderer renderer = new MyPortRenderer();

    public MyPortView(JGraph graph, org.jgraph.graph.CellMapper cm, Object cell) {
        super(cell, graph, cm);
        pg = Graph.getPortGraphics();
        vg = Graph.getVertexGraphics();
    }

    /**
     * Returns the port lacation
     */
    public Point2D getLocation(EdgeView edge) {
        Object modelAnchor = null;
        if (cell instanceof Port)
            modelAnchor = ((Port) cell).getAnchor();
        MyPort port = (MyPort) cell;
        PrimitiveVertex vertex = (PrimitiveVertex) ((MyPort) cell).getParent();
        PortView anchor = (PortView) mapper.getMapping(modelAnchor, false);
        Point2D pos = null;
        boolean isAbsolute = GraphConstants.isAbsolute(allAttributes);
        Point2D offset = GraphConstants.getOffset(allAttributes);
        VertexView vertexView = (VertexView) getParentView();
        if (vertexView != null) {
            if (edge != null) {
                MyPort p = (MyPort) getCell();
                String portType = p.getType();
                if (portType.equals(PortType.SERVER_PORT)) {
                    pos = new Point((int) getBounds().getX(), (int) getBounds()
                            .getCenterY());
                } else if ((portType.equals(PortType.CLIENT_PORT))
                        || ((portType.equals(PortType.COLLECTION_PORT)))) {
                    pos = new Point((int) getBounds().getX()
                            + (int) getBounds().getWidth(), (int) getBounds()
                            .getCenterY());
                } else if (portType.equals(PortType.CONTROLLER_PORT)) {
                    pos = new Point((int) getBounds().getCenterX(),
                            (int) getBounds().getY());
                }
            }
        }
        return pos;
    }

    
    /**
     * Returns the bounds for the port view.
     */
    public Rectangle2D getBounds() {

        Rectangle parentBounds = super.getParentView().getBounds().getBounds();
        String portType = ((MyPort) getCell()).getType();
        boolean isGenerated = ((MyPort) getCell()).isGenerated();
        PrimitiveVertex vertex = (PrimitiveVertex) ((MyPort) getCell())
                .getParent();
        Dimension vertexSize = new Dimension(parentBounds.width, parentBounds.height);
        Rectangle bounds = new Rectangle();
        bounds.height = pg.getPortSize(portType, false).height;
        bounds.width = pg.getPortSize(portType, false).width;
        int nbServerPorts, nbClientPorts, nbControllerPorts;
        //if it's an internal port
        if ((vertex instanceof CompositeVertex)
                && ((CompositeVertex) vertex).isOpen()) {
            bounds.height = pg.getPortSize(portType, true).height;
            bounds.width = pg.getPortSize(portType, true).width;
            nbServerPorts = ((CompositeVertex) vertex)
                    .getListInternalServerPorts().size();
            nbClientPorts = ((CompositeVertex) vertex)
                    .getListInternalClientPorts().size();
            nbControllerPorts = ((CompositeVertex) vertex)
                    .getListInternalControllerPorts().size();
        } 
        //else
        else {
            nbServerPorts = vertex.getListServerPorts().size();
            nbClientPorts = vertex.getListClientPorts().size();
            nbControllerPorts = vertex.getListControllerPorts().size();
        }
        //if the port is a server port
        //the bounds must be at the left of the parent vertex
        if (portType.equals(PortType.SERVER_PORT)) {
            int interval = (vertexSize.height - PortGraphicsInterface.FIRST_PORT_Y)
                    / nbServerPorts;
            if (!isGenerated) {
                bounds.x = parentBounds.x - bounds.width;
            } else {
                bounds.x = parentBounds.x + vertexSize.width - bounds.width
                        - VertexGraphicsInterface.COMPOSITE_MEMBRANE_SIZE;
            }
            bounds.y = PortGraphicsInterface.FIRST_PORT_Y + parentBounds.y
                    + interval * (((MyPort) getCell()).getNumber() - 1);
        } 
		//if the port is a client port
		//the bounds must be at the right of the parent vertex
        else if ((portType.equals(PortType.CLIENT_PORT))
                || (portType.equals(PortType.COLLECTION_PORT))) {
            int interval = (vertexSize.height - PortGraphicsInterface.FIRST_PORT_Y)
                    / nbClientPorts;
            if (!isGenerated) {
                bounds.x = parentBounds.x + vertexSize.width;
            } else {
                bounds.x = parentBounds.x
                        + VertexGraphicsInterface.COMPOSITE_MEMBRANE_SIZE;
            }
            bounds.y = PortGraphicsInterface.FIRST_PORT_Y + parentBounds.y
                    + interval * (((MyPort) getCell()).getNumber() - 1);
        }
		//if the port is a client port
		//the bounds must be at the up of the parent vertex
         else if (portType.equals(PortType.CONTROLLER_PORT)) {
            int interval = vertexSize.width / nbControllerPorts;
            bounds.x = parentBounds.x + interval
                    * (((MyPort) getCell()).getNumber() - 1)
                    + pg.getPortSize(portType, false).width;
            bounds.y = parentBounds.y - bounds.height;
        }
        
        return bounds;
    }

    public CellViewRenderer getRenderer() {
        return renderer;
    }

    public static class MyPortRenderer extends PortRenderer {

		/**
		 * The method is called to paint the port
		 */
        public void paint(Graphics g) {
            PortGraphicsInterface pg = Graph.getPortGraphics();
            MyPort port = (MyPort) view.getCell();
            PrimitiveVertex vertex = (PrimitiveVertex) port.getParent();
            String portType = port.getType();
            Rectangle bounds = g.getClipBounds();
            boolean isInternal = false;
            if ((vertex instanceof CompositeVertex)
                    && (((CompositeVertex) vertex).isOpen())) {
                isInternal = true;
            } else {
                bounds.height = pg.getPortSize(portType, isInternal).height;
                bounds.width = pg.getPortSize(portType, isInternal).width;
            }
            g.setClip(bounds.x, bounds.y, bounds.width, bounds.height);
            if (!((portType.equals(PortType.CONTROLLER_PORT)) && (!(vertex
                    .getControllerDisplay()))))
                pg.drawPort(g, port.getName(), portType, isInternal);
        }

    }

}