/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.swing.gui.api;

/**
 * This interface is mainly used for complicated actions. 
 * In other word, you can you this interface when you use a JDialog to specify parameters.
 * You can easily create simple JDialog without worrying about how the component are displayed.
 * Here is an example of action using the DialogAction:
 * <pre>
 *   public class CreateInterfaceTypeAction 
 *       implements MenuItem, DialogAction {   
 * 
 *       protected BooleanBox booleanBox_;
 *       protected LabelBox nameBox_;
 *
 *       protected void createBox() {
 *           nameBox_ = new LabelBox("Name");
 *           dialog_.addBox(nameBox_);
 *           booleanBox_ = new BooleanBox("Value","TRUE","FALSE",true);
 *           dialog_.addBox(booleanBox_);
 *       }
 *
 *       public void actionPerformed(MenuItemEvent e) throws Exception{
 *           ...     
 *           DialogBox dialog = new DefaultDialogBox("Sample dialog");      
 *           createBox();
 *           dialog.setValidateAction(this);
 *           dialog.show();
 *       }
 *
 *       public int getStatus(TreeView treeView) {
 *           return MenuItem.DISABLED_STATUS;
 *       }
 *
 *       public void executeAction() throws Exception {
 *           String name = nameBox_.getLabel();
 *           boolean isClient = isClientBox_.getValue();
 *           ...
 *       }
 *
 *   }
 * </pre> 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public interface DialogAction {

    /**
     * Executes an action
     * @throws Exception Any exception may be thrown
     */
    public void executeAction() throws Exception;

}