/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.swing.gui.lib;

import java.awt.Component;
import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;

import org.objectweb.util.explorer.swing.gui.api.ValidateReport;


/**
 * This class represents the panel which allow to specify a label
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BooleanBox  
    extends AbstractElementBox{

   //==================================================================
   //
   // Internal states.
   //
   //==================================================================

    protected ButtonGroup group_ = null;

    //==================================================================
    //
    // Constructors.
    //
    //==================================================================

    /** 
     * Default constructor
     * @param label The label of the property
     */
    public BooleanBox(String label) {  
        this(label, "True","False", true);
    }
        
    /** 
     * Default constructor
     * @param label The label of the property
     * @param trueLabel The label of the "true" property
     * @param falseLabel The label of the "false" property
     * @param defaultValue The checked value
     */
    public BooleanBox(String label, String trueLabel, String falseLabel, boolean defaultValue) {
        JRadioButton trueButton, falseButton;
        add(Box.createHorizontalGlue());
        JLabel fieldLabel = new JLabel(label + ": ", SwingConstants.RIGHT);
        fieldLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        fieldLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(fieldLabel);
        add(Box.createHorizontalStrut(5));
        trueButton = new JRadioButton(trueLabel);
        trueButton.setActionCommand("true");
        if(defaultValue)
            trueButton.setSelected(true);
        trueButton.setPreferredSize(new Dimension(110, 20));
        trueButton.setMaximumSize(new Dimension(110, 20));
        add(trueButton);
        add(Box.createHorizontalStrut(5));
        falseButton = new JRadioButton(falseLabel);
        falseButton.setActionCommand("false");
        if(!defaultValue)
            falseButton.setSelected(true);
        falseButton.setPreferredSize(new Dimension(110, 20));
        falseButton.setMaximumSize(new Dimension(110, 20));
        add(falseButton);
        
        group_ = new ButtonGroup();
        group_.add(trueButton);
        group_.add(falseButton);
        
    }

    // ==================================================================
    //
    //  Public methods for ElementBox.
    //
    // ==================================================================

    /**
     * Validates the content of the ElementBox. Indicates if the different value are acceptables or not.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateBox(){
        return new DefaultValidateReport();
    }
    
    /**
     * Returns the Box contains by the ElementBox.
     * @return The Box contains by the ElementBox.
     */
    public Box getBox(){
        return this;
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /** 
     * Returns the specify label
     * @return The value of the boolean box. 
     */
    public boolean getValue() {
        return group_.getSelection().getActionCommand().equals("true");
    }

}
