/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.swing.gui.lib;

import java.awt.Dimension;

import javax.swing.Box;
import javax.swing.JScrollPane;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.swing.api.Explorer;
import org.objectweb.util.explorer.swing.gui.api.ValidateReport;

/**
 * This class represents the panel which displays a DynamicTree. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * 
 * @version 0.1
 */
public class TreeBox 
    extends AbstractElementBox {

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The Tree component. */
    protected Component component_;
    
    /** The Tree interface of the Tree component. */
    protected Tree treeItf_;

    /** The Explorer interface of the Tree component. */
    protected Explorer explorerItf_;
    
    /** TheJScrollPane associated to the tree */
    protected JScrollPane scrollPane_;

    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    /**
     * Default constructor
     * @param tree The DynamicTree to use
     */     
    public TreeBox(Component treeComponent) {
        this(treeComponent,true);   
    }

    /**
     * 
     * @param tree The DynamicTree to use
     * @param isMandatory If the value is mandatory : TRUE, else : FALSE
     */
    public TreeBox(Component treeComponent, boolean isMandatory) {
        super(isMandatory);
        initAttributes(treeComponent);
        scrollPane_ = new JScrollPane(explorerItf_.getTree()); 
        add(scrollPane_);
    }

    /**
     * Invokes just before displaying.
     */
    public void preInitialize(){
        if(explorerItf_!=null){
            explorerItf_.setPopupEnabled(false);
            explorerItf_.setDragAndDropEnabled(false);
        }
    }
    
    /**
     * Invokes just before removing.
     */
    public void postInitialize(){
        if(explorerItf_!=null){
            explorerItf_.setPopupEnabled(true);
            explorerItf_.setDragAndDropEnabled(true);
        }
    }

    //==================================================================
    //
    // Internal methods.
    //
    //==================================================================

    protected void initAttributes(org.objectweb.fractal.api.Component treeComponent){
        component_ = treeComponent;
        if(component_!=null){
            try {
                treeItf_ = (Tree)component_.getFcInterface(Tree.TREE);
                explorerItf_ = (Explorer)component_.getFcInterface(Explorer.EXPLORER);
            } catch (NoSuchInterfaceException e) {
                e.printStackTrace();
            }
        }        
    }
    
    // ==================================================================
    //
    //  Public methods for ElementBox.
    //
    // ==================================================================

    /**
     * Validates the content of the ElementBox. Indicates if the different value are acceptables or not.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateBox(){
        if(isMandatory_){
            if(getObject()==null)
                return new DefaultValidateReport(false,"You must select a component");
        }
        return new DefaultValidateReport();
    }
    
    /**
     * Returns the Box contains by the ElementBox.
     * @return The Box contains by the ElementBox.
     */
    public Box getBox(){
        return this;
    }

    //==================================================================
    //
    // Public methods.
    //
    //==================================================================

    /** Returns the chosen Object */
    public Object getObject(){
        Object o = null;
        Entry entry = treeItf_.getSelectedEntry();
        if (entry != null) {
            o = entry.getValue();
        }
        return o;        
    }

    /**
     * Sets the preferred dimension of the JScrollPane.
     * @param d The preferred dimension of the JScrollPane
     */
    public void setPreferredSize(Dimension d){
        scrollPane_.setPreferredSize(d);        
    }

}