/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.explorer.swing.gui.lib;

/** The java API's imports */
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.SwingConstants;
import javax.swing.JOptionPane;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.swing.api.Explorer;
import org.objectweb.util.explorer.swing.gui.api.TreeChooserObserver;
import org.objectweb.util.explorer.swing.gui.api.ValidateReport;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;

/**
 * This class represents the panel which allows to choose an element in the tree.
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class TreeChooserBox 
    extends AbstractElementBox {

    //==================================================================
    //
    // Internal state.
    //
    //==================================================================

    /** The value of the label */
    protected String label_;

    /** The JButton used to browse the tree */
    protected JButton browseButton_;

    /** The Tree component. */
    protected org.objectweb.fractal.api.Component component_;
    
    /** The Tree interface of the Tree component. */
    protected Tree treeItf_;

    /** The Explorer interface of the Tree component. */
    protected Explorer explorerItf_;
    
    /** */
    protected JTextField textField_;

    /** The choosen object */
    protected Object object_;

    /** The Observer. */
    protected TreeChooserObserver treeChooserObserver_ = null;

    //==================================================================
    //
    // Constructors.
    //
    //==================================================================

    /**
     * Default constructor
     * @param label The label of the box
     * @param tree The DynamicTree to use
     */     
    public TreeChooserBox(String label, org.objectweb.fractal.api.Component treeComponent) {
        this(label,treeComponent,true);   
    }
    
    /**
     * Constructor which specifies if the value is mandatory or not
     * @param label The label of the box
     * @param tree The DynamicTree to use
     * @param isMandatory If the value is mandatory : TRUE, else : FALSE
     */
    public TreeChooserBox(String label, org.objectweb.fractal.api.Component treeComponent, boolean isMandatory) {
        super(isMandatory);
        label_ = label;
        
        initAttributes(treeComponent);

        if(explorerItf_!=null){
            explorerItf_.setPopupEnabled(false);
            explorerItf_.setDragAndDropEnabled(false);
        }
        add(Box.createHorizontalGlue());
        JLabel explorerLabel = new JLabel(label + ": ", SwingConstants.RIGHT);
        explorerLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        explorerLabel.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(explorerLabel);
        add(Box.createHorizontalStrut(5));
        textField_ = new JTextField();
        textField_.setEditable(false);
        textField_.setPreferredSize(new Dimension(200, 20));
        textField_.setMaximumSize(new Dimension(200, 20));
        add(textField_);
        add(Box.createHorizontalStrut(5));
        browseButton_ = new JButton(new BrowseAction(this));
        browseButton_.setPreferredSize(new Dimension(20, 20));
        browseButton_.setMaximumSize(new Dimension(20, 20));
        add(browseButton_);
    }

    // ==================================================================
    //
    //  Internal methods.
    //
    // ==================================================================
        
    protected void initAttributes(org.objectweb.fractal.api.Component treeComponent){
        component_ = treeComponent;
        if(component_!=null){
            try {
                treeItf_ = (Tree)component_.getFcInterface(Tree.TREE);
                explorerItf_ = (Explorer)component_.getFcInterface(Explorer.EXPLORER);
            } catch (NoSuchInterfaceException e) {
                e.printStackTrace();
            }
        }        
    }
    
    // ==================================================================
    //
    //  Public methods for ElementBox.
    //
    // ==================================================================

    /**
     * Validates the content of the ElementBox. Indicates if the different value are acceptables or not.
     * @return The corresponding ValidateReport.
     */
    public ValidateReport validateBox(){
        if(isMandatory_){
            if(getObject()==null)
                return new DefaultValidateReport(false,"The \"" + label_ + "\" value is mandatory");
        }
        return new DefaultValidateReport();
    }

    /**
     * Invokes just before displaying.
     */
    public void preInitialize(){
        if(explorerItf_!=null){
            explorerItf_.setPopupEnabled(false);
            explorerItf_.setDragAndDropEnabled(false);
        }
    }
    
    /**
     * Invokes just before removing.
     */
    public void postInitialize(){
        if(explorerItf_!=null){
            explorerItf_.setPopupEnabled(true);
            explorerItf_.setDragAndDropEnabled(true);
        }
    }
    
    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /** Returns the chosen Object */
    public Object getObject() {
        return object_;
    }

    /** Fixes the TreeChooserObserver */
    public void setTreeChooserObserver(TreeChooserObserver treeChooserObserver) {
        treeChooserObserver_ = treeChooserObserver;
    }

    /**
     * Action which 
     */
    protected class BrowseAction extends AbstractAction {

        /** The parent of the panel */
        protected Component parent_;

        /**
         * Default constructor
         */
        protected BrowseAction(Component parent) {
            super("...", null);
            parent_ = parent;
        }

        public void actionPerformed(java.awt.event.ActionEvent ae) {
            if(treeChooserObserver_ != null) {
                initAttributes(treeChooserObserver_.refresh());
                explorerItf_.setPopupEnabled(false);
            }
            JPanel treePanel = new JPanel();
            treePanel.setLayout(new GridLayout(1, 0));
            treePanel.setPreferredSize(new Dimension(450, 350));
            treePanel.add(new JScrollPane(explorerItf_.getTree()));
            int result = JOptionPane.showOptionDialog((Component)ae.getSource(), treePanel, "Select an object", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE, null, null, null);
            if (result == 0) {
                Entry entry = treeItf_.getSelectedEntry();
                if (entry != null) {
                    Object o = entry.getValue();
                    if (o != null) {
                        object_ = o;
                        textField_.setText(entry.getName().toString());               
                    }
                }

            }
        }

    }

}
