/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swing.lib;

import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.JPanel;

import org.objectweb.util.explorer.api.Panel;
import org.objectweb.util.explorer.api.Table;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;
import org.objectweb.util.explorer.swing.lib.DynamicTable;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class BasicTablePanel 
  implements Panel 
{

	//==================================================================
	//
	// Internal States.
	//
	// ==================================================================

    protected JPanel panel_;

	/**
	 * The used table
	 */
	protected Table table_;

	/**
	 * The used PropertyResolver
	 */
	protected PropertyResolver propertyResolver_;

	/**
	 * The used DescriptionInterpreter
	 */
	protected DescriptionInterpreter descriptionInterpreter_;

	/**
	 * The current tree
	 */
	protected Tree tree_;

    
//    protected AdminCustomization custom_;

//    protected Decoder decoder_;
    
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /*
    public BasicTablePanel(Table table, AdminCustomization custom, Decoder currentDecoder){
        panel_ = new JPanel();
        panel_.setBackground(Color.white);
        panel_.setBorder(BorderFactory.createLineBorder(Color.black));
        table_ = table;
        decoder_ = currentDecoder;
        custom_ = custom;
        //panel_.setLayout(new BoxLayout(panel_, BoxLayout.Y_AXIS));
        panel_.setLayout(new BorderLayout());
    }

    public BasciTablePanel(Table table, AdminCustomization custom){
        this(table, custom, null);
    }
    */
    
    public BasicTablePanel(){
        panel_ = new JPanel();
        panel_.setBackground(Color.white);
        panel_.setBorder(BorderFactory.createLineBorder(Color.black));
        //panel_.setLayout(new BoxLayout(panel_, BoxLayout.Y_AXIS));
        panel_.setLayout(new BorderLayout());
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for ... interface. 
    //
    // ==================================================================    
    
    public void setPropertyResolver(PropertyResolver propertyResolver){
        // TODO: A fractal assembly is needed here!!! (dynamic bindings)
        propertyResolver_ = propertyResolver;
    }
    
    public void setDescriptionInterpreter(DescriptionInterpreter descriptionInterpreter){
        // TODO: A fractal assembly is needed here!!! (dynamic bindings)
        descriptionInterpreter_ = descriptionInterpreter;
    }    
    
    public void setTree(Tree tree){
        // TODO: A fractal assembly is needed here!!! (dynamic bindings)
        tree_ = tree;
    }
    
    public void setTable(Table table){
        table_ = table;
    }
    
    // ==================================================================
    //
    // Public methods for ... interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Panel#selected(org.objectweb.util.explorer.api.TreeView)
     */
    public void selected(TreeView treeView) {
        if(table_!=null){
            DynamicTable dynamicTable = null;
//            if(decoder_!=null)
//                dynamicTable = new DynamicTable(decoder_);    
//            else
                dynamicTable = new DynamicTable();
                dynamicTable.setPropertyResolver(propertyResolver_);
                dynamicTable.setDescriptionInterpreter(descriptionInterpreter_);
                dynamicTable.setTree(tree_);
//            dynamicTable.setAdminCustomization(custom_);
            String[] headers = table_.getHeaders(treeView);
            Object[][] values = table_.getRows(treeView);
            if(values!=null && values.length>0){
                dynamicTable.setData(headers,values);
                //panel_.add(new JScrollPane(dynamicTable));
                panel_.add(dynamicTable.getTableHeader(),BorderLayout.NORTH);
                panel_.add(dynamicTable,BorderLayout.CENTER);
            }
        }       
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Panel#getPanel()
     */
    public Object getPanel() {
        return panel_;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Panel#unselected(org.objectweb.util.explorer.api.TreeView)
     */
    public void unselected(TreeView treeview) {
        // Nothing to do!
    }

}


