/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swing.lib;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.URL;

import javax.swing.Icon;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;

import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.IconProvider;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.menu.lib.DefaultMenuItemTreeView;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;
import org.objectweb.util.explorer.swing.icon.IconFileProvider;
import org.objectweb.util.explorer.swing.icon.ObjectIconProvider;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DynamicTable 
     extends JTable 
{

	//==================================================================
	//
	// Internal States.
	//
	// ==================================================================

    protected TableCellRenderer tableCellRenderer_;
	protected PropertyResolver propertyResolver_;
	protected DescriptionInterpreter descriptionInterpreter_;
	protected Tree tree_;
	protected Icon objectIcon_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    public DynamicTable(){
        super();
        tableCellRenderer_ = new MyTableCellRenderer();
        addMouseListener(new MyMouseAdapter());
        setColumnSelectionAllowed(true);
        setRowSelectionAllowed(true);
        objectIcon_ = (Icon)(new ObjectIconProvider()).newIcon(null);
    }
    
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected PropertyResolver getPropertyResolver(){
        return propertyResolver_;
    }
    
    protected DescriptionInterpreter getDescriptionInterpreter(){
        return descriptionInterpreter_;
    }
    
    protected Tree getTree(){
        return tree_;
    }
    
    protected Icon getImageIcon(Entry entry){
        if(getPropertyResolver()!=null){
            Description desc = getPropertyResolver().resolve(ExplorerConstants.ICON_PROPERTY, entry);
            IconProvider iconProvider = (IconProvider) getDescriptionInterpreter().interprete(desc, null);
            if(iconProvider!=null){
                Object object = iconProvider.newIcon(entry.getValue());
                if(object instanceof Icon){
                    return (Icon)object;
                } else if(object instanceof String) {
                    return (Icon)(new IconFileProvider((String)object)).newIcon(entry.getValue()); 
                } else if (object instanceof URL) {
                    return (Icon)(new IconFileProvider((URL)object)).newIcon(entry.getValue());
                }
            }
        }
        return null;
    }    
    
    //==================================================================
    //
    // Public methods overriding JTable methods.
    //
    //==================================================================

    public TableCellRenderer getCellRenderer(int row, int col) {
        return tableCellRenderer_; 
    }

    public void setPropertyResolver(PropertyResolver propertyResolver){
        // TODO: A fractal assembly is needed here!!! (dynamic bindings)
        propertyResolver_ = propertyResolver;
    }
    
    public void setDescriptionInterpreter(DescriptionInterpreter descriptionInterpreter){
        // TODO: A fractal assembly is needed here!!! (dynamic bindings)
        descriptionInterpreter_ = descriptionInterpreter;
    }

    public void setTree(Tree tree){
        // TODO: A fractal assembly is needed here!!! (dynamic bindings)
        tree_ = tree;
    }
    
    public void setData(String[] headers, Object[][] values){
        setModel(new TableModel(headers, values));                
    }

    //==================================================================
    //
    // Public methods for TableConfiguration interface.
    //
    //==================================================================

    protected class TableModel
            extends AbstractTableModel {
    
        protected String[] headers_;
        protected Object[][] values_; 
    
        public TableModel(String[] headers, Object[][] values){
            headers_ = headers;
            values_ = new Object[values.length][0];
            if(values_!=null){
                for(int i = 0 ; i < values.length ; i++){
                    values_[i] = new Object[values[i].length];
                    for(int j = 0 ; j < values[i].length ; j++){
                        Object value = values[i][j];
                        if(value!=null){
                            if(value instanceof Entry){
                                values_[i][j] = (Entry)value;
                            } else {
                                //values_[i][j] = decoder_.decode(value);
                                Entry newEntry = new DefaultEntry(value.toString(), value);
                                values_[i][j] = newEntry;
                            }
                        }
                    }
                }
            }
        }
    
        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getColumnName(int)
         */
        public String getColumnName(int column) {
            if(headers_!=null)
                return headers_[column];
            return super.getColumnName(column);
        }

        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getColumnCount()
         */
        public int getColumnCount() {
            if(headers_!=null)
                return headers_.length;
            return 0;
            }

        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getRowCount()
         */
        public int getRowCount() {
            if(values_ != null){
                return values_.length;
            }
            return 0; 
        }

        /**
         * Overriding methods
         * @see javax.swing.table.TableModel#getValueAt(int, int)
         */
        public Object getValueAt(int rowIndex, int columnIndex) {
            if(values_ != null)
                return values_[rowIndex][columnIndex];
            return null;
        }
        
        
    }

    /**
     * Mouse Adapter
     * Use to draw Popup Menu
     */
    protected final class MyMouseAdapter extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            popupLayout(e);
        }
        public void mouseReleased(MouseEvent e) {
            popupLayout(e);
        }
        private void popupLayout(MouseEvent e) {
            if (e.isPopupTrigger()) {
                Point point = e.getPoint();
                int row = rowAtPoint(point);
                int col = columnAtPoint(point);
                if((row != -1) && (col != -1)) {
                    Entry object = (Entry)getValueAt(row,col);
                    if(getPropertyResolver()!=null && getDescriptionInterpreter()!=null){
                        Description desc = getPropertyResolver().resolve(ExplorerConstants.MENU_PROPERTY, object, null);
                        JPopupMenu menu = (JPopupMenu) getDescriptionInterpreter().interprete(desc, new DefaultMenuItemTreeView(getTree(),object,null));
                        if (menu != null) {
                            changeSelection(row,col,false,false);
                            menu.show(DynamicTable.this,(int) (point.getX()),(int) (point.getY()));
                        }
                    }
                } 
            }   
        }
    }

    /**
     * Table Cell Renderer
     * Use to put the icon corresponding to the class 
     * of the Value object of the Entry
     */
    protected final class MyTableCellRenderer extends DefaultTableCellRenderer {

        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
            if(value != null) {
                Entry entry = (Entry)value;
                setText(entry.getName().toString());
                Icon icon = getImageIcon(entry); 
                setIcon(icon==null?objectIcon_:icon);
            } else {
                setIcon(null);
            }
            return this; 
        }

    }
    
}


