/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================
$Id: DynamicTree.java,v 1.5 2005/07/06 15:36:02 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.swing.lib;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.Autoscroll;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceContext;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.net.URL;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.Icon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.api.control.SuperController;
import org.objectweb.fractal.api.factory.Factory;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.util.explorer.api.Context;
import org.objectweb.util.explorer.api.DropAction;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.IconProvider;
import org.objectweb.util.explorer.api.Info;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.api.RootContext;
import org.objectweb.util.explorer.api.RootEntry;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.api.TreeView;
import org.objectweb.util.explorer.context.api.ContextContainerFactory;
import org.objectweb.util.explorer.core.common.api.ContextContainer;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.common.api.Synchronization;
import org.objectweb.util.explorer.core.common.api.TextComponent;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.core.common.lib.DefaultTreeView;
import org.objectweb.util.explorer.core.common.lib.RootContextContainer;
import org.objectweb.util.explorer.core.common.lib.TreeBindingFeature;
import org.objectweb.util.explorer.core.dnd.api.DnDDescription;
import org.objectweb.util.explorer.core.dnd.api.DnDDescriptions;
import org.objectweb.util.explorer.core.dnd.lib.DefaultDropTreeView;
import org.objectweb.util.explorer.core.dnd.lib.EntryTransferable;
import org.objectweb.util.explorer.core.menu.lib.DefaultMenuItemTreeView;
import org.objectweb.util.explorer.core.naming.lib.DefaultEntry;
import org.objectweb.util.explorer.core.role.api.RoleManager;
import org.objectweb.util.explorer.core.role.api.RoleProvider;
import org.objectweb.util.explorer.core.root.lib.DefaultRootEntry;
import org.objectweb.util.explorer.interpreter.api.DescriptionInterpreter;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;
import org.objectweb.util.explorer.swing.api.Explorer;
import org.objectweb.util.explorer.swing.icon.IconFileProvider;
import org.objectweb.util.explorer.swing.menu.GenericAction;
import org.objectweb.util.trace.Trace;
import org.objectweb.util.trace.TraceSystem;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DynamicTree 
     extends JTree 
  implements Tree, BindingController, LifeCycleController, Explorer, Autoscroll 
{

    //===================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /**
     * The binding feature to use.
     */
    protected BindingFeature bindingFeature_ = null;

    /**
     * The root node
     */
    protected DefaultMutableTreeNode rootNode_;

    /**
     * Model to handle nodes.
     */
    protected DefaultTreeModel treeModel_;

    /**
     * The context containing the initial entries of the tree.
     */
    protected List initialContext_;

    /**
     * The graphical initial context (The entries might be replaced by their
     * associated context)
     */
    protected ContextContainer graphicalInitialContext_;

    /**
     * List of elements which have already been added into the initial context.
     * This list is used to register the fact that the entry has already been
     * added into the tree. Then, it hasn't to take into account the expansion
     * level defined in the RootEntry.
     */
    protected List alreadyInInitialContext_ = null;

    /**
     * Model to manage node selection
     */
    protected DefaultTreeSelectionModel selectionModel_;
    
    
    /** To know if popup menus have to be displayed or not. */
    protected boolean popupEnabled_ = true;

    /**
     * Registers the state of the node.
     */
    protected Map nodeStates_ = null;

    /**
     * Refers to the selected node.
     */
    protected JMenuBar menuBar_ = null;

    /**
     * The customizable menus.
     */
    protected JMenu actionMenu_ = null;

    /**
     * The customizable menus.
     */
    protected JMenu roleMenu_ = null;

    /**
     * The menu bar of the frame.
     */
    protected JToolBar toolBar_ = null;

    /** The number of element of the toolBar which has to be removed. */
    protected int nbElementsToRemove_ = 0;

    /**
     * To know if the application allows the selection of several roles at the
     * same time.
     */
    protected boolean multipleRoles_ = false;

    // Drag and Drop attributes

    /** To know if the Drag and Drop have to be enabled or not */
    protected boolean dndEnabled_ = true;

    /** The DragSource to use */
    protected DragSource dragSource_;

    /** The DragGestureListener to use */
    protected DragGestureListener dragGestureListener_;

    /** The DragSourceListener to use */
    protected DragSourceListener dragSourceListener_;

    /** The dragAction */
    protected int dragAction_ = DnDConstants.ACTION_COPY_OR_MOVE
            | DnDConstants.ACTION_LINK;

    /** The allowed drop action */
    protected int acceptableActions_ = DnDConstants.ACTION_COPY_OR_MOVE
            | DnDConstants.ACTION_LINK;

    /** The DropTarget */
    protected DropTarget dropTarget_;

    /** The DropTargetListener to use */
    protected DropTargetListener dropTargetListener_;    
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Calls by the constructor
     */
    private void initDragAndDrop(){
        // Drag
        dragSource_ = DragSource.getDefaultDragSource();
        dragSourceListener_ = new TreeDragSourceListener();
        dragGestureListener_ = new TreeDragGestureListener();
        dragSource_.createDefaultDragGestureRecognizer(this, dragAction_, dragGestureListener_);
        
        // Drop
        dropTargetListener_ = new TreeDropTargetListener();
        dropTarget_ = new DropTarget(this, acceptableActions_, dropTargetListener_, true); 
    }    
    
    public DynamicTree() {
        super();
        
        bindingFeature_ = new TreeBindingFeature();
        
        initialContext_ = new Vector();
        alreadyInInitialContext_ = new Vector();

        nodeStates_ = new HashMap();
        //selectedNode_ = "";
        
        rootNode_ = new DefaultMutableTreeNode(new DefaultEntry("Root",graphicalInitialContext_));
        treeModel_ = new MyTreeModel(rootNode_);
        setModel(treeModel_);
        
        selectionModel_ = new DefaultTreeSelectionModel();
        selectionModel_.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        setSelectionModel(selectionModel_);
        addMouseListener(new MyMouseAdapter());
        addTreeWillExpandListener(new MyTreeWillExpandListener());
        addTreeExpansionListener(new MyTreeExpansionListener());
        addTreeSelectionListener(new MyTreeSelectionListener());
        setCellRenderer(new MyTreeCellRenderer());        
        
        initDragAndDrop();
        
        setToolTipText("");
        setShowsRootHandles(true);
        collapseRow(0);
        expandRow(0);     
        setRootVisible(false);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    protected Trace getTrace(){
        return TraceSystem.get("explorer");
    }

    /**
     * Returns the entry associates to a node
     * 
     * @param o The node
     * @return The associated entry
     */
    protected Entry objectToEntry(Object o) {
        if (o != null) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) o;
            return (Entry) node.getUserObject();
        }
        return null;
    }

    /**
     * Gets a string representation of the path
     * 
     * @param node The node
     * @return The path from th root to the node
     * 
     */
    protected String pathToString(DefaultMutableTreeNode node) {
        StringBuffer sf = new StringBuffer();
        Object[] o = node.getUserObjectPath();
        for (int i = 1; i < o.length; i++) {
            sf.append(((Entry) o[i]).getName().toString());
            if (i != o.length - 1)
                sf.append("/");
        }
        return sf.toString();
    }    
    
    /**
     * Gets a string representation of the path
     * 
     * @param node The node
     * @return The path from th root to the node
     * 
     */
    protected String pathToString(TreePath path) {
        if(path!=null)
            return pathToString((DefaultMutableTreeNode)path.getLastPathComponent());
        return "";
        
    }        
    
    protected DefaultMutableTreeNode getChildren(DefaultMutableTreeNode parent, String name){
        Enumeration enumeration = parent.children();
        while (enumeration.hasMoreElements()) {
            DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enumeration.nextElement();
            Entry entry = (Entry)childNode.getUserObject();
            if(name.equals(entry.getName())){
                return childNode;
            }
            
        }
        return null;
    }
    
    /**
     * Provides the path associated to the given representation. 
     * Each node needs to be separated by the '/' character.
     * @param path The representation of the path
     * @return The associated path from the tree.
     */
    protected TreePath stringToPath(String path) {
        if (path != null && !path.equals("")) {
            // Splits the path representation
            // and for each element, tries to obtain the child 
            // of the current element. 
            StringTokenizer st = new StringTokenizer(path,"/");
            DefaultMutableTreeNode parentNode = rootNode_;
            DefaultMutableTreeNode childNode = null;
            while (st.hasMoreTokens()) {
                String currentPath = st.nextToken();
                childNode = getChildren(parentNode, currentPath);
                if(childNode!=null){
                    parentNode = childNode;
                } else {
                    return new TreePath(parentNode.getPath());
                }
            }
            return new TreePath(childNode.getPath());
        }
        return null;
    }
    
    /**
     * Gives the path of the object pointing by the cursor
     * 
     * @param p The position of the cursor
     */
    protected TreePath getPath(Point p) {
        if (p != null) {
            return getPathForLocation((int) p.getX(), (int) p.getY());
        }
        return null;
    }

    /**
     * Return the selected object
     */
    protected Object getSelectedObject(){
        Entry entry = getSelectedEntry();
        if(entry!=null){
            return entry.getValue();
        }
        return null;
    }
    
    /**
     * Returns the parent of the selected entry
     */
    protected Entry getParentEntry() {
        Object object = getLastSelectedPathComponent();
        if (object != null)
            return objectToEntry(((DefaultMutableTreeNode) object).getParent());
        return null;
    }    
    
    /**
     * Provides a view on the tree for the selected node.
     * @return The view on the tree.
     */
    protected TreeView getTreeView() {
        return new DefaultTreeView((Tree)this, getSelectedEntry(), getParentEntry());
    }

    /**
     * Provides a menu item view for the selected node.
     * @return The view on the tree.
     */
    protected MenuItemTreeView getMenuItemTreeView() {
        return new DefaultMenuItemTreeView((Tree)this, getSelectedEntry(), getParentEntry());
    }

    /**
     * Copies the content of the initialContext into the graphicalIntialContext
     */
    protected void createGraphicalInitialContext() {
        // Clears the existing initial context.
        getGraphicalInitialContext().clear();     
        // Finds and copies the root entries
        Description desc = getPropertyResolver().resolve(ExplorerConstants.ROOT_PROPERTY, null);
        if(desc!=null && ! desc.isEmpty()){
            RootContext rootContext = (RootContext) getDescriptionInterpreter().interprete(desc, null);
            RootEntry[] entries = rootContext.getEntries();
            for (int i = 0; i < entries.length; i++) {
                addToInitialContext(entries[i]);
            }
        }
        // Copies the content of the initialContext
        for(int i=0 ; i<initialContext_.size() ; i++) {
            RootEntry rootEntry = (RootEntry)initialContext_.get(i);
            // Create a new RootEntry in oder to avoid having undefined context
            // when the role changes.
            RootEntry newRootEntry = new DefaultRootEntry(rootEntry.getName(), rootEntry.getValue(), rootEntry.getLevel());
            addToInitialContext(newRootEntry);
        }        
     }

    /**
     * Add an object into the initial context and expands the ndoe until a specific level if this one is a context
     *  
     * @param id Name of the object
     * @param object The object to add
     * @param level The level (>0 for a specific level)
     */
    protected void addToInitialContext(RootEntry entry) {
        if (entry!=null) {
            getGraphicalInitialContext().addEntry(entry);
            refreshRootNode();
            //TreePath path = getPathFromInitialContext(id);
            //refreshPath(path);
            String id = entry.getName().toString();
            if(!alreadyInInitialContext_.contains(id)){
                if (entry.getLevel() > 0) {
                    TreePath path = stringToPath(id);
                    if (path != null) {
                        expandPath(path);
                        openTree(path, entry.getLevel() - 1);
                    }
                }
                alreadyInInitialContext_.add(id);
            }
        }
    }    

    /**
     * Open the tree nodes until a specific level
     *
     * @param root The root TreePath to develop
     * @param level The level (>0 for a specific level)
     */
    protected void openTree(TreePath root, int level) {
        if (level > 0) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) root.getLastPathComponent();
            Enumeration enumeration = node.children();
            while (enumeration.hasMoreElements()) {
                DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enumeration.nextElement();
                TreePath p = new TreePath(childNode.getPath());
                expandPath(p);
                openTree(p, level - 1);
            }
            //      }else if (level <0){
            //          for(int i=0;i<getRowCount();i++)
            //          expandRow(i);
        }
    }

    /**
     * Updates the name of the pathes.
     * This method is only usable for initial entry. 
     * It has to be improved to order to be used in a larger context (such as all type of entries)
     * @param currentName The current name of the node.
     * @param newName The new name of the node.
     */
    protected void updateInitialPathes(String currentName, String newName){
        Object[] keys = nodeStates_.keySet().toArray();
        for(int i=0 ; i<keys.length ; i++) {
            if(keys[i].toString().startsWith(currentName)){
                String suffixe = ((String)keys[i]).substring(currentName.length());
                String currentPath = currentName + suffixe;
                String newPath = newName + suffixe;
                nodeStates_.put(newPath,nodeStates_.get(currentPath));
                nodeStates_.remove(currentPath);
            }
        }          
    }    
    
    // ==================================================================
    // Access to Fractal interfaces
    // ==================================================================
    
    /**
     * Provides the PropertyResolver to use.
     */
    protected PropertyResolver getPropertyResolver() {
        try {
            return (PropertyResolver) bindingFeature_.lookupFc(PropertyResolver.PROPERTY_RESOLVER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(PropertyResolver.PROPERTY_RESOLVER + ": interface not found!");
            return null;
        }
    }

    /**
     * Provides the descriptionInterpreter to use.
     */
    protected DescriptionInterpreter getDescriptionInterpreter() {
        try {
            return (DescriptionInterpreter) bindingFeature_.lookupFc(DescriptionInterpreter.DESCRIPTION_INTERPRETER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(DescriptionInterpreter.DESCRIPTION_INTERPRETER + ": interface not found!");
            return null;
        }
    }

    /**
     * Provides the roleManeger to use.
     */
    protected RoleManager getRoleManager() {
        try {
            return (RoleManager) bindingFeature_.lookupFc(RoleManager.ROLE_MANAGER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(RoleManager.ROLE_MANAGER + ": interface not found!");
            return null;
        }
    }

    /**
     * Provides the descriptionInterpreter to use.
     */
    protected RoleProvider getRoleProvider() {
        try {
            return (RoleProvider) bindingFeature_.lookupFc(RoleProvider.ROLE_PROVIDER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(RoleProvider.ROLE_PROVIDER + ": interface not found!");
            return null;
        }
    }
    
    /**
     * Provides the TextComponent to use.
     */    
    protected TextComponent getTextComponent(){
        try {
            return (TextComponent) bindingFeature_.lookupFc(TextComponent.TEXT_COMPONENT);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(TextComponent.TEXT_COMPONENT + ": interface not found!");
            return null;
        }        
    }
    
    /**
     * Provides the TreeTemplate to use.
     */    
    protected org.objectweb.fractal.api.Component getTreeTemplate(){
        try {
            return (org.objectweb.fractal.api.Component) bindingFeature_.lookupFc(Tree.TREE_TMPL);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(Tree.TREE_TMPL + ": interface not found!");
            return null;
        }        
    }

    /**
     * Provides the TreeTemplate to use.
     */    
    protected ContextContainerFactory getContextContainerFactory(){
        try {
            return (ContextContainerFactory) bindingFeature_.lookupFc(ContextContainerFactory.CONTEXT_CONTAINER_FACTORY);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(ContextContainerFactory.CONTEXT_CONTAINER_FACTORY + ": interface not found!");
            return null;
        }        
    }
    
    // ==================================================================
    // Refresh Management
    // ==================================================================

    /**
     * Refreshes all the components bound with the tree via the
     * <code>Synchronisation</code> interface.
     */
    protected void dispatchSynchronization(TreeView treeView){
        String[] clientIftNames = listFc();
        for (int i = 0; i < clientIftNames.length; i++) {
            if(clientIftNames[i].startsWith(Synchronization.SYNCHRONIZATION)){
                try {
                    ((Synchronization)lookupFc(clientIftNames[i])).refresh(treeView);
                } catch (NoSuchInterfaceException e) {
                    getTrace().warn(e.getMessage());
                } catch (Exception e) {
                    getTrace().warn(e.getMessage());
                    e.printStackTrace();
                    JOptionPane.showMessageDialog(null, e.getMessage(), "Error! (" + e.getClass().getName() + ")", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }
    
    protected void dispatchSynchronization(){
        dispatchSynchronization(getTreeView());
    }
    
    /**
     * Refreshes the given path.
     */
    protected void refreshPath(TreePath treePath){
        collapsePath(treePath);
        expandPath(treePath);        
    }    
    
    /**
     * Refreshes the root node. 
     */
    protected void refreshRootNode(){
        refreshPath(new TreePath(rootNode_.getPath()));
    }

    /**
     * Refreshes the action menu depending on the selected object.
     * @param entry The selected entry
     */
    protected void refreshActionMenu(Entry entry){
        if(actionMenu_!=null && entry!=null){
            actionMenu_.removeAll();
            Description desc = getPropertyResolver().resolve(ExplorerConstants.MENU_PROPERTY, entry, getParentEntry());
            if(desc!=null && ! desc.isEmpty()){
                JPopupMenu menu = (JPopupMenu) getDescriptionInterpreter().interprete(desc, getMenuItemTreeView());
                Component[] componentList = menu.getComponents();
                for(int i=0; i<componentList.length;i++){
                    if(JSeparator.class.isAssignableFrom(componentList[i].getClass())){
                        actionMenu_.addSeparator();
                    } else {
                        actionMenu_.add((JMenuItem)componentList[i]);
                    }
                }
                actionMenu_.setEnabled(true);
                //actionMenu_.revalidate();
            } else {
                actionMenu_.setEnabled(false);
            }
        }
    }

    /**
     * Refreshes the tool bar depending on the selected object.
     * @param entry The selected entry
     */
    protected void refreshToolBar(Entry entry){
        if(toolBar_!=null && entry!=null){
            // Removes the old elements
            if(nbElementsToRemove_>0){
                int nbComp = toolBar_.getComponentCount();
                for(int i = nbComp - 1 ; i >= nbComp - nbElementsToRemove_ ; i--)
                    toolBar_.remove(i);
                nbElementsToRemove_ = 0;
            }

            Description desc = getPropertyResolver().resolve(ExplorerConstants.MENU_PROPERTY, entry, getParentEntry());
            if(desc!=null && ! desc.isEmpty()){
                boolean iconDisplayed = false;
                JPopupMenu menu = (JPopupMenu) getDescriptionInterpreter().interprete(desc, getMenuItemTreeView());
                Component[] componentList = menu.getComponents();
                for(int i=0; i<componentList.length;i++){
                    if(!JSeparator.class.isAssignableFrom(componentList[i].getClass())){
                        if(((GenericAction)((JMenuItem)componentList[i]).getAction()).isUserIcon()){
                            // An action must be added
                            toolBar_.add(((JMenuItem)componentList[i]).getAction());
                            nbElementsToRemove_++;
                            iconDisplayed = true;
                        }
                    } else if (iconDisplayed) {
                        // A seperator must be displayed
                        toolBar_.addSeparator();
                        iconDisplayed = false;
                        nbElementsToRemove_++;
                    }    
                }
            }

            if(toolBar_.getComponentCount()==0){
                toolBar_.setVisible(false);
            } else{
                toolBar_.setVisible(false);
                toolBar_.setVisible(true);
            }        
        }
        
    }
        
    /**
     * Refreshes the action menu depending on the selected object.
     * It computes the selected entry.
     * @see DynamicTree#refreshActionMenu(Entry)
     */
    protected void refreshActionMenu(){
        refreshActionMenu(getSelectedEntry());
    }

    /**
     * Refreshes the tool bar depending on the selected object.
     * It computes the selected entry.
     * @see DynamicTree#refreshToolBar(Entry)
     */
    protected void refreshToolBar(){
        refreshToolBar(getSelectedEntry());
    }    
    
    /**
     * Refreshes all the tree. 
     */
    protected void refreshAllPathes(){
        //Entry selectedEntry = getSelectedEntry();
        String selectedPath = pathToString(getSelectionPath());
        createGraphicalInitialContext();        
        //selectedNode_ = pathToString(getSelectionPath()); 
        refreshRootNode();
        selectPath(selectedPath);
        Entry selectedEntry = getSelectedEntry();
        //refreshPanel();
        refreshActionMenu(selectedEntry);
        refreshToolBar(selectedEntry);        
    }

    /**
     * Refreshes the role menu contained in the menu bar. 
     */
    protected void refreshRoleMenu(){
        if(menuBar_!=null){
            String[] roleList = getRoleProvider().getRoleList(true, true);
            String[] currentRoles = getRoleManager().getCurrentRoleIds();
            if(roleMenu_ == null){
                roleMenu_ = new JMenu("Roles");
                menuBar_.add(roleMenu_);
            }
            roleMenu_.removeAll();
            if(roleList.length<=0){
                roleMenu_.setVisible(false);
            } else {
                roleMenu_.setVisible(true);
                if(multipleRoles_){
                    List theRoles = Arrays.asList(currentRoles);
                    for (int i = 0; i < roleList.length; i++) {
                        JCheckBoxMenuItem cbMenuItem = new JCheckBoxMenuItem(new RoleSelected(roleList[i]));
                        cbMenuItem.setSelected(theRoles.contains(roleList[i]));
                        roleMenu_.add(cbMenuItem);
                    }
                } else {
                    ButtonGroup roleGroup = new ButtonGroup();
                    JRadioButtonMenuItem rbMenuItem = null;
                    for(int i=0; i<roleList.length;i++){
                        rbMenuItem = new JRadioButtonMenuItem(new RoleSelected(roleList[i]));
                        if(currentRoles.length>0)
                            rbMenuItem.setSelected(currentRoles[0].equals(roleList[i]));
                        roleGroup.add(rbMenuItem);
                        roleMenu_.add(rbMenuItem);
                    }
                }
            }
        }      
    }
    
    // ==================================================================
    // Wrapper Management
    // ==================================================================
    
    /**
     * Adds an entry to a node.
     * @param parent Node which is the parent of the new entry
     * @param child Entry to add
     * @return The created node
     */
    protected DefaultMutableTreeNode addObject(DefaultMutableTreeNode parent, Entry child) {
        Description desc = getPropertyResolver().resolve(ExplorerConstants.WRAPPER_PROPERTY, child);
        if(desc!=null){
            Context context = (Context) getDescriptionInterpreter().interprete(desc, getTreeView());
            if (context != null) {
                child.setWrapper(getContextContainerFactory().create(context));
            }
        }
        DefaultMutableTreeNode childNode = new DefaultMutableTreeNode(child);
        if (parent == null) {
            parent = rootNode_;
        }
        treeModel_.insertNodeInto(childNode, parent, parent.getChildCount());
        return childNode;
    }    
    
    // ==================================================================
    // Popup Menu Management
    // ==================================================================

    /**
     * Shows the associated PopupMenu, if exists and if displaying is enabled
     * 
     * @param p Point which indicates where is the mouse
     */
    protected void showMenu(Point p) {
        TreePath path = getPath(p);
        if (path != null) {
            selectionModel_.setSelectionPath(path);
            Description desc = getPropertyResolver().resolve(ExplorerConstants.MENU_PROPERTY, getSelectedEntry(), getParentEntry());
            JPopupMenu menu = (JPopupMenu) getDescriptionInterpreter().interprete(desc, getMenuItemTreeView());
            if (menu != null) {
                menu.show((Component) this, (int) (p.getX()), (int) (p.getY()));
            }
        }
    }

    // ==================================================================
    // Icon Management
    // ==================================================================
    
    protected Icon getImageIcon(Entry entry){
        if(getPropertyResolver()!=null){
            Description desc = getPropertyResolver().resolve(ExplorerConstants.ICON_PROPERTY, entry);
            IconProvider iconProvider = (IconProvider) getDescriptionInterpreter().interprete(desc, getTreeView());
            if(iconProvider!=null){
                Object object = iconProvider.newIcon(entry.getValue());
                if(object instanceof Icon){
                    return (Icon)object;
                } else if(object instanceof String) {
                    return (Icon)(new IconFileProvider((String)object)).newIcon(entry.getValue()); 
                } else if (object instanceof URL) {
                    return (Icon)(new IconFileProvider((URL)object)).newIcon(entry.getValue());
                }
            }
        }
        return null;
    }

    // ==================================================================
    // Info Management
    // ==================================================================
    
    protected void displayInfo(Entry entry){
        if (getTextComponent() != null){
            String message = "";
            Description desc = getPropertyResolver().resolve(ExplorerConstants.INFO_PROPERTY, entry);
            if(desc!=null){
                Info info = (Info) getDescriptionInterpreter().interprete(desc, null);
                if (info != null) {
                    message = info.getInfo(getTreeView());
                }
            }            
            getTextComponent().setText(message);
        }
    }
    
    // ==================================================================
    //
    // Public methods for Tree interface.
    //
    // ==================================================================

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#refreshAll()
     */
    public void refreshAll() {
        refreshAllPathes();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.util.explorer.api.Tree#addEntry(java.lang.Object,
     *      java.lang.Object)
     */
    public void addEntry(Object name, Object value) {
        addEntry(name, value, 0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.util.explorer.api.Tree#addEntry(java.lang.Object,
     *      java.lang.Object)
     */
    public void addEntry(Object name, Object value, int level) {
        RootEntry rootEntry = new DefaultRootEntry(name, value, level);
        initialContext_.add(rootEntry);
        addToInitialContext(rootEntry);
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#selectPath(java.lang.String)
     */
    public void 
    selectPath(String path){
        TreePath treePath = stringToPath(path);
        setSelectionPath(treePath);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#clear()
     */
    public void clear() {
        initialContext_.clear();
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#getInitialContextSize()
     */
    public int getInitialContextSize() {
        return getGraphicalInitialContext().getSize();
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#removeEntry(java.lang.Object)
     */
    public void removeEntry(Object name) {
        Iterator it = initialContext_.iterator();
        RootEntry elementToRemove = null;
        while (it.hasNext()) {
            RootEntry element = (RootEntry) it.next();
            if(element.getName().toString().equals(name.toString())) {
                elementToRemove = element;
                break;
            }
        }
        if(elementToRemove!=null)
            initialContext_.remove(elementToRemove);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#renameInitialEntry(java.lang.Object, java.lang.Object)
     */
    public void 
    renameInitialEntry(Object currentName, Object newName){
        Iterator it = initialContext_.iterator();
        while (it.hasNext()) {
            RootEntry element = (RootEntry) it.next();
            if(element.getName().toString().equals(currentName.toString())) {
                element.setName(newName.toString());
                break;
            }
        }
        updateInitialPathes(currentName.toString(), newName.toString());
        refreshAll();
        selectPath(newName.toString());
    }    

    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#duplicate()
     */
    public org.objectweb.fractal.api.Component duplicate() {
        return duplicate(true);
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#getSelectedEntry()
     */
    public Entry getSelectedEntry() {
        Object object = getLastSelectedPathComponent();
        if (object != null)
            return objectToEntry(object);
        return null;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#duplicate(boolean)
     */
    public org.objectweb.fractal.api.Component duplicate(boolean withData) {
        try {
            // Creates a new instance of a tree.
            org.objectweb.fractal.api.Component tmpl = getTreeTemplate();
            Factory factory = (Factory) tmpl.getFcInterface("factory");
            org.objectweb.fractal.api.Component instance = factory.newFcInstance();
            
            // Renames the new instance of the component.
            NameController nameController = (NameController)instance.getFcInterface("name-controller");
            nameController.setFcName("tree-cpt-" + System.currentTimeMillis());
            
            // Moves the new instance of the tree component into the explorer component.
            org.objectweb.fractal.api.Component treeComponent = (org.objectweb.fractal.api.Component)lookupFc(Tree.TREE_ITSELF);
            SuperController sc = (SuperController)treeComponent.getFcInterface("super-controller");
            org.objectweb.fractal.api.Component[] superComponents = sc.getFcSuperComponents();
            if(superComponents !=null && superComponents.length > 0){
                org.objectweb.fractal.api.Component superComponent = superComponents[0];
                ContentController cc = (ContentController)superComponent.getFcInterface("content-controller");
                cc.addFcSubComponent(instance);
            }
            
            // Dynamically connect the new instance of the tree to the existing instances of mandatory components.
            BindingController bc = (BindingController)instance.getFcInterface("binding-controller");
            bc.bindFc(PropertyResolver.PROPERTY_RESOLVER, bindingFeature_.lookupFc(PropertyResolver.PROPERTY_RESOLVER));
            bc.bindFc(DescriptionInterpreter.DESCRIPTION_INTERPRETER, bindingFeature_.lookupFc(DescriptionInterpreter.DESCRIPTION_INTERPRETER));
            bc.bindFc(ContextContainerFactory.CONTEXT_CONTAINER_FACTORY, bindingFeature_.lookupFc(ContextContainerFactory.CONTEXT_CONTAINER_FACTORY));
            
            Fractal.getLifeCycleController(instance).startFc();
            if(withData) {
                Tree tree = (Tree)instance.getFcInterface(Tree.TREE);
                Entry[] entries = getGraphicalInitialContext().getEntries(null);
                for (int i = 0; i < entries.length; i++) {
                    try{
                        tree.addEntry(entries[i].getName(), entries[i].getValue(),1);
                    } catch(Exception e){
                        e.printStackTrace();
                    }
                }
            }
            return instance;
        } catch (NoSuchInterfaceException ex) {
            ex.printStackTrace();
        } catch (IllegalLifeCycleException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (IllegalBindingException ex) {
            ex.printStackTrace();
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new RuntimeException("Error during replication!!!");
        }
        return null;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#renameSelectedNode(java.lang.Object, java.lang.Object)
     */
    public void renameSelectedNode(Object oldName, Object newName) {  
        String selectedPath = pathToString(getSelectionPath());
        String prefixe = selectedPath.substring(0,selectedPath.lastIndexOf(oldName.toString()));
        if(!selectedPath.equals(prefixe)){
            updateInitialPathes(prefixe + oldName, prefixe + newName);
            refreshAll();
            selectPath(prefixe + newName);
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.api.Tree#close()
     */
    public void close() {
        dispatchSynchronization(null);
    }
    
    // ==================================================================
    //
    // Public methods for Explorer interface.
    //
    // ==================================================================

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.util.explorer.swing.api.Explorer#getTree()
     */
    public JTree getTree() {
        return this;
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#setMenuBar(javax.swing.JMenuBar)
     */
    public void setMenuBar(JMenuBar menuBar) {
        menuBar_ = menuBar;
        actionMenu_ = new JMenu("Actions");
        actionMenu_.setEnabled(false);
        menuBar_.add(actionMenu_);
        refreshRoleMenu();
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#setToolBar(javax.swing.JToolBar)
     */
    public void setToolBar(JToolBar toolBar) {
        toolBar_ = toolBar;
        if(toolBar_!=null && toolBar_.getComponents().length==0){
            toolBar_.setVisible(false);
        } else {
            toolBar_.addSeparator();
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#isMultipleRoles()
     */
    public boolean isMultipleRoles() {
        return multipleRoles_;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#setCurrentRoleIds(java.lang.String[])
     */
    public void setCurrentRoles(String[] roleIds) {
        getRoleManager().setCurrentRoleIds(roleIds);
        refreshRoleMenu();
        refreshAllPathes();
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#setMultipleRoles(boolean)
     */
    public void setMultipleRoles(boolean multipleRoles) {
        multipleRoles_ = multipleRoles;
        refreshRoleMenu();
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#isDragAndDropEnabled()
     */
    public boolean isDragAndDropEnabled() {
        return dndEnabled_;
    }
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#isPopupEnabled()
     */
    public boolean isPopupEnabled() {
        return popupEnabled_;
    }
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#setDragAndDropEnabled(boolean)
     */
    public void setDragAndDropEnabled(boolean dndEnabled) {
        dndEnabled_ = dndEnabled;
    }
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.swing.api.Explorer#setPopupEnabled(boolean)
     */
    public void setPopupEnabled(boolean popupEnabled) {
        popupEnabled_ = popupEnabled;
    }
    
    // ==================================================================
    //
    // Public methods for BindingController interface.
    //
    // ==================================================================

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
     *      java.lang.Object)
     */
    public void bindFc(String itfName, Object itfValue)
            throws NoSuchInterfaceException, IllegalBindingException,
            IllegalLifeCycleException {
        bindingFeature_.bindFc(itfName, itfValue);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return bindingFeature_.listFc();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc(String itfName) throws NoSuchInterfaceException {
        return bindingFeature_.lookupFc(itfName);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc(String itfName) throws NoSuchInterfaceException,
            IllegalBindingException, IllegalLifeCycleException {
        bindingFeature_.unbindFc(itfName);
    }

    // ==================================================================
    //
    // Public methods for LifeCycleController interface.
    //
    // ==================================================================

    public String getFcState () {
        return null;
    }

    public void startFc () {
        /*
        System.err.println("startFC");
        System.err.print("initial context initialization... ");
        //graphicalInitialContext_ = getContextContainerFactory().create();
        graphicalInitialContext_ = new RootContextContainer();
        System.err.print("OK");*/
    }

    public void stopFc () {
        // Nothing to do!
    }    
    
    protected ContextContainer getGraphicalInitialContext(){
        if(graphicalInitialContext_==null){
            graphicalInitialContext_ = getContextContainerFactory().create(new RootContextContainer());
            rootNode_ = new DefaultMutableTreeNode(new DefaultEntry("Root",graphicalInitialContext_));
            treeModel_ = new MyTreeModel(rootNode_);
            setModel(treeModel_);
        }
        return graphicalInitialContext_;
    }
    
    // ==================================================================
    //
    // Internal class for JTree management.
    //
    // ==================================================================

    /**
     * Mouse Adapter
     * Use to draw Popup Menu
     */
    final class MyMouseAdapter extends MouseAdapter {
        public void mousePressed(MouseEvent e) {
            popupLayout(e);
        }

        public void mouseReleased(MouseEvent e) {
            popupLayout(e);
        }

        private void popupLayout(MouseEvent e) {
            if (e.isPopupTrigger() && popupEnabled_) {
                showMenu(e.getPoint());
            }
        }
    }

    /**
     * Tree Will Expand Listener
     * Use to complete node : it adds child to current node
     */
    final class MyTreeWillExpandListener implements TreeWillExpandListener {
        public void treeWillCollapse(TreeExpansionEvent event) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
            Entry e = (Entry) currentNode.getUserObject();
            nodeStates_.put(pathToString(currentNode), Boolean.FALSE);
        }

        public void treeWillExpand(TreeExpansionEvent event) {
            DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();
            Entry e = (Entry) currentNode.getUserObject();
            nodeStates_.put(pathToString(currentNode), Boolean.TRUE);
            currentNode.removeAllChildren();
            Entry[] entries = e.getWrapper().getEntries(e.getValue());
            for (int i = 0; i < entries.length; i++) {
                DefaultMutableTreeNode childNode = addObject(currentNode, entries[i]);
            }
            treeModel_.reload(currentNode);
        }
    }    
    
    /**
     * Tree Expansion Listener
     * Use to refresh node
     */
    final class MyTreeExpansionListener implements TreeExpansionListener {
        public void treeCollapsed(TreeExpansionEvent event) {
        }

        public void treeExpanded(TreeExpansionEvent event) {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode) (event.getPath()).getLastPathComponent();
            if (!node.isLeaf()) {   
                Enumeration enumeration = node.children();
                while (enumeration.hasMoreElements()) {
                    DefaultMutableTreeNode childNode = (DefaultMutableTreeNode) enumeration.nextElement();
                    Entry entry = (Entry) childNode.getUserObject();
                    String pathName = pathToString(childNode);
                    if (nodeStates_.containsKey(pathName)) {
                        if (((Boolean)nodeStates_.get(pathName)).booleanValue()) {
                            expandPath(new TreePath(childNode.getPath()));
                        }
                    }
                    //if (selectedNode_.equals(pathName)) {
                    //    setSelectionPath(new TreePath(treeModel_.getPathToRoot(childNode)));
                    //}
                }
            }
        }
    }    
    
    /**
     * Tree Cell Renderer
     * Use to put the icon defined for the given entry.
     */
    final class MyTreeCellRenderer extends DefaultTreeCellRenderer {

        public Component getTreeCellRendererComponent(JTree tree,Object value,boolean selected,boolean expanded,boolean leaf,int row,boolean hasFocus) {
            super.getTreeCellRendererComponent(tree,value,selected,expanded,leaf,row,hasFocus);

            // Put the appropriate icon if one is defined.
            Entry entry = objectToEntry(value);
            Icon icon = getImageIcon(entry);
            if(icon!=null){
                setIcon(icon);
            }
            
            // Put the name of the entry as the label of the node
            setText(entry.getName().toString());

            return this;
        }

    }

    /** 
     * Tree Selection Listener 
     * Use to put the panel corresponding to the selected node
     */
    final class MyTreeSelectionListener implements TreeSelectionListener {

        public void valueChanged(TreeSelectionEvent e) {
            // Compute the selected node
            //TreePath treePath = e.getNewLeadSelectionPath();
            //selectedNode_ = treePath!=null?pathToString(e.getNewLeadSelectionPath()):"";
            
            // Refresh the diplay
            Entry selectedEntry = getSelectedEntry();
            displayInfo(selectedEntry);
            refreshActionMenu(selectedEntry);
            refreshToolBar(selectedEntry);
            dispatchSynchronization();
            
            /*
            refreshPanel();
            refreshActionMenu();
            refreshToolBar();
            */
        }
    }    
    
    /**
     * Tree Model
     * Uses to know if a node is leaf or not.
     */
    final class MyTreeModel extends DefaultTreeModel {

        public MyTreeModel(TreeNode root) {
            super(root);
        }

        public boolean isLeaf(Object node) {
            Entry entry = objectToEntry(node);
            return (entry.getWrapper()==null && !(entry.getValue() instanceof Context));
        }

    }

    // ==================================================================
    //
    // Public methods overriden JComponent methods.
    //
    // ==================================================================
    
    public String getToolTipText(MouseEvent evt) {
        StringBuffer label = new StringBuffer();
        TreePath currentPath = getPathForLocation(evt.getX(), evt.getY());
        if(currentPath!=null){
            Entry entry = objectToEntry(currentPath.getLastPathComponent());
            DnDDescriptions desc = (DnDDescriptions)getPropertyResolver().resolve(ExplorerConstants.DND_PROPERTY, entry);
            if(desc != null && !desc.isEmpty()){
                String[] labels = desc.getActionsDescription();
                if(labels.length>0){
                    label.append("<html>");
                    label.append("Drop action:");
                    for (int i = 0; i < labels.length; i++) {
                        label.append("<br> - " + labels[i]);
                    }
                    label.append("</html>");
                }
            }
        }
        return label.toString();    
    } 
    
    // ==================================================================
    //
    // Public methods for Autoscroll interface.
    //
    // ==================================================================

    public void autoscroll(Point cursorLocn){
        Rectangle rect=getVisibleRect();
        Dimension dim=getSize();
        
        // Vertival autoscroll
        if(cursorLocn.y>rect.y+rect.height-10) {
            rect.y=Math.min(rect.y+10,dim.height-rect.height);
        } else if(cursorLocn.y<rect.y+10) {
            int i = rect.y+rect.height-10;
            rect.y=Math.max(rect.y-10,0);
        }
        
        // Horizontal autoscroll
        if(cursorLocn.x>rect.x+rect.width-10) {
            rect.x=Math.min(rect.x+10,dim.width-rect.width);
        } else if(cursorLocn.x<rect.x+10) {
            rect.x=Math.max(rect.x-10,0);
        }
        
        scrollRectToVisible(rect);
    }
              
    public Insets getAutoscrollInsets(){
        Rectangle rect=getVisibleRect();
        Dimension dim=getSize();
        Insets inset=new Insets(rect.y+10,rect.x+10,dim.height-(rect.y+rect.height)+10,dim.width-(rect.x+rect.width)+10);
        return inset;
    }

    // ==================================================================
    //
    // Internal classes for Drag & Drop.
    //
    // ==================================================================
   
    /**
     * Implementation of DragGestureListener interface.
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
     * 
     * @version 0.1
     */
    protected class TreeDragGestureListener implements DragGestureListener {

        /* 
         * @see java.awt.dnd.DragGestureListener#dragGestureRecognized(java.awt.dnd.DragGestureEvent)
         */
        public void dragGestureRecognized(DragGestureEvent dge) {
            if(dndEnabled_){
                Transferable transferable = new EntryTransferable(DynamicTree.this.getSelectedEntry());
                try {
                    dragSource_.startDrag(dge,DragSource.DefaultLinkDrop, transferable, dragSourceListener_);
                }catch(InvalidDnDOperationException idoe) {
                    System.out.println( idoe );
                }
            } 
        }
    }
    
    /**
     * Implementation of DragSourceListener interface
     * 
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
     * 
     * @version 0.1
     */
    public class TreeDragSourceListener implements DragSourceListener {

        /** The current dropAction */
        protected int dropAction_;
        
        /*
         * DnD problem:
         *   This way to implement a Drag&Drop mechanism seems to be bad 
         *   because the DragSourceDragEvent is not correctly initialized.
         *   Indeed, when the Drag&Drop begins (entering in dragStart state),
         *   no problem is detected, but when the drop action change (call
         *   to the dropActionChanged method), the associated dropAction refers
         *   to an unknown action. That why, in order to know the refering drop 
         *   action we use the current state of the input devices:
         *      16 -> left click                -> Move
         *      18 -> Ctrl + left click         -> Copy
         *      19 -> Ctrl + Shift + left click -> Link
         */

        /**
         * Provides the dropAction depending on the current state of the input devices.
         * @param The current state of the input devices.
         * @return The corresponding dropAction identifier.
         */
        protected int getDropAction(int gestureModifiers){
            if(gestureModifiers==16)
                return DnDConstants.ACTION_MOVE;
            else if(gestureModifiers==18)
                return DnDConstants.ACTION_COPY;
            else if(gestureModifiers==19)
                return DnDConstants.ACTION_LINK;
            else 
                return 0;
        }

        /**
         * Fixes the appropriate icon. 
         */
        protected void setIcon(DragSourceDragEvent dsde){
            DragSourceContext context = dsde.getDragSourceContext();
            dropAction_ = getDropAction(dsde.getGestureModifiers());
            if (dropAction_ == DnDConstants.ACTION_COPY) {
                context.setCursor(DragSource.DefaultCopyDrop);
            } else if (dropAction_ == DnDConstants.ACTION_MOVE) {
                context.setCursor(DragSource.DefaultMoveDrop);
            } else if (dropAction_ == DnDConstants.ACTION_LINK) {
                context.setCursor(DragSource.DefaultLinkDrop);
            } else {
                context.setCursor(DragSource.DefaultLinkNoDrop);
            }
        }

        /**
         * Fixes the appropriate icon. 
         */
        protected void setForbiddenIcon(DragSourceEvent dse){
            DragSourceContext context = dse.getDragSourceContext();
            if (dropAction_ == DnDConstants.ACTION_COPY) {
                context.setCursor(DragSource.DefaultCopyNoDrop);
            } else if (dropAction_ == DnDConstants.ACTION_MOVE) {
                context.setCursor(DragSource.DefaultMoveNoDrop);
            } else if (dropAction_ == DnDConstants.ACTION_LINK) {
                context.setCursor(DragSource.DefaultLinkNoDrop);
            } else {
                context.setCursor(DragSource.DefaultLinkNoDrop);
            }
        }       

        /* 
         * @see java.awt.dnd.DragSourceListener#dragEnter(java.awt.dnd.DragSourceDragEvent)
         */
        public void dragEnter(DragSourceDragEvent dsde) {
            setIcon(dsde);
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dragOver(java.awt.dnd.DragSourceDragEvent)
         */
        public void dragOver(DragSourceDragEvent dsde) {
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dropActionChanged(java.awt.dnd.DragSourceDragEvent)
         */
        public void dropActionChanged(DragSourceDragEvent dsde) {
            setIcon(dsde);
        }
        
        /* 
         * @see java.awt.dnd.DragSourceListener#dragExit(java.awt.dnd.DragSourceEvent)
         */
        public void dragExit(DragSourceEvent dse) {
            setForbiddenIcon(dse);
        }

        /* 
         * @see java.awt.dnd.DragSourceListener#dragDropEnd(java.awt.dnd.DragSourceDropEvent)
         */
        public void dragDropEnd(DragSourceDropEvent dsde) {
        }

    }

    /**
     * Implementation of DropTargetListener interface. 
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
     * 
     * @version 0.1
     */
    protected class TreeDropTargetListener implements DropTargetListener {

        /** The old Path*/
        protected TreePath oldPath_ = null;
        
        /** The currentAction_ */
        protected boolean actionChanged_ = false;

        protected String getDropActionName(int dropAction){
            if(dropAction==DnDConstants.ACTION_MOVE)
                return DnDDescription.MOVE_ACTION;
            else if(dropAction==DnDConstants.ACTION_COPY)
                return DnDDescription.COPY_ACTION;
            else if(dropAction==DnDConstants.ACTION_LINK)
                return DnDDescription.LINK_ACTION;
            else 
                return "";
        }        

        protected String computeActionDesc(Entry entry, String dropAction){
            DnDDescriptions desc = (DnDDescriptions)getPropertyResolver().resolve(ExplorerConstants.DND_PROPERTY, entry);
            if(desc!=null){
                DnDDescription dndDesc = desc.getDropAction(dropAction);
                if(dndDesc !=null){
                    return dndDesc.getLabel();
                }
            }
            return "";
        }
                
        /* 
         * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragEnter(DropTargetDragEvent dtde) {
            dtde.acceptDrag(dtde.getDropAction());
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragOver(DropTargetDragEvent dtde) {
            dtde.acceptDrag(dtde.getDropAction());
            String label = "";
            TreePath path = getPath(dtde.getLocation());
            if(path==null){
                oldPath_ = null;
                if(getTextComponent()!=null){
                    getTextComponent().setText("");
                }                
            } else if(oldPath_ == null || (path != null && !path.equals(oldPath_)) || actionChanged_){
                actionChanged_ = false;
                oldPath_ = path;
                Entry entry = objectToEntry(path.getLastPathComponent());
                String value = computeActionDesc(entry, getDropActionName(dtde.getDropAction()));
                if(getTextComponent()!=null){
                    getTextComponent().setText(value);
                }
            }
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
         */
        public void dropActionChanged(DropTargetDragEvent dtde) {
            dtde.acceptDrag(dtde.getDropAction());
            actionChanged_ = true;
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
         */
        public void dragExit(DropTargetEvent dte) {
        }

        /* 
         * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
         */
        public void drop(DropTargetDropEvent dtde) {
            
            DataFlavor[] df = EntryTransferable.flavors_;
            DataFlavor chosen = null;
            for (int i = 0; i < df.length; i++) {
                if(dtde.isDataFlavorSupported(df[i])){
                    chosen = df[i];
                    break;
                }
            }

            if(getTextComponent()!=null){
                getTextComponent().setText("");
            }
                        
            Entry dragEntry = null;
            // This method is responsible of a bug (it is waiting for ???)
//            dtde.acceptDrop(acceptableActions_);
            try {
                dragEntry = (Entry)dtde.getTransferable().getTransferData(chosen);
            } catch (UnsupportedFlavorException e1) {
                getTrace().warn("[" + getClass().getName() + "] UnsupportedFlavorException: " + e1.getMessage());
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            
            TreePath path = getPath(dtde.getLocation());
            if (path != null) {
                selectionModel.setSelectionPath(path);
                Entry dropEntry = objectToEntry(getLastSelectedPathComponent());
                DnDDescriptions desc = (DnDDescriptions)getPropertyResolver().resolve(ExplorerConstants.DND_PROPERTY, dropEntry);
                if(desc!=null){
                    DnDDescription dndDesc = desc.getDropAction(getDropActionName(dtde.getDropAction()));
                    if(dndDesc !=null){
                        DropAction dropAction = (DropAction)getDescriptionInterpreter().interprete(dndDesc.getCodeDescription(),null);
                        if(dropAction!=null && dragEntry!=null){
                            try{
                                dropAction.execute(new DefaultDropTreeView((Tree)DynamicTree.this, dropEntry, getParentEntry(), dragEntry));
                            }catch(Exception e){
                                JOptionPane.showMessageDialog(null, e.getMessage(), "Drag & Drop error ! (" + e.getClass().getName() + ")", JOptionPane.ERROR_MESSAGE);
                            }
                            DynamicTree.this.refreshAllPathes();
                        }
                    }
                }                
            }
            
        }
    }

    /**
     * This class defines the action performed when a role is selected into the associated menu.
     *
     * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
     *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
     * 
     * @version 0.1
     */
    protected final class RoleSelected extends AbstractAction{

        protected String roleId_;
        
        public RoleSelected(String roleId){
            super(roleId);
            roleId_ = roleId;
        }
        
        public void actionPerformed(ActionEvent e) {
            if(DynamicTree.this.multipleRoles_){
                String[] currentRoles = getRoleManager().getCurrentRoleIds();
                Vector theRoles = new Vector(Arrays.asList(currentRoles));
                if(theRoles.contains(roleId_))
                    theRoles.remove(roleId_);
                else
                    theRoles.add(roleId_);
                getRoleManager().setCurrentRoleIds((String[])theRoles.toArray(new String[theRoles.size()]));
            } else {
                String currentRoleId = getRoleManager().getCurrentRoleIds()[0];
                if(!roleId_.equals(currentRoleId)) {
                    getRoleManager().setCurrentRoleIds(new String[]{roleId_});
                }
            }
            refreshAllPathes();
        }

    }
    
    
}
  
