/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.swing.menu;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import org.objectweb.util.explorer.api.MenuItem;
import org.objectweb.util.explorer.api.MenuItemTreeView;
import org.objectweb.util.explorer.core.menu.api.AcceleratorDescription;
import org.objectweb.util.explorer.core.menu.api.ItemDescription;
import org.objectweb.util.explorer.core.menu.api.MenuItemTreeViewConfiguration;
import org.objectweb.util.explorer.core.menu.api.MnemonicDescription;
import org.objectweb.util.explorer.swing.icon.EmptyIconProvider;
import org.objectweb.util.trace.TraceSystem;

/**
 * Defines the default action wich wraps the MenuItem.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class GenericAction 
     extends AbstractAction 
{
    
    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected MenuItem delegate_;
    protected MenuItemTreeView treeView_;
    protected boolean isUserIcon_ = true;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public GenericAction(ItemDescription itemDesc, MenuItem action, MenuItemTreeView treeView, Icon icon, boolean enabled){
        super(itemDesc.getLabel(),icon);
        
        delegate_ = action;
        treeView_ = treeView;
        
        if(icon==null){
            putValue(SMALL_ICON, new EmptyIconProvider().newIcon(null));
            isUserIcon_ = false;
        }
            
        // Fixes action properties
        putValue(SHORT_DESCRIPTION, itemDesc.getLabel());
        MnemonicDescription md = itemDesc.getMnemonicDescription(); 
        if(md!=null && !md.isEmpty()){
            putValue(MNEMONIC_KEY,new Integer(Character.toUpperCase(md.getMnemonicCharacter())));
        }
        KeyStroke key = getKeyStroke(itemDesc.getAcceleratorDescription());
        if(key!=null){
            putValue(ACCELERATOR_KEY,key);
        }
        setEnabled(enabled);            
    }
    
    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /**
     * Provides the KeyStroke associates to the given Mnemonic. 
     * @param mnemonic The object containing the definition of the keyStroke
     * @return The associated KeyStroke.
     */
    protected KeyStroke getKeyStroke(AcceleratorDescription acceleratorDesc){
        if(acceleratorDesc!=null && !acceleratorDesc.isEmpty()){
            int modifier = 0;
            modifier = (acceleratorDesc.getAltKey()?java.awt.Event.ALT_MASK:0)
                |(acceleratorDesc.getCtrlKey()?java.awt.Event.CTRL_MASK:0)
                |(acceleratorDesc.getShiftKey()?java.awt.Event.SHIFT_MASK:0)
                |(acceleratorDesc.getMetaKey()?java.awt.Event.META_MASK:0);
            if(modifier==0)
                return KeyStroke.getKeyStroke(Character.toUpperCase(acceleratorDesc.getAcceleratorCharacter()));
            else
                return KeyStroke.getKeyStroke(Character.toUpperCase(acceleratorDesc.getAcceleratorCharacter()), modifier);
        }    
        return null;
    }
    
    // ==================================================================
    //
    // Public methods. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent actionEvent) {
        try{
            ((MenuItemTreeViewConfiguration)treeView_).setEvent(actionEvent);
            delegate_.actionPerformed(treeView_);
        } catch (Exception e) {
            TraceSystem.get("explorer").warn(getClass().getName() + " exception: " + e.getMessage());
            JOptionPane.showMessageDialog(null, e.getClass().getName() + ":\n" + e.getMessage(), "Exception (" + actionEvent.getActionCommand() + ")", JOptionPane.ERROR_MESSAGE);
        }
        treeView_.getTree().refreshAll();
    }

    public boolean isUserIcon(){
        return isUserIcon_;
    }
}
