/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================
$Id: DefaultKeyTest.java,v 1.2 2005/07/06 15:36:02 moroy Exp $
====================================================================*/

package org.objectweb.util.explorer.core.common;

import org.objectweb.util.explorer.core.common.lib.DefaultKey;
import org.objectweb.util.explorer.core.role.api.Role;
import org.objectweb.util.explorer.core.role.api.RoleProvider;
import org.objectweb.util.explorer.core.role.lib.DefaultRole;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * Test class for DefaultKey object. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class DefaultKeyTest 
     extends TestCase 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The key ot test. */
    protected DefaultKey testKey_, nullRoleKey_, nullIdKey_, nullRoleAndNullIdKey_, allNullKey_;    
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        Role role = new DefaultRole("test_role");
        testKey_ = new DefaultKey("java","test_id",role);
        nullRoleKey_ = new DefaultKey("java","test_id",null);
        nullIdKey_ = new DefaultKey("java",null,role);
        nullRoleAndNullIdKey_ = new DefaultKey("java",null,null);
        allNullKey_ = new DefaultKey(null,null,null);
    }    
    
    // ==================================================================
    //
    // Public methods (code to test).
    //
    // ==================================================================    

    /*
     * Class under test for boolean equals(Object)
     */
    public void testEqualsObject() {
        Assert.assertTrue(!testKey_.equals(null));
        Assert.assertEquals(testKey_, testKey_);
        Assert.assertEquals(nullRoleKey_, new DefaultKey("java","test_id",null));
        Assert.assertEquals(nullIdKey_, new DefaultKey("java",null,new DefaultRole("test_role")));
        Assert.assertNotSame(nullRoleAndNullIdKey_, new DefaultKey("java",null,null));
        Assert.assertEquals(allNullKey_, new DefaultKey(null,null,null));
        Assert.assertEquals(testKey_, new DefaultKey("java","test_id",new DefaultRole("test_role")));
        
        Object key = new DefaultKey("java","test",new DefaultRole(RoleProvider.DEFAULT_ROLE));
        Assert.assertEquals(key, new DefaultKey("java","test",new DefaultRole(RoleProvider.DEFAULT_ROLE)));
       
    }

    /*
     * Class under test for boolean equals(Object)
     */
    public void testHashCode() {
        Assert.assertEquals(testKey_.hashCode(), testKey_.hashCode());
        Assert.assertEquals(nullRoleKey_.hashCode(), (new DefaultKey("java","test_id",null)).hashCode());
        Assert.assertEquals(nullIdKey_.hashCode(), (new DefaultKey("java",null,new DefaultRole("test_role"))).hashCode());
        Assert.assertEquals(allNullKey_.hashCode(), (new DefaultKey(null,null,null)).hashCode());
        Assert.assertEquals(testKey_.hashCode(), (new DefaultKey("java","test_id",new DefaultRole("test_role"))).hashCode());
        
        Object key = new DefaultKey("java","test",new DefaultRole(RoleProvider.DEFAULT_ROLE));
        Assert.assertEquals(key.hashCode(), (new DefaultKey("java","test",new DefaultRole(RoleProvider.DEFAULT_ROLE))).hashCode());
       
    }
        
}