/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.core.menu;

import org.objectweb.util.explorer.core.code.api.CodeDescription;
import org.objectweb.util.explorer.core.code.lib.BasicCodeDescription;
import org.objectweb.util.explorer.core.menu.api.ItemDescription;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.api.MenuSeparator;
import org.objectweb.util.explorer.core.menu.lib.BasicItemDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicMenuDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicMenuSeparator;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class MenuDescriptionTest 
     extends TestCase 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    protected ItemDescription itemDesc1_ = null, itemDesc2_ = null, sameAsItemDesc1_ = null;
    protected MenuSeparator menuSeparator_ = null;
        
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        CodeDescription codeDesc = new BasicCodeDescription();
        codeDesc.setLanguage("Java");
        codeDesc.setCode("java.lang.Object");
        
        itemDesc1_ = new BasicItemDescription();
        itemDesc1_.setLabel("item 1");
        itemDesc1_.setCodeDescription(codeDesc);
        
        itemDesc2_ = new BasicItemDescription();
        itemDesc2_.setLabel("item 2");
        itemDesc2_.setCodeDescription(codeDesc);
        
        sameAsItemDesc1_ = new BasicItemDescription();
        sameAsItemDesc1_.setLabel("item 1");
        sameAsItemDesc1_.setCodeDescription(codeDesc);
        
        menuSeparator_ = new BasicMenuSeparator();
    }    
    
    // ==================================================================
    //
    // Public methods. 
    //
    // ==================================================================    

    public void testEqualsMethod(){
        MenuDescription menuDesc = new BasicMenuDescription();
        MenuDescription expected = new BasicMenuDescription();
        MenuDescription notExpected = new BasicMenuDescription();
        
        menuDesc.addMenuElement(itemDesc1_);
        menuDesc.addMenuElement(menuSeparator_);
        menuDesc.addMenuElement(itemDesc1_);
        menuDesc.addMenuElement(itemDesc2_);
        ItemDescription id = null;
        menuDesc.addMenuElement(id);
        menuDesc.addMenuElement(new BasicItemDescription());
        menuDesc.addMenuElement(sameAsItemDesc1_);

        notExpected.addMenuElement(itemDesc1_);
        notExpected.addMenuElement(itemDesc2_);        
        notExpected.addMenuElement(menuSeparator_);
        Assert.assertNotSame(notExpected, menuDesc);
        
        expected.addMenuElement(itemDesc1_);
        expected.addMenuElement(menuSeparator_);
        expected.addMenuElement(itemDesc2_);
        Assert.assertEquals(expected, menuDesc);
    }
    
    public void testIsEmptyMethod(){
        MenuDescription menuDesc = new BasicMenuDescription();
        Assert.assertTrue(menuDesc.isEmpty());
        menuDesc.addMenuElement(new BasicItemDescription());
        Assert.assertTrue(menuDesc.isEmpty());        
        menuDesc.addMenuElement(itemDesc1_);
        Assert.assertFalse(menuDesc.isEmpty());
    }   
    
}

