/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

--------------------------------------------------------------------
$Id: LauncherException.java,v 1.2 2005/02/03 18:29:13 contrera Exp $
====================================================================*/

package org.objectweb.util.launcher ;


import java.io.PrintStream ;
import java.io.PrintWriter ;

import org.objectweb.util.misc.api.ExceptionWrapper ;
//import org.objectweb.util.trace.TraceSystem;


/**
 * Wrapping exception for the launcher.
 *
 * <p>
 *   the exception is a runtime wrapping of intial exception for avoiding
 *   try catch and throws statements in methods
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class LauncherException
     extends ExceptionWrapper
{
    /**
     * Wrapping constructor.
     *
     * @param initial - the wrapped exception.
     */
    public LauncherException(Exception initial) { 
        super(initial);
//        TraceSystem.get("launcher").debug("Exception raised: "+initial);
    }

//     /**
//      * Fills in the execution stack trace.
//      */
//     public Throwable
//     fillInStackTrace() {
//         return getException().fillInStackTrace();
//     }

    /**
     * Returns the cause of this throwable or null if the cause 
     * is nonexistent or unknown.
     */
//COMMENTED FOR JAVA 1.3 COMPATIBILITY
//    public Throwable
//    getCause() {
//        return getException().getCause();
//    }

//     /**
//      * Creates a localized description of this throwable.
//      */
//     public String 	
//     getLocalizedMessage() {
//         return getException().getLocalizedMessage();
//     }

    /**
     * Returns the detail message string of this throwable.
     */
    public String 	
    getMessage() {
        return getException().getMessage();
    }

//     /**
//      * Provides programmatic access to the stack trace information 
//      * printed by printStackTrace().
//      */
//     public StackTraceElement[] 	
//     getStackTrace() {
//         return getException().getStackTrace();
//     }

//     /**
//      * Initializes the cause of this throwable to the specified value.
//      */
//     public Throwable 	
//     initCause(Throwable cause) {
//         return getException().initCause(cause);
//     }

    /**
     * Prints this throwable and its backtrace to the standard error stream.
     */
    public void
    printStackTrace() {
        getException().printStackTrace();
    }

    /**
     * Prints this throwable and its backtrace to the specified print stream.
     */
    public void 	
    printStackTrace(PrintStream s) {
        getException().printStackTrace(s);
    }

    /**
     * Prints this throwable and its backtrace to the specified print writer.
     */
    public void 	
    printStackTrace(PrintWriter s) {
        getException().printStackTrace(s);
    }

//     /**
//      * Sets the stack trace elements that will be returned by getStackTrace() 
//      * and printed by printStackTrace() and related methods.
//      */
//     public void
//     setStackTrace(StackTraceElement[] stackTrace){
//         getException().setStackTrace(stackTrace);
//     }
    
    /**
     * Returns a short description of this throwable.
     */
    public String 	
    toString() {
        return getException().toString();
    }
}
