/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): Christophe Contreras.

--------------------------------------------------------------------
$Id: RunDescription.java,v 1.2 2004/09/28 15:19:47 contrera Exp $
====================================================================*/

package org.objectweb.util.launcher.parser ;


/**
 * Representation of a run.<BR>
 * <p>
 * An option describe additionnal informations.
 * It contains:
 * <li>
 *   <ul>an identifier</ul>
 *   <ul>a list of arguments</ul>
 *   <ul>a list of properties</ul>
 *   <ul>a list of Java classes</ul>
 *   <ul>the name of the bootstrap class</ul>
 * </li>
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 * @version 0.1
 */
public class RunDescription
     extends ContextDescription
{
    /** the name of the main class to run */
    protected String mainclass_ ;

    /** the mode of execution */
    protected String mode_ ;

    /**
     * Defines the full name of the main class
     *
     * @param name - the name of the class (with the package) containing 
     * a method public static void main(String[])
     */
    public void setMainclass(String name) { 
        mainclass_ = name ; 
    }
    
    /**
     * Provide the full name of the class to run
     *
     * @return the name of the class (with the package, eg: xxx.xxx.Hello)
     */
    public String getMainclass() { 
        return mainclass_; 
    }

    /**
     * Defines the mode for the execution
     *
     * @param mode - the mode
     */
    public void setMode(String mode) { 
        mode_ = mode ; 
    }

    /**
     * Provide the mode for execution
     *
     * @return the mode (normal, thread, daemon)
     */
    public String getMode() { 
        return mode_; 
    }
}
