
#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <tao/PortableServer/Servant_Base.h>

#include <EnterpriseComponent.h>

#include <dlfcn.h>

#include <cstdio>

#include <iostream>
#include <fstream>
#include <string>
#include <list>
#include <utility>
#include <vector>
#include <signal.h>

using namespace std;

#include <microCCM_set_orb_and_poa.h>

#include <HomeFinder_registration_intfC.h>

#include <MCCM_Service_Integration.h>

void usage( int argc, char** argv )
{
  cerr << "Usage : " << argv[ 0 ] 
       << " description_of_component_libs_and_service_libs.txt" << endl;

  exit( 1 );
};

HomeFinder_registration_intf_ptr obtain_ref_for_home_finder( CORBA::ORB_ptr the_orb )
{
  char* microccm_location = getenv( "microCCM_DIR" );

  if( !microccm_location ) 
    {
      cerr << "The microCCM_DIR env variable is not set" << endl;
      exit( 1 );
    }

  string location_ior( microccm_location ); location_ior += "/etc/ior_home_finder.txt";

  fstream infile( location_ior.c_str(), ios::in );

  if( !infile )
    {
      cerr << "The Home Finder is not running" << endl;
      exit( 1 );
    }

  string the_ior;
  infile >> the_ior;

  CORBA::Object_var ref_obj 
    = the_orb->string_to_object( the_ior.c_str() );  

  HomeFinder_registration_intf_ptr to_return 
    = HomeFinder_registration_intf::_narrow( ref_obj.in() );

  return to_return;
}

// --------------------------------------------------------------------------------

struct Component_library_description
{
  string name_of_shared_lib;
  string idl_home_type;
  string home_exec_entry_point;
};

struct Service_library_description
{
  string name_of_shared_lib;
  string name_of_entry_point;
};

struct Normal_library_description
{
  string name_of_shared_lib;
};

pair< list< Component_library_description >, list< Service_library_description > > 
parse_component_libs_descriptions( const char* name_of_file )
{
  fstream infile( name_of_file, ios::in );

  string read;

  pair< list< Component_library_description >, list< Service_library_description > >
    to_be_returned;

  while( infile.good() )
    {
      infile >> read;

      if( read == "Component_lib" )
	{
	  Component_library_description desc_lib;

	  vector< string* > to_read( 3 ); 
	  to_read[ 0 ] = & ( desc_lib.name_of_shared_lib );
	  to_read[ 1 ] = & ( desc_lib.idl_home_type );
	  to_read[ 2 ] = & ( desc_lib.home_exec_entry_point );

	  for( int ct = 0; ct < 3; ct++ )
	    {
	      infile >> read;
	      cerr << __LINE__ << " just read " << read << endl;

	      if( ( read == "Component_lib" ) || ( read == "Service_lib" ) )
		{
		  cerr << "Error : don't manage to continue read libraries file ..." << endl;
		  cerr << "read == " << read << endl;
		  exit( 1 );
		}

	      *( to_read[ ct ] ) = read;
	    }

	  // ok, now desc_lib should be read ok.
	  to_be_returned.first.push_back( desc_lib );
	}
      else
	if( read == "Service_lib" )
	  {
	    cout << "Reading a Service lib." << endl;
	    Service_library_description sld;

	    vector< string* > to_read( 2 );
	    to_read[ 0 ] = & sld.name_of_shared_lib;
	    to_read[ 1 ] = & sld.name_of_entry_point;

	    for( int ct = 0; ct < 2; ct++ )
	      {
		infile >> read;

		cerr << __LINE__ << " just read " << read << endl;

		if( ( read == "Component_lib" ) || ( read == "Service_lib" ) )
		  {
		    cerr << "Error : don't manage to continue read libraries file ..." << endl;
		    cerr << "read == " << read << endl;
		    exit( 1 );
		  }
		
		*( to_read[ ct ] ) = read;
	      }
	    
	    // ok, now desc_lib should be read ok.
	    to_be_returned.second.push_back( sld );
	  }
    }
  
  return to_be_returned;
}

list< Normal_library_description >
parse_normal_libs_descriptions( const char* name_of_file )
{
  
  fstream infile( name_of_file, ios::in );
  
  string read;
  
  list< Normal_library_description > to_be_returned;
  
  while( infile.good() )
    {
      infile >> read;

      if( read == "Normal_lib" )
	{
	  Normal_library_description nld;
      
	  vector< string* > to_read( 1 );
	  to_read[ 0 ] = & nld.name_of_shared_lib;
      
	  infile >> read;
      
	  cerr << __LINE__ << " just read " << read << endl;
      
	  if( ( read == "Component_lib" ) ||
	      ( read == "Service_lib" )   || 
	      ( read == "Normal_lib") )
	    {
	      cerr << "Error : don't manage to continue read libraries file ..." << endl;
	      cerr << "read == " << read << endl;
	      exit( 1 );
	    }
      
      
	  // ok, now desc_lib should be read ok.
	  to_be_returned.push_back( nld );
	}
    }
  return to_be_returned;
}

// --------------------------------------------------------------------------------

int main( int argc, char** argv )
{

  if( argc < 2 ) usage( argc, argv );

  const char* name_of_file = argv[ 1 ];

  // -- do we need this kind of stuff ?
  int nbargs_for_orb = argc - 1;
  char** args_for_orb = new char* [ nbargs_for_orb ];

  args_for_orb[ 0 ] = strdup( argv[ 0 ] );

  for( int ct = 2; ct < argc; ct++ )
    {
      args_for_orb[ ct - 1 ] = strdup( argv[ ct ] );
    }

  // ------

  CORBA::ORB_var the_orb = CORBA::ORB_init( nbargs_for_orb, args_for_orb );

  // ------

  CORBA::Object_var the_obj = the_orb->resolve_initial_references( "RootPOA" );

  PortableServer::POA_var the_root_poa 
    = PortableServer::POA::_narrow( the_obj.in() );

  PortableServer::POAManager_var the_poa_manager
    = the_root_poa->the_POAManager();

  the_poa_manager->activate();

  // ---------------

  pair< list< Component_library_description >, 
        list< Service_library_description > > libs_descriptions 
    = parse_component_libs_descriptions( name_of_file );

  // -- loading shared libraries corresponding to components --

  for( list< Component_library_description >::iterator iter = libs_descriptions.first.begin();
       iter != libs_descriptions.first.end();
       iter++ )
    {
      const char* name_of_shared_lib = iter->name_of_shared_lib.c_str();
      const char* idl_home_type = iter->idl_home_type.c_str();
      const char* home_exec_entry_point = iter->home_exec_entry_point.c_str();
      cout << "Loading a Comp lib: " << name_of_shared_lib << endl;
      // -------

      void* handle = dlopen( name_of_shared_lib, RTLD_NOW | RTLD_GLOBAL );

      if( !handle )
	{
	  cerr << "Can't manage to open the library" << endl;
	  cerr << "dl_error == " << dlerror() << endl;
	  exit( 1 );
	}

      typedef Components::HomeExecutorBase* (*component_home_creation)( );

      typedef PortableServer::ServantBase* (*component_home_CORBA_creation)( );

      void* symbol = dlsym( handle, home_exec_entry_point );
      
      if( dlerror() != 0 )
	{
	  cerr << "Don't manage to get symbol " << home_exec_entry_point << endl;
	  exit( 1 );
	}

      string for_corba_entry_point = string( "create_servant_for_home" ); // + string( idl_home_type );

      void* symbol_corba_home_factory 
	= dlsym( handle, for_corba_entry_point.c_str() );

      if( dlerror() != 0 )
	{
	  cerr << "Don't manage to get symbol to create Home Servant" << endl;
	  exit( 1 );
	}

      component_home_creation home_create = ( component_home_creation ) symbol;
      
      Components::HomeExecutorBase* the_home_executor = home_create();

      component_home_CORBA_creation to_create_corba_home
	= ( component_home_CORBA_creation ) symbol_corba_home_factory;

      PortableServer::ServantBase* home_servant 
	= to_create_corba_home();

      Components::To_set_home_executor* to_set_home_exec
	= dynamic_cast< Components::To_set_home_executor* >( home_servant );

      if( ! to_set_home_exec )
	{
	  cerr << "There is a problem with a dynamic_cast that should be ok" << endl;
	  exit( 1 );
	}
      
      to_set_home_exec->set_home_executor( the_home_executor );

      microCCM::To_set_poa_orb* to_set_poa_orb
	= dynamic_cast< microCCM::To_set_poa_orb* >( home_servant );

      to_set_poa_orb->set( the_orb.in(), the_root_poa.in() );

      // --- activation of Home Servant now ----
      PortableServer::ObjectId_var objid 
	= the_root_poa->activate_object( home_servant );

      CORBA::Object_ptr obj = the_root_poa->id_to_reference( /* * */objid.in() );

      // --- put this in HomeFinder ----
      HomeFinder_registration_intf_var ref_home_finder
	= obtain_ref_for_home_finder( the_orb.in() );

      Components::CCMHome_var home_ref = Components::CCMHome::_narrow( obj );
      
      // attention, erreur ci-dessous ... il ne faut pas un '/' mais un ':'
      string intf_rep_id = string( "IDL:" ) + idl_home_type + "/1.0";

      ref_home_finder->register_home_by_home_type( home_ref.in(),
						   intf_rep_id.c_str() );

    }

  // --- and now the stuff for Service libraries ----
  
  for( list< Service_library_description >::iterator iter = libs_descriptions.second.begin();
       iter != libs_descriptions.second.end();
       iter++ )
    {
      cout << "Loading a Service lib." << endl;
      const char* name_of_shared_lib = iter->name_of_shared_lib.c_str();
      const char* entry_point = iter->name_of_entry_point.c_str();

      void* handle = dlopen( name_of_shared_lib, RTLD_NOW | RTLD_GLOBAL );

      if( !handle )
	{
	  cerr << "Can't manage to open the library" << endl;
	  cerr << "dl_error == " << dlerror() << endl;
	  exit( 1 );
	}

      void* symbol_service_factory = dlsym( handle, entry_point );

      if( dlerror() != 0 )
	{
	  cerr << "Don't manage to get symbol " << entry_point << endl;
	  exit( 1 );
	}

      typedef MCCM_Service_Integration_Factory* (*creation_service_factory)();

      creation_service_factory csf = ( creation_service_factory ) symbol_service_factory;

      MCCM_Service_Integration_Factory* sif = csf();

      MCCM_Service_Integration_Factories::register_service_integration_factory( sif );
    }



   // --- and now the stuff for Normal libraries ----

  list< Normal_library_description >  normal_libs_descriptions 
    = parse_normal_libs_descriptions( name_of_file );

  for( list< Normal_library_description >::iterator iter = normal_libs_descriptions.begin();
       iter != normal_libs_descriptions.end();
       iter++ )
    {
      const char* name_of_shared_lib = iter->name_of_shared_lib.c_str();

      void* handle = dlopen( name_of_shared_lib, RTLD_NOW | RTLD_GLOBAL );

      if( !handle )
	{
	  cerr << "Can't manage to open the library" << endl;
	  cerr << "dl_error == " << dlerror() << endl;
	  exit( 1 );
	}
    }
  the_orb->run();
  
  
  return 0;
}
