#include <microccm_Acceleration_executor.h>

#include <threading_utils.h>

#include <unistd.h>
#include <cmath>

// --------

class Acceleration_impl : public CCM_Acceleration,
			  public CCM_AccelerationInterface,
			  public Components::SessionComponent
{
public:

  ~Acceleration_impl();

  CCM_AccelerationInterface_ptr get_accel_intf();

  void acceleration_wanted( CORBA::Float value );
  CORBA::Float acceleration_wanted();

  void time_step_control( CORBA::Float value );
  CORBA::Float time_step_control();

  void configuration_complete() throw( Components::InvalidConfiguration );
  void set_session_context( Components::SessionContext_ptr ctx ) throw( Components::CCMException );

  void enable_increase_speed();
  void disable_increase_speed();

private:

  CORBA::Float acceleration_wanted_;
  CORBA::Float time_step_;

  CCM_Acceleration_Context_ptr ctx_;

  pthread_t the_thread_;
  friend void* accel_thread_func( void* );

  threading::protected_resource< bool > speed_is_to_be_increased_;
};

// --------

void Acceleration_impl::time_step_control( CORBA::Float value )
{
  time_step_ = value;
}

CORBA::Float Acceleration_impl::time_step_control()
{
  return time_step_;
}

// --------

void* accel_thread_func( void* s )
{
  Acceleration_impl* accel_impl = ( Acceleration_impl* ) s;

  for( ; ; )
    {
      usleep( lrint( accel_impl->time_step_ * 1e6 ) );
      
      if( accel_impl->speed_is_to_be_increased_ == true )
	{
	  // and now compute the new throttle value 
	  // in order to have "acceleration == acceleration_wanted_"

	  CORBA::Float mass_vehicle
	    = accel_impl->ctx_->get_connection_car_caracs()->mass();

	  CORBA::Float throttle_coeff
	    = accel_impl->ctx_->get_connection_car_caracs()->throttle_factor();

	  CORBA::Float friction_factor
	    = accel_impl->ctx_->get_connection_car_caracs()->friction_factor();

	  CORBA::Float current_speed 
	    = accel_impl->ctx_->get_connection_current_speed()->speed();

	  float alpha 
	    = (  accel_impl->acceleration_wanted_*mass_vehicle 
		 + friction_factor*current_speed*current_speed ) 
	    / throttle_coeff;

	  accel_impl->ctx_->get_connection_the_throttle_port()->throttle_value( alpha );
	}

    }

  return NULL;
}

Acceleration_impl::~Acceleration_impl()
{
  pthread_cancel( the_thread_ );
}

void Acceleration_impl::configuration_complete( ) throw( Components::InvalidConfiguration )
{
  pthread_create( &the_thread_, NULL, accel_thread_func, this );
}

CCM_AccelerationInterface_ptr Acceleration_impl::get_accel_intf()
{
  return this;
}

void Acceleration_impl::acceleration_wanted( CORBA::Float value )
{
  acceleration_wanted_ = value;
}

CORBA::Float Acceleration_impl::acceleration_wanted()
{
  return acceleration_wanted_;
}

void Acceleration_impl::set_session_context( Components::SessionContext_ptr sctx ) throw( Components::CCMException )
{
  ctx_ = CCM_Acceleration_Context::_narrow( sctx );
}

void Acceleration_impl::enable_increase_speed()
{
  speed_is_to_be_increased_.set( true );
}

void Acceleration_impl::disable_increase_speed()
{
  speed_is_to_be_increased_.set( false );
}

// ----------------

class AccelerationHome_impl : public CCM_AccelerationHome
{
public:

  Components::EnterpriseComponent_ptr create() throw( Components::CCMException );
};

Components::EnterpriseComponent_ptr 
AccelerationHome_impl::create() throw( Components::CCMException )
{
  return new Acceleration_impl;
}

// ------------------

extern "C"
{
  Components::HomeExecutorBase* create_home_exec()
  {
    return new AccelerationHome_impl;
  }
}

// ------------------

