#include <Lw_ComponentsC.h>

#include <iostream>
#include <fstream>
#include <string>
#include <map>
#include <utility>
#include <list>
#include <signal.h>
#include <pthread.h>

using namespace std;

#include <microccm_CruiseControlInterfaceC.h>
#include <microccm_BrakeInterfaceC.h>
#include <microccm_EngineInterfaceC.h>
#include <microccm_SpeedInterfaceC.h>

#include <obtain_ref_home_finder.h>
#include <HomeFinderC.h>



using namespace std;


void* thread_func( void* s )
{

 
  SpeedInterface_ptr* bidule = ( SpeedInterface_ptr* ) s;

  for( ; ; )
    {
      usleep( 50 * 1000 );
      cerr << "current velocity is " << ( *bidule )->speed() << endl;
    }

  return NULL;
}


// --------------------------------------------------------------------
int main(int argc, char* argv[])
{

  CORBA::ORB_var the_orb = CORBA::ORB_init( argc, argv );
  
  Components::HomeFinder_var ref_home_finder
    = obtain_ref_for_home_finder( the_orb.in() );

  // ----------------------------------------------------------

  // -- no attributes to set for cruisecontroler
  // -- Components::ConfigValues the_cruisecontroler_config;
  
  // -- attributes for the_acceleration_obj

  Components::ConfigValues the_acceleration_config;

  {
    the_acceleration_config.length( 2 );
    the_acceleration_config[ 0 ].name = CORBA::string_dup( "acceleration_wanted" );
    the_acceleration_config[ 0 ].value <<= ( CORBA::Float ) 1.0;
    
    the_acceleration_config[ 1 ].name = CORBA::string_dup( "time_step_control" );
    the_acceleration_config[ 1 ].value <<= ( CORBA::Float ) 100e-3;
  }

  // -- attributes for the CarCharacteristicsInterface

  Components::ConfigValues the_carcharacteristics_config;
  
  {
    the_carcharacteristics_config.length( 3 );

    the_carcharacteristics_config[ 0 ].name = CORBA::string_dup( "throttle_factor" );
    the_carcharacteristics_config[ 0 ].value <<= ( CORBA::Float ) 10;

    the_carcharacteristics_config[ 1 ].name = CORBA::string_dup( "mass" );
    the_carcharacteristics_config[ 1 ].value <<= ( CORBA::Float ) 1e3;

    the_carcharacteristics_config[ 2 ].name = CORBA::string_dup( "friction_factor" );
    the_carcharacteristics_config[ 2 ].value <<= ( CORBA::Float ) 50;
  }

  // -----------------------------------------------------

  Components::CCMHome_var cruisecontrol_home_ 
    = ref_home_finder->find_home_by_home_type( "IDL:CruiseControlHome/1.0" );

  Components::KeylessCCMHome_var cruisecontrol_home
    = Components::KeylessCCMHome::_narrow( cruisecontrol_home_.in() );

  Components::CCMObject_var the_cruisecontroler_obj
    = cruisecontrol_home->create_component();

  // --

  Components::CCMHome_var acceleration_home_
    = ref_home_finder->find_home_by_home_type( "IDL:AccelerationHome/1.0" );

  Components::HomeConfiguration_var the_home_config_1
    = Components::HomeConfiguration::_narrow( acceleration_home_.in() );

  the_home_config_1->set_configuration_values( the_acceleration_config );

  Components::KeylessCCMHome_var acceleration_home
    = Components::KeylessCCMHome::_narrow( acceleration_home_.in() );

  Components::CCMObject_var the_acceleration_obj
    = acceleration_home->create_component();


  // --

  Components::CCMHome_var desiredspeed_home_
    = ref_home_finder->find_home_by_home_type( "IDL:DesiredSpeedHome/1.0" );
  
  Components::KeylessCCMHome_var desiredspeed_home
    = Components::KeylessCCMHome::_narrow( desiredspeed_home_.in() );

  Components::CCMObject_var the_desiredspeed_obj
    = desiredspeed_home->create_component();

  // --

  Components::CCMHome_var carcharacteristics_home_
    = ref_home_finder->find_home_by_home_type( "IDL:CarCharacteristicsHome/1.0" );

  Components::HomeConfiguration_var the_home_config_2
    = Components::HomeConfiguration::_narrow( carcharacteristics_home_.in() );

  the_home_config_2->set_configuration_values( the_carcharacteristics_config );

  Components::KeylessCCMHome_var carcharacteristics_home
    = Components::KeylessCCMHome::_narrow( carcharacteristics_home_.in() );

  Components::CCMObject_var the_carcaracs_obj
    = carcharacteristics_home->create_component();

  // --

  Components::CCMHome_var throttle_home_
    = ref_home_finder->find_home_by_home_type( "IDL:ThrottleHome/1.0" );

  Components::KeylessCCMHome_var throttle_home
    = Components::KeylessCCMHome::_narrow( throttle_home_.in() );

  Components::CCMObject_var the_throttle_obj
    = throttle_home->create_component();

  // --

  Components::CCMHome_var cruiser_home_
    = ref_home_finder->find_home_by_home_type( "IDL:CruiserHome/1.0" );

  Components::KeylessCCMHome_var cruiser_home
    = Components::KeylessCCMHome::_narrow( cruiser_home_.in() );

  Components::CCMObject_var the_cruiser_obj
    = cruiser_home->create_component();

  // ----------------------------------------------------------

  the_cruisecontroler_obj->connect( "accel_intf", the_acceleration_obj->provide_facet( "accel_intf" ) );
  the_cruisecontroler_obj->connect( "desired_speed", the_desiredspeed_obj->provide_facet( "speed_indic" ) );
  the_cruisecontroler_obj->connect( "cruiser_intf", the_cruiser_obj->provide_facet( "the_cruiser_intf" ) );
  the_cruisecontroler_obj->connect( "current_speed", the_carcaracs_obj->provide_facet( "speed_indic" ) );

  // -----------------------------------------------------------

  the_acceleration_obj->connect( "the_throttle_port", the_throttle_obj->provide_facet( "a_throttle_intf" ) );
  the_acceleration_obj->connect( "current_speed", the_carcaracs_obj->provide_facet( "speed_indic" ) );
  the_acceleration_obj->connect( "car_caracs", the_carcaracs_obj->provide_facet( "car_caracs" ) );

  // -----------------------------------------------------------

  the_carcaracs_obj->connect( "throttle_information", the_throttle_obj->provide_facet( "a_throttle_intf" ) );

  // -----------------------------------------------------------

  the_cruiser_obj->connect( "the_throttle_port", the_throttle_obj->provide_facet( "a_throttle_intf" ) );
  the_cruiser_obj->connect( "desired_speed", the_desiredspeed_obj->provide_facet( "speed_indic" ) );
  the_cruiser_obj->connect( "car_caracs", the_carcaracs_obj->provide_facet( "car_caracs" ) );

  // -----------------------------------------------------------

  the_cruisecontroler_obj->configuration_complete();
  the_acceleration_obj->configuration_complete();
  the_desiredspeed_obj->configuration_complete();
  the_carcaracs_obj->configuration_complete();
  the_throttle_obj->configuration_complete();

  // ----------------------------------------------------------

  // -- and now the game begins ---

  CORBA::Object_var ref_cruisecontrol_intf 
    = the_cruisecontroler_obj->provide_facet( "cc_interface" );

  CruiseControlInterface_ptr cc_intf
    = CruiseControlInterface::_narrow( ref_cruisecontrol_intf.in() );

  // ---

  CORBA::Object_var ref_engine_intf
    = the_cruisecontroler_obj->provide_facet( "eng_interface" );

  EngineInterface_ptr eng_intf
    = EngineInterface::_narrow( ref_engine_intf.in() );

  // ----

  CORBA::Object_var ref_brake_intf
    = the_cruisecontroler_obj->provide_facet( "brk_interface" );

  BrakeInterface_ptr brk_intf
    = BrakeInterface::_narrow( ref_brake_intf.in() );

  // ----

  CORBA::Object_var ref_current_speed_intf
    = the_carcaracs_obj->provide_facet( "speed_indic" );

  SpeedInterface_ptr speed_intf
    = SpeedInterface::_narrow( ref_current_speed_intf.in() );

  // ----

  pthread_t the_thread;

  pthread_create( &the_thread, NULL, thread_func, &speed_intf );

  eng_intf->do_it( on );

  cerr << " engine now on " << endl;
  sleep( 1 );

  brk_intf->do_it( released );

  cerr << " brake released now" << endl;
  sleep( 1 );

  cc_intf->do_it( accel );
  cerr << " Some acceleration now" << endl;

  sleep( 1 );

  cc_intf->do_it( cruise );
  cerr << " Some cruising now" << endl;

  sleep( 3 );

  cc_intf->do_it( cruise_off );
  cerr << "Stopping cruising now" << endl;

  sleep( 10 );

  pthread_cancel( the_thread );
  //  pthread_join( the_thread, NULL );
  
  return 0;
}

