#include <microccm_Cruiser_executor.h>

#include <iostream>
using std::cerr;
using std::endl;

class Cruiser_impl : public CCM_Cruiser,
		     public CCM_CruiserInterface,
		     public Components::SessionComponent
{
 public:

  CCM_CruiserInterface_ptr get_the_cruiser_intf();

  void enable_maintain_speed();
  void disable_maintain_speed();

  void set_session_context( Components::SessionContext_ptr ctx )
    throw( Components::CCMException );

  void configuration_complete() throw( Components::InvalidConfiguration );

 private:

  CCM_Cruiser_Context_ptr ctx_;
};

// -----------

CCM_CruiserInterface_ptr Cruiser_impl::get_the_cruiser_intf()
{
  return this;
}

void Cruiser_impl::set_session_context( Components::SessionContext_ptr ctx )
  throw( Components::CCMException )
{
  ctx_ = CCM_Cruiser_Context::_narrow( ctx );
}

void Cruiser_impl::configuration_complete( ) 
  throw( Components::InvalidConfiguration )
{
}

void Cruiser_impl::enable_maintain_speed()
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  CORBA::Float desired_velocity 
    = ctx_->get_connection_desired_speed()->speed();

  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  CORBA::Float friction_factor 
    = ctx_->get_connection_car_caracs()->friction_factor();

  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  CORBA::Float throttle_factor
    = ctx_->get_connection_car_caracs()->throttle_factor();

  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  // ----

  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  CORBA::Float adequate_throttle_val 
    = friction_factor * desired_velocity * desired_velocity / throttle_factor ;

  // ----

  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  ctx_->get_connection_the_throttle_port()->throttle_value( adequate_throttle_val );
}

void Cruiser_impl::disable_maintain_speed()
{
  ctx_->get_connection_the_throttle_port()->throttle_value( 0.0 );
}

// --------------------------------------------------

class CruiserHome_impl : public CCM_CruiserHome
{
 public:

  Components::EnterpriseComponent_ptr create()
    throw( Components::CCMException );
};

// ---------------------------------------------------

Components::EnterpriseComponent_ptr CruiserHome_impl::create()
  throw( Components::CCMException )
{
  return new Cruiser_impl;
}

// ---------------------------------------------------

extern "C"
{
  Components::HomeExecutorBase* create_home_exec()
  {
    return new CruiserHome_impl;
  }

}

// ------------------------------------------------------

