#include <Stubs_IR3.h>

#include <IFR3_Singleton.h>

#include <obtain_repo_ref.h>
#include <IFR3_client_errors.h>

#include <IDL_to_cpp_mapping.h>

#include <create_equivalent_idl2_interface.h>

#include <iostream>
#include <fstream>
using namespace std;

#include <repid_utils.h>
#include <typecode_utils.h>

#include <Invariable_string_in_place_of.h>
#include <Redispatch_in_place_of.h>

// implementation method hooks
#include <CORBA_cpp_gene_main_servant_constr.h>
#include <CORBA_cpp_gene_main_servant_check_type.h>
#include <CORBA_cpp_gene_main_servant_create_act.h>
#include <CORBA_cpp_gene_main_servant_delete_deact.h>

#include <read_configuration_file.h>

#include <IDL3_to_CORBA_serv_proxy.h>

#include <CORBA_generation_indicators.h>
#include <debug_utils.h>

void usage( const char* name_program )
{
  cerr << " usage : " << endl
       << name_program << " component_type" << " home_type" << " interception_description"
       << endl;
  exit( ERROR_USAGE );
}

int main( int argc, char** argv )
{
  if( ( argc != 3 ) && ( argc != 4 ) ) usage( argv[ 0 ] );
  string name_of_component( argv[ 1 ] );

  // ---
  {
    string name_of_generation_desc;
    if( argc == 4 )
      {
	name_of_generation_desc = argv[ 3 ];
      }
    
    if( name_of_generation_desc != "" )
      {
	CORBA_generation_indicators::read_generation_indications( name_of_generation_desc );
      }
    
  }

  bool only_one_idl_file_per_comp = false;

  char* microccm_dir = getenv( "microCCM_DIR" );

  if( microccm_dir )
    {
      Configuration_file_reader the_reader( string( microccm_dir ) + "/etc/codegen_config.txt"  );
      string codegen_config = the_reader.extract_configuration( "GENERATION_MODE" );
      if( codegen_config == "generate_one_idl_file" ) only_one_idl_file_per_comp = true;
    }

  // --

  CORBA::ORB_ptr the_orb = CORBA::ORB_init( argc, argv );

  CORBA::ComponentIR::Repository_ptr the_ref_repo
    = obtain_the_repo_ref( the_orb );

  IFR3_Singleton::set_instance( the_ref_repo );

  CORBA::Contained_ptr the_component
    = the_ref_repo->lookup( name_of_component.c_str() );

  if( CORBA::is_nil( the_component ) )
    {
      cerr << "IDL3_to_CORBA : don't manage to find component " 
	   << name_of_component << endl;
      exit( ERROR_ITEM_NOT_COMPONENT );
    }

  CORBA::Contained::Description* the_description = the_component->describe();

  if( the_description->kind != CORBA::dk_Component )
    {
      cerr << "I expected " << name_of_component
	   << " to be a component .." << endl;
      exit( ERROR_ITEM_NOT_COMPONENT );
    }

  CORBA::ComponentIR::ComponentDescription* the_comp_desc = NULL;

  //----------------------------------------------------------------------------------
  // ***************   servants corresponding to the provides port  ************
  //----------------------------------------------------------------------------------
  
  if( the_description->value >>= the_comp_desc )
    {
      CORBA::ComponentIR::ProvidesDescriptionSeq& the_provided_interfaces
        = the_comp_desc->provided_interfaces;
      
      int nb_provided_interfaces = the_provided_interfaces.length();
      
      list< Cpp_class_gene* > list_port_servants;
      {
        for( int ct_provided_intf = 0; ct_provided_intf < nb_provided_interfaces; ct_provided_intf++ )
          {
            CORBA::ComponentIR::ProvidesDescription& desc 
	      = the_provided_interfaces[ ct_provided_intf ];
	    
            cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ 
                 << " name_of_port == " <<  desc.name << endl;
	    
            CORBA::Contained_ptr current_intf_ref_as_contained 
              = the_ref_repo->lookup_id( desc.interface_type );
	    
            CORBA::InterfaceDef_ptr current_intf 
              = CORBA::InterfaceDef::_narrow( current_intf_ref_as_contained );

            string current_intf_name( current_intf_ref_as_contained->name() );

            string name_of_generated_class = string( "_microccm_" )+ current_intf_name + "_impl";

	    Cpp_class_gene generated_class = IDL_to_Cpp_mapping( current_intf,
								 name_of_generated_class,
								 false );

            Cpp_class_gene* class_gene_new = new Cpp_class_gene( name_of_generated_class );
	    *class_gene_new = generated_class;

	    list_port_servants.push_back( class_gene_new );

	    if( !only_one_idl_file_per_comp )
	      {
		class_gene_new->add_required_include( string( "<microccm_" ) + current_intf_name + "S.h>" );
	      }

	    do_server_proxy_implementation( *class_gene_new, current_intf );
	    do_mccm_callback_implementations( list_port_servants, current_intf );
	    

	  }

      }

      //----------------------------------------------------------------------------------
      // ***************   Context class impl (Internal interfaces)  ************
      //----------------------------------------------------------------------------------

      string base_name_ctx_class = string( "CCM_" ) + name_of_component + "_Context";

      Cpp_class_gene ctx_class( base_name_ctx_class + "_impl" );
      {
        ctx_class.add_inherited_class( base_name_ctx_class, "public" );
        ctx_class.add_inherited_class( "Components::ExtSessionContext", "public" );

        ctx_class.add_required_include( "<Correspondance_table.h>" );

	// -- Constructor
	Cpp_method ctx_constructor_method; 
	{
	  ctx_constructor_method.set_name_of_method(base_name_ctx_class + string("_impl"));
	  ctx_constructor_method.add_parameter("long" , "nb_added_consumers_max");
	  ctx_class.add_method( ctx_constructor_method );
	  string implem_of_method = ""
	    + get_mccm_debug_trace_warning("CONTEXT");

	  if( the_comp_desc->publishes_events.length() > 0 )
	    {
	      implem_of_method +=
		"nbeventConnectionsMax_ = nb_added_consumers_max;\n"
		"current_indexEventConnections_ = 0;\n"
		"supplier_map_ = new Correspondance_table<CosEventChannelAdmin::ProxyPushSupplier_ptr>();\n"
		"consumer_map_ = new Correspondance_table<Components::EventConsumerBase_ptr>();\n"
		"index_ec_ = 124; \n";
	    }

	      ctx_class.add_implem( ctx_constructor_method, 
				    new Invariable_string_in_place_of( implem_of_method ) );
	}



	// -- insertion of destructor
	{
	  Cpp_method ctx_destructor_method; 
	  ctx_destructor_method.set_name_of_method(string("~") + base_name_ctx_class + string("_impl"));
	  ctx_class.add_method(ctx_destructor_method );
	  
	  string implem_of_method = "" 
	    + get_mccm_debug_trace_warning("CONTEXT")
	    + "\n";
	  
	  if( the_comp_desc->publishes_events.length() > 0 )
	    {
	      implem_of_method +=  
		" for( Correspondance_table< CosEventChannelAdmin::ProxyPushSupplier_ptr >::iterator iter = supplier_map_->begin();\n"
		"     iter != supplier_map_->end();\n"
		"     iter++ )\n"
		"     {\n"
		"       if( ! CORBA::is_nil( iter->value ) )\n"
		"       {\n"
		"	     {\n"
		" 	       CORBA::Object_var dummy_obj = iter->value;\n"
		" 	     }\n"
		"	     iter->value =  CosEventChannelAdmin::ProxyPushSupplier::_nil();\n"
		"        }\n"
		"     }\n"
		"\n"
		"   for( Correspondance_table<Components::EventConsumerBase_ptr >::iterator iter = consumer_map_->begin();"
		"        iter != consumer_map_->end();\n"
		"        iter++ )\n"
		"     {\n"
		"       if( ! CORBA::is_nil( iter->value ) )\n"
		" 	{\n"
		" 	  {\n"
		" 	    CORBA::Object_var dummy_obj = iter->value;\n"
		" 	  }\n"
		" 	  iter->value = Components::EventConsumerBase::_nil();\n"
		" 	}\n"
		"     }\n"
		"    delete supplier_map_;\n"
		"    delete consumer_map_;\n"
		"    CORBA::release(the_event_channel_factory_);\n";

	       for(int ct_publisher=0; ct_publisher< the_comp_desc->publishes_events.length(); ct_publisher++)
		{
		  string name_of_event_source = string( the_comp_desc->publishes_events[ct_publisher].name);
		
		  implem_of_method += 
		    "    CORBA::release(" + name_of_event_source + "_ec_ );\n"
		    "    CORBA::release(" + name_of_event_source + "_ );\n";
		}
	    }
	  
	  ctx_class.add_implem( ctx_destructor_method, 
				new Invariable_string_in_place_of( implem_of_method ) );
	}



	// -- insertion of receptacles methods
        CORBA::ComponentIR::UsesDescriptionSeq& used_interfaces = the_comp_desc->used_interfaces;

        for( int ct = 0; ct < used_interfaces.length(); ct++ )
          {
            string name_of_port( used_interfaces[ ct ].name );

            CORBA::Contained_ptr corresponding_intf_def
              = the_ref_repo->lookup_id( used_interfaces[ ct ].interface_type );

            string typename_of_intf( corresponding_intf_def->name() );

            Cpp_method recept_method; 
            recept_method.set_name_of_method( string( "" ) + "get_connection_" + name_of_port );
            recept_method.set_return_type( typename_of_intf + "_ptr" );

            ctx_class.add_method( recept_method );

            string implem_of_method = ""
	      + get_mccm_debug_trace_warning("CONTEXT")
              + typename_of_intf + "_var to_be_returned\n"
              "= " + typename_of_intf + "::_narrow( ( *ptr_to_receptacles_ )[ " + "\"" + name_of_port + "\" ] );\n"
              "\n"
	      + get_mccm_assert("to_be_returned") +
              "return to_be_returned._retn();\n";

            ctx_class.add_implem( recept_method, 
                                  new Invariable_string_in_place_of( implem_of_method ) );
          }
	
	CORBA::ComponentIR::EventPortDescriptionSeq& publishes_events
	  = the_comp_desc->publishes_events;



	// -- insertion of "init_push_suppliers" method
	if( the_comp_desc->publishes_events.length() > 0 )
	  {
	    Cpp_method init_push_suppliers_method; 
	    {
	      init_push_suppliers_method.set_name_of_method(string( "init_push_suppliers" ));
	      init_push_suppliers_method.add_parameter("CORBA::ORB_ptr", "the_orb");
	      init_push_suppliers_method.set_return_type( "void" );
	      ctx_class.add_method( init_push_suppliers_method );
	      string implem_of_method
		= " init_eventChannelFactory(the_orb); \n";
	    
	      for(int ct_publisher=0; ct_publisher<publishes_events.length(); ct_publisher++)
		{
		  string name_of_event_channel = string( publishes_events[ct_publisher].name) + "_ec_";
		
		  implem_of_method += " " + name_of_event_channel + "= create_new_evt_channel();\n";

		}
	      implem_of_method += "register_push_supplier();\n";
	    
	      ctx_class.add_implem( init_push_suppliers_method, 
				    new Invariable_string_in_place_of( implem_of_method ) );
	    }


	  }

	// -- insertion of "create_new_evt_channel" methods
	if( the_comp_desc->publishes_events.length() > 0 )
	  {
	    Cpp_method create_new_evt_channel;
	    create_new_evt_channel.set_name_of_method( "create_new_evt_channel" );
	    create_new_evt_channel.set_return_type( "CosEventChannelAdmin::EventChannel_ptr" );
	    
	    string implem_of_method = string( "" )
	      + " CosEventChannelAdmin::EventChannel_var to_be_returned;\n"
	      " \n"
	      " index_ec_++; \n"
	      "\n"
	      "  try\n"
	      "    {\n"
	      "     ostringstream ostr; ostr << index_ec_; string index_string = ostr.str();\n"
	      "     to_be_returned = the_event_channel_factory_->create( index_string.c_str(), false );\n"
	      "   }\n"
	      "  catch( CosEventChannelFactory::DuplicateChannel& )\n"
	      "  {\n" 
	      + get_mccm_debug_trace_warning("COMPONENT_IMPLEM_EVENTS") +
	      "   exit(1); \n"
	      "  }\n"
	      " return to_be_returned._retn();\n"
	      "\n";
	  

	    ctx_class.add_method( create_new_evt_channel );
	    ctx_class.add_implem( create_new_evt_channel,
				  new Invariable_string_in_place_of( implem_of_method ) );
	  }


	// -- insertion of "init_eventChannelFactory" method // should mostly be done at Component Server level.
	if( the_comp_desc->publishes_events.length() > 0 )
	  {
	    Cpp_method init_eventChannelFactory_method; 
	    {
	      init_eventChannelFactory_method.set_name_of_method(string( "init_eventChannelFactory" ));
	      init_eventChannelFactory_method.add_parameter("CORBA::ORB_ptr", "the_orb");
	      init_eventChannelFactory_method.set_return_type( "void" );
	      ctx_class.add_method( init_eventChannelFactory_method );
	      string implem_of_method =
		" CORBA::Object_var naming_context_obj = the_orb->resolve_initial_references(\"NameService\");\n"
		" CosNaming::NamingContext_var naming_context = CosNaming::NamingContext::_narrow(naming_context_obj.in());\n"
		" assert(!CORBA::is_nil(naming_context.in()));\n"
	    
		" // get a reference on the event channel factory\n"
		" CosNaming::Name ec_factory_name(1);\n"
		" ec_factory_name.length(1);\n"
		" ec_factory_name[0].id = CORBA::string_dup(EVENT_CHANNEL_FACTORY_NAME);\n"
		" CORBA::Object_var ec_factory_obj = naming_context->resolve(ec_factory_name);\n "
		" the_event_channel_factory_ = CosEventChannelFactory::ChannelFactory::_narrow(ec_factory_obj.in());\n";
	      
	      ctx_class.add_implem( init_eventChannelFactory_method, 
				    new Invariable_string_in_place_of( implem_of_method ) );
	    }
	  }
	



	// -- insertion of "register_push_supplier" method
	{
	if( the_comp_desc->publishes_events.length() > 0 )
	  {
	    Cpp_method register_push_supplier_method; 
	   
	    register_push_supplier_method.set_name_of_method("register_push_supplier");
	    register_push_supplier_method.set_return_type( "void" );
	    
	    string implem_of_method =
	      " CosEventChannelAdmin::SupplierAdmin_var the_supplier_admin;\n";
	    
	    for(int ct_publisher=0; ct_publisher<publishes_events.length(); ct_publisher++)
	      {
		string name_of_event_channel = string( publishes_events[ct_publisher].name) + "_ec_";
		CORBA::ComponentIR::EventDef_var evt_def 
		  = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));
		string name_of_proxypushconsumer =  string( publishes_events[ct_publisher].name) + string("_");
		string name_of_pushsupplier = "ref_" + string( publishes_events[ct_publisher].name) + string("_push_supplier");
		implem_of_method += 
		  " // get a proxy push consumer for " + name_of_event_channel + "and connect the push supplier\n"
		  " the_supplier_admin = " + name_of_event_channel + "->for_suppliers();\n" 
		  + name_of_proxypushconsumer + " = the_supplier_admin->obtain_push_consumer();\n" 
		  " CosEventComm::PushSupplier_var " + name_of_pushsupplier + "= CosEventComm::PushSupplier::_nil();\n" 
		  " " + name_of_proxypushconsumer + "->connect_push_supplier(" + name_of_pushsupplier + ".in());\n";
	      }
	    
	    ctx_class.add_implem( register_push_supplier_method, 
				  new Invariable_string_in_place_of( implem_of_method ) );
	    ctx_class.add_method( register_push_supplier_method );
	  }
	}
	
      

	
	// -- insertion of publishers methods "push_xxx" called by the component
	for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
	  {
	    
	    Cpp_method push_publisher_method; 
	    {
	      CORBA::ComponentIR::EventDef_var evt_def
		= CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));

	      string name_of_publisher = string(publishes_events[ct_publisher].name);
	      string name_of_event = evt_def->name();
	      push_publisher_method.set_name_of_method(string( "push_" ) + name_of_publisher);
	      push_publisher_method.add_parameter(name_of_event + string("*"), "evt");
	      push_publisher_method.set_return_type( "void" );
	      ctx_class.add_method( push_publisher_method );
	      
	      string implem_of_method = 
		" CORBA::Any data;\n"
		" data <<= evt;\n"
		" " + name_of_publisher + "_" + "->push(data);\n"; // proxy_push_consumer->push(data)
	      
	      ctx_class.add_implem( push_publisher_method, 
				    new Invariable_string_in_place_of( implem_of_method ) );
	    }
	  }

	ctx_class.add_member( "Correspondance_table< CORBA::Object_ptr >*", "ptr_to_receptacles_" );

	if( the_comp_desc->publishes_events.length() > 0 )
	  {
	    ctx_class.add_member("long", "nbeventConnectionsMax_");
	    ctx_class.add_member("long", "current_indexEventConnections_");
	    ctx_class.add_member("Correspondance_table<CosEventChannelAdmin::ProxyPushSupplier_ptr >*", "supplier_map_");
	    ctx_class.add_member("Correspondance_table<Components::EventConsumerBase_ptr >*", "consumer_map_");
	    ctx_class.add_member("CosEventChannelFactory::ChannelFactory_ptr", "the_event_channel_factory_");
	    ctx_class.add_member("long", "index_ec_");
	  }
	
	for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
	  {
	    CORBA::ComponentIR::EventDef_var evt_def 
	      = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));
	    string name_of_publisher = string(publishes_events[ct_publisher].name);
	    string name_of_event = evt_def->name();
	    
	    ctx_class.add_member("CosEventChannelAdmin::EventChannel_ptr", name_of_publisher + "_ec_");
	    ctx_class.add_member( "CosEventChannelAdmin::ProxyPushConsumer_ptr", name_of_publisher + "_" );
	    //	    ctx_class.add_member("CosEventComm::PushSupplier_ptr", "ref_" + name_of_event + "_push_supplier");
	  }
	
      }// end of Context_impl class
      
      

      //----------------------------------------------------------------------------------
      // ***************    generation of publisher proxy "xxx_proxy"       **************
      //----------------------------------------------------------------------------------
      
      CORBA::ComponentIR::EventPortDescriptionSeq& publishes_events
	= the_comp_desc->publishes_events;
      
      list< Cpp_class_gene* > list_publisher_proxies;
      {
      for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
	{
	  
	  CORBA::ComponentIR::EventDef_var evt_def 
	    = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));
	  string name_of_event = evt_def->name();
	  
	  string name_of_publisher = string(publishes_events[ct_publisher].name);
	  string name_of_event_consumer = name_of_event + "Consumer";
	  string name_of_generated_class 
	    = name_of_publisher + "_proxy";
	  
 	  Cpp_class_gene* source_proxy_class = new Cpp_class_gene(name_of_generated_class);
	  list_publisher_proxies.push_back(source_proxy_class);

	  // 	  string the_interface_name( the_interface_def->name() );
	  
	  // 	  string::size_type pos_microccm = the_interface_name.find( "_microccm_" );
	  // 	  the_interface_name.erase( pos_microccm, pos_microccm + string( "_microccm_" ).size() );
	  
	  source_proxy_class->add_required_include( "<sstream>" );
	  
	  // -- insertion of "constructor" method
	  Cpp_method constructor_method; 
	  {
	    constructor_method.set_name_of_method(name_of_publisher + "_proxy");
	    constructor_method.add_parameter(ctx_class.get_name_of_class() + "*", "context");
	    source_proxy_class->add_method( constructor_method );
	    
	    string implem_of_method = 
	      "context_ = context;\n";
	    
	    source_proxy_class->add_implem( constructor_method, 
					   new Invariable_string_in_place_of( implem_of_method ) );
	  }
	  
	  // -- insertion of "set_context" method
	  Cpp_method set_context_method; 
	  {
	    set_context_method.set_name_of_method(string("set_context"));
	    set_context_method.add_parameter(ctx_class.get_name_of_class() + "*", "context" );
	    set_context_method.set_return_type( "void" );
	    source_proxy_class->add_method( set_context_method );
	    
	    string implem_of_method = 
	      "context_ = context;\n";
	    
	    source_proxy_class->add_implem( set_context_method, 
					   new Invariable_string_in_place_of( implem_of_method ) );
	  }

	  // -- insertion of "get_context" method
	  Cpp_method get_context_method; 
	  {
	    get_context_method.set_name_of_method(string("get_context"));
	    get_context_method.set_return_type( ctx_class.get_name_of_class() + "*" );
	    source_proxy_class->add_method( get_context_method );
	    
	    string implem_of_method = 
	      "return context_;\n";
	    
	    source_proxy_class->add_implem( get_context_method, 
					   new Invariable_string_in_place_of( implem_of_method ) );
	  }

	  // -- insertion of "connect" method
	  Cpp_method connect_consumer_method; 
	  {
	    connect_consumer_method.set_name_of_method(string("connect"));
	    connect_consumer_method.add_parameter(string(name_of_event + "Consumer_ptr").c_str(),"the_consumer");
	    connect_consumer_method.set_return_type("Components::Cookie*" );
	    source_proxy_class->add_method( connect_consumer_method );
	    
	    string implem_of_method = 
	      " // narrow the consumer to a PushConsumer\n"
	      " assert(!CORBA::is_nil(the_consumer));\n"
	      " CosEventComm::PushConsumer_ptr the_push_consumer = the_consumer->get_pushConsumer();\n"
	      " assert(!CORBA::is_nil(the_push_consumer));\n\n"
	      " // create a sequence of octets.."
	      " if(get_nbConnections() > get_nbConnectionsMax()) throw Components::ExceededConnectionLimit();\n"
	      "\n"
	      " Components::Cookie* key = increment_eventsConnectionKey();\n"
	      " CORBA::OctetSeq seq = key->cookieValue;\n"
	      "\n"
	      " // make a connection to the event channel dedicated to this source\n"
	      " CosEventChannelAdmin::ConsumerAdmin_var consumer_admin = get_EventChannel()->for_consumers ();\n"
	      " CosEventChannelAdmin::ProxyPushSupplier_ptr proxy_push_supplier = consumer_admin->obtain_push_supplier();\n"
	      "\n"
	      " // subscribe the push consumer to the event service\n"
	      " proxy_push_supplier->connect_push_consumer(the_push_consumer);\n\n"
	      " // fill the maps\n"
	      " ostringstream string_key; string_key << (long)(key->cookieValue[0]);\n"
	      " (*get_context()->supplier_map_)[string_key.str().c_str()] = CosEventChannelAdmin::ProxyPushSupplier::_duplicate(proxy_push_supplier);\n"
	      " (*get_context()->consumer_map_)[string_key.str().c_str()] = Components::EventConsumerBase::_duplicate(the_consumer);\n\n"
	      " return key;\n";
	    
	      source_proxy_class->add_implem( connect_consumer_method, 
					   new Invariable_string_in_place_of( implem_of_method ) );
	  }


		  
	  // -- insertion of "disconnect" method
	  Cpp_method disconnect_consumer_method; 
	  {
	    disconnect_consumer_method.set_name_of_method(string("disconnect"));
	    disconnect_consumer_method.add_parameter("const Components::Cookie*", "cookie");
	    disconnect_consumer_method.set_return_type(name_of_event_consumer + "_ptr");
	    source_proxy_class->add_method( disconnect_consumer_method );
	    
	    string implem_of_method = 
	      " CORBA::OctetSeq key = cookie->cookieValue;\n\n"
	    
	      " // get data from the maps\n"
	      " ostringstream string_key; string_key << (long)cookie->cookieValue[0];\n"
	      " const char* c_string_key = string_key.str().c_str();\n"
	      " CosEventChannelAdmin::ProxyPushSupplier_ptr the_push_supplier = (*get_context()->supplier_map_)[c_string_key];\n"
	      " Components::EventConsumerBase_ptr the_consumer_base = (*get_context()->consumer_map_)[c_string_key];\n"
	      " " + name_of_event_consumer + "_var the_consumer = " + name_of_event_consumer + "::_narrow(the_consumer_base);\n"
	      "if(CORBA::is_nil(the_consumer.in())) throw Components::InvalidConnection();\n\n"
	    
	      " // disconnect the push consumer\n"
	      " the_push_supplier->disconnect_push_supplier();\n"
	      " the_push_supplier = CosEventChannelAdmin::ProxyPushSupplier::_nil();\n\n"
	    
	      " // erase the key into maps\n"
	      " CORBA::release(the_push_supplier);\n"
	      " (*get_context()->supplier_map_).erase(c_string_key);\n"
	      " CORBA::release(the_consumer_base);\n"
	      " (*get_context()->consumer_map_).erase(c_string_key);\n"
	      " return the_consumer._retn();\n";


	    source_proxy_class->add_implem( disconnect_consumer_method, 
					   new Invariable_string_in_place_of( implem_of_method ) );
	  }
	  
	  
	  
	  // -- insertion of "get_indexConnections" method
	  Cpp_method get_indexConnections_method; 
	  {
	    get_indexConnections_method.set_name_of_method("get_indexConnections");
	    get_indexConnections_method.set_return_type( "long" );
	    source_proxy_class->add_method(get_indexConnections_method);
	    
	    string implem_of_method = 
	      " return context_->current_indexEventConnections_;\n";
	    
	    source_proxy_class->add_implem( get_indexConnections_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	  }

	  // -- insertion of "set_indexConnections" method
	  Cpp_method set_indexConnections_method; 
	  {
	    set_indexConnections_method.set_name_of_method( "set_indexConnections" );
	    set_indexConnections_method.add_parameter ("long", "indexConnections" );
	    set_indexConnections_method.set_return_type( "void" );
	    source_proxy_class->add_method( set_indexConnections_method );
	    
	    string implem_of_method = 
	      " context_->current_indexEventConnections_ = indexConnections;\n";
	    
	    source_proxy_class->add_implem( set_indexConnections_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	  }
	  
	  
	  // -- insertion of "get_nbConnections" method
	  Cpp_method get_nbConnections_method; 
	  {
	    get_nbConnections_method.set_name_of_method( "get_nbConnections" );
	    get_nbConnections_method.set_return_type( "int" );
	    source_proxy_class->add_method( get_nbConnections_method );
	    
	    string implem_of_method = 
	      "  return context_->supplier_map_->size();\n";
	    
	    source_proxy_class->add_implem( get_nbConnections_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	  }

	  
	  // -- insertion of "get_nbConnectionsMax" method
	  Cpp_method get_nbConnectionsMax_method; 
	  {
	    get_nbConnectionsMax_method.set_name_of_method( "get_nbConnectionsMax" );
	    get_nbConnectionsMax_method.set_return_type( "long" );
	    source_proxy_class->add_method( get_nbConnectionsMax_method );
	    
	    string implem_of_method = 
	      " return context_->nbeventConnectionsMax_;\n";
	    
	    source_proxy_class->add_implem( get_nbConnectionsMax_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	  }


	  // -- insertion of "increment_eventsConnectionKey" method
	  Cpp_method increment_eventsConnectionKey_method; 
	  {
	    increment_eventsConnectionKey_method.set_name_of_method( "increment_eventsConnectionKey" );
	    increment_eventsConnectionKey_method.set_return_type( "Components::Cookie*" );
	    increment_eventsConnectionKey_method.add_exception_thrown("Components::ExceededConnectionLimit");
	    source_proxy_class->add_method( increment_eventsConnectionKey_method );
	    
	    string implem_of_method = 
	      "   // memory leak on key of Cookie type\n"
	      "   // proceed with caution for sequence release \n"
	      "   Components::Cookie* key =  new Components::Cookie;\n"
	      "   long index_connections = get_indexConnections();\n"
	      "   key->cookieValue.length(1);\n"
	      "   key->cookieValue[0] = ++index_connections;\n"
	      "   set_indexConnections(index_connections);\n"
	      "   return key;\n";
	    
	    source_proxy_class->add_implem( increment_eventsConnectionKey_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	  }


	  // -- insertion of "get_EventChannel" method
	  Cpp_method get_EventChannel; 
	  {
	    get_EventChannel.set_name_of_method( "get_EventChannel" );
	    get_EventChannel.set_return_type( "CosEventChannelAdmin::EventChannel_ptr" );
	    source_proxy_class->add_method( get_EventChannel );
	    
	    string implem_of_method = 
	      "   return  CosEventChannelAdmin::EventChannel::_duplicate(context_->" + name_of_publisher + "_ec_" + ");\n";
	    
	    source_proxy_class->add_implem( get_EventChannel, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	  }

	  
	  // MEMBERS
	  source_proxy_class->add_member(ctx_class.get_name_of_class() + "*", "context_");

	}// for (publisher..)
      }// list<source_proxy>


      // proxy for sinks
      //
      CORBA::ComponentIR::EventPortDescriptionSeq& consumes_events
	= the_comp_desc->consumes_events;
      
      list< Cpp_class_gene* > list_sink_proxies; 
      list< Cpp_class_gene* > list_sink_pushconsumer_proxies; 
      {
	for(int ct_consumer = 0; ct_consumer < consumes_events.length(); ct_consumer++)
	  {
	    
	    CORBA::ComponentIR::EventDef_var evt_def 
	      = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(consumes_events[ct_consumer].event));

	    string name_of_event = evt_def->name();
	    string name_of_consumer = string(consumes_events[ct_consumer].name);
	    string name_of_event_consumer = name_of_event + "Consumer";
	    string name_of_generated_class 
	      = name_of_component + "__" + name_of_event_consumer + "_impl";
	    
	    Cpp_class_gene* sink_proxy_class = new Cpp_class_gene( name_of_generated_class );
	    list_sink_proxies.push_back(sink_proxy_class);
	    sink_proxy_class->add_inherited_class( string( "POA_" ) + name_of_event_consumer,
						   "public" );
	  
	    // 	  string the_interface_name( the_interface_def->name() );
	  
	    // -- insertion of "constructor" method
	    Cpp_method constructor_method; 
	    {
	      constructor_method.set_name_of_method(name_of_generated_class);
	      constructor_method.add_parameter(" CCM_" + name_of_component + "_ptr", "the_comp");
	      constructor_method.add_parameter("const char *", "name_of_port");
	      sink_proxy_class->add_method( constructor_method );
	    }
	    string implem_of_method = 
	      " for_this_port_ = CORBA::string_dup(name_of_port);\n"
	      " consumer_ = the_comp;\n"
	      " cos_pushconsumer_ref_ = 0;\n";
	  
	    sink_proxy_class->add_implem( constructor_method, 
					  new Invariable_string_in_place_of( implem_of_method ) );


	    
	    // -- insertion of "destructor" method
	    {
	      Cpp_method destructor_method; 
	      destructor_method.set_name_of_method(string("~") + name_of_generated_class);
	      sink_proxy_class->add_method( destructor_method );
	    
	      string implem_of_method = 
		"  CORBA::string_free(for_this_port_);\n"
		"  CORBA::release(cos_pushconsumer_ref_);\n";
	      
	      sink_proxy_class->add_implem( destructor_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	    }
	    

	    // -- insertion of "push_event" method
	    Cpp_method push_event_method; 
	    {
	      push_event_method.set_name_of_method("push_event");
	      push_event_method.add_parameter("Components::EventBase*", "evt");
	      push_event_method.set_return_type( "void" );
	      push_event_method.add_exception_thrown("CORBA::SystemException");
	      sink_proxy_class->add_method( push_event_method );
	    }
	    implem_of_method = 
	      "  " + name_of_event + "* typed_evt = " + name_of_event + "::_downcast(evt);\n"
	      "  this->push(typed_evt);\n";

	    sink_proxy_class->add_implem( push_event_method, 
					 new Invariable_string_in_place_of( implem_of_method ) );
	  
	    
	    // -- insertion of "push" method
	    Cpp_method push_method; 	  
	    {
	      push_method.set_name_of_method("push");
	      push_method.add_parameter(name_of_event + "*", "data");
	      push_method.set_return_type( "void" );
	      push_method.add_exception_thrown("CORBA::SystemException");
	      sink_proxy_class->add_method( push_method );
	      
	      implem_of_method = "";
	      for( int ct_consumes = 0; ct_consumes < consumes_events.length(); ct_consumes++ )
		{
		  string name_of_port( consumes_events[ ct_consumes ].name );
		  
		  if( name_of_event == scoped_name( consumes_events[ ct_consumes ].event ) )
		    {
		      implem_of_method += 
			"   if( strcmp( for_this_port_.in() , \"" + name_of_port + "\" ) == 0 )\n"
			"     { \n"
			"        consumer_->push_" + name_of_port + "( data );\n"
			"        return;\n"
			"     } \n"
			"\n";
		    }
		  
		}
	      
	      implem_of_method += " exit( 2345 ); // should not there unless there is a bug ..\n";
	      
	      sink_proxy_class->add_implem( push_method, 
					    new Invariable_string_in_place_of( implem_of_method ) );
	    }
	  
	  
	    // -- insertion of "get_pushconsumer" method
	    Cpp_method get_pushconsumer_method; 
	    {
	      get_pushconsumer_method.set_name_of_method("get_pushConsumer");
	      get_pushconsumer_method.set_return_type( "CosEventComm::PushConsumer_ptr" );
	      get_pushconsumer_method.add_exception_thrown("CORBA::SystemException");
	      
	      implem_of_method =
		"  if(CORBA::is_nil(cos_pushconsumer_ref_))\n"
		"  {\n"
		"  cerr << \"cos_pushconsumer_ref_ doesn't exists!\" << endl;\n"
		"  }\n"
		"  return cos_pushconsumer_ref_;\n";
	      
	    }
	    sink_proxy_class->add_method( get_pushconsumer_method );
	    sink_proxy_class->add_implem( get_pushconsumer_method, 
					  new Invariable_string_in_place_of( implem_of_method ) );
	    // members
	    sink_proxy_class->add_member("CORBA::String_var", "for_this_port_");
	    sink_proxy_class->add_member("CosEventComm::PushConsumer_ptr", "cos_pushconsumer_ref_");
	    sink_proxy_class->add_member(string("CCM_") + name_of_component + "_ptr", "consumer_");
	    
	  
	
	  // -------------  insertion of "xxxPushConsumer" class ----------------
	  name_of_generated_class 
	    = name_of_component + "__" + name_of_event + "PushConsumer";
	  
 	  Cpp_class_gene* sink_pushconsumer_class = new Cpp_class_gene( name_of_generated_class );
	  list_sink_pushconsumer_proxies.push_back(sink_pushconsumer_class);
	  
	  sink_pushconsumer_class->add_inherited_class( string( "POA_CosEventComm::PushConsumer"),
						"public" );
	  
	  // -- insertion of "constructor" method
	  {
	    Cpp_method constructor_method; 
	    constructor_method.set_name_of_method(name_of_generated_class);
	    constructor_method.add_parameter(name_of_component + "__" + name_of_event_consumer + "_impl*", "consumer");
	    sink_pushconsumer_class->add_method( constructor_method );
	    string implem_of_method = 
	      " myeventconsumer_impl_ = consumer;\n";
	    sink_pushconsumer_class->add_implem( constructor_method, 
						 new Invariable_string_in_place_of( implem_of_method ) );
	  }
	    

	  // -- insertion of "push" method
	  {
	    Cpp_method push_method; 
	    push_method.set_name_of_method("push");
	    push_method.add_parameter("const CORBA::Any&", "any_data");
	    
	    push_method.set_return_type( "void" );
	    push_method.add_exception_thrown("CORBA::SystemException");
	  
	    sink_pushconsumer_class->add_method( push_method );
	    implem_of_method = 
	      " " + name_of_event + "* myevent_data=0;\n"  
	      " any_data >>= myevent_data;\n"
	      " myeventconsumer_impl_->push_event(myevent_data);";
	    sink_pushconsumer_class->add_implem( push_method, 
						 new Invariable_string_in_place_of( implem_of_method ) );
	  }
	  
	  
	  // -- insertion of "diconnect_push_consumer_method" method
	  {  
	    Cpp_method disconnect_push_consumer_method; 
	    
	    disconnect_push_consumer_method.set_name_of_method("disconnect_push_consumer");
	    disconnect_push_consumer_method.add_exception_thrown("CORBA::SystemException");
	    disconnect_push_consumer_method.set_return_type( "void" );
	  
	    sink_pushconsumer_class->add_method( disconnect_push_consumer_method );
	    sink_pushconsumer_class->add_implem( disconnect_push_consumer_method, 
						 new Invariable_string_in_place_of(""));
	  }  
	  
	  
	  sink_pushconsumer_class->add_member(name_of_component +  "__" + name_of_event_consumer + "_impl*", "myeventconsumer_impl_");
	  
	  
	} // for(in ct_consumer=0;...)
      }

      
      
      
      
//       class MyEvent1PushConsumer : public POA_CosEventComm::PushConsumer
//       {
//       public:
// 	MyEvent1PushConsumer(MyEvent1Consumer_impl* myevent1consumer);
// 	void push(const CORBA::Any& data) throw(CORBA::SystemException);
// 	virtual void disconnect_push_consumer()  throw(CORBA::SystemException);
	
//       public:
// 	MyEvent1Consumer_impl* myevent1consumer_impl_;
//       };

     
      
      //----------------------------------------------------------------------------------
      // ***************    generation for main component servant      ************
      //----------------------------------------------------------------------------------

      CORBA::InterfaceDef_ptr the_interface_def
	= create_equivalent_idl2_interface( the_ref_repo, the_comp_desc );

      string name_of_generated_class 
	= string( the_interface_def->name() ) + "_impl";

      Cpp_class_gene class_gene_result 
	= IDL_to_Cpp_mapping( the_interface_def, name_of_generated_class, false );

      string the_interface_name( the_interface_def->name() );

      string::size_type pos_microccm = the_interface_name.find( "_microccm_" );
      the_interface_name.erase( pos_microccm, pos_microccm + string( "_microccm_" ).size() );

      class_gene_result.add_inherited_class( string( "POA_" ) + the_interface_name,
					     "public" );

      // MEMBERS of the class ...

      class_gene_result.add_required_include( "<Correspondance_table.h>" );

      if( !only_one_idl_file_per_comp )
	{
	  class_gene_result.add_required_include( string( "<microccm_" ) + the_interface_name + "S.h>" );
	}

      class_gene_result.add_member( "MCCM_Service_Integration_Points*", "the_service_integration_points_" );
      
      {
	Cpp_method get_service_integration_points_meth; 
	get_service_integration_points_meth.set_prototype( "  MCCM_Service_Integration_Points* get_service_integration_points();" );
	
	class_gene_result.add_method( get_service_integration_points_meth );
	string implem = "" 
	  + get_mccm_debug_trace_warning("INTERCEPTORS") +
	  "return the_service_integration_points_;";
	
	class_gene_result.add_implem( get_service_integration_points_meth,
				      new Invariable_string_in_place_of ( implem ) );
      }
      
      for( int ct_provided_interface = 0; 
	   ct_provided_interface < nb_provided_interfaces; 
	   ct_provided_interface++ )
        {
	  CORBA::ComponentIR::ProvidesDescription& desc = the_provided_interfaces[ ct_provided_interface ];
	  string name_of_current_port( desc.name );

	  CORBA::Contained_ptr ref_intfdef_as_contained
	    = the_ref_repo->lookup_id( desc.interface_type );

	  string name_of_current_intf( ref_intfdef_as_contained->name() );

          string type_member = string( "_microccm_" ) + name_of_current_intf + "_impl";
          string name_member = string( "obj_impl_" ) + name_of_current_port + "_";

          class_gene_result.add_member( type_member + "*", name_member );
	  class_gene_result.add_member( "PortableServer::ObjectId_var", 
					string( "" ) + "objid_" + name_of_current_port + "_" );
        }

      class_gene_result.add_member( ctx_class.get_name_of_class()+"*", "ctx_component_" );
      class_gene_result.add_member( "PortableServer::POA_ptr", "the_poa_" );
      class_gene_result.add_member( "CORBA::ORB_ptr", "the_orb_" );
      class_gene_result.add_member( "Correspondance_table< CORBA::Object_ptr >", "refs_for_receptacles_" );
      class_gene_result.add_member( "Correspondance_table< CORBA::Object_ptr >", "refs_for_facets_" );
      class_gene_result.add_member( string( "CCM_" ) + name_of_component + "_ptr", "the_comp_instance_" );
      class_gene_result.add_member( "Components::CCMHome_ptr", "the_home_" );
      
      
      for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
	    {
	      CORBA::ComponentIR::EventDef_var evt_def
		= CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));

	      string name_of_publisher = string(publishes_events[ct_publisher].name);
	      class_gene_result.add_member( name_of_publisher + "_proxy *", "proxy_" + name_of_publisher + "_" );
	    }
      
       for(int ct_consumer = 0; ct_consumer < consumes_events.length(); ct_consumer++)
	    {
	      CORBA::ComponentIR::EventDef_var evt_def 
		= CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(consumes_events[ct_consumer].event));

	      string name_of_consumer = string(consumes_events[ct_consumer].name);
	      string name_of_event = string(evt_def->name());
	      class_gene_result.add_member( name_of_component + "__" + name_of_event + string("Consumer_impl*"), 
					    string("proxy_") + name_of_event + string("Consumer_") );
	      class_gene_result.add_member( name_of_event + string( "Consumer_ptr " ),
					    name_of_consumer + "_" );
	      class_gene_result.add_member(  string("Correspondance_table<") + 
					     name_of_component + "__" + 
					     name_of_event + string("PushConsumer*")  + string(" >"), 
					     name_of_event + string( "PushConsumers_" ) );

	    }

      
      // ADDITION of some methods (not inherited these ones ...)

      // -- insertion of Constructor
      {
        string name_of_constr_method = name_of_generated_class;
        Cpp_method constructor_method;
        constructor_method.set_name_of_method( name_of_constr_method );
	constructor_method.add_parameter("CORBA::ORB_ptr", "the_orb");
	constructor_method.add_parameter("PortableServer::POA_ptr", "the_poa");
	constructor_method.add_parameter("Components::CCMHome_ptr", "the_home");
	class_gene_result.add_method( constructor_method );

        // add here implementation of constructor method
        class_gene_result.add_implem( name_of_constr_method,
                                    new CORBA_cpp_gene_main_servant_constr( the_comp_desc ) );
        
      }

      // -- insertion of Destructor
      {
        string name_of_destr_method = string( "~" ) + name_of_generated_class;
        Cpp_method destructor_method;
        destructor_method.set_name_of_method( name_of_destr_method );
	class_gene_result.add_method( destructor_method );

	
        // add here implementation of destructor method
	
	string destructeur_impl =
	  get_mccm_debug_trace_warning("COMPONENT_IMPLEM") +
	  "for( Correspondance_table< CORBA::Object_ptr >::iterator iter = refs_for_receptacles_.begin();\n"
	  "    iter != refs_for_receptacles_.end();\n"
	  "    iter++ )\n"
	  "{\n"
	  "   if( ! CORBA::is_nil( iter->value ) )\n"
	  "   {\n"
	  "     {\n"
	  "      CORBA::Object_var dummy_obj = iter->value;\n"
	  "     }\n"
	  "     iter->value = CORBA::Object::_nil();\n"
	  "   }\n"
	  "}\n"
	  "\n"
	  "for( Correspondance_table< CORBA::Object_ptr >::iterator iter = refs_for_facets_.begin();\n"
	  "     iter != refs_for_facets_.end();\n"
	  "     iter++ )\n"
	  "{\n"
	  "   if( ! CORBA::is_nil( iter->value ) )\n"
	  "   {\n"
	  "       {\n"
	  "         CORBA::Object_var dummy_obj = iter->value;\n"
	  "        }\n"
	  "      iter->value = CORBA::Object::_nil();\n"
	  "    }\n"
	  "}\n\n"
	
	  "delete ctx_component_;\n"
	  "delete the_comp_instance_;\n"
	  "deactivate_and_delete_objs();\n"
	  "CORBA::release(the_home_);\n";
	
	for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ++ct_publisher)
	  {
	    string name_of_publisher = string(publishes_events[ct_publisher].name);
	    destructeur_impl +=  
	      " delete proxy_" + name_of_publisher + "_;\n";
	  }



	
	// ------------------------

        class_gene_result.add_implem( name_of_destr_method,
                                    new Invariable_string_in_place_of( destructeur_impl ) );
      }

      // -- insertion of  "void create_and_activate_objs()" method
      {
        Cpp_method create_and_act_objs;
	{
	  create_and_act_objs.set_name_of_method( "create_and_activate_objs" );
	  create_and_act_objs.set_return_type( "void" );
	}
	class_gene_result.add_method( create_and_act_objs );
        // add here implementation for "create_and_activate_objs"
        class_gene_result.add_implem( "create_and_activate_objs",
                                    new CORBA_cpp_gene_main_servant_create_act( the_comp_desc, 
										the_ref_repo ) );
       }

      // -- insertion of "void deactivate_and_delete_objs()" method
      {
        Cpp_method deactivate_and_del_method;
	{
	  deactivate_and_del_method.set_name_of_method( "deactivate_and_delete_objs" );
	  deactivate_and_del_method.set_return_type( "void" );
	}
	class_gene_result.add_method( deactivate_and_del_method );
        // add here implementation for "deactivate_and_delete_objs" method
        class_gene_result.add_implem( "deactivate_and_delete_objs", 
                                      new CORBA_cpp_gene_main_servant_delete_deact( the_comp_desc, 
										    the_ref_repo ) );
      }
      
      // -- insertion of "check_type_for()" method
      {
	Cpp_method check_type_for_method;
	{
	  check_type_for_method.set_name_of_method( "check_type_for" );
	  check_type_for_method.add_parameter( "const char*", "name" );
	  check_type_for_method.add_parameter( "CORBA::Object_ptr", "ref" );
	  check_type_for_method.set_return_type( "bool" );
	}
	class_gene_result.add_method( check_type_for_method );

	class_gene_result.add_implem( "check_type_for",
                                     new CORBA_cpp_gene_main_servant_check_type( the_comp_desc,
										 the_ref_repo ) );
      }

      // -- addition of _set_configuration( const Components::ConfigValues& ) method

      {
	Cpp_method set_configuration_method;
	set_configuration_method.set_prototype( "void _set_configuration( const Components::ConfigValues& cfg_vals )" );

	CORBA::ExtAttrDescriptionSeq& attributes_seq
	  = the_comp_desc->attributes;

	string implem = string( "" ) 
	  + get_mccm_debug_trace_warning("COMPONENT_IMPLEM")
	  + "for( int counter = 0; counter < cfg_vals.length(); counter++ ) \n"
	  + "  { \n";

	for( int ct_attributes = 0; ct_attributes < attributes_seq.length(); ct_attributes++ )
	  {
	    CORBA::ExtAttributeDescription& attr_desc = attributes_seq[ ct_attributes ];
	    string attr_name = string( attr_desc.name );

	    string type_attr;
	    string any_extraction_add_on = "";

	    if( ( is_primitive( attr_desc.type.in() ) ) || ( attr_desc.type->kind() == CORBA::tk_enum ) )
	      {
		type_attr = IDL_to_cpp_arg_mapping( attr_desc.type.in(), arg_passing_in );
	      }
	    else
	      {
		type_attr = attr_desc.type->name();
		any_extraction_add_on = " &"; 
	      }

	    // implementations sets attributes accordingly on underlying executor



	    implem += 
	      " if( strcmp( cfg_vals[ counter ].name.in(), \"" + attr_name + "\" ) == 0 ) \n"
	      "   {  \n"
	      "     " + type_attr + " extracted_from_any; \n"
	      "     cfg_vals[ counter ].value >>= " + any_extraction_add_on + "extracted_from_any; \n"
	      "     the_comp_instance_->" + attr_name + "( extracted_from_any );  \n"
	      " }\n"
	      " \n";

	  }

	implem += "} \n";

	class_gene_result.add_method( set_configuration_method );
	class_gene_result.add_implem( set_configuration_method, new Invariable_string_in_place_of( implem ) );
      }


      // --- ADDITION of implementations for inherited methods ...

      // get_ccm_home_method
      {
	string impl_get_ccm_home =   
	  get_mccm_debug_trace_warning("COMPONENT_IMPLEM")
	  + get_mccm_assert("the_home_") +
	  "return Components::CCMHome::_duplicate( the_home_ );\n";
	class_gene_result.add_implem( "get_ccm_home", 
				      new Invariable_string_in_place_of( impl_get_ccm_home ) );
      }

      // configuration complete
      {
	string impl_configuration_complete =
	  get_mccm_debug_trace_warning("COMPONENT_IMPLEM")
	  + get_mccm_assert("the_comp_instance_")
	  = " return the_comp_instance_->configuration_complete(); \n";

	class_gene_result.add_implem( "configuration_complete",
				    new Invariable_string_in_place_of( impl_configuration_complete ) );
      }

      // remove
      {
	string impl_remove = 
	  get_mccm_debug_trace_warning("COMPONENT_IMPLEM") +
	  " exit( 4 ); \n";
	class_gene_result.add_implem( "remove",
				    new Invariable_string_in_place_of( impl_remove ) );
      }

      // provide_facet
      {
	string impl_provide_facet = 
	  get_mccm_debug_trace_warning("COMPONENT_IMPLEM_FACETS_RECPS")
	  + get_mccm_assert("name") +
	  "if( refs_for_facets_.exists( name ) ) \n"
	  "{\n"
	  "return CORBA::Object::_duplicate( refs_for_facets_[ name ] );\n"
	  "}\n"
	  "else\n"
	  "throw Components::InvalidName();\n";
	
	class_gene_result.add_implem( "provide_facet",
				      new Invariable_string_in_place_of( impl_provide_facet ) );
      }

      // connect
      {
	string impl_connect = ""
	  + get_mccm_debug_trace_warning("COMPONENT_IMPLEM_FACETS_RECPS")
	  + get_mccm_assert("connection")
	  + get_mccm_assert("name") +
	  "if( ! refs_for_receptacles_.exists( name ) ) throw Components::InvalidName();\n"
	  "\n"
	  "if( ! CORBA::is_nil( refs_for_receptacles_[ name ] ) ) \n"
	  "{\n"
	  "   throw Components::AlreadyConnected();\n"
	  "}\n"
	  "\n"
	  "bool ok = check_type_for( name, connection );\n"
	  "if( ! ok ) throw Components::InvalidConnection();\n"
	  "\n"
	  "refs_for_receptacles_[ name ] = CORBA::Object::_duplicate( connection );\n"
	  "\n"
	  "// return a convenient cookie .. que retourner dans le cas non receptacle multiple ??\n"
	  "Components::Cookie* to_be_returned = new Components::Cookie;\n"
	  "\n"
	  "return to_be_returned;\n"
	  "// xxx attention, receptacles multiples pas encore supportes.\n";

	class_gene_result.add_implem( "connect",
				    new Invariable_string_in_place_of( impl_connect ) );
      }

      // disconnect
      {
	string impl_disconnect = ""
	  + get_mccm_debug_trace_warning("COMPONENT_IMPLEM_FACETS_RECPS")
	  + get_mccm_assert("name") +
	  "// can be factorized ...\n"
	  "if( ! refs_for_receptacles_.exists( name ) ) throw Components::InvalidName();\n"
	  "\n"
	  "if( CORBA::is_nil( refs_for_receptacles_[ name ] ) ) \n"
	  "  {\n"
	  "     throw Components::NoConnection();\n"
	  "  }\n"
	  "else\n"
	  "{\n"
	  "   // what follows is not possible\n"
	  "   // CORBA::Object::_release( refs_for_receptacles_[ name ] );\n"
	  "   // so use the following :\n"
	  "   {\n"
	  "      CORBA::Object_var the_obj = refs_for_receptacles_[ name ];\n"
	  "   }\n"
	  "\n"
	  "   refs_for_receptacles_[ name ] = CORBA::Object::_nil();\n"
	  "}\n"
	  "\n"
	  "// multiple receptacles still not handled here ...\n"
	  "// (exception Components::InvalidConnection is thrown when cookie does not\n"
	  "// correspond to an existing connection)\n";

	class_gene_result.add_implem( "disconnect",
				      new Invariable_string_in_place_of( impl_disconnect ) );
      }
      
      
      for(int ct_consumer = 0; ct_consumer < consumes_events.length(); ct_consumer++)
	{
	  CORBA::ComponentIR::EventDef_var evt_def 
	    = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(consumes_events[ct_consumer].event));
	  string name_of_port = string(consumes_events[ct_consumer].name);
	  string name_of_consumer = string(evt_def->name()) + "Consumer";
	  
	  // insertion of "get_consumer_xxx"
	  {
	    Cpp_method get_consumer_xxx;
	    
	    get_consumer_xxx.set_name_of_method( string("get_consumer_") + name_of_port);
	    get_consumer_xxx.set_return_type( name_of_consumer + "_ptr");
	    class_gene_result.add_method(get_consumer_xxx );
	    
	    string implem_of_method =
	      "  return " + name_of_consumer + "::_duplicate(" + name_of_port + "_);\n";
	    
	    class_gene_result.add_implem( get_consumer_xxx,
					  new Invariable_string_in_place_of( implem_of_method ));
	    
	  }
	}

	

      

      // -- insertion of "get_consumer" method
      //       Cpp_method get_consumer_method;
      // 	{
      // 	  get_consumer_method.set_name_of_method( "get_consumer" );
      // 	  get_consumer_method.add_parameter( "const char*", "sink_name" );
      // 	  get_consumer_method.set_return_type( "Components::EventConsumerBase_ptr" );
      // 	}
      // 	class_gene_result.add_method( get_consumer_method );	  
	string impl_get_consumer = "" 
	  + get_mccm_debug_trace_warning("COMPONENT_IMPLEM_EVENTS")
	  + get_mccm_assert("sink_name");
	  " string the_string(sink_name);\n";
	
	for(int ct_consumer = 0; ct_consumer < consumes_events.length(); ct_consumer++)
	  {
	    CORBA::ComponentIR::EventDef_var evt_def = 
	      CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(consumes_events[ct_consumer].event));
	    
	    impl_get_consumer += 
	      " if ( strcmp( sink_name, \"" +  string(consumes_events[ct_consumer].name) + "\" ) == 0)\n"
	      " {\n"
	      "  return get_consumer_" + string( consumes_events[ct_consumer].name ) + "();\n"
	      " }\n";
	  }
	impl_get_consumer += " return 0;\n";
    	class_gene_result.add_implem("get_consumer" , 
				     new Invariable_string_in_place_of( impl_get_consumer ));
	
	// -- insertion of "subscribe" method
	// 	Cpp_method subscribe_method;
	// 	{
// 	  subscribe_method.set_name_of_method( "subscribe");
// 	  subscribe_method.add_parameter("const char*", "publisher_name");
// 	  subscribe_method.add_parameter("Components::EventConsumerBase_ptr", "subscriber");
// 	  subscribe_method.set_return_type( "Components::Cookie*");
// 	  subscribe_method.add_exception_thrown("CORBA::SystemException");
// 	  subscribe_method.add_exception_thrown("Components::InvalidName");
// 	  subscribe_method.add_exception_thrown("Components::InvalidConnection");
// 	  subscribe_method.add_exception_thrown("Components::ExceededConnectionLimit");
// 	}
	string impl_subscribe_method = "" 
	  + get_mccm_debug_trace_warning("COMPONENT_IMPLEM_EVENTS")
	  + get_mccm_assert("publisher_name")
	  + get_mccm_assert("subscriber") +
	  " string the_string(publisher_name);\n";
	
	if( publishes_events.length() != 0)
	  {
	    for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
	      {
		CORBA::ComponentIR::EventDef_var evt_def = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));
		string name_of_publisher = string(publishes_events[ct_publisher].name);
		string name_of_event = string(evt_def->name());
		
		impl_subscribe_method += 
		  " if (the_string ==" + string( "\"") + name_of_publisher + string("\"") + ")\n"
		  "   {\n"
		  "    " + name_of_event + "Consumer_ptr the_" + name_of_event + "Consumer = " + name_of_event + "Consumer::_narrow(subscriber);\n"
		  "    if(CORBA::is_nil(" + "the_" + name_of_event + "Consumer" + ")) throw Components::InvalidConnection();\n"
		  "    assert(!CORBA::is_nil(" + "the_" + name_of_event + "Consumer));\n"
		  "    return proxy_" + name_of_publisher + "_" + "->connect(" + "the_" + name_of_event + "Consumer);\n"
		  "   }\n"
		  " else";
	      }
	    impl_subscribe_method += 
	      "\n"
	      "   {\n"
	      "    throw Components::InvalidName();\n"
	      "   }\n";
	  }
	impl_subscribe_method += "   return 0;\n";
    
	//	class_gene_result.add_method( subscribe_method );
	class_gene_result.add_implem("subscribe" , 
				     new Invariable_string_in_place_of( impl_subscribe_method ));


	
	
	// -- insertion of "unsubscribe" method
	{
	  Cpp_method unsubscribe_method;  
	  
	  string impl_unsubscribe_method = ""
	    + get_mccm_debug_trace_warning("COMPONENT_IMPLEM_EVENTS")
	    + get_mccm_assert("publisher_name") + "\n" +
	    " string the_string(publisher_name);\n";
	    
	  if( publishes_events.length() != 0)
	    {
	      for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
		{
		  CORBA::ComponentIR::EventDef_var evt_def = CORBA::ComponentIR::EventDef::_narrow(the_ref_repo->lookup_id(publishes_events[ct_publisher].event));
		  string name_of_publisher = string(publishes_events[ct_publisher].name);
		  string name_of_event = string(evt_def->name());
		  
		  impl_unsubscribe_method += 
		    " if (the_string == " + string("\"") + name_of_publisher + string("\"") + ")\n"
		    "   {\n"
		    "    proxy_" + name_of_publisher + "_" + "->disconnect(&ck);\n"
		    "   }\n"
		    " else";
		}
	      impl_unsubscribe_method +=
		"\n"
		"   {\n"
		"    throw Components::InvalidName();\n"
		"   }\n";
	    }
	  
	  
	  class_gene_result.add_implem("unsubscribe" , 
				     new Invariable_string_in_place_of( impl_unsubscribe_method ));
	}
    

    
	

	// -- insertion of "connect_consumer" method
// 	Cpp_method connect_consumer_method;
// 	{
// 	  connect_consumer_method.set_name_of_method( "connect_consumer");
// 	  connect_consumer_method.add_parameter("const char*", "emitter_name");
// 	  connect_consumer_method.add_parameter(" Components::EventConsumerBase_ptr", "consumer");
// 	  connect_consumer_method.set_return_type( "void");
// 	  connect_consumer_method.add_exception_thrown("CORBA::SystemException");
// 	  connect_consumer_method.add_exception_thrown("Components::InvalidName");
// 	  connect_consumer_method.add_exception_thrown("Components::InvalidConnection");
// 	  connect_consumer_method.add_exception_thrown("Components::AlreadyConnected");
// 	}  
// 	class_gene_result.add_method( connect_consumer_method );
	
	string impl_connect_consumer_method =
	  " // TODO for emitter...\n";
	
	class_gene_result.add_implem( "connect_consumer", 
				      new Invariable_string_in_place_of( impl_connect_consumer_method ));
	
	
	// -- insertion of "disconnect_consumer" method
	// 	Cpp_method disconnect_consumer_method;
	// 	{
	// 	  disconnect_consumer_method.set_name_of_method( "disconnect_consumer");
	// 	  disconnect_consumer_method.add_parameter("const char*", "emitter_name");
	// 	  disconnect_consumer_method.set_return_type( "Components::EventConsumerBase*");
	// 	  disconnect_consumer_method.add_exception_thrown("CORBA::SystemException");
	// 	  disconnect_consumer_method.add_exception_thrown("Components::InvalidName");
	// 	  disconnect_consumer_method.add_exception_thrown("Components::NoConnection");
	// 	}
	// 	  class_gene_result.add_method( disconnect_consumer_method );
	
	string impl_disconnect_consumer_method =
	  " // TODO for emitter...\n";
	
	class_gene_result.add_implem( "disconnect_consumer", 
				      new Invariable_string_in_place_of( impl_disconnect_consumer_method ));
	string base_name = string( "microccm_" ) + name_of_component + "_CORBA";

 	// -- insertion of "attributes" method
 	{
	  CORBA::ExtAttrDescriptionSeq& attributes_seq
	    = the_comp_desc->attributes;

	  for(int ct_attributes = 0; ct_attributes < attributes_seq.length(); ct_attributes++)
	    {
	      CORBA::ExtAttributeDescription& attr_desc = attributes_seq[ ct_attributes ];
	      string attr_name = string( attr_desc.name );

	      cerr << " Currently treating attribute of name " << attr_name << endl;

	      {
		Cpp_method get_attribute_method;
		string attr_type_ret = IDL_to_cpp_arg_mapping(attr_desc.type.in(), arg_passing_return);
		get_attribute_method.set_prototype( attr_type_ret + " " + attr_name + string("( )") + " throw (CORBA::SystemException)" );

		// class_gene_result.add_method( get_attribute_method );

		string impl_get_attribute_method =
		" return the_comp_instance_->" + attr_name +"();\n";

		cerr << "Adding implementation for get attribute of name " << attr_name << endl;

		cerr << get_attribute_method << endl;

		class_gene_result.add_implem( get_attribute_method,
				      new Invariable_string_in_place_of( impl_get_attribute_method ));
	      }

	      if( attr_desc.mode != CORBA::ATTR_READONLY )
		{
		  Cpp_method set_attribute_method;
		  string attr_type_in = IDL_to_cpp_arg_mapping(attr_desc.type.in(), arg_passing_in);
		  set_attribute_method.set_prototype(string("void ") + attr_name + string("(") + attr_type_in + " attribute " + string(")") 
						     + " throw (CORBA::SystemException)");

		  // class_gene_result.add_method( set_attribute_method );

		  string impl_set_attribute_method =
		    " the_comp_instance_->" + attr_name +"( attribute );\n";

		  cerr << "Adding implementation for set attribute of name " << attr_name << endl;

		  class_gene_result.add_implem( set_attribute_method, 
					      new Invariable_string_in_place_of( impl_set_attribute_method ));
		}

	    }

	}


	//---------------------------------------------------------------------------------
	// ************************   writing component in file     ***********************
	//---------------------------------------------------------------------------------
	
	string name_h_file = base_name + ".h";
	string name_cpp_file = base_name + ".cpp";
	
	fstream outfile_h( name_h_file.c_str(), ios::out );
	fstream outfile_cpp( name_cpp_file.c_str(), ios::out );
	
	outfile_h << "#ifndef " << base_name << + "_h" << endl;
	outfile_h << "#define " << base_name << + "_h" << endl<< endl;
      
      
	outfile_h << "#include <microccm_" << name_of_component << "_executor.h>" << endl;
	outfile_h << "#include <orbsvcs/orbsvcs/CosNamingC.h>" << endl;
	outfile_h << "#include <orbsvcs/orbsvcs/CosEventChannelAdminS.h>" << endl;
	outfile_h << "#include <CosEventChannelFactoryC.h>" << endl;
	outfile_h << "#include <EventFactory_config.h>" << endl;
	outfile_h << endl;
	outfile_h << "#include <MCCM_Service_Integration.h>" << endl;


	if( !only_one_idl_file_per_comp )
	{
	  CORBA::ComponentIR::EventPortDescriptionSeq& consumes_events
	    = the_comp_desc->consumes_events;
	
	  CORBA::ComponentIR::EventPortDescriptionSeq& publishes_events 
	    = the_comp_desc->publishes_events;
	
	  for( int ct_consumes = 0; ct_consumes < consumes_events.length(); ct_consumes++ )
	    {
	      string name_of_evttype = scoped_name( consumes_events[ ct_consumes ].event );
	      outfile_h << "#include <microccm_" << name_of_evttype << "C.h>" << endl;
	      outfile_h << "#include <microccm_" << name_of_evttype << "S.h>" << endl;
	      
	    }
	  
	  for( int ct_publishes = 0; ct_publishes < publishes_events.length(); ct_publishes++ )
	    {
	      string name_of_evttype = scoped_name( publishes_events[ ct_publishes ].event );
	      outfile_h << "#include <microccm_" << name_of_evttype << "C.h>" << endl;
	      outfile_h << "#include <microccm_" << name_of_evttype << "S.h>" << endl;
	    }
	}

      outfile_cpp << "#include <" << name_h_file << ">" << endl;

      outfile_cpp << "#include <string>" << endl
		  << "#include <debug_opt.h>" << endl
		  << "using namespace std;" << endl
		  << endl;

      for( list< Cpp_class_gene* >::iterator iter = list_port_servants.begin();
	   iter != list_port_servants.end();
	   iter++ )
	{
	  pair< string, string > generated_code = ( *iter )->generate_code();
	  outfile_h << generated_code.first << endl;
	  outfile_cpp << generated_code.second << endl;
	}

      // -- context class ---
      outfile_h << ctx_class.generate_code().first << endl;
      outfile_cpp << ctx_class.generate_code().second << endl;

      // -- source proxy classes ---
      for( list< Cpp_class_gene* >::iterator iter = list_publisher_proxies.begin();
	   iter != list_publisher_proxies.end();
	   iter++ )
	{
	  pair< string, string > generated_code = (*iter)->generate_code();
	  outfile_h << generated_code.first << endl;
	  outfile_cpp << generated_code.second << endl;
	}

      // -- sink proxy classes ---
      for( list< Cpp_class_gene* >::iterator iter = list_sink_proxies.begin();
	   iter != list_sink_proxies.end();
	   iter++ )
	{
	  pair< string, string > generated_code = (*iter)->generate_code();

	  outfile_h << "#ifndef " << (*iter)->get_name_of_class() << "_h" << endl;
	  outfile_h << "#define " << (*iter)->get_name_of_class() << "_h" << endl;
	  outfile_h << generated_code.first << endl;
	  outfile_h << "#endif"  << endl << endl;
	  outfile_cpp << "#ifndef " << (*iter)->get_name_of_class() << "_cpp" << endl;
	  outfile_cpp << "#define " << (*iter)->get_name_of_class() << "_cpp" << endl;
	  outfile_cpp << generated_code.second << endl;
	  outfile_cpp << "#endif"  << endl << endl;
	}

      // -- sink pushconsumer proxy classes ---
      for( list< Cpp_class_gene* >::iterator iter = list_sink_pushconsumer_proxies.begin();
	   iter != list_sink_pushconsumer_proxies.end();
	   iter++ )
	{
	  pair< string, string > generated_code = (*iter)->generate_code();
	  outfile_h << "#ifndef " << (*iter)->get_name_of_class() << "_h" << endl;
	  outfile_h << "#define " << (*iter)->get_name_of_class() << "_h" << endl; 
	  outfile_h << generated_code.first << endl;
	  outfile_h << "#endif"  << endl << endl;
	  
	  outfile_cpp << "#ifndef " << (*iter)->get_name_of_class() << "_cpp" << endl;
	  outfile_cpp << "#define " << (*iter)->get_name_of_class() << "_cpp" << endl;
	  outfile_cpp << generated_code.second << endl;
	  outfile_cpp << "#endif"  << endl << endl;
	}

            
      // -- component servant class ---
      pair< string, string > generated_code2 = class_gene_result.generate_code();
      outfile_h << generated_code2.first << endl;
      outfile_cpp << generated_code2.second << endl;


      //---------------------------------------------------------------------------------
      // ************************     generation of Home     ***************************
      //---------------------------------------------------------------------------------
      
      string name_of_home( argv[ 2 ] );
      
      string name_generated_class = string( "" ) + "_microccm_" + name_of_home + "_impl";
      
      // HomeConfiguation class
      //
      Cpp_class_gene home_gene_class( name_generated_class );
      home_gene_class.add_inherited_class( string( "POA_" ) + name_of_home,
					     "public" );

      home_gene_class.add_inherited_class( "Components::To_set_home_executor", "public" );
      home_gene_class.add_inherited_class( "microCCM::To_set_poa_orb", "public" );
      home_gene_class.add_required_include( "<microCCM_set_orb_and_poa.h>" );

      // Members of the class
      //      home_gene_class.add_member("CCM_" + name_of_home +"*", "the_component_home_");

      home_gene_class.add_member( "Correspondance_table< " + name_of_generated_class + "* >", "impl_for_components_" );
      home_gene_class.add_member( "Components::HomeExecutorBase*", "the_component_home_" );
      //   home_gene_class.add_member("PortableServer::POA_ptr", "poa_ref_");
      //      home_gene_class.add_member("CORBA::ORB_ptr", "the_orb_"); 

      // Methods of the class

      // ------ insertion of "set_home_executor( Components::HomeExecutorBase* )" method ---

      {
	Cpp_method to_set_home_executor;
	{
	  to_set_home_executor.set_prototype( "void set_home_executor( Components::HomeExecutorBase* hex_b )" );
	  home_gene_class.add_method( to_set_home_executor );
	  string implem = get_mccm_debug_trace_warning("HOME_IMPLEM")
	    + "  the_component_home_ = hex_b; ";
	  home_gene_class.add_implem( "set_home_executor",
				      new Invariable_string_in_place_of( implem  ) );
	  
	}

      }

      // --------------- insertion of create() method  -------------------
      {
	Cpp_method home_create_method;	
	{
	  home_create_method.set_name_of_method( "create" );
	  home_create_method.set_return_type( name_of_component + "_ptr" );
	  home_create_method.add_exception_thrown("CORBA::SystemException");
	  home_create_method.add_exception_thrown("Components::CreateFailure");
	  //	  constr_method.add_parameter( string( "CCM_" ) + current_intf_name + "_ptr",
	  //		       "delegated_to" );
	}
	
	string impl_home_create_method = 
	  get_mccm_debug_trace_warning("HOME_IMPLEM") +
	  "   _microccm_" + name_of_component + "_impl* to_be_returned_0 \n"
	  "           = new _microccm_" + name_of_component + "_impl(the_orb_, poa_ref_, this->_this() );\n"
	  "\n"
	  
	  "   // add event factories\n ";
	
	CORBA::ComponentIR::EventPortDescriptionSeq& consumes_events
	  = the_comp_desc->consumes_events;

	for(int ct_events = 0; ct_events < consumes_events.length(); ct_events++)
	  {
	    CORBA::Contained_ptr consumes_def_as_contained
	      = the_ref_repo->lookup_id( consumes_events[ ct_events ].id );
	    
	    CORBA::ComponentIR::EventPortDef_var event_p_def
	      = CORBA::ComponentIR::EventPortDef::_narrow( consumes_def_as_contained );
	    
	    string name_of_event = string( event_p_def->event()->name() );
	    string name_of_event_factory = "vt_" + name_of_event + "_factory";
	    
	    impl_home_create_method += 
	      "   #ifndef " + name_of_event_factory + "_cpp\n"
	      "   #define " + name_of_event_factory + "_cpp\n"
	      "   " + name_of_event + "_init * " +  name_of_event_factory + "= 0;\n"
	      "   " + name_of_event_factory + "= new " + name_of_event + "_init;\n"
	      "   the_orb_->register_value_factory (" + name_of_event_factory + "->tao_repository_id(), " + name_of_event_factory + ");\n" 
	      "   " + name_of_event_factory + "->_remove_ref();\n"
	      "#endif\n\n";
	  }
	
	impl_home_create_method += 
	  "CCM_" + name_of_home +"* the_comp_home = dynamic_cast< CCM_" + name_of_home +"* >( the_component_home_ );\n"
	  "   Components::EnterpriseComponent_ptr created = the_comp_home->create();\n"
	  "   Components::SessionComponent_ptr aux = dynamic_cast< Components::SessionComponent* >( created );\n"
	  "   // Monolithic executor creation\n"
	  "   if( aux ) // as in specification, pass ref to SessionContext when SessionComponent is implemented.\n"
	  "   {\n"
	  "     aux->set_session_context( to_be_returned_0->ctx_component_ );\n"
	  "   }\n"
	  "\n"
	  "   to_be_returned_0->the_comp_instance_  = dynamic_cast< CCM_" + name_of_component + "* >( created );\n"
	  "\n"
	  "   to_be_returned_0->create_and_activate_objs();\n"
	  "\n"
	  " // -- check here for non-functional services passed as configurations\n"
	  "\n"
	  "Components::ConfigValues component_configs;\n"
	  "\n"
	  "int counter = 0;\n"
	  "\n"
	  "for( int ct = 0; ct < the_cfg_values_.length(); ct++ )\n"
	  "{\n"
	  "  if( strncmp( the_cfg_values_[ ct ].name.in(), \"__\" , 2) != 0 )\n"
	  "    {\n"
	  "       counter++;\n"
	  "    }\n"
	  "}\n"
	  "\n"
	  "  component_configs.length( counter );\n"
	  "\n"
	  "  int comp_configs_counter = 0;\n"
	  "\n"
	  "  for( int ct = 0; ct < the_cfg_values_.length(); ct++ )\n"
	  "{\n"
	  "   if( strncmp( the_cfg_values_[ ct ].name.in(), \"__\" , 2) == 0 )\n"
	  "{\n"
	  "    // this refers to a technical service\n"
	  "   char* for_real_name = CORBA::string_dup( the_cfg_values_[ ct ].name.in() ); \n"
	  "\n"
	  "   MCCM_Service_Integration* service_integ\n"
	  "       = MCCM_Service_Integration_Factories::create( & (for_real_name[ 2 ]) ,\n"
	  "                     the_cfg_values_[ ct ].value );\n"
	  "\n"
	  "  CORBA::string_free( for_real_name ); \n"
	  "   MCCM_Service_Integration_Points* the_sip \n"
	  "      = to_be_returned_0->get_service_integration_points();\n"
	  "\n"
	  "  service_integ->install_at_service_access_points( the_sip );\n"
	  "}\n"
	  "else\n"
	  "{\n"
	  "  component_configs[ comp_configs_counter ].name \n"
	  "      = CORBA::string_dup( the_cfg_values_[ ct ].name.in() );\n"
	  "\n"
	  "// checker la memoire ci-dessous vis-a-vis de l'affectation du Any \n"
	  "// dans l'autre Any ... \n"
	  "\n"
	  "  component_configs[ comp_configs_counter ].value\n"
	  "   = the_cfg_values_[ ct ].value; \n"
	  "\n"
	  "   comp_configs_counter++;\n"
	  "}\n"
	  "}\n"
	  "\n"
	  "// --------------------------------------------------------------------------------------\n"
	  "\n"
          "   to_be_returned_0->_set_configuration( component_configs );\n"
	  "\n"
	  "   PortableServer::ObjectId_var the_objid  = poa_ref_->activate_object( to_be_returned_0 );"
	  "\n"
	  "   CORBA::Object_ptr ref_my_component = poa_ref_->id_to_reference( the_objid.in() );\n"
	  "\n"
	  "    char* objid_string = PortableServer::ObjectId_to_string(the_objid.in());\n"
	  "    impl_for_components_[objid_string] = to_be_returned_0;\n"
	  "\n"
	  "return " + name_of_component + "::_narrow( ref_my_component );\n"
	  "\n";

		  
	home_gene_class.add_implem(home_create_method,
				      new Invariable_string_in_place_of( impl_home_create_method ) );
	home_gene_class.add_method( home_create_method );
      }


      // ------------ insertion of create_component method --------------
      {
      Cpp_method home_create_component_method; 
	{
	  home_create_component_method.set_name_of_method( "create_component" );
	  home_create_component_method.set_return_type( "Components::CCMObject_ptr" );
	  home_create_component_method.add_exception_thrown("CORBA::SystemException");
	  home_create_component_method.add_exception_thrown("Components::CreateFailure");
	  
	  //	  constr_method.add_parameter( string( "CCM_" ) + current_intf_name + "_ptr",
	  //		       "delegated_to" );
	}
	
	string impl_home_create_component_method = "" +
	  get_mccm_debug_trace_warning("HOME_IMPLEM")
	  + "  return create(); \n";
	
	home_gene_class.add_implem(home_create_component_method,
				   new Invariable_string_in_place_of( impl_home_create_component_method ) );
	home_gene_class.add_method( home_create_component_method );
      }
      
      

      // ------------ insertion of remove_component method --------------
      {
	Cpp_method home_remove_component_method; 
	
	{
	  home_remove_component_method.set_name_of_method( "remove_component" );
	  home_remove_component_method.set_return_type( "void" );
	  home_remove_component_method.add_parameter( "Components::CCMObject_ptr", "obj" );
	  home_remove_component_method.add_exception_thrown("Components::RemoveFailure");
	}
	string impl_home_remove_component_method = 
	  get_mccm_debug_trace_warning("HOME_IMPLEM") +
	  "  PortableServer::ObjectId_var oid = poa_ref_->reference_to_id(obj);\n"
	  "  char* oid_string = PortableServer::ObjectId_to_string(oid.in());\n"
	  "  poa_ref_->deactivate_object(oid);\n"
	  "  " + name_of_generated_class + "*" + " comp = impl_for_components_[oid_string];\n"
	  "  impl_for_components_.erase(oid_string);\n"
	  "  delete comp;\n";
      
	home_gene_class.add_implem(home_remove_component_method,
				   new Invariable_string_in_place_of( impl_home_remove_component_method ) );
	home_gene_class.add_method( home_remove_component_method );
	
      }
      

      // ------------ insertion of set_configurator_component method --------------
      {
	Cpp_method home_set_configurator_method; 
	
	{
	  home_set_configurator_method.set_name_of_method( "set_configurator" );
	  home_set_configurator_method.set_return_type( "void" );
	  home_set_configurator_method.add_parameter( "Components::Configurator_ptr", "configurator_ref");
	home_set_configurator_method.add_exception_thrown("CORBA::SystemException");
	}

	string impl_home_set_configurator_method = " throw CORBA::NO_IMPLEMENT();\n";
	home_gene_class.add_implem(home_set_configurator_method,
				   new Invariable_string_in_place_of(impl_home_set_configurator_method));
	home_gene_class.add_method( home_set_configurator_method );
      }


      // ------------ insertion of set_configuration_values method --------------

      home_gene_class.add_member( "Components::ConfigValues", "the_cfg_values_" );

      {
	Cpp_method home_set_configuration_values_method; 
	
	{
	  home_set_configuration_values_method.set_name_of_method( "set_configuration_values" );
	  home_set_configuration_values_method.set_return_type( "void" );
	  home_set_configuration_values_method.add_parameter( "const Components::ConfigValues&", "cfgValues");
	  home_set_configuration_values_method.add_exception_thrown( "CORBA::SystemException" );
	}
      
	string impl_home_set_configuration_values_method = "" +
	  get_mccm_debug_trace_warning("HOME_IMPLEM") +
	  " the_cfg_values_ = cfgValues;\n";

	home_gene_class.add_implem(home_set_configuration_values_method,
				   new Invariable_string_in_place_of(impl_home_set_configuration_values_method));
	home_gene_class.add_method( home_set_configuration_values_method );
      }

      
      // ------------ insertion of complete_component_configuration_values method --------------
      {
	Cpp_method home_complete_component_configuration_method; 
	
	{
	  home_complete_component_configuration_method.set_name_of_method( "complete_component_configuration" );
	  home_complete_component_configuration_method.set_return_type( "void" );
	  home_complete_component_configuration_method.add_parameter( "CORBA::Boolean", "boolean");
	  home_complete_component_configuration_method.add_exception_thrown("CORBA::SystemException");
      }
	
	string impl_home_complete_component_configuration_method = " throw CORBA::NO_IMPLEMENT();\n";
	home_gene_class.add_implem(home_complete_component_configuration_method,
				   new Invariable_string_in_place_of(impl_home_complete_component_configuration_method));
	home_gene_class.add_method( home_complete_component_configuration_method );
      }


      // ------------ insertion of disable_home_configuration method --------------
      {
	Cpp_method disable_home_configuration_method; 
	
	{
	  disable_home_configuration_method.set_name_of_method( "disable_home_configuration" );
	  disable_home_configuration_method.set_return_type( "void" );
	  disable_home_configuration_method.add_exception_thrown("CORBA::SystemException");
	}
      
	string impl_disable_home_configuration_method = " throw CORBA::NO_IMPLEMENT();\n";
	home_gene_class.add_implem(disable_home_configuration_method,
				   new Invariable_string_in_place_of(impl_disable_home_configuration_method));
	home_gene_class.add_method( disable_home_configuration_method );
      }

      outfile_h << home_gene_class.generate_code().first << endl;
      outfile_cpp << home_gene_class.generate_code().second << endl;

      outfile_cpp << " extern \"C\"" << endl
		  << "{" << endl
		  << endl
		  << "PortableServer::ServantBase* create_servant_for_home()" << endl
		  << "{" << endl
		  << "return new " << name_generated_class << ";" << endl
		  << "}" << endl
		  << endl
		  << "void delete_servant_for_home( PortableServer::ServantBase* svnt )" << endl
		  << "{" << endl
		  << name_of_generated_class << "* ptr = dynamic_cast< " << name_of_generated_class << "* >( svnt );" << endl
		  << "delete ptr;" << endl
		  << "}"
		  << endl
		  << "};" << endl;

      cerr << "home corba generation OK" << endl;
      
      outfile_h << "#endif // " << base_name << "_h" << endl;
      outfile_h.close();
      outfile_cpp.close();
    }

  return 0;
}
