#include <Stubs_IR3.h>

#include <unistd.h>

#include <iostream>
#include <fstream>
#include <string>
#include <typeinfo>
using namespace std;

#include <IFR3_client_errors.h>
#include <dump_intf_from_ir.h>
#include <compute_idl_item_dependencies.h>
#include <repid_utils.h>
#include <typecode_utils.h>

#include <obtain_repo_ref.h>

#include <read_configuration_file.h>

void usage( const char* name_of_program )
{
  cerr << "Incorrect usage of " << name_of_program << endl;
  cerr << name_of_program << " Component_item Home_item" << endl;
  exit( ERROR_USAGE );
}

#include <list>
using namespace std;

class fstream_manager
{
public:

  enum fstream_manager_policy { same_fstream, different_fstreams };

  fstream_manager() : unique_stream_( NULL ) 
  {
  }

  ~fstream_manager()
  {
    if( ( current_policy_ == same_fstream ) && ( unique_stream_ ) )
      {
	delete unique_stream_;
      }
    
    for( list< fstream* >::iterator iter = created_streams_.begin();
	 iter != created_streams_.end();
	 iter++ )
      {
	delete *iter;
      }
  }

  void set_policy( fstream_manager_policy the_policy ) 
  {
    current_policy_ = the_policy;
  }

  fstream& do_open( string name_of_file )
  {
    if( current_policy_ == same_fstream )
      {
	if( ! unique_stream_ )
	  {
	    unique_stream_ = new fstream( name_of_file.c_str(), ios::out /* ios_base::out */ );
	  }
	
	return *unique_stream_;
      }
    else
      {
	fstream* to_be_returned = new fstream( name_of_file.c_str(), ios::out /* ios_base::out */ );
	return *to_be_returned;
      }
  }

  void do_close()
  {
    if (unique_stream_) unique_stream_->close();

    for( list< fstream* >::iterator iter = created_streams_.begin();
	 iter != created_streams_.end();
	 iter++ )
      {
	( *iter )->close();
      }
  }

private:

  list< fstream* > created_streams_;

  fstream* unique_stream_;
  fstream_manager_policy current_policy_;
};

// -------------------------- =================== -------------------------------------

int main( int argc, char** argv )
{
  if( argc != 3 )
    {
      usage( argv[ 0 ] );
    }

  CORBA::ORB_ptr the_orb = CORBA::ORB_init( argc, argv );

  CORBA::ComponentIR::Repository_ptr the_repo_ref = obtain_the_repo_ref( the_orb );

  string name_of_IDL_item_to_generate_for( argv[ 1 ] );
  string name_of_component = name_of_IDL_item_to_generate_for;

  CORBA::Contained_ptr the_item 
    = the_repo_ref->lookup( name_of_IDL_item_to_generate_for.c_str() );

  CORBA::ComponentIR::ComponentDef_ptr the_component_item
    = CORBA::ComponentIR::ComponentDef::_narrow( the_item );

  if( CORBA::is_nil( the_item ) )
    {
      cerr << name_of_IDL_item_to_generate_for << " no such IDL component item" << endl;
      exit( ERROR_NO_IDL_ITEM_OK );
    }

  char* microccm_dir_value = getenv( "microCCM_DIR" );

  if( !microccm_dir_value ) 
    {
      cerr << " You need to set the microCCM_DIR env variable" << endl;
      exit( 1 );
    }

  Configuration_file_reader conf_file_reader( string( microccm_dir_value ) + "/etc/codegen_config.txt" );

  string codegen_config = conf_file_reader.extract_configuration( "GENERATION_MODE" );

  fstream_manager the_fstream_manager;
  bool do_not_generate_microccm_includes;

  if( codegen_config == "generate_one_idl_file" )
    {
      do_not_generate_microccm_includes = true;
      the_fstream_manager.set_policy( fstream_manager::same_fstream );
    }
  else
    {
      do_not_generate_microccm_includes = false;
      the_fstream_manager.set_policy( fstream_manager::different_fstreams );
    }

  // ---

  string name_of_output_file 
    = string( "microccm_" ) + string( the_item->name() ) + ".idl";

  fstream& component_out_file = the_fstream_manager.do_open( name_of_output_file.c_str() );

  // ---

  CORBA::Contained::Description* the_description = the_item->describe();

  if( the_description->kind != CORBA::dk_Component )
    {
      cerr << "I expected " << name_of_IDL_item_to_generate_for 
	   << " to be a component .." << endl;
      exit( ERROR_ITEM_NOT_COMPONENT );
    }

  // for each port, find dependencies of involved type (interface or eventtype)
  // and regenerate IDL declaration files for each.

  CORBA::ComponentIR::ComponentDescription* the_comp_desc = NULL;

  if( the_description->value >>= the_comp_desc )
    {
      cerr << "name of component == " << the_comp_desc->name 
	   << " rep id == " << the_comp_desc->id << endl;

      cerr << "defined_in == " << the_comp_desc->defined_in 
	   << "base component == " << the_comp_desc->base_component << endl;

      CORBA::ComponentIR::ProvidesDescriptionSeq& what_is_provided
	= the_comp_desc->provided_interfaces;

      // ---

      CORBA::InterfaceDefSeq base_interfaces;
      base_interfaces.length( 1 );

      CORBA::Contained_ptr ccm_object_intf
	= the_repo_ref->lookup_id( "IDL:omg.org/Components/CCMObject:1.0" );

      if( CORBA::is_nil( ccm_object_intf ) )
	{
	  cerr << __PRETTY_FUNCTION__ << ": " << __LINE__ << "...probleme" << endl;
	  exit( 1 );
	}

      base_interfaces[ 0 ] 
	= CORBA::InterfaceDef::_narrow( ccm_object_intf );

      // ---- 

      string name_of_equivalent_intf 
	= string( "_microccm_" ) + string( the_comp_desc->name );

      string repid_of_equivalent_interface 
	= string( "IDL:" ) + name_of_equivalent_intf + string( ":1.0" );

      CORBA::InterfaceDef_ptr the_created_interface
	= the_repo_ref->create_interface( repid_of_equivalent_interface.c_str(),
					  name_of_equivalent_intf.c_str(),
					  "1.0",
					  base_interfaces
					  );

      cerr << "inter. created." << endl;

      the_created_interface->base_interfaces( base_interfaces );

      // --- and now redump attribute types, etc ... ---

      const CORBA::ExtAttrDescriptionSeq& the_attr_desc = the_comp_desc->attributes;

      for( int ct = 0; ct < the_attr_desc.length(); ct++ )
	{
	  try
	    {
	      CORBA::Contained_ptr attr_idl_type
		= the_repo_ref->lookup_id( the_attr_desc[ ct ].type->id() );

	      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

	      Dump_idl_item_as_idl_language::dump_with_dependencies( component_out_file,
								     attr_idl_type,
								     the_repo_ref );

	      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	    }
	  catch( ... )
	    {
	      cerr << "native attribute" << endl;
	    }

	}



      //
      // **************  Generating Provided interfaces ***************
      //
      cerr << "********   Generating Facets   **********" << endl;
      int nb_provided_interfaces = what_is_provided.length();
      cerr << "nb provided interfaces: " << nb_provided_interfaces << endl;

      for( int ct_provided_interface = 0; 
	   ct_provided_interface < nb_provided_interfaces; 
	   ct_provided_interface++ )
	{
	  string name_of_port 
	    = CORBA::string_dup( what_is_provided[ ct_provided_interface ].name ); 

	  string interface_type 
	    = CORBA::string_dup( what_is_provided[ ct_provided_interface ].interface_type );

	  CORBA::Contained_ptr the_related_interface
	    = the_repo_ref->lookup_id( interface_type.c_str() );

	  CORBA::IDLType_ptr the_provided_interface_as_idl_type
	    = CORBA::IDLType::_narrow( the_related_interface );

	  {
	    cerr << "Generating IDL code for interface " << the_related_interface->name() << endl;
	    string name_of_generated_file 
	      = string( "microccm_" ) + string( the_related_interface->name() ) + ".idl";
	    
	    fstream& outfile_for_intf = the_fstream_manager.do_open ( name_of_generated_file );
	    
	    Dump_idl_item_as_idl_language::dump_with_dependencies( outfile_for_intf, 
								   the_related_interface, 
								   the_repo_ref );
	    //	    outfile_for_intf.close();
	  }

	}



      //
      // **************  Generating Used interfaces ***************
      //
      cerr << "********   Generating Receptacles   **********" << endl;

      CORBA::ComponentIR::UsesDescriptionSeq& what_is_used = the_comp_desc->used_interfaces;
      int nb_used_interfaces = what_is_used.length();

      for( int ct_used_interface = 0; ct_used_interface < nb_used_interfaces; ct_used_interface++ )
	{
	  CORBA::ComponentIR::UsesDescription& the_uses_desc = what_is_used[ ct_used_interface ];

	  CORBA::Contained_ptr used_interface_as_contained 
	    = the_repo_ref->lookup_id( the_uses_desc.interface_type );

	  {
	    string name_of_output_file 
	      = string( "microccm_" ) + string( used_interface_as_contained->name() ) + ".idl";

	    fstream& idl_output = the_fstream_manager.do_open( name_of_output_file );

	    Dump_idl_item_as_idl_language::dump_with_dependencies( idl_output, 
								   used_interface_as_contained, 
								   the_repo_ref );
	  }
	}

      CORBA::ExtAttrDescriptionSeq& the_attrib_seq = the_comp_desc->attributes;
      int nb_attribs = the_attrib_seq.length();

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << " nb_attribs == " << nb_attribs <<  endl;

      for( int ct_attrib = 0; ct_attrib < nb_attribs; ct_attrib++ )
	{
	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

	  CORBA::ExtAttributeDescription& the_description = the_attrib_seq[ ct_attrib ];

	  CORBA::TypeCode_ptr the_typecode = the_description.type.in();

	  CORBA::IDLType_ptr as_idltype = typecode_to_idltype( the_typecode, the_repo_ref );

	  Repid current_repid( string( the_description.id.in() ) );

	  current_repid.add_prefix_to_beginning( "_microccm_" );
	  string new_id = current_repid.get();

	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	  cerr << " Creating attribute on equivalent interface " << the_description.name
	       << "," << new_id << endl;

	  the_created_interface->create_attribute( new_id.c_str(),
						   the_description.name,
						   the_description.version,
						   as_idltype,
						   the_description.mode );
	}


      //
      // **************  Generating component's Event Sources and Sinks ***************
      //
      cerr << "**************  Generating component's Event Sources and Sinks ***************" << endl;
      // iteration on components publishes / subscribe ports and generation of equivalent
      // IDL for eventtypes ...

      CORBA::ComponentIR::EventPortDescriptionSeq& consumes_events 
	= the_comp_desc->consumes_events;

      for( int ct_consumes = 0; ct_consumes < consumes_events.length(); ct_consumes++ )
	{
	  string name_of_evttype = scoped_name( consumes_events[ ct_consumes ].event );
	  string output_file_basename = string( "microccm_" ) + name_of_evttype;
	  string output_file_name = output_file_basename + ".idl";

	  fstream& outfile = the_fstream_manager.do_open( output_file_name );

	  outfile << "#ifndef " << output_file_basename << "_idl" << endl
		  << "#define " << output_file_basename << "_idl" << endl;

	  outfile << "#include <orbsvcs/orbsvcs/CosEventComm.idl>" << endl;
	  outfile << "#include <Lw_Components.idl>" << endl << endl;

	  CORBA::Contained_ptr evtdef_as_contained 
	    = the_repo_ref->lookup_id( consumes_events[ ct_consumes ].id );

	  CORBA::ComponentIR::ConsumesDef_ptr the_cons_def
	    = CORBA::ComponentIR::ConsumesDef::_narrow( evtdef_as_contained );

	  Dump_idl_item_as_idl_language::dump_idl_eventtype_as_idl_language( outfile,
									     the_cons_def->event(),
									     the_repo_ref );

	  outfile << "#endif // " << output_file_name << "_idl" << endl;
	  //	  outfile.close();
	}

      // ------------------ 

      CORBA::ComponentIR::EventPortDescriptionSeq& publishes_events 
	= the_comp_desc->publishes_events;

      for( int ct_publishes = 0; ct_publishes < publishes_events.length(); ct_publishes++ )
	{
	  string name_of_evttype = scoped_name( publishes_events[ ct_publishes ].event );
	  string output_file_basename = string( "microccm_" ) + name_of_evttype;
	  string output_file_name = output_file_basename + ".idl";

	  fstream& outfile = the_fstream_manager.do_open( output_file_name );
	  outfile << "#ifndef " << output_file_basename << "_idl" << endl
		  << "#define " << output_file_basename << "_idl" << endl;

	  outfile << "#include <orbsvcs/orbsvcs/CosEventComm.idl>" << endl;
	  outfile << "#include <Lw_Components.idl>" << endl << endl;

	  CORBA::Contained_ptr evtdef_as_contained 
	    = the_repo_ref->lookup_id( publishes_events[ ct_publishes ].id );

	  CORBA::ComponentIR::PublishesDef_ptr the_pub_def
	    = CORBA::ComponentIR::PublishesDef::_narrow( evtdef_as_contained );

	  Dump_idl_item_as_idl_language::dump_idl_eventtype_as_idl_language( outfile,
									     the_pub_def->event(),
									     the_repo_ref );

	  outfile << "#endif // " << output_file_name << "_idl" << endl;
	  //	  outfile.close();
	}

      // -----------------------------
      
      //       string name_of_output_file 
      // 	= string( "microccm_" ) + string( the_item->name() ) + ".idl";
      //       fstream component_out_file( name_of_output_file.c_str(), ios::out );

      component_out_file << "#include <orbsvcs/orbsvcs/CosEventComm.idl>" << endl;
      component_out_file << "#include <Lw_Components.idl>" << endl << endl;


      //
      // **************  Generating Component ***************
      //
      cerr << "********    Generating Component  **********" << endl;
      if( ! do_not_generate_microccm_includes )
	{

      for( int ct_consumes = 0; ct_consumes < consumes_events.length(); ct_consumes++ )
	{
	  string name_of_evttype = scoped_name( consumes_events[ ct_consumes ].event );
	  component_out_file << "#include <microccm_" << name_of_evttype << ".idl>" << endl;
	}

      for( int ct_publishes = 0; ct_publishes < publishes_events.length(); ct_publishes++ )
	{
	  string name_of_evttype = scoped_name( publishes_events[ ct_publishes ].event );
	  component_out_file << "#include <microccm_" << name_of_evttype << ".idl>" << endl;
	}

      for( int ct_provided_interface = 0; 
	   ct_provided_interface < nb_provided_interfaces; 
	   ct_provided_interface++ )
	{
	  CORBA::Contained_ptr as_contained 
	    = the_repo_ref->lookup_id( what_is_provided[ ct_provided_interface ].interface_type );

	  string name_current_interface( as_contained->name() );

	  component_out_file << "#include <" << "microccm_" << name_current_interface << ".idl>" << endl;
	}

      for( int ct_used_interface = 0;
	   ct_used_interface < nb_used_interfaces;
	   ct_used_interface++ )
	{
	  CORBA::Contained_ptr as_contained
	    = the_repo_ref->lookup_id( what_is_used[ ct_used_interface ].interface_type );

	  string name_current_interface( as_contained->name() );

	  component_out_file << "#include <" << "microccm_" << name_current_interface << ".idl>" << endl;
	}

	}

      component_out_file << endl;

       Dump_idl_item_as_idl_language::dump_idl_interface_as_idl_language( component_out_file, 
   									  the_created_interface );
      

      //
      // **************  Generating Home ***************
      //
      cerr << " **************  Generating Home ***************" << endl;
      string name_of_home_searched( argv[2] );

      CORBA::Contained_ptr ref_home_as_contained 
	= the_repo_ref->lookup( name_of_home_searched.c_str() );

      if( CORBA::is_nil( ref_home_as_contained ) )
	{
	  cerr << "Error : " << name_of_home_searched 
	       << " is not present in Interface Repository" << endl;
	  exit( ERROR_NO_IDL_ITEM_OK );
	}

      CORBA::ComponentIR::HomeDef_ptr ref_home
	= CORBA::ComponentIR::HomeDef::_narrow( ref_home_as_contained );

      Dump_idl_item_as_idl_language::dump_idl_home_as_idl_language( component_out_file, ref_home );
      return NO_ERROR;
    }

  the_fstream_manager.do_close();

  return false;

}

