#include <IDL_to_cpp_mapping.h>
#include <Invariable_string_in_place_of.h>

#include <iostream>

#include <IFR3_Singleton.h>

#include <repid_utils.h>

using namespace std;

Cpp_class_gene IDL_to_Cpp_mapping( CORBA::InterfaceDef_ptr the_intf_def,
				   string name_of_class_to_generate,
				   bool abstract_class_or_not )
{
  string interface_type_name;

  if( name_of_class_to_generate == "don\'t know" ) 
    interface_type_name = the_intf_def->name();
  else
    interface_type_name = name_of_class_to_generate;

  Cpp_class_gene to_be_returned( interface_type_name );

  CORBA::InterfaceDef::FullInterfaceDescription* the_intf_descr 
    = the_intf_def->describe_interface();

  // ---- iteration on methods  ------------------------------------------

  CORBA::OpDescriptionSeq& all_methods  = the_intf_descr->operations;

  const int nb_methods = all_methods.length();

  for( int ct_method = 0; ct_method < nb_methods; ct_method++ )
    {
      //      cerr << "methods ... from FullInterfaceDescription == " 
      //           << all_methods[ ct_method ].name << " defined in " << all_methods[ ct_method ].defined_in 
      //           << endl;
    }

  for( int ct_method = 0; ct_method < nb_methods; ct_method++ )
    {
      CORBA::OperationDescription& op_desc = all_methods[ ct_method ];
      Cpp_method the_method; 

      if ( abstract_class_or_not )  the_method.set_virtual_decl( is_pure_virtual );

      //      cerr   << " IDL_to_Cpp_mapping currently treating " << op_desc.name << " method"
      //	     << endl;

      the_method.set_name_of_method( string( op_desc.name ) );

      // result type
      {
	CORBA::TypeCode_ptr the_result_type = op_desc.result;
	string name_returned_type = IDL_to_cpp_arg_mapping( the_result_type, 
							    arg_passing_return );	
	the_method.set_return_type( name_returned_type );
      }

      // parameters
      int nb_params = op_desc.parameters.length();

      for( int ct_param = 0; ct_param < nb_params; ct_param++ )
       	{

	  CORBA::ParameterDescription& the_param_description 
	    = op_desc.parameters[ ct_param ];

	  CORBA::TypeCode_ptr the_arg_type = the_param_description.type;

	  arg_passing_mode the_arg_passing_mode;

	  {
	    if( the_param_description.mode == CORBA::PARAM_IN ) the_arg_passing_mode = arg_passing_in;
	    if( the_param_description.mode == CORBA::PARAM_OUT ) the_arg_passing_mode = arg_passing_out;
	    if( the_param_description.mode == CORBA::PARAM_INOUT ) the_arg_passing_mode = arg_passing_inout;
	  }

	  string type_of_param = IDL_to_cpp_arg_mapping( the_arg_type, the_arg_passing_mode );
	  string name_of_param( the_param_description.name );
       	  the_method.add_parameter( type_of_param, name_of_param );
 	}

      // exceptions
      //       {

      //       }

      // because TAO problem :-(
      if( ! abstract_class_or_not )
	{
	  the_method.add_exception_thrown( "CORBA::SystemException "); 
	}

      to_be_returned.add_method( the_method );
    }

  // ---- iteration on attributes --------------------------------------

  {
    CORBA::AttrDescriptionSeq& the_attr_seq = the_intf_descr->attributes;

    for( int ct = 0; ct < the_attr_seq.length(); ct++ )
      {
	string name_of_attribute( the_attr_seq[ ct ].name );

	{
	  string for_result_get_method = IDL_to_cpp_arg_mapping( the_attr_seq[ ct ].type.in() ,
								 arg_passing_return );

	  Cpp_method get_method; get_method.set_name_of_method( name_of_attribute );
	  get_method.set_return_type( for_result_get_method );	    

	  if( abstract_class_or_not ) get_method.set_virtual_decl( is_pure_virtual );
	  if( !abstract_class_or_not ) get_method.add_exception_thrown( "CORBA::SystemException" );

	  to_be_returned.add_method( get_method );
	}

	{
	  if( the_attr_seq[ ct ].mode == CORBA::ATTR_NORMAL )
	    {
	      string for_param_set_method = IDL_to_cpp_arg_mapping( the_attr_seq[ ct ].type.in() ,
								     arg_passing_in );
	      
	      Cpp_method set_method; set_method.set_name_of_method( name_of_attribute );
	      set_method.set_return_type( "void" );

	      set_method.add_parameter( for_param_set_method, "attribute" );

	      if( abstract_class_or_not ) set_method.set_virtual_decl( is_pure_virtual );
	      if( !abstract_class_or_not ) set_method.add_exception_thrown( "CORBA::SystemException" );

	      to_be_returned.add_method( set_method );
	    }
	}


      }
  
  }
  
  // -------------------------------------------------------------------

  return to_be_returned;
}

// -----------------------------------------------------------------------------------

#include <IDL_to_cpp_mapping_aux.h>

/**. Constructor for Cpp_mapping_basic_arg_conv class. This class provide methods to 
 * convert basic IDL arguments (short, long, longlong, etc.) to CORBA C++ mapping.
 * 
 * @param name A string representing the name of the argument.
 */
Cpp_mapping_basic_arg_conv::Cpp_mapping_basic_arg_conv( string corba_corresp_name )
  : corba_corresp_name_( corba_corresp_name )
{
}


/**. This method converts a basic argument (short, long, longlong, etc.) to 
 * its CORBA C++ mapping depending on its mode: return, in, inout, out.
 *
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_basic_arg_conv::convert( string name, arg_passing_mode apm )
{
  if( apm == arg_passing_return ) return corba_corresp_name_;
  if( apm == arg_passing_in ) return corba_corresp_name_;
  if( apm == arg_passing_inout ) return corba_corresp_name_ + "&";
  if( apm == arg_passing_inout ) return corba_corresp_name_ + "&";
}

/**. This method converts an enum argument to its CORBA C++ 
 * mapping depending on its mode: return, in, inout, out.
 *
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_enum_conv::convert( string name, arg_passing_mode apm )
{
  if( apm == arg_passing_return ) return name;
  if( apm == arg_passing_in ) return name;
  if( apm == arg_passing_inout ) return name + "&";
  if( apm == arg_passing_out ) return name + "&";
}



/**. This method converts a CORBA object reference argument to its CORBA C++ 
 * mapping depending on its mode: return, in, inout, out.
 *
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_objref_conv::convert( string name, arg_passing_mode apm )
{
  if( apm == arg_passing_return ) return name + "_ptr";
  if( apm == arg_passing_in ) return name + "_ptr";
  if( apm == arg_passing_inout ) return name + "_ptr&";
  if( apm == arg_passing_out ) return name + "_ptr&";
}


/**. This method converts a struct argument to its CORBA C++ 
 * mapping depending on its mode: return, in, inout, out.
 *
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_struct_conv::convert( string name, arg_passing_mode apm )
{
  //  cerr << "argument is of type struct." << endl;

  bool is_variable = is_variable_size( name.c_str() );

  if( !is_variable )
    {
      if( apm == arg_passing_return ) return name;
      if( apm == arg_passing_in ) return string( "const " ) + name + "&";
      if( apm == arg_passing_inout ) return  name + "&";
      if( apm == arg_passing_out ) return name + "&";
    }
  else
    {
      if( apm == arg_passing_return ) return name + "*";
      if( apm == arg_passing_in ) return string( "const " ) + name + "&";
      if( apm == arg_passing_inout ) return  name + "&";
      if( apm == arg_passing_out ) return name + "*&";      
    }
}



CORBA::TypeCode_var extract_TypeCode(CORBA::TypeCode_ptr typecode)
{
  if(typecode)
    {
      CORBA::TypeCode_var tc_result = typecode->content_type();
      
      if(tc_result->kind() == CORBA::tk_alias)
	{
	  extract_TypeCode(tc_result);
	}
      else
	{
	  return tc_result;
	}
    }
}

bool Cpp_mapping_struct_conv::is_variable_size( const char* name_of_struct )
{
  //  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
  bool unbounded = false;
  
  CORBA::ComponentIR::Repository_ptr the_ref_repo 
    = IFR3_Singleton::get_instance();
  
  CORBA::Contained_ptr struct_contained
    = the_ref_repo->lookup( name_of_struct);

  if(  CORBA::is_nil( struct_contained ) )
    {
      cerr << "IDL3_to_CORBA : don't manage to find component " 
	   << name_of_struct << endl;
      //      exit( )
    }

  CORBA::StructDef_ptr struct_def = CORBA::StructDef::_narrow(struct_contained);

  CORBA::StructMemberSeq* struct_members = struct_def->members();

  int nb_struct_members = struct_members->length();

  for(int ct_struct_members=0; ct_struct_members < nb_struct_members; ct_struct_members++)
    {
      CORBA::TypeCode_var typecode_member =  (*struct_members)[ct_struct_members].type;

      if(typecode_member->kind() == CORBA::tk_alias)
	{
	  typecode_member  = extract_TypeCode(typecode_member);
	}
      

      switch (typecode_member->kind()) 
	{

	case CORBA::tk_any: 
	  return unbounded = true;
	  break;
	  
	case CORBA::tk_string : 
	  if(!typecode_member->length()) return unbounded = true;
	  break;
	  
	case CORBA::tk_wstring:
	  if(!typecode_member->length()) return unbounded = true;
	  break;
	  
	case CORBA::tk_sequence:
	  if(!typecode_member->length()) return unbounded = true;
	  break;
	  
	case CORBA::tk_objref:
	  return unbounded = true;
	  break;

	case CORBA::tk_struct:
	  is_variable_size( (*struct_members)[ct_struct_members].name);
	  break;
	}
    }
  cerr << "this is either a simple type, or not defined !!" << endl;
  return unbounded;
}


Cpp_mapping_wstring_string_conv::Cpp_mapping_wstring_string_conv( string wstring_or_string )
{
  if( wstring_or_string == "string" )
    {
      corresponding_base_name_ = "char";
    }
  else
    {
      corresponding_base_name_ = "wchar";
    }
}


/**. This method converts a wstring argument to its CORBA C++ 
 * mapping depending on its mode: return, in, inout, out.
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_wstring_string_conv::convert( string name, arg_passing_mode apm )
{
  if( apm == arg_passing_return ) 
    return corresponding_base_name_ + "*";

  if( apm == arg_passing_in ) 
    return string( "const " ) + corresponding_base_name_ + "*";

  if( apm == arg_passing_inout )
    return corresponding_base_name_ + "*&";

  if( apm == arg_passing_out )
    return corresponding_base_name_ + "*&";
}

/**. This method converts a sequence argument to its CORBA C++ 
 * mapping depending on its mode: return, in, inout, out.
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_sequence_conv::convert( string name, arg_passing_mode apm )
{
  if( apm == arg_passing_return ) return name + "*";
  if( apm == arg_passing_in ) return string( "const " ) + name + "&";
  if( apm == arg_passing_inout ) return name + "&";
  if( apm == arg_passing_out ) return name + "*&";
}

/**. This method converts an any argument to its CORBA C++ 
 * mapping depending on its mode: return, in, inout, out.
 * @see Cpp_mapping_arg_conv::convert
 * @param name A string representing the name of the basic argument to convert.
 * @param apm  An enum arg_passing_mode taking among values: arg_passing_return, arg_passing_in, arg_passing_inout, arg_passing_out.
 * @return A string containing the concatenation of characters representing the c++ mapping of the type.
 */
string Cpp_mapping_any_conv::convert( string name, arg_passing_mode apm )
{
  if( apm == arg_passing_return ) return "CORBA::Any*";
  if( apm == arg_passing_in ) return "const CORBA::Any&";
  if( apm == arg_passing_inout ) return "CORBA::Any&";
  if( apm == arg_passing_out ) return "CORBA::Any* &";
}

// -----------------------------------------------------------------------------------

string IDL_to_cpp_arg_mapping( CORBA::TypeCode_ptr type, arg_passing_mode the_mode )
{
  map< CORBA::TCKind, Cpp_mapping_arg_conversion* > primitive_types_mapping;
  map< CORBA::TCKind, Cpp_mapping_arg_conversion* >& m = primitive_types_mapping;

  // -------------------------------------------------------------

  m[ CORBA::tk_void ] = new Cpp_mapping_basic_arg_conv( "void" );

  // -------------------------------------------------------------

  m[ CORBA::tk_short ] = new Cpp_mapping_basic_arg_conv( "CORBA::Short" );
  m[ CORBA::tk_long ] = new Cpp_mapping_basic_arg_conv( "CORBA::Long" );
  m[ CORBA::tk_longlong ] = new Cpp_mapping_basic_arg_conv( "CORBA::LongLong" );
  m[ CORBA::tk_ushort ] = new Cpp_mapping_basic_arg_conv( "CORBA::UShort" );
  m[ CORBA::tk_ulong ] = new Cpp_mapping_basic_arg_conv( "CORBA::ULong" );
  m[ CORBA::tk_ulonglong ] = new Cpp_mapping_basic_arg_conv( "CORBA::ULongLong" );
  m[ CORBA::tk_float ] = new Cpp_mapping_basic_arg_conv( "CORBA::Float" );
  m[ CORBA::tk_double ] = new Cpp_mapping_basic_arg_conv( "CORBA::Double" );
  m[ CORBA::tk_longdouble ] = new Cpp_mapping_basic_arg_conv( "CORBA::LongDouble" );
  m[ CORBA::tk_boolean ] = new Cpp_mapping_basic_arg_conv( "CORBA::Boolean" );
  m[ CORBA::tk_char ] = new Cpp_mapping_basic_arg_conv( "CORBA::Char" );
  m[ CORBA::tk_wchar ] = new Cpp_mapping_basic_arg_conv( "CORBA::WChar" );
  m[ CORBA::tk_octet ] = new Cpp_mapping_basic_arg_conv( "CORBA::Octet" );

  m[ CORBA::tk_enum ] = new Cpp_mapping_enum_conv;
  m[ CORBA::tk_objref ] = new Cpp_mapping_objref_conv;
  m[ CORBA::tk_struct ] = new Cpp_mapping_struct_conv;

  // --- fixed union, variable union .. see later ... xxx
  // tk_union 

  m[ CORBA::tk_string ] = new Cpp_mapping_wstring_string_conv( "string" );
  m[ CORBA::tk_wstring ] = new Cpp_mapping_wstring_string_conv( "wstring" );
  m[ CORBA::tk_sequence ] = new Cpp_mapping_sequence_conv;

  // --- arrays .. see later ... xxx 
  //    tk_array

  m[ CORBA::tk_any ] = new Cpp_mapping_any_conv;

  // --- fixed .. see later xxx 
  //    tk_fixed
  
  // --- valuetype .. see later xxx
  //   tk_value

  CORBA::TCKind the_tckind = type->kind();

  // xxx le cas des alias serait mieux comme traite a l'exterieur
  // xxx de cette fonction

  if( CORBA::tk_alias == the_tckind )
    {
      //      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      CORBA::ComponentIR::Repository_ptr the_ref_repo 
	= IFR3_Singleton::get_instance();

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      cerr << type->id() << endl;

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      if( CORBA::is_nil( the_ref_repo ) )
	{
	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	  exit( 1 );
	}

      CORBA::Contained_ptr ref_alias_def_as_contained
	= the_ref_repo->lookup_id( type->id() );

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      if( CORBA::is_nil( ref_alias_def_as_contained ) )
	{
	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	  exit( 1 );
	}

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      CORBA::AliasDef_ptr ref_alias
	= CORBA::AliasDef::_narrow( ref_alias_def_as_contained );

     // find aliased type ...
      CORBA::IDLType_ptr aliased_type 
	= ref_alias->original_type_def();

      //      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      CORBA::SequenceDef_var the_aliased_type_as_seq 
	= CORBA::SequenceDef::_narrow( aliased_type );

      if( ! CORBA::is_nil( the_aliased_type_as_seq ) )
	{
	  return m[ CORBA::tk_sequence ]->convert( ref_alias->name(), the_mode );
	};

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
      
      // xxx attention a la limitation : probleme si typedef de typedefs ??
      return IDL_to_cpp_arg_mapping( aliased_type->type(), the_mode );
    }

  if( is_basic_type( the_tckind ) ) 
    {
      //      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

      if( m.find( the_tckind ) == m.end() )
	{
	  //	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	}

      return m[ the_tckind ]->convert( "dummy_name", the_mode );
    }
  else
    {
      //      cerr << __PRETTY_FUNCTION__ << " type->name() == " << type->name() << endl;
      return m[ the_tckind ]->convert( scoped_name( type->id() ), // string( type->name() )  , 
				       the_mode );
    }

}

// ----------------------------------------------------------------------------------

Cpp_class_gene& add_duplicate_and_narrow_methods( Cpp_class_gene& the_class_gene )
{
  // _duplicate static method
  {
    Cpp_method duplicate_method;
    duplicate_method.set_name_of_method( "_duplicate" );
    duplicate_method.is_static();
    duplicate_method.set_return_type( the_class_gene.get_name_of_class() + "*" );
    duplicate_method.add_parameter( the_class_gene.get_name_of_class() + "*", "ref" );

    the_class_gene.add_method( duplicate_method );
    the_class_gene.add_implem( duplicate_method,
				   new Invariable_string_in_place_of( "\nreturn ref;\n" ) );
  }

  // _narrow static method
  {
    Cpp_method narrow_method;
    narrow_method.set_name_of_method( "_narrow" );
    narrow_method.is_static();
    narrow_method.set_return_type( the_class_gene.get_name_of_class() + "*" );
    narrow_method.add_parameter( "microCCM::Object_ptr", "ref" );

    the_class_gene.add_method( narrow_method );
    
    string implem_of_method 
      = "return dynamic_cast< " + the_class_gene.get_name_of_class() + "* > ( ref );\n";

    the_class_gene.add_implem( narrow_method, 
			     new Invariable_string_in_place_of( implem_of_method ) );
  }

  return the_class_gene;

}
