#include <Stubs_IR3.h>
#include <IDL_to_cpp_mapping.h>

#include <typecode_utils.h>

#include <iostream>
#include <list>
#include <memory>
using namespace std;

#include <Contained_wrapper.h>
#include <compute_idl_item_dependencies.h>
#include <tree.h>

/**. This method computes dependencies for a Contained object in an IR repository.
 *
 * @param tree_iter A subnode CORBA::Contained object on ywhich dependencies has to be computed.
 * @return None
 */

bool Compute_idl_item_dependencies::is_subnode_already_processed(CORBA::Contained_ptr item)
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
  cerr << "item repId to find: " <<  item->id() << endl;
  map< string , bool >::iterator iter = processed_subnode_map.find(string(item->id()));

  for( map< string, bool >::iterator iter = processed_subnode_map.begin();
       iter !=  processed_subnode_map.end();
       iter++)
    {
      cerr << "item found: " << iter->first << endl;
    }


  if( iter != processed_subnode_map.end() )
    {
      cerr << "subnode found !" << endl;
      return true;
    }
  return false;
}

/**. This method computes dependencies for a Contained object in an IR repository.
 *
 * @param repo_ref The IR repository where the Contained object is located.
 * @return None
 */
bool Compute_idl_item_dependencies::is_typecode_authorised(CORBA::TypeCode_ptr typecode)
{
  if(typecode)
    {
      if( (!is_primitive(typecode)) && 
	  (!is_valuetype(typecode)) )
	//	  (typecode->kind() != CORBA::tk_sequence) )
	{
	  return true;
	}
    }
  return false;
}


list< CORBA::Contained_ptr >
Compute_idl_item_dependencies::compute_interface_dependencies( CORBA::InterfaceDef_ptr the_intf_def, 
				CORBA::ComponentIR::Repository_ptr the_repo_ref )
{
  list< CORBA::Contained_ptr > the_dependencies_level_1;

  // base interfaces dependencies ...
  cerr << "***** base interfaces dependencies .." << endl;
  CORBA::InterfaceDefSeq* the_intf_def_seq = the_intf_def->base_interfaces();

  for( int ct = 0; ct < the_intf_def_seq->length(); ct++ )
    {
      CORBA::InterfaceDef_ptr current_intf = ( *the_intf_def_seq )[ ct ];
      //      cerr << "  " << ct << ": " << endl;
      the_dependencies_level_1.push_back( current_intf );
    }

  // operations
  cerr << "***** operation dependencies .." << endl;

  CORBA::InterfaceDef::FullInterfaceDescription_var the_full_desc 
    = the_intf_def->describe_interface();

  const CORBA::OpDescriptionSeq& all_operations = the_full_desc->operations;

  for( int ct = 0; ct < all_operations.length(); ct++ )
    {
      const CORBA::OperationDescription& op_desc = all_operations[ ct ];

      CORBA::TypeCode_var result_type = op_desc.result;
       
    //   if( result_type->kind() == CORBA::tk_alias)
// 	{
// 	  result_type =  extract_TypeCode(result_type);
// 	}
      

      if(result_type.in())
	{  
	  if( is_typecode_authorised( result_type.in()) )
	    {
	      string name_of_type = (string)result_type->name();

	      // recuperer IDL item in IR3
	      CORBA::Contained_ptr the_result_type_as_contained 
		= the_repo_ref->lookup_id( result_type->id() );

	      if(strcmp(result_type->name(), the_full_desc->name) != 0)
		{
		  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

		  if(! CORBA::is_nil(the_result_type_as_contained))
		    {
		      cerr << "  result type: " << ct << "-" << the_result_type_as_contained->name() << endl;
		      the_dependencies_level_1.push_back( the_result_type_as_contained );
		    }
		  else
		    {
		      cerr << "****Error in result type contained with repid: " 
			   << result_type->id() << "is NULL" << endl;
		    }
		}
	    }

	}

      //  mettre aussi ici les dpendances pour les arguments .. et pour les exceptions, etc ...

      const CORBA::ParDescriptionSeq& param_descriptions = op_desc.parameters;

      for( int ct_param = 0; ct_param < param_descriptions.length(); ct_param++ )
	{
	  const CORBA::ParameterDescription& param_desc = param_descriptions[ ct_param ];
	  CORBA::TypeCode_var the_type_code = param_desc.type;

	  // to be processed after, in the Contained_wrapper..
	  //  if( the_type_code->kind() == CORBA::tk_alias)
	  // 	    {
	  // 	      the_type_code =  extract_TypeCode(the_type_code);
	  // 	    }

	  //cerr << "name of param: " << the_type_code->name() << endl;
	  //      look for alias...
	  //

	  if( is_typecode_authorised( the_type_code.in() ))
	    {
	      cerr << "name of param: " << the_type_code->name() << endl;
	      CORBA::Contained_ptr the_result_type_as_contained
		= the_repo_ref->lookup_id( the_type_code->id() );

	      if((! CORBA::is_nil(the_result_type_as_contained)) && 
		 (the_result_type_as_contained != NULL))
		    {
		      cerr << "  arguments dependencies: " << ct_param << "-" << the_type_code->id() << endl ;
		      the_dependencies_level_1.push_back( the_result_type_as_contained );
		    }
		  else
		    {
		      cerr << "**** Error in params contained with repid: " <<  the_type_code->id() << " is NULL" << endl;
		    }
	    }
	  else
	    {
	      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	      cerr << the_type_code->kind() << endl;
	    }
	}


      // et ici les exceptions lancees par cette methode.
      const CORBA::ExcDescriptionSeq& exceptions_raised = op_desc.exceptions;
      for( int ct_except = 0; ct_except < exceptions_raised.length(); ct_except++ )
	{
	  const CORBA::ExceptionDescription& exc_desc = exceptions_raised[ ct_except ];
	  CORBA::Contained_ptr used_exception = the_repo_ref->lookup_id( exc_desc.id );

	  if(! CORBA::is_nil(used_exception))
	    {
	      cerr << " exceptions dependencies" << ct_except << "-" << used_exception->name() << endl;
	      the_dependencies_level_1.push_back( used_exception );
	    }
	  else
	    {
	      cerr << "**** Error in exceptions contained with repid: " << exc_desc.id << "is NULL" << endl;
	    }
	}

    } //  for( int ct = 0; ct < all_operations.length()...

  cerr << "***** attributes dependencies .." << endl;
  const CORBA::AttrDescriptionSeq& all_attributes = the_full_desc->attributes;

  // do it for the attributes types ...
  for( int ct_attribute = 0; ct_attribute < all_attributes.length(); ct_attribute++ )
    {
      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
      const CORBA::AttributeDescription& attrib_desc = all_attributes[ ct_attribute ];
    
      CORBA::TypeCode_var attrib_tc = attrib_desc.type;
      if( attrib_tc->kind() == CORBA::tk_alias)
	{
	  attrib_tc = extract_TypeCode(attrib_tc);
	}

      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
      if( is_typecode_authorised( attrib_tc.in()) )
	{
	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
	  CORBA::Contained_ptr attribute_cont = the_repo_ref->lookup_id( attrib_tc->id());
	  if(! CORBA::is_nil(attribute_cont))
	    {
	      cerr << " attribute dependencies" << ct_attribute << "-" << attribute_cont->name() << endl;
	      the_dependencies_level_1.push_back(attribute_cont );
	    }
	  else
	    {
	      cerr << "**** Error in attributes;  Contained with repid: " <<  attrib_desc.type->id() << "is NULL" << endl;
	    }
	}
    }
  
  // TODO : remove duplicates obtained ...
  return the_dependencies_level_1;
}

list< CORBA::Contained_ptr >
Compute_idl_item_dependencies::compute_struct_dependencies( CORBA::StructDef_ptr the_struct_def,
			     CORBA::ComponentIR::Repository_ptr the_repo_ref )
{
  list< CORBA::Contained_ptr > to_be_returned;

  CORBA::StructMemberSeq_var the_struct_members = the_struct_def->members();

  for( int ct = 0; ct < the_struct_members->length(); ct++ )
    {
      const CORBA::StructMember& current_member
	= ( *the_struct_members )[ ct ];

      CORBA::TypeCode_var tc_result = current_member.type;
      
      if( tc_result->kind() == CORBA::tk_alias)
	{
	  tc_result = extract_TypeCode(current_member.type);
	}
    
      if(is_typecode_authorised( tc_result.in()))
	{
	  CORBA::Contained_ptr dependency 
	    = the_repo_ref->lookup_id( tc_result->id() );
	  
	  if(! CORBA::is_nil(dependency))
	    {
	      cerr << " struct dependencies" << ct <<  "- " << dependency->name() << endl;
	      to_be_returned.push_back( dependency );
	    }
	  else
	    {
	      cerr << "**** Error in struct;  Contained with repid: " <<  tc_result->id() << "is NULL" << endl;
	    }
	}
    }

  return to_be_returned;
}

// xxx a factoriser avec compute_struct_dependencies car c'est presque du idem.

list< CORBA::Contained_ptr >
Compute_idl_item_dependencies::compute_exception_dependencies( CORBA::ExceptionDef_ptr the_except_def,
				CORBA::ComponentIR::Repository_ptr the_repo_ref )
{
  auto_ptr< CORBA::StructMemberSeq > the_except_members( the_except_def->members() );

  list< CORBA::Contained_ptr > to_be_returned;

  for( int ct_member = 0; ct_member < the_except_members->length(); ct_member++ )
    {
      const CORBA::StructMember& current_member
	= ( *the_except_members )[ ct_member ];
      
      CORBA::TypeCode_var exc_tc = current_member.type;
      
      if( exc_tc->kind() == CORBA::tk_alias)
	{
	  exc_tc = extract_TypeCode(current_member.type);
	}

      if( is_typecode_authorised( exc_tc.in()))
	{
	  CORBA::Contained_ptr dependency 
	    = the_repo_ref->lookup_id( exc_tc->id() );

	  if(! CORBA::is_nil(dependency))
	    {
	      cerr << " struct dependencies" << ct_member <<  "- " << dependency->name() << endl;
	      to_be_returned.push_back( dependency );
	    }
	  else
	    {
	      cerr << "**** Error in exception;  Contained with repid: " <<  exc_tc->id() << "is NULL" << endl;
	    }
	 
	}
    }

  return to_be_returned;
}

// ----------------------------------------------------------------------------

/**. This method computes dependencies for a Contained object in an IR repository.
 * It encapsulates the contained object in wrapper (containing methods to compute dependencies),
 * It compute dependencies (call to "get_dependencies" method which returns the list of 
 * dependent subnodes Contained objects).
 * It creates the corresponding number of subnodes
 * And, for each subnode add dependencies (recursively - depth first)
 *
 * @param tree_iter A subnode CORBA::Contained object on which dependencies has to be computed.
 * @param repo_ref The IR repository where the Contained object is located.
 * @return None
 */

void Compute_idl_item_dependencies::add_dependencies( tree< CORBA::Contained_ptr >::tree_iterator& tree_iter,
						      CORBA::ComponentIR::Repository_ptr ref_repo )
{

  CORBA::Contained_ptr current_contained = tree_iter.get();

  // create item wrapper
  Contained_wrapper* current_wrapper 
    = Contained_wrapper_factory::create_contained_wrapper( current_contained, ref_repo );

  // compute item dependencies
  list< CORBA::Contained_ptr > current_dependencies 
    = current_wrapper->get_dependencies();

  // register item as processed in the map
  processed_subnode_map[string(  current_contained->id() ) ]  = true;
  cerr << "item repId new: " <<  current_contained->id() << endl;

  cerr << "dep size: " << current_dependencies.size() << endl;

  // create all corresponding subnodes for item
  for( int ct = 0; ct < current_dependencies.size(); ct++ ) 
    {
      cerr << "create sub_node " << endl;
      tree_iter.create_sub_node( );
    }

  // iterate on subnodes to compute all items dependencies (recursively) if needed.
  int sub_node_counter = 0;
  for( list< CORBA::Contained_ptr >::iterator iter = current_dependencies.begin();
       iter != current_dependencies.end();
       iter++)
    {
      tree_iter.down( sub_node_counter );
      tree_iter.set( *iter );
      if(!is_subnode_already_processed(*iter))
	{
	  cerr << "add dependencies for: " << (*iter)->name() << endl;
	  add_dependencies( tree_iter, ref_repo );
	}
      tree_iter.up( );
      sub_node_counter++;
    }

}

/**. This method creates the tree and launch the dependency computation 
 * for a Contained object in an IR repository.
 * It builds a tree in which nodes are the needed IDL items (nested interfaces , operations, attributes...) contained in the CORBA::Contained object. It calls "add_dependencies" method to construct the tree recursively.
 *
 * @param item A CORBA::Contained object on which dependencies has to be computed.
 * @param repo_ref The IR repository where the Contained object is located.
 * @return A populated tree representing the Contained object dependencies.
 */
tree< CORBA::Contained_ptr > 
Compute_idl_item_dependencies::compute_idl_item_dependencies( CORBA::Contained_ptr item,
			       CORBA::ComponentIR::Repository_ptr repo_ref )
{
  tree< CORBA::Contained_ptr > dependencies_tree;

  tree< CORBA::Contained_ptr >::tree_iterator tree_iter 
    = dependencies_tree.get_head();

  tree_iter.set( item );

  add_dependencies( tree_iter, repo_ref );

  return dependencies_tree;
}


map<string, bool> Compute_idl_item_dependencies::processed_subnode_map;
