#include <dump_intf_from_ir.h>

#include <string>
#include <map>
using namespace std;

#include <tree.h>
#include <compute_idl_item_dependencies.h>

#include <Contained_wrapper.h>
#include <repid_utils.h>

#include <typecode_utils.h>
#include <dump_intf_from_ir.h>

using namespace std;

/**. This method dumps an interface to an output stream. It looks for
 * data contained in the interface: methods and their parameters, attributes.
 * @param os   An output stream (file, or stdout)
 * @param the_intf_def  An InterfaceDef object from the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_interface_as_idl_language( ostream& os, 
					 CORBA::InterfaceDef_ptr the_intf_def )
{
  CORBA::InterfaceDefSeq* base_interfaces 
    = the_intf_def->base_interfaces();

  CORBA::InterfaceDef::FullInterfaceDescription* the_description;

  try
    {
      the_description = the_intf_def->describe_interface();
    }
  catch( CORBA::SystemException& exc )
    {
      cerr << exc << " is the exception catched .." << endl;
    }

  string name_interface_ = the_intf_def->name(); 

  string name_interface = name_interface_;

  if( name_interface_.find( "_microccm_" ) == 0 )
    {
      name_interface = name_interface_.substr( string( "_microccm_" ).size() );
    }

  os << "#ifndef " << name_interface << "_defined" << endl
     << "#define " << name_interface << "_defined" << endl;

  os << "interface " << name_interface; 

  if( base_interfaces->length() > 0 ) 
    {
      os << " : " 
	 << scoped_name( ( *base_interfaces )[ 0 ]->describe_interface()->id ); 
    }

  for( int ct = 1; ct < base_interfaces->length(); ct++ )
    {
      os << ", " 
	 << scoped_name( ( *base_interfaces )[ ct ]->describe_interface()->id ); 
    }

  os << endl
     << "{"
     << endl;

  // operations

  const CORBA::OpDescriptionSeq& the_operations = the_description->operations;

  for( int ct_operation = 0; 
       ct_operation < the_operations.length(); 
       ct_operation++ )
    {
      const CORBA::OperationDescription& current_operation_def 
	= the_operations[ ct_operation ];

      if( strcmp( current_operation_def.defined_in, the_description->id ) != 0 )
        {
          cerr << __PRETTY_FUNCTION__ << "operation defined in " 
               << current_operation_def.defined_in << endl;

          cerr << __PRETTY_FUNCTION__ << "desc of interface == " 
               << the_description->id << endl;

        continue;
        }

      CORBA::TypeCode_ptr the_typecode = current_operation_def.result;

      cerr << __PRETTY_FUNCTION__ << " method == " << current_operation_def.name << endl;

      string return_type = do_tc_kind_to_string( the_typecode );

      os << "    " << return_type << " " << current_operation_def.name << "( ";

      // now iteration on arguments.
      const CORBA::ParDescriptionSeq& the_parameters_description 
	= current_operation_def.parameters;

      int len_params_description = the_parameters_description.length();
      for( int ct_param = 0; ct_param < len_params_description; ct_param++ )
	{
	  const CORBA::ParameterDescription& param_desc 
	    = the_parameters_description[ ct_param ];

	  string parameter_passing_mode;

	  {
	    if( param_desc.mode == CORBA::PARAM_IN ) 
	      parameter_passing_mode = "in";

	    if( param_desc.mode == CORBA::PARAM_OUT ) 
	      parameter_passing_mode = "out";

	    if( param_desc.mode == CORBA::PARAM_INOUT ) 
	      parameter_passing_mode = "inout";
	  }

	  CORBA::TypeCode_ptr type_param = param_desc.type;
	  string param_type = do_tc_kind_to_string( type_param );

	  os << " " << parameter_passing_mode << " " 
	     << param_type << " " << param_desc.name;

	  if( ct_param < the_parameters_description.length() - 1 )
	    {
	      os << ",";
	    }
	}

      os << ")";

      // now exceptions raised ...      
      const CORBA::ExcDescriptionSeq& exceptions_raised 
	= current_operation_def.exceptions;

      if( exceptions_raised.length() > 0 )
	{
	  os << " raises( ";

	  for( int ct_except = 0; 
	       ct_except < exceptions_raised.length(); 
	       ct_except++ )
	    {
	      os << exceptions_raised[ ct_except ].name;

	      if( ct_except < exceptions_raised.length() - 1 )
		{
		  os << ", ";
		};
	    }

	  os << " )";
	}
      os << ";" << endl;
    }

  // attributes
  const CORBA::AttrDescriptionSeq& attributes_desc 
    = the_description->attributes;

  int nb_attributes = attributes_desc.length();
  
  cerr << __PRETTY_FUNCTION__ << ": nb_attributes ==" << nb_attributes << endl;

  for( int ct_attribute = 0; 
       ct_attribute < nb_attributes; 
       ct_attribute++ )
    {
      const CORBA::AttributeDescription& attrib_desc 
	= attributes_desc[ ct_attribute ];

      os << "    ";
      if( attrib_desc.mode == CORBA::ATTR_READONLY ) os << " readonly";

      os << " attribute";
      CORBA::TypeCode_ptr the_typecode = attrib_desc.type;

      os << " " << do_tc_kind_to_string( the_typecode ) 
	 << " " << attrib_desc.name << ";" << endl;
    }

  // other nested interfaces, exceptions .. to see in a forthcoming step.

  // xxx

  os << "};" << endl;
  os << endl;

  os << "#endif // " << endl << endl;

  return;
}

/**. This method dumps a component's home to an output stream. It looks for
 * data contained in the defined IDL3 home: component, home's name and 
 * generates the IDL equivalent interface of the home (Explicit, Implicit 
 * and the Home itself) in the IDL equivalent component's file.
 *
 * @param outfile   An output stream (file, or stdout)
 * @param home_def  An HomeDef object from the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_home_as_idl_language( ostream& outfile, CORBA::ComponentIR::HomeDef_ptr home_def )
{

  CORBA::Contained::Description* contained_desc;

  contained_desc = home_def->describe();

  CORBA::ComponentIR::HomeDescription* the_home_desc = 0;

  contained_desc->value >>= the_home_desc;

  const char* name_of_home = the_home_desc->name;

  string name_of_home_string( name_of_home );

  CORBA::String_var deduced_name_of_component
    = home_def->managed_component()->name();

  cerr << "managed components ... == " 
       << the_home_desc->managed_component << endl
       << "name == " << the_home_desc->name << endl
       << "home repid == " << the_home_desc->id << endl
       << "base home == " << the_home_desc->base_home << endl
       << endl;

  cerr << "deduced_name_of_component == " << deduced_name_of_component << endl;
  
  outfile << "#ifndef " << name_of_home_string << "_defined" << endl
	  << "#define " << name_of_home_string << "_defined" << endl;

  outfile << endl;

  outfile << "interface " << name_of_home << "Explicit : " <<  /* "Components::CCMHome" */  "Components::HomeConfiguration" << endl
	  << "{" << endl
	  << " // equivalent operations - TODO " << endl
	  << "};" << endl
	  << endl
	  << "interface " << name_of_home << "Implicit : Components::KeylessCCMHome" << endl
	  << "{" << endl
	  << "   " << deduced_name_of_component << " create() raises( Components::CreateFailure );" << endl
	  << "};" << endl
	  << endl
	  << "interface " << name_of_home << " : " << name_of_home << "Explicit, " << name_of_home << "Implicit" << endl
	  << "{" << endl
	  << "};" << endl;
  
  outfile << "#endif // " << endl << endl;
}


/**. This method dumps a struct to an output stream. In the interface repository, 
 * it looks for members contained in the struct, and dump them to the given output 
 * stream.
 * @param os   An output stream (file, or stdout)
 * @param the_intf_def  A StructDef object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_struct_as_idl_language( ostream& os, CORBA::StructDef_ptr the_struct_def )
{
  os << "#ifndef " << the_struct_def->name() << "_defined" << endl
     << "#define " << the_struct_def->name() << "_defined" << endl;
  
  os << "struct " << the_struct_def->name() << endl
     << "{" << endl;

  CORBA::StructMemberSeq* the_struct_members = the_struct_def->members();

  for( int ct_member = 0; ct_member < the_struct_members->length(); ct_member++ )
    {
      const CORBA::StructMember& the_struct_member = ( *the_struct_members )[ ct_member ];
      os << "    " << do_tc_kind_to_string( the_struct_member.type ) 
	 << " " << the_struct_member.name << ";" << endl;
    }

  os << "};" << endl;
  os << endl;

  os << "#endif // " << endl << endl;

  return;
}


/**. This method dumps an exception to an output stream. In the interface repository, 
 * it looks for the exception, and dump it to the given output stream.
 * @param os   An output stream (file, stdout, ...)
 * @param the_except_def  An ExceptionDef  object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_exception_as_idl_language( ostream& os, CORBA::ExceptionDef_ptr the_except_def )
{

  os << "#ifndef " << the_except_def->name() << "_defined" << endl
     << "#define " << the_except_def->name() << "_defined" << endl;

  os << "exception " << the_except_def->name() << endl
     << "{" << endl;

  CORBA::StructMemberSeq* the_except_members = the_except_def->members();

  for( int ct_member = 0; ct_member < the_except_members->length(); ct_member++ )
    {
      const CORBA::StructMember& the_struct_member = ( *the_except_members )[ ct_member ];
      os << "    " << do_tc_kind_to_string( the_struct_member.type ) 
	 << " " << the_struct_member.name << ";" << endl;
    }

  os << "};" << endl;
  os << endl << endl;

  os << "#endif // " << endl << endl;

  return;
}


class Contained_id_dumper
{
public:

  Contained_id_dumper( ostream& os ) : os_( os ) { }

  void operator()( CORBA::Contained_ptr cont_ )
  {
    os_ << cont_->id() << endl;
  }

private:

  ostream& os_;
};

// -------------------------------------------------------------------------

class Contained_idl_lang_dumper
{
public:

  Contained_idl_lang_dumper( ostream& os,
			     CORBA::ComponentIR::Repository_ptr the_repo ) 
    : os_( os ) 
  { 
    the_repo_ = the_repo;
  }

  void operator()( CORBA::Contained_ptr cont_ )
  {
    Contained_wrapper* the_contained 
      = Contained_wrapper_factory::create_contained_wrapper( cont_, the_repo_  );

    the_contained->dump_as_idl_language( os_ );
  }

private:

  CORBA::ComponentIR::Repository_ptr the_repo_;
  ostream& os_;
};


/**. This method launches the dependencies computation, makes the tree traversal, 
 * which finally, dumps all the needed elements in the output IDL file (os).
 *
 * @param os             An output stream (file, stdout, ...)
 * @param contained_item A CORBA::Contained_ptr object on which dependencies have to be found.
 * @param the_repo       A  CORBA::ComponentIR::Repository_ptr object 
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_with_dependencies( ostream& os,
			     CORBA::Contained_ptr contained_item,
			     CORBA::ComponentIR::Repository_ptr the_repo )
{
  if( CORBA::is_nil( contained_item ) )
    {
      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
      exit( 2 );
    }

  tree< CORBA::Contained_ptr > all_dependencies
    = Compute_idl_item_dependencies::compute_idl_item_dependencies( contained_item, the_repo );

  Contained_idl_lang_dumper the_dumper( os, the_repo );

  //  Contained_id_dumper the_dumper( cerr );

  do_tree_traversal( all_dependencies, the_dumper );

  return;
}


/**. This method dumps all event sinks to an output stream. In the interface repository, 
 * it looks for the consumes ports, and dump them to the given output stream, thanks to 
 * EventDef dump method.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_comp_desc  A ComponentDescription_ptr  object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_sinks_as_idl_language( ostream& os,
								    CORBA::ComponentIR::ComponentDescription* the_comp_desc,
								    CORBA::ComponentIR::Repository_ptr the_repo_ref)
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
  CORBA::ComponentIR::EventPortDescriptionSeq consumes_events_seq = the_comp_desc->consumes_events;
  int nb_consumes_ports = consumes_events_seq.length();
  
  for (int ct_consumes_port = 0; ct_consumes_port < nb_consumes_ports; ct_consumes_port++)
    {
      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
      CORBA::ComponentIR::EventPortDescription& consumes_event_desc 
	= consumes_events_seq[ct_consumes_port];
      
      CORBA::Contained_ptr consumes_event_cont
	= the_repo_ref->lookup_id(consumes_event_desc.id);
      
      // For each consumer get the event itself, and its description.
      CORBA::ComponentIR::ConsumesDef_ptr consumes_ref 
	= CORBA::ComponentIR::ConsumesDef::_narrow( consumes_event_cont );

      dump_idl_eventtype_as_idl_language(os, consumes_ref->event(),the_repo_ref );
    }
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
}


/**. This method dumps all event sources to an output stream. In the interface repository, 
 * it looks for the publishes ports, and dump them to the given output stream, thanks to 
 * EventDef dump method.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_comp_desc  A ComponentDescription_ptr  object defined in the interface repository.
 * @return none
 * 
 */

void Dump_idl_item_as_idl_language::dump_idl_sources_as_idl_language( ostream& os,
								      CORBA::ComponentIR::ComponentDescription* the_comp_desc,
								      CORBA::ComponentIR::Repository_ptr the_repo_ref)
								      
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
  CORBA::ComponentIR::EventPortDescriptionSeq publishes_events_seq = the_comp_desc->publishes_events;
  int nb_publishes_ports = publishes_events_seq.length();
  
  for (int ct_publishes_port = 0; ct_publishes_port < nb_publishes_ports; ct_publishes_port++)
    {      
      cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
      CORBA::ComponentIR::EventPortDescription& publishes_event_desc 
	= publishes_events_seq[ct_publishes_port];
      
      CORBA::Contained_ptr publishes_event_cont
	= the_repo_ref->lookup_id(publishes_event_desc.id);
      
      // For each consumer get the event itself, and its description.
      CORBA::ComponentIR::PublishesDef_ptr publishes_ref 
	= CORBA::ComponentIR::PublishesDef::_narrow( publishes_event_cont );

      dump_idl_eventtype_as_idl_language(os, publishes_ref->event(), the_repo_ref);
    }
}


/**. This method dumps a component's eventtype to an output stream. In the interface repository, 
 * it looks for the value type description, and dump it to the given output stream.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_except_def  An EventDef object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_eventtype_as_idl_language(ostream& os, 
								       CORBA::ComponentIR::EventDef_ptr the_event_def,
								       CORBA::ComponentIR::Repository_ptr the_repo_ref)
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
  CORBA::ExtValueDef::ExtFullValueDescription* value_desc = the_event_def->describe_ext_value();

  //
  // Base value
  //
  CORBA::ValueDef_ptr inherited_value_def = the_event_def->base_value();
  if(!CORBA::is_nil(inherited_value_def))
    {
      dump_idl_valuetype_as_idl_language(os, CORBA::ExtValueDef::_narrow(inherited_value_def), the_repo_ref);
    }

  //
  // Inherited abstract value type Components::EventBase
  //
  CORBA::Contained_ptr eventbase_cont
    = the_repo_ref->lookup_id( "IDL:omg.org/Components/EventBase:1.0" );
  if( CORBA::is_nil( eventbase_cont ) )
    {
      cerr << __PRETTY_FUNCTION__ << ": " << __LINE__ << "...probleme" << endl;
      exit( 1 );
    }
   
  // BUG TAO !! --> do we generate EventBase after valuetype generation.
  //   CORBA::ValueDefSeq abstract_inherited_values;
  // abstract_inherited_values.length(1);
  // abstract_inherited_values[ 0 ] 
  //     = CORBA::ValueDef::_narrow( eventbase_cont );
  // the_event_def->abstract_base_values(abstract_inherited_values); // hyp: this method doesn't set the abstract_base_values.

   
   
  //
  // Valuetype
  //
  dump_idl_valuetype_as_idl_language(os, the_event_def, the_repo_ref);
  
  //
  // Interface
  //
  // defining name and repository id of interface
  string name_of_equivalent_intf = string(the_event_def->name()) + string("Consumer");
  string repid_of_equivalent_interface 
    = string( "IDL:" ) + name_of_equivalent_intf + string( ":1.0" );

  os << "#ifndef " << name_of_equivalent_intf << "_defined" << endl
     << "#define " << name_of_equivalent_intf << "_defined" << endl;

  os << "interface" << " " << name_of_equivalent_intf << " "
     << ":" << " " << "Components::EventConsumerBase" << endl;
  os << "{" << endl;
  os << "  void push( in" << " " << string(the_event_def->name()) << " " << "evt" << ");" << endl;
  os << "  CosEventComm::PushConsumer get_pushConsumer();" << endl;
  os << "};" << endl;
  os << endl;

  os << "#endif" << endl << endl;

  //   // defining base interfaces
  //   CORBA::InterfaceDefSeq base_interfaces;
  //   base_interfaces.length( 1 );
  //   cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
    
  //   CORBA::Contained_ptr eventconsumerbase_cont
  //     = the_repo_ref->lookup_id( "IDL:omg.org/Components/EventConsumerBase:1.0" );
  
  //   if( CORBA::is_nil( eventconsumerbase_cont ) )
  //     {
  //       cerr << __PRETTY_FUNCTION__ << ": " << __LINE__ << "...probleme" << endl;
  //       exit( 1 );
  //     }
  
  //   base_interfaces[ 0 ] 
  //     = CORBA::InterfaceDef::_narrow( eventconsumerbase_cont );
  //     cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
    
  //   CORBA::InterfaceDef_ptr eventconsumer_interface_def =
  //     the_event_def->create_interface(repid_of_equivalent_interface.c_str(),
  // 				    name_of_equivalent_intf.c_str(),
  // 				    "1.0",
  // 				    base_interfaces);
  
  //   eventconsumer_interface_def->base_interfaces( base_interfaces );
  //   //  eventconsumer_interface_def->add_operation(push);
  //   CORBA::PrimitiveDef_var push_result_prim = the_repo_ref->get_primitive(CORBA::pk_void);
  //   CORBA::ParDescriptionSeq& params;
  //   params.length(1);
  //   params[0].name = CORBA::String("event");
  //   // creation typecode
  //   params[0].type = sqds;
  //   params[0].type_def = sqds;
  //   params[0].mode = PARAM_IN;

  //   eventconsumer_interface_def->create_operation("IDL:"+ the_event_def->name() + " push:1.0",
  //  						"push",
  //  						"1.0"
  //   						push_result_prim->type() ,
  //  						CORBA::OP_NORMAL,
  //  						push_params,
  //  						push_exec,
  //  						push_ctxt);

  //   eventconsumer_interface_def->add_operation(get_pushconsumer);
  //   dump_idl_interface_as_idl_language(os, eventconsumer_interface_def);
}


/**. This method dumps a value type to an output stream. In the interface repository, 
 * it looks for the value type extended description, and dump it to the given output stream.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_except_def  An EventDef object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_valuetype_as_idl_language(ostream& os, 
								       CORBA::ExtValueDef_ptr the_value_def, 
								       CORBA::ComponentIR::Repository_ptr the_repo_ref ) 
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;  
  CORBA::ExtValueDef::ExtFullValueDescription* value_desc = the_value_def->describe_ext_value();

  os << "#ifndef " << value_desc->name << "_defined" << endl
     << "#define " << value_desc->name << "_defined" << endl;

  os << "valuetype " << value_desc->name;

  // inherited value
  CORBA::ValueDef_ptr inherited_value_def = the_value_def->base_value();
  if(!CORBA::is_nil(inherited_value_def))
    {
      os << "," << " " << endl << " " << inherited_value_def->name() << endl;
    }

  // BUG TAO !! --> we add EventBase
  os <<  ":" << " " << endl << " " << "Components::EventBase" << endl;
  
  // abstract inherited values (todo...)
  cerr << "******************" << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;  
  
  //   CORBA::ValueDefSeq* abstract_inherited_values = the_value_def->abstract_base_values();
  //   if(abstract_inherited_values != NULL)
  //     {
  //       cerr << "******************" << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;  
  //       int nb_abstract_values = abstract_inherited_values->length();
  //       cerr << "nb ab values: " << nb_abstract_values << endl;  
  //       for(int ct_abstract_value = 0; ct_abstract_value < nb_abstract_values; ct_abstract_value++)
  // 	{ 
  // 	  cerr << "value name: " << (*abstract_inherited_values)[ct_abstract_value]->name() << endl;
  // 	  os << "," << " " << endl << " " << (*abstract_inherited_values)[ct_abstract_value]->name() << endl;
  // 	}
  //     }
  
  os << "{" << endl;


  //
  // Attributes
  //
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;  
  CORBA::ExtAttrDescriptionSeq attributes_desc = value_desc->attributes;
  int nb_value_attributes = attributes_desc.length();
  for(int ct_value_attributes = 0; ct_value_attributes < attributes_desc.length(); ct_value_attributes++)
    {
      dump_idl_attribute_as_idl_language(os, attributes_desc[ct_value_attributes]);
    }


  //      
  // Value members
  //
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
  CORBA::ValueMemberSeq members_desc = value_desc->members;
  int nb_value_members = members_desc.length();
  for(int ct_value_member = 0; ct_value_member < nb_value_members; ct_value_member++)
    {
      CORBA::ValueMember& member_desc = members_desc[ct_value_member];
      CORBA::TypeCode_var tc =  member_desc.type;
      cout <<" " << ct_value_member << " " << member_desc.name << " of type: " << tc->kind() << endl;
      string visibility_string;
      {
	if( member_desc.access == CORBA::PRIVATE_MEMBER ) 
	  visibility_string = "private";
	
	if( member_desc.access == CORBA::PUBLIC_MEMBER ) 
	    visibility_string = "public";
      }
      os << "    " << visibility_string
	 << " " <<  do_tc_kind_to_string(tc.in())
	 << " " << member_desc.name  << ";" << endl;
    }
  
  
  //
  // Operations
  //

  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

  CORBA::OpDescriptionSeq operations_desc = value_desc->operations;
  for(int ct_op_desc = 0; ct_op_desc < operations_desc.length(); ct_op_desc++)
    {
      dump_idl_operation_as_idl_language(os, operations_desc[ct_op_desc]);
    }

  os << "};" << endl;

  os << "#endif //" << endl << endl;
}


/**. This method dumps a value type to an output stream. In the interface repository, 
 * it looks for the value type extended description, and dump it to the given output stream.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_except_def  An EventDef object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_attribute_as_idl_language(ostream& os, CORBA::ExtAttributeDescription& attribute_desc )
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;  
  CORBA::TypeCode_var tc =  attribute_desc.type;
  cout <<"  " << attribute_desc.name << "of type: " << attribute_desc.type->name() << endl; 
  
  // only "read_only" mode
  if( attribute_desc.mode == CORBA::ATTR_READONLY ) os << " readonly";
  os << "attribute ";
  os << " " << attribute_desc.type->name();
  os << " " << attribute_desc.name << ";" << endl;
    
  //  os << " " << do_tc_kind_to_string( the_typecode ) 
  //     << " " << attrib_desc.name << ";" << endl;
}


/**. This method dumps a value type to an output stream. In the interface repository, 
 * it looks for the value type extended description, and dump it to the given output stream.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_except_def  An EventDef object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_operation_as_idl_language(ostream& os, CORBA::OperationDescription& operation_desc )
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

//   if( strcmp( operation_desc.defined_in, the_description->id ) != 0 )
//     {
//       cerr << __PRETTY_FUNCTION__ << "operation defined in " 
// 	   << operation_desc.defined_in << endl;
      
//       //      cerr << __PRETTY_FUNCTION__ << "desc of interface == " 
//       //	   << the_description->id << endl;

//       continue;
//     }

  CORBA::TypeCode_var the_typecode = operation_desc.result;

  cerr << __PRETTY_FUNCTION__ << " method == " << operation_desc.name << endl;

  string return_type = do_tc_kind_to_string( the_typecode.in() );

  os << "    " << return_type << " " << operation_desc.name << "( ";

  // now iteration on arguments.
  const CORBA::ParDescriptionSeq& the_parameters_description 
    = operation_desc.parameters;

  int len_params_description = the_parameters_description.length();
  for( int ct_param = 0; ct_param < len_params_description; ct_param++ )
    {
      const CORBA::ParameterDescription& param_desc 
	= the_parameters_description[ ct_param ];

      string parameter_passing_mode;

      {
	if( param_desc.mode == CORBA::PARAM_IN ) 
	  parameter_passing_mode = "in";
	
	if( param_desc.mode == CORBA::PARAM_OUT ) 
	  parameter_passing_mode = "out";

	if( param_desc.mode == CORBA::PARAM_INOUT ) 
	  parameter_passing_mode = "inout";
      }

      CORBA::TypeCode_ptr type_param = param_desc.type;
      string param_type = do_tc_kind_to_string( type_param );

      os << " " << parameter_passing_mode << " " 
	 << param_type << " " << param_desc.name;

      if( ct_param < the_parameters_description.length() - 1 )
	{
	  os << ",";
	}
    }

  os << ")";

  dump_idl_exceptions_as_idl_language(os, operation_desc.exceptions);

  os << ";" << endl;
    
}


/**. This method dumps a value type to an output stream. In the interface repository, 
 * it looks for the value type extended description, and dump it to the given output stream.
 *
 * @param os   An output stream (file, stdout, ...)
 * @param the_except_def  An EventDef object defined in the interface repository.
 * @return none
 * 
 */
void Dump_idl_item_as_idl_language::dump_idl_exceptions_as_idl_language(ostream& os, CORBA::ExcDescriptionSeq& exceptions_desc )
{
  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;
    
  if( exceptions_desc.length() > 0 )
    {
      os << " raises( ";
      
      for( int ct_except = 0; 
	   ct_except < exceptions_desc.length(); 
	   ct_except++ )
	{
	  os << exceptions_desc[ ct_except ].name;

	  if( ct_except < exceptions_desc.length() - 1 )
	    {
	      os << ", ";
	    };
	}

      os << " )";
    }
}
