#ifndef tree_h
#define tree_h

#include <vector>
using namespace std;

template< typename T >
class tree
{
 public:

  struct tree_storage
  {
    void remove_all_subnodes();

    T data;

    tree_storage* upper_node;
    vector< tree_storage* > sub_nodes;

    int depth;
  };

  tree();

  ~tree();

  tree( const tree< T >& t )
  {    
    t.no_cleaning_ = true;

    {
      head_ = t.head_;
      for( int ct = 0; ct < head_.sub_nodes.size(); ct++ )
	{
	  head_.sub_nodes[ ct ]->upper_node = &head_;
	}
    }

    t.do_no_destruction(); // for the moment.
  }

  void do_no_destruction() const 
  {
    no_cleaning_ = true; 
  }

  // --- tree_iterator inner class ---

  class tree_iterator
    {
    public:

      struct cannot_go_up { };
      struct cannot_go_down { };

      friend class tree< T >;

      void up() throw( cannot_go_up );
      void down( int index ) throw( cannot_go_down );

      int create_sub_node( ); 
      int nb_sub_nodes( ) const;

      int depth() const;

      T& get( );
      void set( const T& t );

    private:

      tree_iterator( tree_storage* initial );

      tree_storage* current_node_;
    };

  tree_iterator get_head();

 private:

  mutable bool no_cleaning_;
  tree_storage head_;
};

template< typename T, typename S >
void do_iterate( typename tree< T >::tree_iterator iter, S& action )
{
  //  action( iter.get() );

  for( int ct = 0; ct < iter.nb_sub_nodes(); ct++ )
    {
      iter.down( ct );
      do_iterate< T, S >( iter, action );
      iter.up();
    }

  action( iter.get() );
}

template< typename T, typename S >
void do_tree_traversal( tree< T >& the_tree, S& s )
{
  typename tree< T >::tree_iterator iter = the_tree.get_head();

  do_iterate< T, S >( iter, s );
}

#include <tree_impl.h>

#endif // tree_h
