#include <typecode_utils.h>

#include <list>
#include <algorithm>
#include <map>
#include <iostream>
using namespace std;

bool is_primitive( CORBA::TypeCode_ptr the_typecode )
{
  list< CORBA::TCKind > primitive_types;

  primitive_types.push_back( CORBA::tk_void );
  primitive_types.push_back( CORBA::tk_short );
  primitive_types.push_back( CORBA::tk_long );
  primitive_types.push_back( CORBA::tk_ushort );
  primitive_types.push_back( CORBA::tk_ulong );
  primitive_types.push_back( CORBA::tk_float );
  primitive_types.push_back( CORBA::tk_double );
  primitive_types.push_back( CORBA::tk_boolean );
  primitive_types.push_back( CORBA::tk_char );
  primitive_types.push_back( CORBA::tk_octet );
  primitive_types.push_back( CORBA::tk_any );
  primitive_types.push_back( CORBA::tk_TypeCode );
  primitive_types.push_back( CORBA::tk_Principal );
  primitive_types.push_back( CORBA::tk_string );
  primitive_types.push_back( CORBA::tk_longlong );
  primitive_types.push_back( CORBA::tk_ulonglong );
  primitive_types.push_back( CORBA::tk_longdouble );
  primitive_types.push_back( CORBA::tk_wchar );
  primitive_types.push_back( CORBA::tk_wstring );
  primitive_types.push_back( CORBA::tk_fixed );

  if( find( primitive_types.begin(), primitive_types.end(), 
	    the_typecode->kind() ) != primitive_types.end() )
    {
      return true;
    }
  else
    {
      return false;
    }

}


bool is_valuetype( CORBA::TypeCode_ptr the_typecode )
{
  if(the_typecode->kind() == CORBA::tk_value)
    {
      return true;
    }
  return false;
}


string do_tc_kind_to_string( CORBA::TypeCode_ptr the_type_code )
{
  cerr << __PRETTY_FUNCTION__ << ":" <<  __LINE__ << endl;
  CORBA::TCKind the_kind = the_type_code->kind();

  map< CORBA::TCKind, string > tc_kind_to_string;
  {
    tc_kind_to_string[ CORBA::tk_void ] = "void";
    tc_kind_to_string[ CORBA::tk_short ] = "short";
    tc_kind_to_string[ CORBA::tk_long ] = "long";
    tc_kind_to_string[ CORBA::tk_ushort ] = "unsigned short";
    tc_kind_to_string[ CORBA::tk_ulong ] = "unsigned long";
    tc_kind_to_string[ CORBA::tk_float ] = "float";
    tc_kind_to_string[ CORBA::tk_double ] = "double";
    tc_kind_to_string[ CORBA::tk_boolean ] = "boolean";
    tc_kind_to_string[ CORBA::tk_char ] = "char";
    tc_kind_to_string[ CORBA::tk_octet ] = "octet";
    tc_kind_to_string[ CORBA::tk_any ] = "any";
    tc_kind_to_string[ CORBA::tk_TypeCode ] = "TypeCode";
    tc_kind_to_string[ CORBA::tk_Principal ] = "Principal"; // xxx ???
    //      tc_kind_to_string[ CORBA::tk_objref ];
    //      tc_kind_to_string[ CORBA::tk_struct ];
    //       tc_kind_to_string[ CORBA::tk_union ];
    //       tc_kind_to_string[ CORBA::tk_enum ];
    tc_kind_to_string[ CORBA::tk_string ] = "string";
    //      tc_kind_to_string[ CORBA::tk_sequence ];
    //      tc_kind_to_string[ CORBA::tk_array ];
    //      tc_kind_to_string[ CORBA::tk_alias ];
    //      tc_kind_to_string[ CORBA::tk_except ];
    tc_kind_to_string[ CORBA::tk_longlong ] = "long long";
    tc_kind_to_string[ CORBA::tk_ulonglong ] = "unsigned long long";
    tc_kind_to_string[ CORBA::tk_longdouble ] = "long double";
    tc_kind_to_string[ CORBA::tk_wchar ] = "wchar";
    tc_kind_to_string[ CORBA::tk_wstring ] = "wstring";
    tc_kind_to_string[ CORBA::tk_fixed ] = "fixed";
    //    tc_kind_to_string[ CORBA::tk_value ] ;
    //      tc_kind_to_string[ CORBA::tk_value_box ];
    //      tc_kind_to_string[ CORBA::tk_native ];
    //      tc_kind_to_string[ CORBA::tk_abstract_interface ];
    //      tc_kind_to_string[ CORBA::tk_local_interface ];
    //      tc_kind_to_string[ CORBA::tk_component ];
    //      tc_kind_to_string[ CORBA::tk_home ];
    //      tc_kind_to_string[ CORBA::tk_event ];
  }

  bool is_primitive = false;

  map< CORBA::TCKind, string >::iterator iter = tc_kind_to_string.find( the_kind );

  if( iter != tc_kind_to_string.end() ) is_primitive = true;

  if( is_primitive ) 
    {
      cerr << "****** type : " <<  tc_kind_to_string[ the_kind ] << endl;
      return tc_kind_to_string[ the_kind ];
    }

  if( the_kind == CORBA::tk_objref )
    {
      return string( the_type_code->name() );
    }

  if( the_kind == CORBA::tk_struct )
    {
      return string( the_type_code->name() );
    }

  if( the_kind == CORBA::tk_alias )
    {
      return string( the_type_code->name() );
//       CORBA::Contained_ptr the_alias
//         = the_repo_ref->lookup_id( the_typecode->id() );

//       CORBA::IDLType_ptr aliased_type = CORBA::IDLType::_narrow( the_alias );

//       CORBA::TypeCode_ptr corresponding_typecode
//         = aliased_type->type();

//       return do_tc_kind_to_string( corresponding_typecode );
    }
  
  if( the_kind == CORBA::tk_value )
    {
      return string( the_type_code->name() );
    }

  if( the_kind == CORBA::tk_enum )
    {
      return string( the_type_code->name() );
    }

  cerr << __PRETTY_FUNCTION__ << ": need to finish implementation ..." << endl;
  exit( 1 );
}



CORBA::IDLType_ptr typecode_to_idltype( CORBA::TypeCode_ptr the_type_code,
					CORBA::ComponentIR::Repository_ptr the_repo_ref )
{
  CORBA::TCKind the_kind = the_type_code->kind();

  map< CORBA::TCKind, CORBA::PrimitiveKind > tc_kind_to_primitive_kind;
  {
    tc_kind_to_primitive_kind[ CORBA::tk_void ] = CORBA::pk_void;
    tc_kind_to_primitive_kind[ CORBA::tk_short ] = CORBA::pk_short;
    tc_kind_to_primitive_kind[ CORBA::tk_long ] = CORBA::pk_long;
    tc_kind_to_primitive_kind[ CORBA::tk_ushort ] = CORBA::pk_ushort;
    tc_kind_to_primitive_kind[ CORBA::tk_ulong ] = CORBA::pk_ulong;
    tc_kind_to_primitive_kind[ CORBA::tk_float ] = CORBA::pk_float;
    tc_kind_to_primitive_kind[ CORBA::tk_double ] = CORBA::pk_double;
    tc_kind_to_primitive_kind[ CORBA::tk_boolean ] = CORBA::pk_boolean;
    tc_kind_to_primitive_kind[ CORBA::tk_char ] = CORBA::pk_char;
    tc_kind_to_primitive_kind[ CORBA::tk_octet ] = CORBA::pk_octet;
    tc_kind_to_primitive_kind[ CORBA::tk_any ] = CORBA::pk_any;
    //    tc_kind_to_primitive_kind[  ] = CORBA::pk_TypeCode;
    //    tc_kind_to_primitive_kind[  ] = CORBA::pk_Principal; // ??
    tc_kind_to_primitive_kind[ CORBA::tk_string ] = CORBA::pk_string;
    //    tc_kind_to_primitive_kind[  ] = CORBA::pk_objref; // ??
    tc_kind_to_primitive_kind[ CORBA::tk_longlong ] = CORBA::pk_longlong;
    tc_kind_to_primitive_kind[ CORBA::tk_longdouble ] = CORBA::pk_longdouble;
    tc_kind_to_primitive_kind[ CORBA::tk_wchar ] = CORBA::pk_wchar;
    tc_kind_to_primitive_kind[ CORBA::tk_wstring ] = CORBA::pk_wstring;
    //    tc_kind_to_primitive_kind[  ] = CORBA::pk_value_base; // ??
  }

  bool is_primitive = false;

  map< CORBA::TCKind, CORBA::PrimitiveKind >::iterator iter = tc_kind_to_primitive_kind.find( the_kind );

  if( iter != tc_kind_to_primitive_kind.end() ) is_primitive = true;

  if( is_primitive ) 
    {
      return the_repo_ref->get_primitive( iter->second );
    }

  if( ( the_kind == CORBA::tk_objref ) || ( the_kind = CORBA::tk_struct ) )
    {
      CORBA::Contained_ptr contained = the_repo_ref->lookup_id( the_type_code->id() );
      return CORBA::IDLType::_narrow( contained );
    }

  cerr << __PRETTY_FUNCTION__ << ": need to finish implementation ..." << endl;
  exit( 1 );
}

