#include <MCCM_Service_Integration.h>

#include <Trace_integrationC.h>
#include <Trace_integrationS.h>

class Trace_bidon_integration : public MCCM_Service_Integration
{
public:
  void install_at_service_access_points( MCCM_Service_Integration_Points* sip );
  void accept_configuration( const CORBA::Any& an_any );

  char* name_of_facet_;
  char* name_of_method_;
};

class Trace_interceptor : public MCCM_Interceptor
{
public:
  void invoke( MCCM_Args* args );
  void set_callback( MCCM_Callback* );

private:

  MCCM_Callback* the_callback_;
};

class Trace_bidon_integration_factory : public MCCM_Service_Integration_Factory
{
public:

  char* name();
  MCCM_Service_Integration* create();
};

// ----------------------------------------------------------------------------------------

void Trace_bidon_integration::accept_configuration( const CORBA::Any& config )
{
  const StringSeq* the_string_seq;
  config >>= the_string_seq;

  name_of_facet_ = CORBA::string_dup( ( *the_string_seq )[ 0 ] );
  name_of_method_ = CORBA::string_dup( ( *the_string_seq )[ 1 ] );
}

void 
Trace_bidon_integration::install_at_service_access_points( MCCM_Service_Integration_Points* sip )
{
  sip->set_in_place_of_interceptor( name_of_facet_, name_of_method_, new Trace_interceptor );
}

// ----------------------------------------------------------------------------------------

void Trace_interceptor::invoke( MCCM_Args* args )
{
  cerr << "Before call .." << endl;
  the_callback_->invoke( args );
  cerr << "After call .." << endl;
}

void Trace_interceptor::set_callback( MCCM_Callback* the_callback )
{
  the_callback_ = the_callback;
}

// ----------------------------------------------------------------------------------------

char* Trace_bidon_integration_factory::name()
{
  return "trace_bidon";
}

MCCM_Service_Integration* Trace_bidon_integration_factory::create() 
{
  return new Trace_bidon_integration;
}

// ----------------------------------------------------------------------------------------

extern "C"
{
  void register_trace_bidon_service()
  {
    Trace_bidon_integration_factory* tbi = new Trace_bidon_integration_factory;

    MCCM_Service_Integration_Factories::register_service_integration_factory( tbi );
  }
};
