#ifndef PSOS
#define _UNIX
#endif
/* ------------------------------------------------ */
/*			Logiciel TFTP Client Serveur			*/
/*													*/
/*	Realisateur			: R.SIMON					*/
/*	Date de creation	: 03/10/2000				*/
/*													*/
/* ------------------------------------------------ */
#ifdef _VALNT
#include <time.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <winsock.h>
#define CLOSE		closesocket
#define PRINTF(a)	trace_bl(a)
#endif

#ifdef _UNIX
#include "sys/types.h"
#include "sys/socket.h"
#include "netinet/in.h"
#include "errno.h"
#include "string.h"
#include "stdio.h"
#include "sys/time.h"
#define CLOSE		close
#define PRINTF(a)	printf a
#endif

#ifdef PSOS
#include "psos.h"
#include "prepc.h"
#include "pna.h"
#include "errno.h"
#include "string.h"
#include "stdio.h"
#include "time.h"
#define CLOSE		close
#ifndef _BOOTRTIP
#define PRINTF(a)	printf a
#else
#ifdef __PC__
#define PRINTF(a)	printf a
#else
#define PRINTF(a)	Print a
#endif /* __PC__ */
#endif /* _BOOTRTIP */
#endif /* PSOS */

#include "tftp.h"
#include "tftp_def.h"
#include "dataio.h"
#include "support.h"

#ifdef _VALNT
// pour trace_bl pour ValNT
#include "gen.h"
#include "sdit.h"
#endif

static  t_ctx_tftp ctx_tftp [K_RESERVE_CTX+1];
static unsigned char bufferTftp[600];
static int sizeBlocData;
static t_access_file defaultAccessFile;

/* ne sont plus static */
int g_comportement;
int arret_process_tftp;
unsigned short port_server_tftp;

static int base_timeout;
static int nb_max_retry;
static int max_timeout;

#ifdef _SIMERROR
static int perteAck  = 0;
static int perteData = 0;
#endif

char *error_tftp_string[] =
{
	"Error undefined",
	"File not found",
	"Access violation",
	"Disk Full",
	"Illegal Operation",
	"Unknown Transfer Id",
	"File already exist",
	"No such User",
	"Incoherent TFTP Frame",
	"Max Retry",
	"No TFTP Resource",
};

int transmit_error (int ind_ctx, short error);
int transmit_bloc_data (int ind_ctx, int op);

char *decode_name (int lg_trame, int *lg_name, char *name, char *pstr)
{
	int current_lg = *lg_name;
	while (current_lg < lg_trame)
	{
		*name = *pstr;
		if (*pstr == 0)
			break;
		else
		{
			name++;
			pstr++;
			current_lg++;
		}
	}
	*lg_name = current_lg;
	return pstr;
}

/* -------------------------------------------------------------------- */
/*    Fonctions de gestion des contextes de transfert                   */
/*				searchNewentry recherche une nouvelle entree            */
/*				freeCtx libere le contexte passe en parametre           */
/*				searchCtx recherche une entree a partir des coordonnees */
/*                        de l'emetteur de la trame                     */
/* -------------------------------------------------------------------- */
int searchNewentry ()
{
	int ind;

	for (ind = 0; ind < K_MAX_CTX; ind++)
		if (ctx_tftp[ind].state == K_FREE_STATE)
		{
			ctx_tftp[ind].state = K_WAIT_FREE;
			ctx_tftp[ind].timeout_emission = 0x7FFFFFFF;
			break;
		}

	if (ind == K_MAX_CTX)
		ind = -1;

	return ind;
}

void freeCtx (int ctx_index)
{

	if (ctx_tftp[ctx_index].sock_number != ctx_tftp[0].sock_number)
		CLOSE (ctx_tftp[ctx_index].sock_number);

	if (ctx_index < K_RESERVE_CTX)
	{
		if (ctx_tftp[ctx_index].fd_file != (FILE *)NULL)
		{
			ctx_tftp[ctx_index].accessFile->fclose (ctx_tftp[ctx_index].fd_file);
			ctx_tftp[ctx_index].fd_file = (FILE *)NULL;
		}

		if (ctx_tftp[ctx_index].callback != 0)
		{
			ctx_tftp[ctx_index].callback (ctx_tftp[ctx_index].error,
											ctx_tftp[ctx_index].args_callBack);
			arret_process_tftp = 1;
		}

#ifdef _TRACE_TFTP  
		/* affichage des statistiques */
		//      		PRINTF (("\nNb frames received    : %08d", ctx_tftp[ctx_index].tramesrecues));
		//       		PRINTF (("\nNb frames transmitted : %08d", ctx_tftp[ctx_index].tramesemises));
#endif
		//		PRINTF (("\n\nNb bytes received    : %08d", ctx_tftp[ctx_index].bytesreceived));
		//		PRINTF (("\nNb bytes transmitted : %08d\n", ctx_tftp[ctx_index].bytestransmitted));
 
		//		PRINTF (("\nNb frames ACK        : %08d", ctx_tftp[ctx_index].tramesack));
		//		PRINTF (("\nNb frames ACK Retry  : %08d", ctx_tftp[ctx_index].tramesackretry));
		//		PRINTF (("\nNb frames DATA       : %08d", ctx_tftp[ctx_index].tramesdata));
		//		PRINTF (("\nNb frames DATA Retry : %08d", ctx_tftp[ctx_index].tramesdataretry));
		arret_process_tftp = 1;
	}

	memset ( (char *)&ctx_tftp[ctx_index], 0, sizeof(struct t_ctx_tftp));
	ctx_tftp[ctx_index].state = K_FREE_STATE;
	ctx_tftp[ctx_index].timeout_emission = 0x7FFFFFFF;
}

int searchCtx (struct sockaddr_in *from, int local_sock)
{
	int index = 0;

	while (index < K_MAX_CTX)
	{
		if ((from->sin_port == ctx_tftp[index].port_dest) &&
			(from->sin_addr.s_addr == ctx_tftp[index].ip_dest) &&
			(local_sock == ctx_tftp[index].sock_number) )
			break;
		index++;
	}
	if (index == K_MAX_CTX)
		index = -1;

	return index;
}

/* -------------------------------------------------------------------- */
/*    Fonctions d'emission des trames TFTP                              */
/*				sendPacketTftp emet une trame sur une socket UDP        */
/*				transmit_ack prepare l'mission d'une trame TFTP ACK	*/
/*					 a partir des donnees du contexte de transfert		*/
/*				transmit_bloc_data prepare l'mission d'une trame	*/
/*                       TFTP DATA  a partir des donnees du contexte    */
/*						 de transfert                                   */
/*				transmit_error prepare l'mission d'une trame		*/
/*						 TFTP ERROR a partir des parametres             */
/* -------------------------------------------------------------------- */

int sendPacketTftp (int ind_ctx, unsigned char *buffer, int len)
{
	struct sockaddr_in to;
	int cr;
	long current_time;
	long *ptr_time = &current_time;
	int new_timeout;

	memset ( (char *)&to, 0, sizeof (struct sockaddr_in) );
	to.sin_family = AF_INET;
	to.sin_port = htons(ctx_tftp[ind_ctx].port_dest);
	to.sin_addr.s_addr = htonl(ctx_tftp[ind_ctx].ip_dest);
	cr = sendto (ctx_tftp[ind_ctx].sock_number, (char *)buffer, len, 0,
#ifdef PSOS
				(struct sockaddr_in *)&to, sizeof (struct sockaddr));
#else
				(struct sockaddr *)&to, sizeof (struct sockaddr));
#endif
	ctx_tftp[ind_ctx].tramesemises++;
	new_timeout = base_timeout * (nb_max_retry - ctx_tftp[ind_ctx].nbretry + 1);
	if (new_timeout > max_timeout)
		new_timeout = max_timeout;

	ctx_tftp[ind_ctx].timeout_emission = time (ptr_time) + new_timeout - 10;
	return cr;
}

int transmit_ack (int ind_ctx, int op)
{
	struct t_tftp_header tftpHeader;
	int cr;
#ifdef _SIMERROR
	int simuperte;
#endif

	ctx_tftp[ind_ctx].state = K_WAIT_DATA;
	if (op == K_REPEAT)
	{
		ctx_tftp[ind_ctx].tramesackretry++;
		ctx_tftp[ind_ctx].packetNumber--;
	}

	tftpHeader.op_tftp   = htons (K_TFTP_ACK);
	tftpHeader.u.num_pkt = htons (ctx_tftp[ind_ctx].packetNumber);
#ifdef _SIMERROR
	if (ctx_tftp[ind_ctx].port_dest == port_server_tftp)
		simuperte = perteAck % 50;

	if (simuperte != 4)
#endif
	cr = sendPacketTftp (ind_ctx, (unsigned char *)&tftpHeader, sizeof (struct t_tftp_header));
#ifdef _SIMERROR
	else
	{
		long current_time;
		long *ptr_time = &current_time;

		/* simulation de l'envoi de 2 fois la mme trame */
/*		cr = sendPacketTftp (ind_ctx, (unsigned char *)&tftpHeader, sizeof (struct t_tftp_header)); */
/*		cr = sendPacketTftp (ind_ctx, (unsigned char *)&tftpHeader, sizeof (struct t_tftp_header)); */
		/* simulation d'une perte de trame */
		ctx_tftp[ind_ctx].timeout_emission = time (ptr_time) + base_timeout;
		cr = sizeof (struct t_tftp_header);
	}
	perteAck++;
#endif

	if (cr >= 0)
/*
 *	{
 *		PRINTF (("\nError transmitting. ERRNO = %d", errno));
 *		PRINTF (("\nError for transmitting ACK TFTP Packet"));
 *	}
 *	else
*/
		ctx_tftp[ind_ctx].tramesack++;

	ctx_tftp[ind_ctx].packetNumber++;

	return cr;
}

void receive_request (int ind_ctx, unsigned char *buffer, int lg, struct sockaddr_in *from)
{
	char *pstr;
	int lg_name;
	char file_name[512];
	char mode[256];
	struct t_tftp_header *tftpHeader = (struct t_tftp_header *)buffer;

	ctx_tftp[ind_ctx].port_dest = from->sin_port;
	ctx_tftp[ind_ctx].ip_dest   = from->sin_addr.s_addr;
	ctx_tftp[ind_ctx].sock_number = ctx_tftp[0].sock_number;
	ctx_tftp[ind_ctx].accessFile  = ctx_tftp[0].accessFile;
	/* Extraction du nom du fichier */
	pstr = (char *)buffer + 2;
	lg_name = 2;
	pstr = decode_name (lg, &lg_name, file_name, pstr);
	/* Extraction du mode */
	pstr++;
	pstr = decode_name (lg, &lg_name, mode, pstr);
	if (lg_name == lg)
	{
		/* Trame TFTP incoherente */
		ctx_tftp[ind_ctx].error = K_ERROR_INCTRAME;
		transmit_error (ind_ctx, K_ERROR_INCTRAME);
		freeCtx (ind_ctx);
	}
	else
	{
		ctx_tftp[ind_ctx].nbretry = nb_max_retry;
		ctx_tftp[ind_ctx].packetNumber = 0;
		if (tftpHeader->op_tftp == K_TFTP_READ_REQUEST)
		{
			/* Ouverture du fichier */
			/* Initialisation du contexte de transfert */
			/* Lecture du permier bloc de 512 octets   */
			/* Envoi du premier bloc */
			/* Ouverture du fichier en write */
		  //			PRINTF (("\nTransfert du fichier %s\n", file_name));
			ctx_tftp[ind_ctx].fd_file =
				ctx_tftp[ind_ctx].accessFile->fopen ((char *)file_name, "r");
			if (ctx_tftp[ind_ctx].fd_file == (FILE *)NULL)
			{
				ctx_tftp[ind_ctx].error = K_ERROR_FILENOTFOUND;
				transmit_error (ind_ctx, K_ERROR_FILENOTFOUND);
				freeCtx (ind_ctx);
			}
			else
				transmit_bloc_data (ind_ctx, K_NEXT);
		}
		else
		{
		  //			PRINTF (("\nReception du fichier %s", file_name));
			/* Ouverture du fichier en write */
			ctx_tftp[ind_ctx].fd_file =
					ctx_tftp[ind_ctx].accessFile->fopen ((char *)file_name, "r");
			if (ctx_tftp[ind_ctx].fd_file == (FILE *)NULL)
			{
				ctx_tftp[ind_ctx].fd_file =
						ctx_tftp[ind_ctx].accessFile->fopen ((char *)file_name, "w");
				if (ctx_tftp[ind_ctx].fd_file == (FILE *)NULL)
				{
					ctx_tftp[ind_ctx].error = K_ERROR_ACCESSVIOLATION;
					transmit_error (ind_ctx, K_ERROR_ACCESSVIOLATION);
					freeCtx (ind_ctx);
				}
				else
				{
					ctx_tftp[ind_ctx].state = K_WAIT_DATA;
					transmit_ack (ind_ctx, K_NEXT);
				}
			}
			else
			{
				/* Le fichier existe : on interdit le transfert */
				ctx_tftp[ind_ctx].error = K_ERROR_FILEEXIST;
				transmit_error (ind_ctx, K_ERROR_FILEEXIST);
				freeCtx (ind_ctx);
			}
		}
	}
}

void receive_bloc_data (int ind_ctx, struct t_tftp_header *tftpHeader, int lg)
{
	int lg_data;

	if (tftpHeader->u.num_pkt == htons (ctx_tftp[ind_ctx].packetNumber))
	{
		ctx_tftp[ind_ctx].tramesdata++;
		/* Reception d'une trame de donnes */
		/* Ecriture des donnees dans le fichier */
		lg_data = lg - sizeof (struct t_tftp_header);
		ctx_tftp[ind_ctx].bytesreceived  += lg_data;
		if (lg_data != 0)
		{
			/* Appel de la routine d'ecriture des data dans le fichiers */
			ctx_tftp[ind_ctx].lg_data = 
					ctx_tftp[ind_ctx].accessFile->fwrite ((unsigned char *)tftpHeader + 
										sizeof (struct t_tftp_header),
										1, lg_data, ctx_tftp[ind_ctx].fd_file);
		}
		else
			ctx_tftp[ind_ctx].lg_data = 1;

		if (ctx_tftp[ind_ctx].lg_data == 0)
		{
			ctx_tftp[ind_ctx].error = K_ERROR_DISKFULL;
			transmit_error (ind_ctx, K_ERROR_DISKFULL);
			freeCtx (ind_ctx);
		}
		else
		{
			if ((ctx_tftp[ind_ctx].packetNumber % K_AVANCEMENT) == 0)
			{
				if ((ctx_tftp[ind_ctx].packetNumber % (K_AVANCEMENT * 40)) == 0)
					PRINTF (("\n"));
				PRINTF (("."));
			}
			transmit_ack (ind_ctx, K_NEXT);
			if (lg_data < sizeBlocData)
			{
				/* le transfert est termin */
				/*		Fermeture du fichier */
				/*		Libration du contexte de transfert */
				PRINTF (("\n"));
				ctx_tftp[ind_ctx].error = K_DONE;
				freeCtx (ind_ctx);
			}
		}
	}
}

int transmit_bloc_data (int ind_ctx, int op)
{
	struct t_tftp_header *tftpHeader = (struct t_tftp_header *)bufferTftp;
	unsigned char *pstr = bufferTftp;
	unsigned char *bufferatransferer = ctx_tftp[ind_ctx].buffer;
	int cr;
	int i;
#ifdef _SIMERROR
	int simuperte;
#endif

	if (op == K_NEXT)
	{
		ctx_tftp[ind_ctx].lg_data =
				ctx_tftp[ind_ctx].accessFile->fread (ctx_tftp[ind_ctx].buffer, 1,
									sizeBlocData,
									ctx_tftp[ind_ctx].fd_file);
		ctx_tftp[ind_ctx].bytestransmitted += ctx_tftp[ind_ctx].lg_data;
	}
	else
	{
		ctx_tftp[ind_ctx].tramesdataretry++;
		ctx_tftp[ind_ctx].packetNumber--;
	}

	ctx_tftp[ind_ctx].state = K_WAIT_ACK;
	ctx_tftp[ind_ctx].packetNumber++;

	tftpHeader->op_tftp = htons(K_TFTP_DATA);
	tftpHeader->u.num_pkt = htons(ctx_tftp[ind_ctx].packetNumber);

	pstr = bufferTftp + sizeof (struct t_tftp_header);

	for (i = 0; i < ctx_tftp[ind_ctx].lg_data; i++)
		*pstr++ = *bufferatransferer++;

#ifdef _SIMERROR
	if (ctx_tftp[ind_ctx].port_dest == port_server_tftp)
		simuperte = perteAck % 50;

	if (simuperte != 7)
#endif
	cr = sendPacketTftp (ind_ctx, (unsigned char *)bufferTftp,
						ctx_tftp[ind_ctx].lg_data + sizeof (struct t_tftp_header));
#ifdef _SIMERROR
	else
	{
		long current_time;
		long *ptr_time = &current_time;

		/* simulation de l'envoi de 2 fois la mme trame */
		/* cr = sendPacketTftp (ind_ctx, (unsigned char *)bufferTftp,
						ctx_tftp[ind_ctx].lg_data + sizeof (struct t_tftp_header));*/
		/* cr = sendPacketTftp (ind_ctx, (unsigned char *)bufferTftp,
						ctx_tftp[ind_ctx].lg_data + sizeof (struct t_tftp_header));*/
		/* simulation d'une perte de trame */
		ctx_tftp[ind_ctx].timeout_emission = time (ptr_time) + base_timeout;
		cr = ctx_tftp[ind_ctx].lg_data + sizeof (struct t_tftp_header);
	}
	perteData++;
#endif
	if (cr < 0)
		PRINTF (("\nError for transmitting DATA TFTP Packet"));
	else
		ctx_tftp[ind_ctx].tramesdata++;
	return cr;
}

int transmit_error (int ind_ctx, short error)
{
	struct t_tftp_header *tftpHeader = (struct t_tftp_header *)bufferTftp;
	unsigned char *pstr = bufferTftp;
	int cr;

	if ( (error > K_MAX_ERROR_NUM) || (error < 0) )
		error = K_ERROR_UNDEFINED;

	tftpHeader->op_tftp = htons (K_TFTP_ERROR);
	tftpHeader->u.error = htons (error);

	pstr += sizeof (struct t_tftp_header);
	strcpy ((char *)pstr, error_tftp_string[error]);

	PRINTF (("\nERROR TFTP = %d", error));

	cr = sendPacketTftp (ind_ctx, bufferTftp, strlen (error_tftp_string[error]) +
											sizeof (struct t_tftp_header) + 1);
/*
 *	if (cr < 0)
 *	{
 *		PRINTF (("\nError transmitting. ERRNO = %d", errno));
 *		PRINTF (("\nError for transmitting ERROR TFTP Packet"));
 *	}
*/
	return cr;
}

int transmit_tftp_request (int ind_ctx)
{
	int cr = 0;
	struct t_tftp_header *tftpHeader = (struct t_tftp_header *)bufferTftp;
	char *pstr = (char *)bufferTftp;
	int lg;

	ctx_tftp[ind_ctx].state = K_WAIT_ACK_REQUEST;
	if (ctx_tftp[ind_ctx].mode == K_MODE_READ)
	{
		tftpHeader->op_tftp = htons(K_TFTP_READ_REQUEST);
		ctx_tftp[ind_ctx].packetNumber = 1;
	}
	else
	{
		tftpHeader->op_tftp = htons(K_TFTP_WRITE_REQUEST);
	}

	pstr += 2;
	strcpy (pstr, ctx_tftp[ind_ctx].remote_file);
	pstr += strlen (ctx_tftp[ind_ctx].remote_file) + 1;
	strcpy (pstr, K_MODE_STR);

	pstr += strlen (K_MODE_STR);

	lg = strlen (K_MODE_STR) + strlen (ctx_tftp[ind_ctx].remote_file) + 2 + 2;
	cr = sendPacketTftp (ind_ctx, (unsigned char *)bufferTftp, lg);

/*
 *	if (cr < 0)
 *	{
 *		PRINTF (("\nError transmitting. ERRNO = %d", errno));
 *		PRINTF (("\nError for transmitting REQUEST TFTP Packet"));
 *	}
*/
	return cr;
}

void traitement_timeout ()
{
	int i;
	int cr;
	int decr;
	long current_time;
	long *ptr_time = &current_time;

	current_time = time (ptr_time);

	for (i = 1; i < K_MAX_CTX; i++)
	{
		decr = 0;
		if (ctx_tftp[i].state != K_FREE_STATE)
		{
			if (ctx_tftp[i].timeout_emission <  current_time)
			{
				if (ctx_tftp[i].nbretry != 0)
				{
#ifdef _TRACE_TFTP
					PRINTF (("\nReemission avec etat = %d", ctx_tftp[i].state));
#endif
					if (ctx_tftp[i].state == K_WAIT_DATA)
						cr = transmit_ack (i, K_REPEAT);
					else if (ctx_tftp[i].state == K_WAIT_ACK)
						cr = transmit_bloc_data (i, K_REPEAT);
					else if (ctx_tftp[i].state == K_WAIT_ACK_REQUEST)
						cr = transmit_tftp_request (i);
					else
						decr = -1;
					if ( decr  == 0)
						ctx_tftp[i].nbretry--;
				}
				else
					if (ctx_tftp[i].state != K_FREE_STATE)
					{
						/* Arret du transfert */
						ctx_tftp[i].error = K_ERROR_MAXRETRY;
						freeCtx (i);
					}
			}
		}
	}
}

int creer_socket_and_bind (int ctx_ind)
{
	struct sockaddr_in sa;
	int cr;

   	if ( (ctx_tftp[ctx_ind].sock_number = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
   	{
   		PRINTF (("\nCan't create the client socket"));
		ctx_tftp[ctx_ind].error = K_ERROR_UNDEFINED;
       	cr = -1;
	}
	else
	{
		memset( &sa, 0, sizeof( sa ) );
		sa.sin_family = AF_INET;
		sa.sin_port = 0;
		sa.sin_addr.s_addr = INADDR_ANY;
#ifdef PSOS
		cr = bind (ctx_tftp[ctx_ind].sock_number, (struct sockaddr_in *)&sa, sizeof (sa));
#else
		cr = bind (ctx_tftp[ctx_ind].sock_number, (struct sockaddr *)&sa, sizeof (sa));
#endif
		if (cr < 0)
		{
	   		PRINTF (("\nCan't bind the client socket"));
			ctx_tftp[ctx_ind].error = K_ERROR_UNDEFINED;
	   		cr = -1;
		}
	}
	return cr;
}

void start_tftp_transfer (void)
{
	int i;
	int cr;

	for (i = 0; i < K_MAX_CTX; i++)
		if ((ctx_tftp[i].state == K_WAIT_TRF) &&
			(ctx_tftp[i].mode != K_NO_MODE) )
	{
		cr = creer_socket_and_bind (i);
		if (cr == 0)
		{
		  //			PRINTF (("\nOuverture du fichier %s\n", ctx_tftp[i].local_file));
			if (ctx_tftp[i].mode == K_MODE_READ)
			{
				ctx_tftp[i].fd_file =
						ctx_tftp[i].accessFile->fopen (ctx_tftp[i].local_file, "r");
				if (ctx_tftp[i].fd_file  > (FILE *)NULL)
				{
					PRINTF (("\nfichier %s exist", ctx_tftp[i].local_file));
					ctx_tftp[i].error = K_ERROR_FILEEXIST;
					cr = -1;
				}
				else
				{
					ctx_tftp[i].fd_file =
							ctx_tftp[i].accessFile->fopen (ctx_tftp[i].local_file, "w");
					if (ctx_tftp[i].fd_file == (FILE *)0)
					{
       					ctx_tftp[i].error = K_ERROR_DISKFULL;
						cr = -1;
					}
				}
			}
			else if (ctx_tftp[i].mode == K_MODE_WRITE)
			{
				ctx_tftp[i].lg_data = sizeBlocData;
				ctx_tftp[i].fd_file =
						ctx_tftp[i].accessFile->fopen (ctx_tftp[i].local_file, "r");
				if (ctx_tftp[i].fd_file  == (FILE *)NULL)
				{
					PRINTF (("\nfichier %s not found", ctx_tftp[i].local_file));
					ctx_tftp[i].error = K_ERROR_FILENOTFOUND;
					cr = -1;
				}
			}
			else
			{
				ctx_tftp[i].error = K_ERROR_UNDEFINED;
				cr = -1;
			}
		}
		if (cr == 0)
		  {
		    PRINTF (("OK--> commencer transfert.\n"));
		    transmit_tftp_request (i);	
		  }
		else
		  {
		    PRINTF (("Pb--> liberation contexte.\n"));
		    freeCtx (i);
		  }
	}
}

/* -------------------------------------------------------------------- */
/*    Fonction de traitement de la recption d'une trame UDP             */
/* -------------------------------------------------------------------- */

void receive_datagram_tftp (unsigned char *buffer, int lg,
							struct sockaddr_in *from, int local_sock)
{
	int ind_ctx;
	struct t_tftp_header *tftpHeader = (struct t_tftp_header *)buffer;
	int cr;
	struct sockaddr_in local_from;

	from->sin_port = htons(from->sin_port);
	from->sin_addr.s_addr = htonl(from->sin_addr.s_addr);
	tftpHeader->op_tftp = htons(tftpHeader->op_tftp);
	memcpy ((char *)&local_from, (char *)from, sizeof (struct sockaddr_in) );

#ifdef _TRACE_TFTP
	//	PRINTF (("\nReception d'un paquet UDP. Port Emetteur = %d IP Addr = %08x Lg = %d",
	//					from->sin_port, from->sin_addr.s_addr, lg));
	//	PRINTF (("\nOperation TFTP = %d", tftpHeader->op_tftp));
#endif

	ind_ctx = searchCtx (from, local_sock);

	if (ind_ctx == -1)
	{
		/* recherche a partir du numero de socket serveur */
		local_from.sin_port = port_server_tftp;
		ind_ctx = searchCtx (&local_from, local_sock);
		/* Si le contexte est trouve on met a jour le contexte */
		if (ind_ctx != -1)
			ctx_tftp[ind_ctx].port_dest = from->sin_port;
	}

	if (ind_ctx == -1)
	{
		if ((tftpHeader->op_tftp == K_TFTP_READ_REQUEST) ||
			(tftpHeader->op_tftp == K_TFTP_WRITE_REQUEST) )
		{
			/* Recherche d'un contexte de libre */
			ind_ctx = searchNewentry ();
			if (ind_ctx < 0)
			{
				ind_ctx = K_RESERVE_CTX;

				ctx_tftp[ind_ctx].port_dest = from->sin_port;
				ctx_tftp[ind_ctx].ip_dest   = from->sin_addr.s_addr;
				ctx_tftp[ind_ctx].state     = K_WAIT_FREE;
				ctx_tftp[ind_ctx].fd_file = (FILE *)0;
				ctx_tftp[ind_ctx].error = K_ERROR_NOSUCHUSER;
				cr = creer_socket_and_bind (ind_ctx);
				if (cr == 0)
					transmit_error (ind_ctx, K_ERROR_NOSUCHUSER);
				freeCtx (ind_ctx);
			}
			else
				receive_request (ind_ctx, buffer, lg, from);
		}
		else
		{
			/* on n'emet une trame d'erreur que sur reception
			 * d'une trame de donnees. Sur reception d'une trame ACK
			 * aucun traitement n'est effectue
			*/
			if (tftpHeader->op_tftp == K_TFTP_DATA)
			{
				/* Emission d'une trame d'erreur */
				/* Erreur : operation illegale   */
				ind_ctx = K_RESERVE_CTX;

				ctx_tftp[ind_ctx].port_dest = from->sin_port;
				ctx_tftp[ind_ctx].ip_dest   = from->sin_addr.s_addr;
				ctx_tftp[ind_ctx].state     = K_WAIT_FREE;
				ctx_tftp[ind_ctx].error = K_ERROR_ILLEGALOP;
				ctx_tftp[ind_ctx].fd_file = (FILE *)0;
				cr = creer_socket_and_bind (ind_ctx);
				if (cr == 0)
					transmit_error (ind_ctx, K_ERROR_ILLEGALOP);
				freeCtx (ind_ctx);
			}
		}
	}
	else
	{
		ctx_tftp[ind_ctx].nbretry = nb_max_retry;
		ctx_tftp[ind_ctx].tramesrecues++;
		if ((tftpHeader->op_tftp == K_TFTP_READ_REQUEST) ||
			(tftpHeader->op_tftp == K_TFTP_WRITE_REQUEST) )
		{
			/* Emission d'une trame d'erreur */
			/* Erreur : operation illegale   */
			ctx_tftp[ind_ctx].error = K_ERROR_ILLEGALOP;
			transmit_error (ind_ctx, K_ERROR_ILLEGALOP);
			freeCtx (ind_ctx);
		}
		else
		{
			if (tftpHeader->op_tftp == K_TFTP_DATA)
				receive_bloc_data (ind_ctx, tftpHeader, lg);
			else if (tftpHeader->op_tftp == K_TFTP_ACK)
			{
				/* Reception d'une trame ACK */
				/*	Controle du numero acquitt */
				/*	Si OK : envoi du prochain bloc */
				/*		sinon rejet de la trame */
				if (tftpHeader->u.num_pkt == htons (ctx_tftp[ind_ctx].packetNumber))
				{
					ctx_tftp[ind_ctx].tramesack++;
					if (ctx_tftp[ind_ctx].lg_data < sizeBlocData)
					{
						/* le transfert est termine :
						 *		Fermeture du fichier,
						 *		Liberation du contexte de transfert
						*/
						PRINTF (("\n"));
						ctx_tftp[ind_ctx].error = K_DONE;
						freeCtx (ind_ctx);
					}
					else
					{
						if ((ctx_tftp[ind_ctx].packetNumber % K_AVANCEMENT) == 0)
						{
							if ((ctx_tftp[ind_ctx].packetNumber % (K_AVANCEMENT * 40)) == 0)
								PRINTF (("\n"));
							PRINTF (("."));
						}
						/* Envoi du bloc suivant */
						transmit_bloc_data (ind_ctx, K_NEXT);
					}
				}
			}
			else if (tftpHeader->op_tftp == K_TFTP_ERROR)
			{
				/* Arret du transfert :                                      */
				/*	Liberation du contexte                                   */
				/*	Renvoi d'un compte rendu au demandeur du service         */
				/*	s'il s'agit de la fonction TFTP client                   */
				/*  incrementation d'un compteur s'il s'agit de TFTP serveur */
				ctx_tftp[ind_ctx].error = htons(tftpHeader->u.error);
				freeCtx (ind_ctx);
			}
			else
			{
				/* Emission d'une trame d'erreur */
				/* Erreur : operation illegale   */
				transmit_error (ind_ctx, K_ERROR_ILLEGALOP);
				ctx_tftp[ind_ctx].error = K_ERROR_ILLEGALOP;
				freeCtx (ind_ctx);
			}
		}
	}
}

int tftp_request (int mode, char *local_file_name, char *remote_file,
					unsigned long ip_server, unsigned short port_server,
					t_fct_callBack callback, void *args_callback,
					t_access_file *accessFile)
{
	int ind_ctx;
	int cr;

	cr = 0;
	if (g_comportement != K_TFTP_SERVEUR)
	{
		ind_ctx = searchNewentry ();
		if (ind_ctx >= 0)
		{
			ctx_tftp[ind_ctx].nbretry = nb_max_retry;
			if (port_server == 0)
				ctx_tftp[ind_ctx].port_dest = port_server_tftp;
			else
				ctx_tftp[ind_ctx].port_dest = port_server;

			ctx_tftp[ind_ctx].ip_dest   = ip_server;
			strcpy (ctx_tftp[ind_ctx].local_file, local_file_name);
			strcpy (ctx_tftp[ind_ctx].remote_file, remote_file);
			ctx_tftp[ind_ctx].state = K_WAIT_TRF;
			ctx_tftp[ind_ctx].mode = mode;
			ctx_tftp[ind_ctx].callback = callback;
			ctx_tftp[ind_ctx].args_callBack = args_callback;
			if (accessFile == (t_access_file *)0 )
				ctx_tftp[ind_ctx].accessFile = &defaultAccessFile;
			else
				ctx_tftp[ind_ctx].accessFile = accessFile;
			/* ouverture du fichier */
			if ((mode != K_MODE_WRITE) && (mode != K_MODE_READ))
				cr = -1;
			if (cr == -1)
				freeCtx (ind_ctx);
		}
		else
			cr = -1;
	}
	else
		cr = -1;

	return cr;
}

int init_tftp (int comportement, t_access_file *accessFile,
				unsigned short port_server, t_conf_tftp *conf_tftp)
{
	struct sockaddr_in sa;
	int ret = 0;
	int crFct;
	int lg_from, i;
	int *plg_from = &lg_from;

	if (conf_tftp->size_bl_data > K_MAX_DATA_TFTP)
		sizeBlocData = K_MAX_DATA_TFTP;
	else if (conf_tftp->size_bl_data < K_MIN_DATA_TFTP)
		sizeBlocData = K_MIN_DATA_TFTP;
	else
		sizeBlocData = conf_tftp->size_bl_data;

	g_comportement = comportement;
	base_timeout = conf_tftp->base_timeout;
	max_timeout = conf_tftp->max_timeout;
	nb_max_retry = conf_tftp->nb_max_retry;

	memset (&ctx_tftp, 0, (sizeof (struct t_ctx_tftp) * K_MAX_CTX) );
	defaultAccessFile.fopen  = fopen;
	defaultAccessFile.fclose = fclose;
	defaultAccessFile.fread  = fread;
	defaultAccessFile.fwrite = fwrite;
	if (port_server == 0)
		port_server_tftp = K_PORT_TFTP_SERVER;
	else
		port_server_tftp = port_server;

	if (comportement != K_TFTP_CLIENT)
	{
		ctx_tftp[0].state = K_WAIT_TRF;
		/* creation de la socket du serveur TFTP */
   		if ( (ctx_tftp[0].sock_number = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
   		{
#ifdef _VALNT
			ret = WSAGetLastError ();
			PRINTF (("\nSocket Create Failed -> %ld",ret));
#else
   	 		PRINTF (("\nCan't create the TFTP Server\n"));
#endif
			ret = -1;
		}
		else
		{
			ctx_tftp[0].mode = K_NO_MODE; 
			ctx_tftp[0].timeout_emission = 0x7FFFFFFF;
			if (accessFile == (t_access_file *)0)
				ctx_tftp[0].accessFile = &defaultAccessFile;
			else
				ctx_tftp[0].accessFile = accessFile;

			memset( &sa, 0, sizeof( sa ) );
			sa.sin_family = AF_INET;
			sa.sin_port = port_server_tftp;
			sa.sin_addr.s_addr = INADDR_ANY;
#ifdef PSOS
			crFct = bind (ctx_tftp[0].sock_number, (struct sockaddr_in *)&sa, sizeof (sa));
#else
			crFct = bind (ctx_tftp[0].sock_number, (struct sockaddr *)&sa, sizeof (sa));
#endif
			if (crFct < 0)
   			{
   	 			PRINTF (("\nCan't bind the TFTP Server. CR = %08x", errno));
				ret = -1;
			}
		}
	}
	else
	{
		ctx_tftp[0].state = K_WAIT_FREE;
		ctx_tftp[0].timeout_emission = 0x7FFFFFFF;
		if (accessFile == (t_access_file *)0)
			ctx_tftp[0].accessFile = &defaultAccessFile;
		else
			ctx_tftp[0].accessFile = accessFile;
	}

	for (i = 1; i < K_MAX_CTX; i++)
	{
		ctx_tftp[i].state = K_FREE_STATE;
		ctx_tftp[i].timeout_emission = 0x7FFFFFFF;
	}
	return ret;
}

int initialize_rfd (fd_set *rfds)
{
	int biggest_sock = 0;
	int i;

	FD_ZERO(rfds);
	biggest_sock = 0;
	for (i = 0; i < K_MAX_CTX; i++)
	{
		if (ctx_tftp[i].sock_number != 0)
		{
			FD_SET(ctx_tftp[i].sock_number, rfds);
			if (ctx_tftp[i].sock_number > biggest_sock)
				biggest_sock = ctx_tftp[i].sock_number;
		}
	}
	return biggest_sock;

}
int run_tftp (int parametre_arret)
{
	fd_set      rfds;
	int      biggest_sock;
	struct timeval timeout;
	unsigned char BufferUdp[K_SIZE_BUFFER];
	struct sockaddr_in from;
	int ret = 0;
	int crFct = 0;
	int sock;
	int i;
	int lg_data;
	int lg_from;
	int *plg_from = &lg_from;

	arret_process_tftp = 0;
	if (ret == 0)
	{
		while (1 == 1)
		{
			biggest_sock = initialize_rfd (&rfds);
			if (biggest_sock == 0)
			{
				start_tftp_transfer ();
				biggest_sock = initialize_rfd (&rfds);
			}

			timeout.tv_sec = base_timeout;
			timeout.tv_usec = 0;
			ret = select (biggest_sock+1, (fd_set *)&rfds, (fd_set *)NULL, (fd_set *)NULL,
						  (struct timeval *)&timeout);
			if (ret < 0)
			{
#ifdef _VALNT
				PRINTF (("\nErreur lors de l'appel de la primitive Select -> %ld", WSAGetLastError() ));
#else
				PRINTF (("\nErreur lors de l'appel de la primitive Select -> %ld", errno));
#endif
				/* Erreur lors de l'appel de la primitive select */
				/* liberation de tous les contextes */
				for (i = 1; i < K_MAX_CTX; i++)
					freeCtx (i);
				crFct = -1;
				break;
			}
			if (ret > 0)
			{
				for (sock = 1; sock <= biggest_sock; sock++)
				{
					if (FD_ISSET (sock, &rfds) != 0)
					{
						/* Reception d'un paquet UDP */
						/* Lecture du paquet, recherche du contexte associe et activation du traitement */
						*plg_from = sizeof (from);
						lg_data = recvfrom (sock, (char *)BufferUdp, K_SIZE_BUFFER, 0,
#ifdef PSOS
											(struct sockaddr_in *)&from, plg_from);
#else
											(struct sockaddr *)&from, plg_from);
#endif 

						if ( lg_data > 0)
 							receive_datagram_tftp (BufferUdp, lg_data, &from, sock);
						FD_CLR (sock, &rfds);
					}
				}
			}
			if (ret == 0)
				traitement_timeout ();
			start_tftp_transfer ();
			/* Permet de sortir de la boucle dans le mode TFTP CLIENT */
			if ((parametre_arret != 0) &&  (arret_process_tftp != 0) && g_comportement == K_TFTP_CLIENT)
				break;
		}
	}
	return crFct;
}
/* for compatibility compiler */
