/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.command.lib;

// Package dependencies.
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.openccm.ast.api.FileScope;
import org.objectweb.openccm.command.lib.CompilerGeneratorBase;

/**
 * Implementation of the cif_jimpl command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class CIFtoJavaImpl
     extends CompilerGeneratorBase
  implements org.objectweb.corba.command.api.CIFtoJavaImpl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Output directory Option. */
    private org.objectweb.util.cmdline.api.OptionArgument odir_; // -d <directory>

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** The default constructor. */
    public
    CIFtoJavaImpl()
    {
        // Calls the CompilerGeneratorBase constructor.
        super(new DefaultCommandLine("cif_jimpl",
                                     "file",
                                     new String[] {
                                        "Compile an OMG CIDL file and generate its associated Java implementation templates."
                                     },
                                     true)
             );

        // Inits internal state.
        // Specify output base directory
        odir_ = new DefaultOptionArgument(
                        new String[] { "-d" },
                        "outputdir",
                        new String[] {
                          "Generate Java implementations templates into directory <outputdir>,",
                          "default is generated/"
                        },
                        "generated" );
        getCommandLine().addOption(odir_);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Application
    //
    // ==================================================================

    /**
     * Runs the application.
     *
     * @param args - The command line arguments.
     *
     * @return The status.
     */
    public int
    run(java.lang.String[] args)
    {
        String filename = args[0];

        // Inits the OpenCCM Abstract Syntax Tree.
        if(!initAST())
            return -1;

        // Compiles the file.
        FileScope fileScope = compile(filename);
        if(fileScope == null)
            return -1;

        // Generates Java implementations templates
        if (!generate_cif_jimpl(fileScope,
                                filename, // Input File
                                getOutputDirectory())) // Base directory for Java Mapping
            return -1;

        // All is OK.
        return 0;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnIR3
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.CommandOnAST
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Compiler
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.command.api.Generator
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.command.api.CIDLtoCIF
    //
    // ==================================================================

    /**
     * Obtains the output directory.
     *
     * @result The output directory.
     */
    public String
    getOutputDirectory()
    {
        return odir_.getArgument();
    }

    /**
     * Generates Java implementations template files for the CIF.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope The source AST FileScope returned by the compile() method.
     * @param inputfile - The source file.
     * @param base_dir - The base directory for java mapping.
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_cif_jimpl(org.objectweb.openccm.ast.api.FileScope filescope,
                       String inputfile,
                       String base_dir)
    {
       // Launchs the CIF_Jimpl generator.
        org.objectweb.corba.generator.cif.api.CIF_JavaImplGenerator gen =
            new org.objectweb.corba.generator.cif.lib.CIF_JavaImplGenerator(getAST());
        try
        {
            gen.cif_to_java_impl(base_dir, filescope);
        }
        catch(org.objectweb.openccm.generator.common.lib.GenerationException ex)
        {
            report_exception(ex);
            return false;
        }
        return true;
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        CIFtoJavaImpl cif2jimpl = new CIFtoJavaImpl();
        cif2jimpl.runMain(args);
    }
}
