/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif.lib;

// Package dependencies.
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.OperationDecl;
import org.objectweb.openccm.ast.api.InterfaceDecl;


/**
 * This class generates CIF implementation templates files for a component home.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class HomeImplMapping
  implements org.objectweb.corba.generator.cif.api.HomeImplMapping
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Utility class to convert types in Java*/
    static public org.objectweb.openccm.generator.translator.idl2java.api.IDL_JavaTranslator translator_;

    /** The composition containing the home. */
    private org.objectweb.openccm.ast.utils.api.CompositionInfo comp_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param composition - The composition to manage.
     **/
    public HomeImplMapping(org.objectweb.openccm.ast.api.CompositionDecl composition)
    {
        comp_ = new org.objectweb.openccm.ast.utils.lib.CompositionInfo(composition);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Inits the static field.
     **/
    static
    {
        translator_ = new org.objectweb.openccm.generator.translator.idl2java.lib.IDL_JavaTranslator();
    }

    /**
     * Map the "create_executor_segment" operation.
     *
     * @param clazz - Add methods to the Home class.
     **/
    private void
    map_create_segments_op(ClassObject clazz)
    {
        MethodObject method = null;
        ParameterObject param = null;

        // Map "read_value" operation
        method = new MethodObjectImpl();
        method.addComment(" Method for the org.objectweb.ccm.runtime.cif.api.SegmentHome interface.");
        method.addComment(" ");
        method.addComment(" Create an executor segment from its identifier.");
        method.addComment(" ");
        method.addComment(" @param segid - The executor segment identifier.");
        method.addComment(" ");
        method.addComment(" @return - The executor segment created.");
        method.setName("create_executor_segment");
        method.setReturnType("org.omg.Components.ExecutorSegmentBase");

        method.getImpl().setMacro("HOME_CREATE_SEGMENT");
        method.getImpl().addContextValue("package", translator_.getPackage(comp_.getHomeExecutor()));
        method.getImpl().addContextValue("segments", comp_.getSegments());
        method.getImpl().addContextValue("component", comp_.getComponent());

        param = new ParameterObjectImpl();
        param.setName("segid");
        param.setType("int");
        method.addParameter(param);
        clazz.addMethod(method);
    }

    /**
     * Map the "create_home" operation.
     *
     * @param clazz - Add methods to the Home class.
     **/
    private void
    map_create_home_op(ClassObject clazz)
    {
        MethodObject method = null;

        method = new MethodObjectImpl();
        method.addComment(" This method is called by the OpenCCM Component Server");
        method.addComment(" to create a home instance.");
        method.addComment(" ");
        method.addComment(" @return - The created home.");
        method.setName("create_home");
        method.setReturnType("org.omg.Components.HomeExecutorBase");
        method.setStatic(true);

        method.getImpl().setMacro("HOME_CREATE_HOME");
        method.getImpl().addContextValue("package", translator_.getPackage(comp_.getHomeExecutor()));
        method.getImpl().addContextValue("home_name", comp_.getHome().getName() );

        clazz.addMethod(method);
    }

    /**
     * Map business operations.
     *
     * @param clazz - Add methods to the Home class.
     **/
    private void
    businessOperations(ClassObject clazz)
    {
        Declaration[] decls = null;
        OperationDecl op = null;
        InterfaceDecl itf = null;
        MethodObject method = null;

        itf = comp_.getHome().getLocalMapping();
        decls = itf.getContents(false, DeclarationKind.dk_operation);

        for (int i=0;i<decls.length;i++)
        {
            op = (org.objectweb.openccm.ast.api.OperationDecl)decls[i];

            if ( (!op.getParent().getAbsoluteName().startsWith("::Components")) &&
                 (!op.getName().equals("create")) )
            {
                method = translator_.map_operation(op);
                method.getImpl().setMacro("BUSINESS_OP");
                method.getImpl().addContextValue("translator", translator_);
                method.getImpl().addContextValue("obj", op);
                clazz.addMethod(method);
            }
        }

    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.generator.cif.api.HomeImplMapping.
    //
    // ==================================================================

    /**
     * Generate Home template class.
     *
     * @param repository - The java repository.
     **/
    public void
    toJavaHomeTemplate(Repository repository)
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;

        clazz = new ClassObjectImpl(comp_.getHome().getName()+"Impl");
        clazz.addComment(" This is the CIDL-based implementation of ");
        clazz.addComment(" OMG IDL3 "
                         + comp_.getHome().getId()
                         + " home type.");
        clazz.addComment("");
        clazz.addComment(" This class inherits from the generated home executor skeleton class");
        clazz.addComment(" defined by OMG CIDL "
                         + translator_.getAbsoluteName( comp_.getHomeExecutor() )
                         );
        clazz.addComment("");
        clazz.addComment(" @author OpenCCM CIF_Jimpl Compiler.");
        clazz.setModifier(ModifierKindImpl.mk_public);
        clazz.setPackage( translator_.getPackage(comp_.getHomeExecutor()) );
        clazz.addInheritedObject( translator_.getAbsoluteName(comp_.getHomeExecutor()) );

        // Add the default constructor
        ct = new ConstructorObjectImpl();
        clazz.addConstructor(ct);

        // Add Home methods
        map_create_segments_op(clazz);
        map_create_home_op(clazz);
        businessOperations(clazz);

        // Add object to the java repository
        repository.addObject(clazz);
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
