/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif.lib;

// Package dependencies.
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
import org.objectweb.openccm.ast.api.SegmentDecl;


/**
 * This class generates CIF implementation templates files for component segments.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class SegmentImplMapping
  implements org.objectweb.corba.generator.cif.api.SegmentImplMapping
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Utility class to convert types in Java*/
    static public org.objectweb.openccm.generator.translator.idl2java.api.IDL_JavaTranslator translator_;

    /** The composition containing the segment. */
    private org.objectweb.openccm.ast.utils.api.CompositionInfo comp_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param composition - The composition to manage.
     **/
    public SegmentImplMapping(org.objectweb.openccm.ast.api.CompositionDecl composition)
    {
        comp_ = new org.objectweb.openccm.ast.utils.lib.CompositionInfo(composition);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Inits the static field.
     **/
    static
    {
        translator_ = new org.objectweb.openccm.generator.translator.idl2java.lib.IDL_JavaTranslator();
    }

    /**
     * Map segment operation (facets and sinks interfaces).
     *
     * @param clazz - Add methods to the Home class.
     * @param seg - Add methods to the Home class.
     **/
    private void
    businessOperations(ClassObject clazz,
                       SegmentDecl seg)
    {
        String[] facets = null,
                 sinks = null;

        // For each facet implemented, map corresponding interface operations
        facets = seg.getFacetList().getStrings();
        for (int i=0; i<facets.length; i++)
        {
            org.objectweb.openccm.ast.api.InterfacePortDecl itf_port = null;
            org.objectweb.openccm.ast.api.ComponentDecl current = null;

            current = comp_.getComponent();
            while ((current != null) && (itf_port == null))
            {
                itf_port = (org.objectweb.openccm.ast.api.InterfacePortDecl)current.lookup(facets[i]);
                current = current.getBaseComponent();
            }
            translator_.mapInterfaceOps( clazz,
                                         itf_port.getLocalMapping() );
        }

        // For each sink implemented, map corresponding interface operations
        sinks = seg.getSinkList().getStrings();
        for (int i=0; i<sinks.length; i++)
        {
            org.objectweb.openccm.ast.api.EventPortDecl event_port = null;
            org.objectweb.openccm.ast.api.ComponentDecl current = null;

            current = comp_.getComponent();
            while ((current != null) && (event_port == null))
            {
                event_port = (org.objectweb.openccm.ast.api.EventPortDecl)current.lookup(sinks[i]);
                current = current.getBaseComponent();
            }
            translator_.mapInterfaceOps( clazz,
                                         event_port.getEvent().getLocalConsumerMapping() );
        }
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.generator.cif.api.SegmentImplMapping.
    //
    // ==================================================================

    /**
     * Generate Component segment template classes.
     *
     * @param repository - The java repository.
     **/
    public void
    toJavaSegmentTemplates(Repository repository)
    {
        ClassObject clazz = null;
        ConstructorObject ct = null;
        SegmentDecl[] segments = null;

        segments =  comp_.getSegments();
        for (int i=0; i<segments.length; i++)
        {
            clazz = new ClassObjectImpl(segments[i].getName()+"Impl");
            clazz.addComment(" This is the CIDL-based implementation of the");
            clazz.addComment(" OMG IDL3 "
                            + segments[i].getId()
                            + " segment type.");
            clazz.addComment("");
            clazz.addComment(" @author OpenCCM CIF_Jimpl Compiler.");
            clazz.setModifier(ModifierKindImpl.mk_public);
            clazz.setPackage( translator_.getPackage(comp_.getHomeExecutor()) );
            clazz.addInheritedObject( translator_.getPackage(comp_.getHomeExecutor()) +
                                      "." + segments[i].getName() );

            // Add the default constructor
            ct = new ConstructorObjectImpl();
            clazz.addConstructor(ct);

            // Add Component methods
            businessOperations(clazz, segments[i]);

            // Add object to the java repository
            repository.addObject(clazz);
        }
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
