/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.dependencies.lib;

// Package dependencies.
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.ComponentDecl;
import org.objectweb.openccm.ast.api.HomeDecl;
import org.objectweb.openccm.ast.api.InterfaceDecl;
import org.objectweb.openccm.ast.api.InterfacePortDecl;
import org.objectweb.openccm.ast.api.EventPortDecl;
import org.objectweb.openccm.ast.api.TypeRef;
import org.objectweb.openccm.ast.api.TypeKind;
import org.objectweb.openccm.ast.api.EventDecl;
import org.objectweb.openccm.ast.api.OperationDecl;
import java.util.Set;


/**
 * This class allows you to get classes dependencies for an IDL3 declaration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class IDL3DeclarationDependencies
     extends IDL2DeclarationDependencies
  implements org.objectweb.corba.generator.dependencies.api.IDL3DeclarationDependencies
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     */
    public
    IDL3DeclarationDependencies()
    {
        // Init internal state
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for org.objectweb.openccm.generator.cidl.api.IDL3DeclarationDependencies
    //
    // ==================================================================

    /**
     * Get Home File dependencies.
     * This includes all inherited Homes.
     *
     * @param home - The home declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getHomeDependencies(HomeDecl home,
                        Set visited)
    {
        Set files = null;
        String path = null;
        InterfaceDecl[] bases = null;

        // Test if this declaration was visited
        if ( visited.contains(home) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(home);
        }

        path = _translator.getAsDirectory(home);
        files = new java.util.HashSet();

        // Home local Interfaces
        addStantardIdlFiles(files, path + "CCM_" + home.getName() + "Explicit");
        addOperationsClass(files, path + "CCM_" + home.getName() + "Explicit");
        addStantardIdlFiles(files, path + "CCM_" + home.getName() + "Implicit");
        addOperationsClass(files, path + "CCM_" + home.getName() + "Implicit");
        addStantardIdlFiles(files, path + "CCM_" + home.getName());
        addOperationsClass(files, path + "CCM_" + home.getName());


        // Home remote Interfaces
        addStantardIdlFiles(files, path + home.getName() + "Explicit");
        addOperationsClass(files, path + home.getName() + "Explicit");
        addPoaIdlFiles(files, path + home.getName() + "Explicit");
        addStubClass(files, path + home.getName() + "Explicit");
        addStantardIdlFiles(files, path + home.getName() + "Implicit");
        addOperationsClass(files, path + home.getName() + "Implicit");
        addPoaIdlFiles(files, path + home.getName() + "Implicit");
        addStubClass(files, path + home.getName() + "Implicit");
        addStantardIdlFiles(files, path + home.getName());
        addOperationsClass(files, path + home.getName());
        addPoaIdlFiles(files, path + home.getName());
        addStubClass(files, path + home.getName());

        // Home skeletons
        addObjectClass(files, path + home.getName() + "CCM");
        addObjectClass(files, path + home.getName() + "SkeletonInterceptor");
        addObjectClass(files, path + home.getName() + "StubInterceptor");

        // Home Meta-Information
        addObjectClass(files, path + home.getName() + "_homeMI");
        addObjectClass(files, path + home.getName() + "_interfaceMI");

        // Home implemented interfaces
        bases = home.getSupportedInterfaceList().getInterfaces();
        for (int i=0; i<bases.length; i++)
        {
            files.addAll( getInterfaceDependencies(bases[i], true, visited) );
        }

        // Home base home(s)
        if (home.getBaseHome() != null)
        {
            files.addAll( getHomeDependencies(home.getBaseHome(), visited) );
        }

        // Home contents dependencies
        files.addAll( getContentsDependencies(home, visited) );

        return files;
    }

    /**
     * Get Component File dependencies.
     * This includes all inherited Components.
     *
     * @param comp - The Component declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getComponentDependencies(ComponentDecl comp,
                             Set visited)
    {
        Set files = null;
        String path = null;
        InterfaceDecl[] bases = null;

        files = new java.util.HashSet();

        // Test if this declaration was visited
        if ( visited.contains(comp) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(comp);
        }

        path = _translator.getAsDirectory(comp);

        // Component local interfaces
        addStantardIdlFiles(files, path + "CCM_" + comp.getName());
        addOperationsClass(files, path + "CCM_" + comp.getName());
        addStantardIdlFiles(files, path + "CCM_" + comp.getName() + "_Context");
        addOperationsClass(files, path + "CCM_" + comp.getName() + "_Context");
        addStantardIdlFiles(files, path + "CCM_" + comp.getName() + "_Executor");
        addOperationsClass(files, path + "CCM_" + comp.getName() + "_Executor");

        // Component remote interfaces
        addStantardIdlFiles(files, path + comp.getName());
        addOperationsClass(files, path + comp.getName());
        addPoaIdlFiles(files, path + comp.getName());
        addStubClass(files, path + comp.getName());

        // Component skeletons
        addObjectClass(files, path + comp.getName() + "CCM");
        addObjectClass(files, path + comp.getName() + "MonolithicWrapper");
        addObjectClass(files, path + comp.getName() + "SkeletonInterceptor");
        addObjectClass(files, path + comp.getName() + "StubInterceptor");

        // Component Meta-Information
        addObjectClass(files, path + comp.getName() + "_componentMI");
        addObjectClass(files, path + comp.getName() + "_interfaceMI");

        // Component implemented interfaces
        bases = comp.getSupportedInterfaceList().getInterfaces();
        for (int i=0; i<bases.length; i++)
        {
            files.addAll( getInterfaceDependencies(bases[i], true, visited) );
        }

        // Component base component(s)
        if (comp.getBaseComponent() != null)
        {
            files.addAll( getComponentDependencies(comp.getBaseComponent(), visited) );
        }

        // Component contents dependencies
        files.addAll( getContentsDependencies(comp, visited) );

        return files;
    }

    /**
     * Get Interface Port Declaration File dependencies.
     * This includes all inherited interfaces.
     *
     * @param itf_port - The interface port declaration.
     * @param server - If true, include server class files.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getInterfacePortDependencies(InterfacePortDecl itf_port,
                                 boolean server,
                                 Set visited)
    {
        Set files = null;
        String path = null,
               name = null;

        // Test if this declaration was visited
        if ( visited.contains(itf_port) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(itf_port);
        }

        path = _translator.getAsDirectory( itf_port.getParent() );
        files = new java.util.HashSet();
        name = itf_port.getInterface().getName();

        // Add CCM local interfaces
        if (server)
        {
            addStantardIdlFiles( files, path + "CCM_" + name );
            addOperationsClass( files, path + "CCM_" + name );
        }

        // Add CCM interface skeletons
        files.addAll( getInterfaceDependencies(itf_port.getInterface(), server, visited) );
        if (server)
        {
            addObjectClass(files, path + name + "SkeletonInterceptor");
            addObjectClass(files, path + name + "StubInterceptor");
        }

        // Interface Meta-Information
        if (server)
        {
            addObjectClass(files, path + name + "_interfaceMI");
        }

        return files;
    }

    /**
     * Get Event port Declaration File dependencies.
     * This includes all inherited interfaces.
     *
     * @param event_port - The event port declaration.
     * @param server - If true, include server class files.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getEventPortDependencies(EventPortDecl event_port,
                             boolean server,
                             Set visited)
    {
        Set files = null;
        String path = null,
               name = null;

        // Test if this declaration was visited
        if ( visited.contains(event_port) )
        {
            return files;
        }
        else
        {
            // Add this to visited declarations
            visited.add(event_port);
        }

        path = _translator.getAsDirectory( event_port.getEvent() );
        name = event_port.getEvent().getName();
        files = new java.util.HashSet();

        // Add CCM local interfaces
        if (server)
        {
            addStantardIdlFiles( files, path + "CCM_" + name + "Consumer" );
            addOperationsClass( files, path + "CCM_" + name + "Consumer" );
        }

        // Add CCM interface skeletons
        addStantardIdlFiles( files, path + name + "Consumer" );
        addStubClass( files, path + name + "Consumer" );
        addOperationsClass( files, path + name + "Consumer" );
        if (server)
        {
            addPoaIdlFiles( files, path + name + "Consumer" );
            addObjectClass(files, path + name + "ConsumerSkeletonInterceptor");
            addObjectClass(files, path + name + "ConsumerStubInterceptor");
            addObjectClass(files, path + name + "ConsumerWrapper");
        }

        // Add event and event base event dependencies
        files.addAll( getValueTypeDependencies(event_port.getEvent(), visited) );
        addObjectClass(files, path + name + "FactoryHelper");

        return files;
    }

    /**
     * Get dependencies for an IDL3 declaration.
     *
     * @param decl - Get dependencies for this declaration.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getDeclarationDependencies(Declaration decl,
                               Set visited)
    {
        Set files = null;

        files = new java.util.HashSet();

        // Test if this declaration was visited (to avoid following tests)
        if ( visited.contains(decl) )
        {
            return files;
        }

        if (decl.getDeclKind() == DeclarationKind.dk_component)
        {
            files.addAll( getComponentDependencies((ComponentDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_consumes)
        {
            files.addAll( getEventPortDependencies((EventPortDecl) decl, true, visited) );
        }
        else if ( (decl.getDeclKind() == DeclarationKind.dk_emits)
                  || (decl.getDeclKind() == DeclarationKind.dk_publishes) )
        {
            files.addAll( getEventPortDependencies((EventPortDecl) decl, false, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_provides)
        {
            files.addAll( getInterfacePortDependencies((InterfacePortDecl) decl, true, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_uses)
        {
            files.addAll( getInterfacePortDependencies((InterfacePortDecl) decl, false, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_home)
        {
            files.addAll( getHomeDependencies((HomeDecl) decl, visited) );
        }
        else if (decl.getDeclKind() == DeclarationKind.dk_event)
        {
            files.addAll( getValueTypeDependencies((EventDecl) decl, visited) );
        }
        else if ( (decl.getDeclKind() == DeclarationKind.dk_factory)
                  ||(decl.getDeclKind() == DeclarationKind.dk_finder) )
        {
            files.addAll( getOperationDependencies((OperationDecl) decl, visited) );
        }
        else
        {
            // Call to IDL2 dependencies
            files.addAll( super.getDeclarationDependencies(decl, visited) );
        }

        return files;
    }

    /**
     * Get Type Reference dependencies class files.
     *
     * @param aconst - Get dependencies from this type reference.
     * @param visited - List of previously visited declarations. Must not be null.
     *
     * @return The list of dependencies as a set of Files.
     */
    public Set
    getTypeRefDependencies(TypeRef type,
                           Set visited)
    {
        TypeKind tk = null;
        Set files = new java.util.HashSet();

        tk = type.getTypeKind();

        // If type is a declaration, add declaration dependencies
        if (type.isDeclaration())
        {
            Declaration decl = (Declaration) type;

            if (tk == TypeKind.tk_component)
            {
                files.addAll( getComponentDependencies((ComponentDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_home)
            {
                files.addAll( getHomeDependencies((HomeDecl) decl, visited) );
            }
            else if (tk == TypeKind.tk_event)
            {
                files.addAll( getValueTypeDependencies((EventDecl) decl, visited) );
            }
            else
            {
                // Call to IDL2 dependencies
                files.addAll( super.getTypeRefDependencies(type, visited) );
            }
        }
        return files;
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
