/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s):

====================================================================*/

package org.objectweb.openccm.descriptor.componentassembly.gui;

import org.objectweb.openccm.descriptor.componentassembly.beans.*;
import org.objectweb.openccm.descriptor.componentassembly.*;

/**
 * This is an extension to the Apollon generated framework around componentassembly DTD
 *  it looks for subelements that could contain files to archive
 *  and calls their own zipper extensions to do the same job
 *  any found file name is sent to a zip factory
 *
 */
public class ComponentfilesZipper
{
    // ==================================================================
    //
    // Public methods
    //
    // ==================================================================

    /**
     *  The zipCall method used to go through the element children
     *
     * @param ComponentfilesBean componentfiles the element to parse
     * @param ZipFactory zip_factory the factory to send file names
     */
    public void
    zipCall(
        ComponentfilesBean componentfiles
        , org.objectweb.openccm.packaging.ZipFactory zip_factory
        )
    {
        // retrieves the elements that may contain files to zip and call their extension
        java.util.List subelement_list;

        // component files
        subelement_list
         = componentfiles.getComponentfileList();
        // goes through each elements and call its zip method
        for (java.util.Iterator i = subelement_list.iterator() ; i.hasNext() ; )
        {
            ComponentfileBean sub_element
             = (ComponentfileBean) i.next();

            ComponentfileZipper sub_zipper
             = (ComponentfileZipper) sub_element
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(sub_element, zip_factory);
        }
    }
}
