/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s):

====================================================================*/

package org.objectweb.openccm.descriptor.softpkg.gui;

import org.objectweb.openccm.descriptor.softpkg.beans.*;
import org.objectweb.openccm.descriptor.softpkg.*;

/**
 * This is an extension to the Apollon generated framework around softpkg DTD
 *  it looks for subelements that could contain files to archive
 *  and calls their own zipper extensions to do the same job
 *  any found file name is sent to a zip factory
 *
 */
public class DependencyZipper
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     *  The constructor taking an event as parameter
     */
    public void
    zipCall(
        DependencyBean dependency
        , org.objectweb.openccm.packaging.ZipFactory zip_factory
        )
    {
        // Softpkg ref file
        SoftpkgrefBean softpkgref
         = (SoftpkgrefBean) dependency.getSoftpkgref();
        // goes through sub element and call its zip method
        if (softpkgref != null)
        {
            SoftpkgrefZipper sub_zipper
             = (SoftpkgrefZipper) softpkgref
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(softpkgref, zip_factory);
        }

        // valuetype factory
        ValuetypefactoryBean valuetypefactory
         = (ValuetypefactoryBean)
            dependency.getValuetypefactory();

        // goes through sub element and call its zip method
        if (valuetypefactory != null)
        {
            ValuetypefactoryZipper sub_zipper
             = (ValuetypefactoryZipper) valuetypefactory
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(valuetypefactory, zip_factory);
        }

        FileinarchiveBean fileinarchive
         = (FileinarchiveBean)
            dependency.getFileinarchive();

        // goes through sub element and call its zip method
        if (fileinarchive != null)
        {
            FileinarchiveZipper sub_zipper
             = (FileinarchiveZipper) fileinarchive
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();

            sub_zipper.zipCall(fileinarchive, zip_factory);
        }
    }
}
