/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.packaging;

// Package dependencies
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.openccm.descriptor.componentassembly.gui.ComponentassemblyZipper;

/**
 * This is the action class for building an archive for an assembly
 *
 */
public class AssemblyPackager
  implements MenuItem
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Private methods.
    //
    // ==================================================================


    // ==================================================================
    //
    // public methods.
    //
    // ==================================================================

    /**
     *  Status of the menu item
     *
     * @param treeView to obtain the activated node
     */
    public int
    getStatus(org.objectweb.util.browser.api.TreeView treeView)
    {
        return org.objectweb.util.browser.api.MenuItem.ENABLED_STATUS;
    }

    /**
     *  The action method taking an event as parameter
     */
    public void
    actionPerformed(MenuItemTreeView e)
    throws Exception
    {
        String action = e.getActionEvent().getActionCommand();

        // gets the calling instance
        org.objectweb.apollon.framework.Bean zeus_instance
            = (org.objectweb.apollon.framework.Bean)
                e.getSelectedObject();

        // gets the calling entry
        org.objectweb.util.browser.api.Entry entry
         = e.getSelectedEntry();

        // gets the file name for the assembly descriptor
        // it is registered as an absolute path in the GUI for this entry
        String descriptor_filename
         = CCMSplitPane.get_file_by_id(
            (String) entry.getName()
            );

        // calls the ZIP maker method
        make_zip (zeus_instance, descriptor_filename);
    }

    /**
     *  A method to build ZIP archives
     *   called from menu action, or from the static call
     *
     * @param zeus_instance  the META used as CAD descriptor
     * @param descriptor_filename  the name of the CAD file
     */
    public void
    make_zip (
        org.objectweb.apollon.framework.Bean zeus_instance,
        String descriptor_filename
        )
    throws Exception
    {
        if (zeus_instance.idBean().equals("[Componentassembly] (id?)"))
        {
            throw new Exception(
                "Undefined Assembly ID for "
                +descriptor_filename);
        }
        else
        {
            java.io.File zipfile
             = new java.io.File(
                zeus_instance.idBean()
                +".aar"
                );

            System.err.println("Starting Assembly ARchive creation...");

            // creates a new zip factory for this package
            ZipFactory zip_factory
            = new ZipFactory();

            zip_factory.setZipName(zipfile.getAbsolutePath());

            // gets the descriptor file for the described Assembly
            java.io.File original_descriptor_file
             = new java.io.File(
                descriptor_filename
                );
            // creating the file to be zipped
            java.io.File temp_descriptor_file
             = new java.io.File(
                "META-INF"
                +System.getProperty("file.separator")
                +original_descriptor_file.getName()
                );

            try
            {
                // marshaling the temporary descriptor
                zeus_instance.marshalBean(temp_descriptor_file);
            } catch (Exception except) {
                except.printStackTrace();
            }

            // adds the descriptor to the zip archive
            zip_factory
                .addFileToZip(
                    temp_descriptor_file.getPath()
                    );

            // launches the ZIP extensions in the SOFTPKG framework
            ComponentassemblyZipper zipper
            = (ComponentassemblyZipper) zeus_instance
                .getExtensionManager()
                .getExtensionByName("gui.Zipper")
                .getInstance();
            zipper.zipCall(
                (org.objectweb.openccm.descriptor
                .componentassembly.beans.ComponentassemblyBean)
                    zeus_instance
                , zip_factory);

            // builds the archive after file retrieving
            try
            {
                zip_factory.buildZip();
                System.err.println("End of the AAR creation, writing file:");
                System.err.println(zipfile.getAbsolutePath());
            } catch (Exception exc) {
                System.err.println("Assembly archive could not be built. See following stacktrace");
                exc.printStackTrace();
            }
        }
    }
}
