/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Contreras
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.openccm.packaging.cmdline;


/**
 * This implements the AAR option treatment for OpenCCM Pack-Assembling Tool
 *
 * @author <a href="mailto:christophe.contreras@lifl.fr">Christophe Contreras</a>
 *
 */

public class OptionAAR
     extends OptionBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor.
     *
     * @param application The associated PATApplication.
     */
    public
    OptionAAR(PATApplication application)
    {
        setLabels(new String[] { "-AAR" });
        setDescription(new String[] { "Builds Assembly ARchives from Component Assembly Descriptors"});
        setApplication(application);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.packaging.cmdline.OptionBase
    //
    // ==================================================================

    /**
     * Gets the base option label.
     *
     * @return The base option label.
     */
    public String
    getBaseOptionLabel()
    {
        return "-AAR";
    }

    /**
     * Consumes the option value.
     *
     * @param current The option value.
     */
    public void
    consumeOption(String current)
    {
        getApplication().displayGUI(false);
        getApplication().setAARMaker(true);
    }

    // ==================================================================
    //
    // Other public methods.
    //
    // ==================================================================
}
