/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Mathieu Vadet, Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.generator.idl.lib;

// Package dependencies.
import org.objectweb.openccm.ast.api.AST;
import org.objectweb.openccm.ast.api.AnyValue;
import org.objectweb.openccm.ast.api.AttributeDecl;
import org.objectweb.openccm.ast.api.ComponentBase;
import org.objectweb.openccm.ast.api.ComponentDecl;
import org.objectweb.openccm.ast.api.ConstantDecl;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.DeclarationKind;
import org.objectweb.openccm.ast.api.EnumDecl;
import org.objectweb.openccm.ast.api.ExceptionDecl;
import org.objectweb.openccm.ast.api.ExceptionList;
import org.objectweb.openccm.ast.api.HomeDecl;
import org.objectweb.openccm.ast.api.Initializer;
import org.objectweb.openccm.ast.api.InterfaceDecl;
import org.objectweb.openccm.ast.api.InterfaceList;
import org.objectweb.openccm.ast.api.OperationDecl;
import org.objectweb.openccm.ast.api.Parameter;
import org.objectweb.openccm.ast.api.Scope;
import org.objectweb.openccm.ast.api.StructDecl;
import org.objectweb.openccm.ast.api.TypeRef;
import org.objectweb.openccm.ast.api.UnionDecl;
import org.objectweb.openccm.ast.api.UnionMember;
import org.objectweb.openccm.ast.api.UsesDecl;
import org.objectweb.openccm.ast.api.ValueDecl;
import org.objectweb.openccm.ast.api.ValueMemberDecl;
import org.objectweb.openccm.ast.api.WithNameTypeRef;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.generator.translator.ast2idl.api.AST_IDLTranslator;

import java.util.ArrayList;
import java.util.List;


/**
 * This class visits AST declarations and generate the matching IDL3.
 *
 * @author <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class IDL3Generator
     extends org.objectweb.openccm.generator.common.lib.GeneratorBase
  implements org.objectweb.openccm.generator.idl.api.IDL3Generator
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     * Utility class to convert types in IDL
     */
    public AST_IDLTranslator translator_;

    /**
     * The last prefix defined in the file
     */
    protected String last_prefix_;

    /**
     * Print the macro directive ?
     */
    protected boolean print_macro_;


    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public IDL3Generator(AST ast)
    {
        // Call the GeneratorBase constructor
        super(ast);

        // Init internal states
        translator_ = new org.objectweb.openccm.generator.translator.ast2idl.lib.AST_CIDLTranslator();
        last_prefix_ = "";
        print_macro_ = true;
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Get the Macro Name from a declaration.
     *
     * @param target_decl - The scope to visit.
     *
     * @return The macro name.
     */
    protected String
    get_macro(Scope target_decl)
    {
        int idx, idx2;
        String abs_name = null,
               res = "";
        try{
            abs_name = target_decl.getAbsoluteName();

            idx = abs_name.indexOf(':', 2);
            idx2 = 2;
            while (idx!=-1)
            {
                res = res + abs_name.substring(idx2, idx) + '_';
                idx2 = idx+2;
                idx = abs_name.indexOf(':', idx2);
            }
            res = res + abs_name.substring(idx2);
            res = "__" + res.toUpperCase() + "__";
        }catch(Exception e){
            res = "__NO_NAME__";
            print_macro_ = false;
        }
        return res;
    }

    /**
     * Method called after generation.
     * Generally used to close files.
     **/
    protected void
    after_generation()
    {
        close("out");
    }


    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     *
     * @param outputfile - The name where declarations will be generated.
     * @param imports - IR declarations to import.
     * @param app_name - The application's name.
     */
    public void
    initialize( String outputfile,
                ArrayList imports,
                String app_name )
    {
        List list = new java.util.ArrayList();

        /** Initialize environment **/

        // Templates to use.
        list.clear();
        list.add("org/objectweb/openccm/generator/common/common.vm");
        list.add("org/objectweb/openccm/generator/idl/idl3.vm");
        setLibrary(list);

        // The output file
        open(outputfile, "out");

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
        put("gen", this);
        put("declarator", "");
        put("limited_types" , new Long(org.objectweb.openccm.ast.api.DeclarationKind.dk_all));
        put("imports", imports);
    }

    /**
     * Generates contents of a scope.
     * name must be a valid declaration name.
     *
     * @param name - The name of the declaration to visit.
     *
     * @throws GenerationException
     */
    public void
    generate(String name)
    throws GenerationException
    {
        Scope target_decl = null;

        try{
            target_decl = (Scope)getDeclaration(name);
        }catch(ClassCastException ex){
            String msg = name + "is not a scope!";
            throw new GenerationException(msg);
        }
        generate(target_decl);
    }

    /**
     * Generates contents of a scope.
     *
     * @param target_decl - The scope to visit.
     *
     * @throws GenerationException
     */
    public void
    generate(Scope target_decl)
    throws GenerationException
    {
        target_decl_ = target_decl;

        // System.out.println("### Visiting scope (generate) :  " + target_decl.getAbsoluteName());
        put("obj", target_decl);
        put("FILE_MACRO", get_macro(target_decl));
        map("FILE_HEADER");
        visit(target_decl);
        after_generation();
    }

    /**
     * Generates contents of the root scope.
     * Declarations types must match to limited types.
     *
     * @param target_decl - The scope to visit.
     * @param limited_types - A logical combination of DeclarationKind.
     *
     * @throws GenerationException
     */
    public void
    generate(Scope target_decl, long limited_types)
    throws GenerationException
    {
        List vect = null;

        vect = getDeclarations(target_decl, limited_types);
        put("limited_types" , new Long(limited_types));
        put("FILE_MACRO", get_macro(target_decl));
        map("FILE_HEADER");
        forward(vect);
        map("END_LINE");
        visit(vect);
        after_generation();
    }

    /**
     * Close an idl stream.
     *
     * @param id - The identifiant of the file to close.
     */
    public void
    close(String id)
    {
        if ( print_macro_ )
            print("\n#endif", id);
        super.close(id);
    }

    // ===========================================================
    //
    // Useful methods for IDL3 Mapping.
    //
    // ===========================================================

    /**
     * Check if id is an idl keyword.
     *
     * @param id - The string to check
     *
     * @return A valid name.
     **/
    public String
    checkName(String id)
    {
        return translator_.checkKeywords(id);
    }

    /**
     * Get a valid absolute name from a declaration,
     * i.e. check keywords.
     *
     * @param decl - The declaration to get the name.
     *
     * @return A valid absolute name.
     */
    public String
    getAbsoluteName(Declaration decl)
    {
        return translator_.getAbsoluteName(decl);
    }

    /**
     * Utility method to check forward declarations.
     *
     * @return The associated search mask.
     **/
    public long
    forward_check()
    {
        return (DeclarationKind.dk_interface +
                DeclarationKind.dk_local_interface +
                DeclarationKind.dk_abstract_interface +
                DeclarationKind.dk_component +
                DeclarationKind.dk_value +
                DeclarationKind.dk_struct +
                DeclarationKind.dk_union +
                DeclarationKind.dk_event);
    }

    /**
     * Utility method to check interface declarations.
     *
     * @return The associated search mask.
     **/
    public long
    interface_check()
    {
        return (DeclarationKind.dk_attribute +
                DeclarationKind.dk_constant +
                DeclarationKind.dk_exception +
                DeclarationKind.dk_operation +
                DeclarationKind.dk_alias +
                DeclarationKind.dk_struct +
                DeclarationKind.dk_union +
                DeclarationKind.dk_enum +
                DeclarationKind.dk_native);
    }

    /**
     * Utility method to check component declarations.
     *
     * @return The associated search mask.
     **/
    public long
    component_check()
    {
        return (DeclarationKind.dk_provides +
                DeclarationKind.dk_uses +
                DeclarationKind.dk_emits +
                DeclarationKind.dk_publishes +
                DeclarationKind.dk_consumes +
                DeclarationKind.dk_attribute);
    }

    /**
     * Utility method to check home declarations.
     *
     * @return The associated search mask.
     **/
    public long
    home_check()
    {
        return (DeclarationKind.dk_factory +
                DeclarationKind.dk_finder +
                interface_check());
    }

    /**
     * Utility method to check value declarations.
     *
     * @return The associated search mask.
     **/
    public long
    value_check()
    {
        return (DeclarationKind.dk_value_member +
                interface_check());
    }

    /**
     * Get an attribute of the class DeclarationKind.
     *
     * @param att - The attribute to get.
     *
     * @return The double value of the attribute.
     **/
    public long
    getDeclarationKind(String att)
    {
        java.lang.reflect.Field field = null;
        long res = 0;

        try{
            field = Class.forName("org.objectweb.openccm.ast.api.DeclarationKind").getField(att);
            res = field.getLong(field);
        }catch(Exception e){
            e.printStackTrace();
        }

        return res;
    }

    // ===========================================================
    //
    // Methods for IDL3 Mapping.
    //
    // ===========================================================

    /**
     ** Generate a macro if necessary
     **/
    public String
    macro(Declaration obj)
    {
        String res = "";
        String version = obj.getVersion();
        String id = obj.getId();
        String prefix = obj.getPrefix();

        // System.err.println("Macro for " + obj.getAbsoluteName() + " : " + prefix);
        if (!prefix.equals(last_prefix_))
        {
            res = res + "#pragma prefix \""+prefix+"\"\n";
            last_prefix_ = prefix;
        }
        else
        {
            String p_id = obj.getParent().getId();
            int idx1 = p_id.indexOf(':');
            int idx2 = p_id.lastIndexOf(':');
            if (id.indexOf(p_id.substring(idx1, idx2))==-1)
                res = res + "#pragma id "+obj.getName()+" \""+id+"\"\n";
        }
        if (!version.equals("1.0"))
            res = res + "#pragma version "+obj.getName()+" "+version+"\n";

        return res;
   }

    /**
     * Generate structured members contained in a declaration
     * (i.e. a struct, an exception or a declaration).
     *
     * @param obj - The declaration to visit
     **/
    public void
    struct_members(Declaration obj, String obj_declarator)
    {
        boolean is_inner = false;
        int multiple_idx = -1;
        WithNameTypeRef[] members = new WithNameTypeRef[0];

        // Retrieve members of the declaration
        try
        {
            members = ((ExceptionDecl) obj).getMemberList().getStructMembers();
        }catch(ClassCastException e){
            try{
                members = ((StructDecl) obj).getMemberList().getStructMembers();
            }catch(ClassCastException ex){
                members = ((UnionDecl) obj).getMemberList().getUnionMembers();
            }
        }
//        System.err.println("members de struct_members : " + members);

        // Compute each member
        for (int i=0; i<members.length; i++)
        {
            String name = checkName( members[i].getName() );
            TypeRef type = members[i].getType();

            // check for inner struct, union or enum members
            if ((type.getTypeKind() == org.objectweb.openccm.ast.api.TypeKind.tk_struct) ||
                (type.getTypeKind() == org.objectweb.openccm.ast.api.TypeKind.tk_union) ||
                (type.getTypeKind() == org.objectweb.openccm.ast.api.TypeKind.tk_enum))
            {
                Declaration decl = null;
                decl = (Declaration)type;
                if (decl.getParent().getId().equals(obj.getId()))
                    is_inner = true;
                else
                    is_inner = false;
            }
            else
                is_inner = false;


            // compute the declarator
            String declarator = "";
            // check for multiple declarators
            //  e.g.  long l1, l2, l3;
            if ( (i<members.length-1) && (type==members[i+1].getType()) )
                multiple_idx = i;
            while ( (i<members.length-1) && (type==members[i+1].getType()) )
                i++;

            if (multiple_idx==-1)
                declarator = name + translator_.toIDLPostfix(type);
            else
            {
                for (int j=multiple_idx; j<i+1; j++)
                    declarator = declarator + checkName(members[j].getName())
                                 + translator_.toIDLPostfix(members[j].getType()) + ", ";
                declarator = declarator.substring(0, declarator.length()-2);
                multiple_idx = -1;
            }

            if (is_inner)
            {
                // inline struct, union or enum
                Declaration decl = null;
                String id = "";

                decl = (Declaration)type;
                id = org.objectweb.openccm.ast.lib.DeclarationKindImpl.toString(decl.getDeclKind()).toUpperCase()+ "_INLINED";

                put("obj", decl);
                put("declarator", declarator);
                map(id);
            }
            else
            {
                put("type_spec", translator_.toIDL(type) );
                put("name", declarator);
                map("STRUCT_MEMBER");
            }
        }//end for

        put("declarator",obj_declarator);
        put("obj", obj);
    }


    /**
     * Generate union members for an union.
     *
     * @param union - The union to visit
     **/
    public void
    union_members(UnionDecl union, String obj_declarator)
    {
        UnionMember[] members = null;
        boolean is_default = false;
        boolean print = true;
        boolean is_inner = false;
        int di = union.getDefaultIndex();

        members = union.getMemberList().getUnionMembers();

//union.getContents(true, DeclarationKind.dk_all);
//System.err.println("default index:"+di);
        for (int i=0; i<members.length; i++)
        {
            String name = checkName( members[i].getName() );
            TypeRef type = members[i].getType();
            AnyValue val = members[i].getAnyValue();

            // check that the i and i+1 element spec are different.
            // else just print the label
            if ( ( i<members.length-1 ) &&
                 ( name.equals( checkName(members[i+1].getName()) ) )
                )
                print = false;
            else
                print = true;

// TODO : Correct the bug with default index (always -1)
//            if (di == i)
            if (translator_.toString(val, union.getDiscriminator()).equals(""))
                is_default = true;
            else
                is_default = false;
//System.err.println("Default="+is_default+", i:"+i);

            // check for inner struct, union or enum members
            if ( (type.getTypeKind() == org.objectweb.openccm.ast.api.TypeKind.tk_struct) ||
                 (type.getTypeKind() == org.objectweb.openccm.ast.api.TypeKind.tk_union) ||
                 (type.getTypeKind() == org.objectweb.openccm.ast.api.TypeKind.tk_enum))
            {
                Declaration decl = null;
                decl = (Declaration)type;
                if (decl.getParent().getId().equals(union.getId()))
                    is_inner = true;
                else
                    is_inner = false;
            }
            else
                is_inner = false;

            if (!print)
            {
                // print the label
                if (is_default)
                    print("default :\n");
                else
                {
                    String value = translator_.toString(val, union.getDiscriminator());
                    print("case " + value + " :\n");
                }
            }
            else if (is_inner)
            {
                // print the label first
                if (is_default)
                    print("default :\n");
                else
                {
                    String value = translator_.toString(val, union.getDiscriminator());
                    print("case " + value + " :\n");
                }

                // inline struct, union or enum
                Declaration decl = null;
                String id = "";

                decl = (Declaration)type;
                id = org.objectweb.openccm.ast.lib.DeclarationKindImpl.toString(decl.getDeclKind()).toUpperCase()+
                    "_INLINED";

                put("obj", decl);
                put("declarator", name+translator_.toIDLPostfix(type));
                // reajust the tabulation
                //print("/inc");
                map(id);
                //print("/dec");
            }
            else
            {
                put("value", translator_.toString(val, union.getDiscriminator()));
                put("type_spec",translator_.toIDL(type));
                put("name", name+translator_.toIDLPostfix(type));
                if (is_default)
                    map("UNION_DEFAULT_MEMBER");
                else
                    map("UNION_MEMBER");
            }
        }//end for
        put("declarator",obj_declarator);
        put("obj", union);
   }


    /**
     * Generate members of an enumeration
     *
     * @param enumDecl - The enumeration.
     **/
    public void
    enum_members(EnumDecl enumDecl)
    {
        String[] members = enumDecl.getMemberList().getStrings();
        String res = "";
        int i;

        for (i=0; i<members.length-1; i++)
            print( members[i] + ",\n");

        if (i<members.length)
            print( members[i] );
    }


    /**
     * Generate the modifier of a ValueDecl
     *
     * @param value - The ValueDecl concerned.
     **/
    public void
    modifier(ValueDecl value)
    {
        if (value.isCustom())
            print("custom ");
        else if (value.isAbstract())
            print("abstract ");
    }


    /**
     * Generate inherited component for a component.
     *
     * @param comp - The component concerned.
     *
     * @return The resulting string.
     **/
    public String
    base(ComponentDecl comp)
    {
        ComponentDecl inh = comp.getBaseComponent();
        if (inh != null)
            return (" : " + getAbsoluteName(inh));
        return "";
   }

    /**
     * Generate inherited home.
     *
     * @param home - The home concerned.
     *
     * @return The resulting string.
     **/
    public String
    base(HomeDecl home)
    {
        HomeDecl inh = home.getBaseHome();
        if (inh != null)
            return (" : "+ getAbsoluteName(inh));
        return "";
   }


    /**
     * Generate supported interfaces.
     *
     * @param comp - The list of interaces concerned.
     *
     * @return The resulting string.
     **/
    public String
    supports(InterfaceDecl[] itfs)
    {
        String res = "";

        if ( (itfs!=null) && (itfs.length>0) )
        {
            res += "\n          supports ";
            int i;
            for (i=0; i<itfs.length-1; i++)
                res += getAbsoluteName(itfs[i]) + ", ";
            res += getAbsoluteName(itfs[i]);
        }
        return res;
  }

    /**
     * Generate supported interfaces for a component base.
     *
     * @param comp - The component base concerned.
     *
     * @return The resulting string.
     **/
    public String
    supported(ComponentBase comp)
    {
        InterfaceList list = null;

        list = comp.getSupportedInterfaceList();
        if (list != null)
            return supports( list.getInterfaces() );
        return "";
    }

    /**
     * Generate supported interfaces for a value declaration.
     *
     * @param comp - The value declaration concerned.
     *
     * @return The resulting string.
     **/
    public String
    supported(ValueDecl value)
    {
        InterfaceList list = null;

        list = value.getSupportedInterfaceList();
        if (list != null)
            return supports( list.getInterfaces() );
        return "";
    }

    /**
     * Generate inherited interfaces for an interface.
     *
     * @param itf - The interface concerned.
     *
     * @return The string format.
     **/
    public String
    bases(InterfaceDecl itf)
    {
        String res = "";
        Declaration[] itfs = itf.getInheritedInterfaceList().getInterfaces();

        if (itfs.length>0)
            res += "\n              : ";

        for (int i=0; i<itfs.length; i++)
        {
            res += getAbsoluteName(itfs[i]);
            if (i<itfs.length-1)
                res += ",\n                " ;
        }
        return res;
    }

    /**
     * Generate mode for an operation.
     *
     * @param op - The operation concerned.
     *
     * @return The string format.
     **/
    public String
    mode(OperationDecl op)
    {
        if (op.isOneway())
            return("oneway ");
        else
            return "";
    }

    /**
     * Convert a parameter mode in String format
     *
     * @param param - The parameter concerned.
     *
     * @return The string format.
     **/
    public String
    toStringMode(Parameter param)
    {
        if ( param.isIn() )
            return "in ";
        else if ( param.isOut() )
            return "out ";
        else
            return "inout ";
    }

    /**
     * Generate type for a type reference.
     *
     * @param obj - The type reference concerned.
     *
     * @return The string format.
     **/
    public String
    type(TypeRef obj)
    {
        return translator_.toIDL(obj);
    }

    /**
     * Generate parameters for an operation.
     *
     * @param op - The operation concerned.
     *
     * @return The string format.
     **/
    public String
    params(OperationDecl op)
    {
        String res = "";
        Parameter[] par = op.getParameterList().getParameters();

        for (int i=0; i<par.length; i++)
            res += toStringMode(par[i]) + translator_.toIDL(par[i].getType())
                   + " " + checkName(par[i].getName()) + ", ";

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }

    /**
     * Generate exceptions raises.
     *
     * @param exc - The list of exceptions concerned.
     *
     * @return The string format.
     **/
    public String
    raise(String raises_format, ExceptionList list)
    {
        ExceptionDecl[] excs = list.getExceptions();
        String res = "";

        if ( (excs != null) && (excs.length>0))
        {
            int i;

            res += ("\n   " + raises_format + "(");
            for (i=0; i<excs.length-1; i++)
                res += getAbsoluteName(excs[i]) + ", ";
            res += getAbsoluteName(excs[i]) + ")";
        }
        return res;
    }

    /**
     * Generate exceptions raises for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return the string format.
     **/
    public String
    raises(AttributeDecl attr)
    {
        return raise("raises", attr.getExceptionList());
    }

    /**
     * Generate getexceptions raises for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return The string format.
     **/
    public String
    getraises(AttributeDecl attr)
    {
        return raise("getraises", attr.getGetExceptionList());
    }

    /**
     * Generate getexceptions raises for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return the string format.
     **/
    public String
    setraises(AttributeDecl attr)
    {
        return raise("setraises", attr.getSetExceptionList());
    }

    /**
     * Generate access mode for an attribute.
     *
     * @param attr - The attribute concerned.
     *
     * @return the string format.
     **/
    public String
    access(AttributeDecl attr)
    {
        if (attr.isReadonly())
            return "readonly ";
        return "";
    }

    /**
     * Generate multiple use declaration.
     *
     * @param uses - The use declaration concerned.
     *
     * @return The string format.
     **/
    public String
    multiple(UsesDecl uses)
    {
        if (uses.isMultiple())
            return "multiple ";
        else
            return "";
    }

    /**
     * Generate primary key for a home.
     *
     * @param home - The home concerned.
     *
     * @return The string format.
     **/
    public String
    primary_key(HomeDecl home)
    {
        Declaration pk = home.getPrimaryKey();

        if (pk == null)
            return "";
        else
            return ("\n     primary key " + getAbsoluteName(pk));
   }

    /**
     * Generate base value for a value declaration.
     *
     * @param value - The value declaration concerned.
     *
     * @return the string format.
     **/
    public String
    base_value(ValueDecl value)
    {
        ValueDecl base = value.getBaseValue();
        if (base!=null)
        {
            if (value.isTruncatable())
                return " : truncatable " + getAbsoluteName(base);
            else
                return " : " + getAbsoluteName(base);
        }
        return "";
   }


    /**
     * Generate abstract values for a value declaration.
     *
     * @param value - The value declaration concerned.
     *
     * @return The resulting string.
     **/
    public String
    abstract_values(ValueDecl value)
    {
        ValueDecl[] bases = value.getAbstractValueList().getValues();
        String res = "";

        if (bases.length>0)
        {
            if (value.getBaseValue() == null)
                res += " : ";
            else
                res += ", ";
            int i;
            for (i=0; i<bases.length-1; i++)
                res += getAbsoluteName(bases[i]) + ", ";
            res += getAbsoluteName(bases[i]);
        }
        return res;
    }

    /**
     * Generate visibility for a value member.
     *
     * @param vm - The value member concerned.
     *
     * @return The string format.
     **/
    public String
    visibility(ValueMemberDecl vm)
    {
        if (vm.isPrivate())
            return "private";
        else
            return "public";
    }

    /**
     * Translate an array into its idl format.
     *
     * @param obj - The array to translate.
     *
     * @return The resulting string.
     **/
    public String
    array(TypeRef obj)
    {
        return translator_.toIDLPostfix(obj);
    }

    /**
     * Generate contexts for an operation.
     *
     * @param op - The operation concerned.
     *
     * @return The string format.
     **/
    public String
    contexts(OperationDecl op)
    {
        String[] ctxs = op.getContextList().getStrings();
        String res = "";

        if (ctxs.length > 0)
        {
            res += " context(";
            for (int i=0; i<ctxs.length; i++)
                res += "\"" + ctxs[i] + "\", ";
            res += ")";
        }

        return res;
    }


    /**
     * Generate value for a constant.
     *
     * @param constant - The constant declaration concerned.
     *
     * @return The string format.
     **/
    public String
    value(ConstantDecl constant)
    {
        return translator_.toString(constant.getAnyValue(), constant.getType());
    }


    /**
     * Generate initializers parameters for an initializer.
     *
     * @param obj - The initializer concerned.
     *
     * @return The resulting string.
     **/
    public String
    initializer_params(Initializer obj)
    {
        Parameter[] params = null;
        params = obj.getParameterList().getParameters();
        String res = "";

        for (int i=0; i<params.length; i++)
        {
            res += toStringMode(params[i]);
            res += translator_.toIDL(params[i].getType());
            res += " " + checkName(params[i].getName()) + ", ";
        }

        if (!res.equals(""))
            res = res.substring(0, res.length()-2);

        return res;
    }


    /**
     * Generate initializers (i.e. factory) for a value Declaration.
     *
     * @param value - The value declaration concerned.
     **/
    public void
    initializers(ValueDecl value)
    {
        Initializer[] ops = value.getInitializerList().getInitializers();

        for (int i=0;i<ops.length;i++)
        {
            put("init", ops[i]);
            map("INITIALIZER");
        }
    }
}
