/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet, Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ir3;

// Package dependencies
import org.omg.CORBA.ValueDef;

/** 
 * Implementation of the CORBA::ValueDef reference.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.4
 */

public class ValueDef_ref
     extends IDLType_ref
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** 
     * TODO.
     */
    public
    ValueDef_ref(IRObject_impl owner)
    {
        // Calls the IDLType_ref constructor.
        super(owner);
    }

    /** 
     * TODO.
     */
    public
    ValueDef_ref(IRObject_impl owner,
                 ValueDef_impl value)
    {
        // Calls the IDLType_ref constructor.
        super(owner, value);
    }

    // ==================================================================
    //
    // Static public methods.
    //
    // ==================================================================

    /** 
     * Cuts the dependency to a set of values.
     */
    static public void
    cutDependencies(ValueDef_ref[] values)
    {
        if (values != null)
        {
            for(int i=0; i<values.length; i++)
                values[i].cutDependency();
        }
    }

    /** 
     * Computes the CORBA::RepositoryIdSeq for an ValueDef_ref[].
     */
    static public String[]
    computeRepositoryIdSeq(ValueDef_ref[] values)
    {
        String[] result = new String[values.length];

        for(int i=0; i<values.length; i++)
            result[i] = values[i].getId();

        return result;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Obtains its implementation.
     */
    public ValueDef_impl
    getImpl()
    {
        return (ValueDef_impl)(super.getBaseImpl());
    }

    /** 
     * Obtains its ValueDef reference.
     */
    public ValueDef
    asValueDef()
    {
        if(getTarget() == null) return null;
        return getImpl().asValueDef();
    }

    /** 
     * Obtains its repositoryID.
     */
    public String
    getId()
    {
        if (getTarget() == null) return "";
        return getImpl().id();
    }
}
