/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.task;

//Package dependencies
import java.util.Iterator;


/**
 * This class manages PreProcessor Options.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class CPPoptions
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** Inhibit generation of #-lines with line-number information if true. */
    private boolean line_info_;

    /** List of macros to define. */
    private java.util.List defines_;

    /** List of macros to undefine. */
    private java.util.List undefines_;

    /** List of pathes to include. */
    private java.util.List includes_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public CPPoptions()
    {
        // Init internal states
        line_info_ = false; 
        defines_ = new java.util.ArrayList();
        undefines_ = new java.util.ArrayList();
        includes_ = new java.util.ArrayList();
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Get Launcher arguments declaration.
     * 
     * @return Launcher arguments declaration string.
     */
    protected String
    getCPParguments()
    {
        String args = "";
        MacroType macro = null;
        
        for (Iterator it = defines_.iterator(); it.hasNext();)
        {
            macro = (MacroType) it.next();
            args += "-D" + macro.getName();
            if (macro.getValue() != null)
                args += "=" + macro.getValue();
            args += " "; 
        }
        for (Iterator it = undefines_.iterator(); it.hasNext();)
        {
            args += "-U" + it.next() + " "; 
        }
        for (Iterator it = includes_.iterator(); it.hasNext();)
        {
            args += "-I" + it.next() + " "; 
        }
        if (line_info_)
            args += "-P ";
        
        return args;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Inhibit generation of #-lines with line-number information if true (-P).
     * 
     * @param b - The value to set.
     */
    public void
    setLineinfo(boolean b)
    {
        line_info_ = b;
    }
    
    /**
     * Define a new macro (-D).
     * Default is 1.
     * 
     * @param outputdir - The output directory name.
     */
    public void
    addConfiguredDefine(MacroType macro)
    {
        defines_.add(macro); 
    }
    
    /**
     * Remove any definition for name (-U).
     * 
     * @param str - The macro name to undefine.
     */
    public void
    addConfiguredUndefine(StringType str)
    {
        undefines_.add(str.getName());
    }
    
    /**
     * Put dir in the include file search path(-I).
     * 
     * @param dir - The macro name to undefine.
     */
    public void
    addConfiguredIncludepath(StringType dir)
    {
        includes_.add(dir.getName()); 
    }
    
}
