/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Alex Andrushchak.

====================================================================*/

package org.objectweb.openccm.pss.demo1;

/**
 * This is the client application to access Form.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public class FormImpl
     extends FormPOA
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The basic session */
    org.omg.CosPersistentState.Session _session;

    /** The Person storage home */
    SH_Person _person_home;

    /** The Address storage home */
    SH_Address _address_home;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * Default Constructor.
     * 
     * @param session - The PSS session associated to this Form.
     */
    public FormImpl(org.omg.CosPersistentState.Session session)
    {
        _session = session;
        try
        {
            _person_home = (SH_Person)
                                _session.find_storage_home("PSDL:openccm.objectweb.org/pss/demo1/SH_Person:1.0");
        }catch(org.omg.CosPersistentState.NotFound e){
            e.printStackTrace();
        }
        try
        {
            _address_home = (SH_Address)
                                _session.find_storage_home("PSDL:openccm.objectweb.org/pss/demo1/SH_Address:1.0");
        }catch(org.omg.CosPersistentState.NotFound e){
            e.printStackTrace();
        }
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Operation createPerson
     */
    public void
    createPerson( String first_name,
                  String last_name,
                  int age,
                  String address_line1,
                  String address_line2,
                  String address_city,
                  String address_zip)
    {
        Address a = null;
        Person p = null;

        try{
            a = _address_home._create( address_line1, address_line2, address_city, address_zip );

            p = _person_home.my_create( first_name, last_name );
        }catch( Exception e ){
            e.printStackTrace();
        }

        try{
            org.omg.CosPersistentState.CatalogBase cat = null;

            // Get the session
            cat = p.get_storage_home().get_catalog();

            // Update the storage type
            p.age(age);
            p.adress( a.get_pid() );
            cat.flush();
        }catch(Exception e){
            e.printStackTrace();
        }
    }

    /**
     * Operation removePerson
     */
    public void removePerson( java.lang.String first_name,
                              java.lang.String last_name )
    {
        Person p = null;
        Address adr = null;

        try
        {
            p = _person_home.find_by_mykey( first_name, last_name );
            adr = p.adress();

            if ( adr != null )
            {
                adr.destroy_object();
            }
            p.destroy_object();
        }
        catch ( org.omg.CosPersistentState.NotFound ex )
        {
            System.out.println( "This person was not found..." );
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
    }

    /**
     * Operation showPerson
     */
    public String showPerson( java.lang.String first_name,
                              java.lang.String last_name )
    {
        try
        {
            Person p = _person_home.find_by_mykey( first_name, last_name );

            return p.print();
        }
        catch ( org.omg.CosPersistentState.NotFound ex )
        {
            return "This person was not found...\n";
        }
        catch( Exception e )
        {
            e.printStackTrace();
        }
        return null;
    }
}
