/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.api;

// Package dependencies
import org.objectweb.openccm.ast.api.AbstractStorageTypeDecl;
import org.objectweb.openccm.ast.api.StorageHomeFactoryDecl;
import org.objectweb.openccm.ast.api.StorageTypeDecl;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.generator.java.ast.api.InterfaceObject;

/**
 * This is the interface for PSDL factory mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 * 
 * @version 0.1
 */

public interface PSDLFactoryMapping
{
    /** 
     * Set the PSDL StorageHome Factory Declaration to map. 
     *
     * @param factory - The factory to map.
     */
     public void
     setFactory(StorageHomeFactoryDecl factory);

    /**
     * Map a factory definition into Java methods.
     *
     * @param generator - The PSDL to Java generator.
     * @param obj - The StorageHome mapping object.
     * @param ast - The abstract storage type defining this factory.
     * @param st - The storage type managed by the storage home.
     */
    public void
    toJava(PSDL2JavaGenerator generator,
           InterfaceObject obj,
           AbstractStorageTypeDecl ast,
           StorageTypeDecl st)
    throws GenerationException;
}
