/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.api;

// Package dependencies
import org.objectweb.openccm.ast.api.FileScope;


/**
 * Interface of the psdl_java command.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public interface PSDLtoJavaCommand
         extends org.objectweb.openccm.command.api.Compiler,
                 org.objectweb.openccm.command.api.Generator
{
    /**
     * Obtains the output directory.
     *
     * @result The output directory.
     */
    public String
    getOutputDirectory();

    /**
     * Obtains the user properties file name.
     *
     * @result The user properties file name.
     */
    public String
    getUserPropertiesFile();

    /**
     * Obtains the persistent framework choosen.
     *
     * @result The persistent framework choosen.
     */
    public String
    getBackend();

    /**
     * Generates PSDL Mapping from an AST FileScope instance.
     *
     * Note that checkComponentRepository() and initAST() methods
     * must be called before.
     *
     * @param filescope The source AST FileScope returned by the compile() method.
     * @param inputfile - The source file.
     * @param base_dir - The base directory for java mapping.
     * @param user_properties_file - The user jdo properties file.
     * @param backend - The target persistent framework.
     *
     * @return True if ok, else false.
     */
    public boolean
    generate_java_mapping(FileScope filescope,
                          String inputfile,
                          String base_dir,
                          String user_properties_file,
                          String backend);
}
