/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.lib;

//Package dependencies
/** To access AST. */
import org.objectweb.openccm.ast.api.AbstractStorageHomeDecl;
import org.objectweb.openccm.ast.api.AbstractStorageTypeDecl;
import org.objectweb.openccm.ast.api.StorageHomeKeyDecl;
import org.objectweb.openccm.ast.api.StorageHomeFactoryDecl;
import org.objectweb.openccm.ast.api.PsdlOperationDecl;
import org.objectweb.openccm.ast.api.Declaration;
import org.objectweb.openccm.ast.api.DeclarationKind;
/** To access Java AST. */
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
/** Others. */
import java.util.List;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.pss.generator.common.api.PSDL2JavaGenerator;


/**
 * This class generates Abstract Storage Home Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */

public class AbstractStorageHomeMapping
  implements org.objectweb.openccm.pss.generator.common.api.AbstractStorageHomeMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Abstract Storage Home declaration. */
    private AbstractStorageHomeDecl ash_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     */

    public AbstractStorageHomeMapping()
    {
        // Init internal state
        ash_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Set the PSDL Abstract StorageHome Declaration to map. 
     *
     * @param ash - The Abstract Storage Home to map.
     */
     public void
     setAbstractStorageHome(AbstractStorageHomeDecl ash)
     {
         ash_ = ash;
     }
     
    /**
     * Map an Abstract Storage Home definition into a java Interface.
     *
     * @param generator - The PSDL to Java generator.
     */
    public void
    toJava(PSDL2JavaGenerator generator)
    throws GenerationException
    {
        InterfaceObject itf = null;
        MethodObject method = null;
        AbstractStorageHomeDecl[] inherited = null;
        List vect = null;
        PsdlOperationDecl[] operations = null;
        AbstractStorageTypeDecl ast = null;
        StorageHomeKeyDecl[] keys = null;
        StorageHomeFactoryDecl[] factories = null;
        Declaration decl = null;

        itf = new InterfaceObjectImpl(ash_.getName());
        itf.addComment(" Abstract Storage Home "+ash_.getName());
        itf.addComment("");
        itf.addComment(" @author OpenCCM PSDL Compiler");
        itf.setModifier(ModifierKindImpl.mk_public);
        itf.setPackage(generator.getTranslator().getPackage(ash_));

        // Add Inherited Abstract Storage Home by ash
        inherited = ash_.getAbstractStorageHomeList().getAbstractStorageHomes();
        for (int i=0; i<inherited.length; i++)
            itf.addInheritedObject(generator.getTranslator().getAbsoluteName(inherited[i]));

        if (inherited.length < 1)
            itf.addInheritedObject("org.omg.CosPersistentState.StorageHomeBase");

        generator.getJavaRepository().addObject(itf);

        // Get the managed Abstract Storage Type
        ast = ash_.getAbstractStorageType();

        // Map Factories
        vect = generator.getDeclarations(ash_, DeclarationKind.dk_storage_home_factory);
        factories = (StorageHomeFactoryDecl[])vect.toArray(new StorageHomeFactoryDecl[0]);
        for (int i=0; i<factories.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.PSDLFactoryMapping factory = null;
            
            factory = (org.objectweb.openccm.pss.generator.common.api.PSDLFactoryMapping) 
                            MappingObjectFactory.getClass("PSDLFactoryMapping");
            factory.setFactory(factories[i]);
            factory.toJava(generator, itf, ast, null);
        }

        // Map Keys
        vect = generator.getDeclarations(ash_, DeclarationKind.dk_storage_home_key);
        keys = (StorageHomeKeyDecl[])vect.toArray(new StorageHomeKeyDecl[0]);
        for (int i=0; i<keys.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.PSDLKeyMapping key = null;
            
            key = (org.objectweb.openccm.pss.generator.common.api.PSDLKeyMapping) 
                        MappingObjectFactory.getClass("PSDLKeyMapping");
            key.setKey(keys[i]);
            key.toJava(generator, itf, ast);
        }

         // Map Operations
        vect = generator.getDeclarations(ash_, DeclarationKind.dk_psdl_operation);
        operations = (PsdlOperationDecl[])vect.toArray(new PsdlOperationDecl[0]);
        for (int i=0; i<operations.length; i++)
        {
            org.objectweb.openccm.pss.generator.common.api.PSDLOperationMapping op = null;
            
            op = (org.objectweb.openccm.pss.generator.common.api.PSDLOperationMapping)
                        MappingObjectFactory.getClass("PSDLOperationMapping");
            op.setOperation(operations[i]);
            op.toJava(generator.getTranslator(), itf);
        }
   }

}
