/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Alex Andrushchak.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.runtime.hibernate.lib;

import org.objectweb.openccm.pss.runtime.common.lib.PIDHelper;

/**
 * This class establish a session with a Hibernate implementation.
 * It implements the org.objectweb.openccm.services.pss.api.Connector interface.
 *
 * @author <a href="mailto:pretend@ukr.net">Alex Andrushchak</a>
 *
 * @version 0.1
 */

public class Session
     extends org.objectweb.openccm.pss.runtime.hibernate.lib.CatalogBase
  implements org.objectweb.openccm.pss.runtime.hibernate.api.Session
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param id - The catalog id.
     * @param session - The associted Hibernate Session.
     */
    public Session(int id, net.sf.hibernate.Session session)
    {
        super(id, session);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Catalog
    //
    // ==================================================================

    /**
     * Retrieve a storage object with its pid.
     */
    public java.lang.Object
    find_by_pid(byte[] the_pid)
    throws org.omg.CosPersistentState.NotFound
    {
        // System.out.println("Session.find_by_pid: the_pid=" + the_pid);
        org.omg.CosPersistentState.StorageHomeBase sh = null;

        // Get the object storage home with its Repository ID
        sh = find_storage_home(PIDHelper.get_RID(the_pid));

        return sh.find_by_short_pid(PIDHelper.get_short_pid(the_pid));
    }

    /**
     * Operation flush
     */
    public void
    flush()
    {
        commit_tx();
    }

    /**
     * Operation refresh
     */
    public void
    refresh()
    {
        rollback_tx();
    }

    /**
     * Operation free_all
     */
    public void
    free_all()
    {
        // For each Storage Home, free registered storage types incarnations
        // TO DO
    }

    /**
     * Operation close
     */
    public void
    close()
    {
        flush();
        try {
            get_hibernate_session().close();
        } catch (net.sf.hibernate.HibernateException ex) {
            throw new RuntimeException(ex);
        }
    }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.common.api.CatalogBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase
    //
    // ==================================================================
}
