/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Alex Andrushchak.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.runtime.hibernate.lib;

import org.objectweb.openccm.pss.runtime.common.lib.PIDHelper;


/**
 *  This is a base class for Storage Homes implemented with Hibernate.
 *
 * @author <a href="mailto:pretend@ukr.net">Alex Andrushchak</a>
 *
 * @version 0.1
 */

public abstract class StorageHomeBase
    extends org.objectweb.openccm.pss.runtime.common.lib.StorageHomeBase
    implements org.objectweb.openccm.pss.runtime.hibernate.api.StorageHomeBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StorageHomeBase()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.StorageHomeBase
    //
    // ==================================================================

    /**
     * Retrieve a storage object with its short pid.
     *
     * @param short_pid - The short pid to find.
     *
     * @throws org.omg.CosPersistentState.NotFound ifstorage object can't be found.
     *
     * @return The binding storage object.
     */
    public java.lang.Object
    find_by_short_pid(byte[] short_pid)
    throws org.omg.CosPersistentState.NotFound
    {
        String sshort_pid = new String(short_pid);
        // System.out.println("StorageHomeBase.find_by_short_pid: sshort_pid=" + sshort_pid);
        Integer identifier = null;
        try {
            identifier = new Integer(sshort_pid);
        } catch (NumberFormatException ex) {
            ex.printStackTrace();
            throw new org.omg.CosPersistentState.NotFound();
        }

        // First, search in storage type incarnations
        java.util.Iterator it = _storage_type_incarnations.iterator();
        org.omg.CosPersistentState.StorageObject so = null;

        while(it.hasNext())
        {
            so = (org.omg.CosPersistentState.StorageObject) it.next();
            if ( java.util.Arrays.equals(so.get_short_pid(), short_pid) )
            {
                return so;
            }
        }

        // If not found, search in the database
        java.lang.Object obj = null;
        net.sf.hibernate.Session session = null;
        org.objectweb.openccm.pss.runtime.common.api.CatalogBase catalog = null;
        java.lang.Class clazz = null;

        session = ((org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase) get_catalog()).
                get_hibernate_session();

        catalog = (org.objectweb.openccm.pss.runtime.common.api.CatalogBase) get_catalog();
        clazz = catalog.get_connector().get_storage_object_factory( get_st_rid() );

        try {
            obj = session.load(clazz, identifier);
            _storage_type_incarnations.add(obj);
            ((org.objectweb.openccm.pss.runtime.common.api.StorageObject) obj).initialize(this);
        } catch (net.sf.hibernate.HibernateException ex) {
            ex.printStackTrace();
            throw new org.omg.CosPersistentState.NotFound();
        }
        return obj;
    }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.common.api.StorageHomeBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.hibernate.api.StorageHomeBase
    //
    // ==================================================================

    /**
     * Create a pid for a storage type.
     *
     * @param short_pid - The short_pid of the storage type to encode.
     *
     * @return The storage type PID.
     */
    public byte[]
    create_pid(String short_pid)
    {
        return PIDHelper.encode(get_rid(), short_pid);
    }

    /**
     * Must be called before create method invokation.
     */
    public void
    before_create()
    {
        ((org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase) get_catalog()).begin_tx();
    }

    /**
     * Must be called after create method invokation.
     *
     * @param st - The storage type to make persistent.
     */
    public void
    after_create(org.objectweb.openccm.pss.runtime.hibernate.api.StorageObject st)
    {
        org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase catalog = null;
        net.sf.hibernate.Session session = null;
        st.initialize(this);

        // Commit the storage Type
        catalog = (org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase) get_catalog();
        session = catalog.get_hibernate_session();

        try {
            session.save(st);
        } catch (net.sf.hibernate.HibernateException ex) {
            throw new RuntimeException(ex);
        }

        catalog.commit_tx();

    }

    /**
     * Must be called after create method invokation.
     *
     * @param st - The storage type to make persistent.
     */
    public Object
    find_by_key(String[] fields_names, String[] fields_types, java.lang.Object[] key_values)
    {
        net.sf.hibernate.Session session = null;
        String query = null;
        java.lang.Class clazz = null;
        java.util.List l = null;
        org.objectweb.openccm.pss.runtime.common.api.StorageObject so;
        net.sf.hibernate.type.Type[] types = TypeMapping.toTypes(fields_types);

        // Get the hibernate session
        session = ((org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase) get_catalog()).
                get_hibernate_session();

        // Get ST class
        clazz = ((org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase) get_catalog()).
                    get_connector().get_storage_object_factory(get_st_rid());

        // Create the query
        query = "from " + clazz.getName() + " where ";
        for (int i=0; i<fields_names.length; i++) {
            query += fields_names[i] + "=?";
            if (i < fields_names.length-1) {
                query += " and ";
            }
        }
       
        try {
            l = session.find(query, key_values, types);
        } catch (net.sf.hibernate.HibernateException ex) {
            throw new RuntimeException(ex);
        }

        if ( 1 == l.size() ) {
            so = (org.objectweb.openccm.pss.runtime.common.api.StorageObject) l.get(0);
            so.initialize(this);
            return so;
        }
        return null;
    }
}
