/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Alex Andrushchak.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.runtime.hibernate.lib;

/**
 *  This is a base class for Storage Objects implemented with Hibernate.
 *
 * @author <a href="mailto:pretend@ukr.net">Alex Andrushchak</a>
 *
 * @version 0.1
 */

public abstract class StorageObjectBase
    extends org.objectweb.openccm.pss.runtime.common.lib.StorageObjectBase
    implements org.objectweb.openccm.pss.runtime.hibernate.api.StorageObject
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StorageObjectBase()
    {
        super();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    /**
     * Create the short PID.
     */
    private String
    create_short_pid()
    {
        org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase catalog = null;
        net.sf.hibernate.Session session = null;

        // Get the persistent manager
        catalog = (org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase)
                        get_storage_home().get_catalog();
        session = catalog.get_hibernate_session();

        try {
            return session.getIdentifier(this).toString();
        } catch (net.sf.hibernate.HibernateException ex) {
            throw new RuntimeException(ex);
        }
    }


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.StorageObject
    //
    // ==================================================================

    /**
     * Destroy the associated Storage Object.
     */
    public void
    destroy_object()
    {
        org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase catalog = null;
        net.sf.hibernate.Session session = null;

        // Get the persistent manager
        catalog = (org.objectweb.openccm.pss.runtime.hibernate.api.CatalogBase)
                        get_storage_home().get_catalog();
        session = catalog.get_hibernate_session();

        // Remove the storage type
        catalog.begin_tx();
        try {
            session.delete(this);
        } catch (net.sf.hibernate.HibernateException ex) {
            throw new RuntimeException(ex);
        }
        catalog.commit_tx();
   }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.common.api.StorageObject
    //
    // ==================================================================
    /**
     * Get the PID.
     */
    public byte[]
    get_pid()
    {
        if (_pid == null)
        {
            org.objectweb.openccm.pss.runtime.hibernate.api.StorageHomeBase sh = null;

            sh = (org.objectweb.openccm.pss.runtime.hibernate.api.StorageHomeBase) get_storage_home();
            _pid = sh.create_pid( create_short_pid() );
        }
        return _pid;
    }


    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.hibernate.api.StorageObject
    //
    // ==================================================================

}
