/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.runtime.jdo.lib;


/**
 * CatalogBase is a base class for JDO Catalog.
 * It implements the org.objectweb.openccm.pss.runtime.api.Catalog interface.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public abstract class CatalogBase
              extends org.objectweb.openccm.pss.runtime.common.lib.CatalogBase
           implements org.objectweb.openccm.pss.runtime.jdo.api.CatalogBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * Persistent Manager reference.
     */
    private javax.jdo.PersistenceManager _persistence_manager;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param id - The catalog id.
     * @param pm - The associted JDO Persistence Manager.
     */
    public CatalogBase(int id, javax.jdo.PersistenceManager pm)
    {
        super(id);
        _persistence_manager = pm;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Catalog
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.common.api.CatalogBase
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.jdo.api.CatalogBase
    //
    // ==================================================================

    /**
     * Get the JDO Persistence Manager.
     *
     * @return The associated persistence manager.
     */
    public javax.jdo.PersistenceManager
    get_persistence_manager()
    {
        return _persistence_manager;
    }

    /**
     * Begin a transaction in the JDO Persistence Manager.
     */
    public void
    begin_tx()
    {
        javax.jdo.Transaction t = get_persistence_manager().currentTransaction();
        if ( !t.isActive() )
            t.begin();
    }

    /**
     * Commit a transaction in the JDO Persistence Manager.
     */
    public void
    commit_tx()
    {
        javax.jdo.Transaction t = get_persistence_manager().currentTransaction();
        if ( t.isActive() )
        {
            t.commit();
            begin_tx();
        }
    }

    /**
     * Rollback a transaction in the JDO Persistence Manager.
     */
    public void
    rollback_tx()
    {
        javax.jdo.Transaction t = get_persistence_manager().currentTransaction();
        if ( t.isActive() )
        {
            t.rollback();
            begin_tx();
        }
    }
}
