/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: CCMASTModelCreateUtils.java,v 1.3 2005/04/01 20:04:08 contrera Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.ast;

import ispuml.mdaTransformation.model.jmi.JmiModelCreateUtils;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.File;
import java.util.Collection;

import javax.jmi.reflect.RefPackage;
import org.objectweb.openccm.ast.api.FileScope;
import org.objectweb.openccm.ast.api.Scope;



/**
 * Class containing utilities for the CCM Abstract Syntax Tree (IDL repository).
 * @author Pierre Carpentier
 */

public class CCMASTModelCreateUtils extends JmiModelCreateUtils {

    /** The CCM Abstract Syntax Tree. */
    private static org.objectweb.openccm.ast.lib.Repository ast;

    /**
     * Gets the CCM Abstract Syntax Tree.
     * @return The CCM AST.
     */
    public static org.objectweb.openccm.ast.lib.Repository getAst() {
        return ast;
    }
    
    
	/**
	 * Create a new model to the OpenCCM AST.
	 * @return The new Scope (root of the model).
	 */
	public static Scope createModel() {
        org.omg.CORBA.ComponentIR.Repository ir = org.objectweb.openccm.corba.TheInterfaceRepository.getRepository();
        ast = new org.objectweb.openccm.ast.lib.Repository(org.objectweb.openccm.ir3.api.ComponentRepositoryHelper.narrow(ir));

        FileScope scope = ast.startFileScope("");
        scope.create();
        
		return scope;
	}

	/**
	 * Get the underlying implementation of the extent.
	 * @param extent The RefPackage.
	 * @return The outermost package.
	 */
	public static RefPackage getModelImpl(RefPackage extent) {
		return extent.refOutermostPackage();
	}

	/**
	 * Read model from XMI.
	 * @param filename The XMI filename which contains the model.
     * @param extent The outermost package.
     * @param importer The class which is able to import the XMI file.
	 */
	public static void readModel(String filename, RefPackage extent, String importer) {
		try {
			readModel(new FileInputStream(new File(filename)), extent, importer);
		} catch (java.lang.Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Read model from XMI.
	 * @param input The InputStream of the XMI file.
     * @param extent The outermost package.
     * @param importer The class which is able to import the XMI file.
	 */
	public static void readModel(InputStream input, RefPackage extent, String importer) {
		try {
			// Taylored
            Class importerClass = Class.forName(importer);
            Object importerInstance = importerClass.newInstance();
            java.lang.reflect.Method [] methods = importerClass.getMethods();
            java.lang.reflect.Method initiateMethod = null;
            for (int i=0 ; i<methods.length ; i++) {
                if (methods[i].getName().equals("initiate")) {
                    initiateMethod = methods[i];
                }
            }
            if (initiateMethod == null)
                throw new Exception("Method initiate not found in " + importer + " class.");

            Object initParam [] = { extent };
            initiateMethod.invoke(importerInstance, initParam);
            Class parseParamType [] = { InputStream.class };
            java.lang.reflect.Method parseMethod = importerClass.getMethod("parse", parseParamType);
            Object parseParam [] = { input };
            parseMethod.invoke(importerInstance, parseParam);
		} catch (java.lang.Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Write model to XMI.
     * @param filename The XMI filename which will contain the model.
     * @param extent The outermost package.
     * @param exporter The class which is able to export the XMI file.
	 */
	public static void writeModel(String filename, RefPackage extent, String exporter) {
        throw new UnsupportedOperationException("Write model not implemented");
	}

	/**
	 * Get all roots from model.
	 * @param elementName
     * @return The list of the roots.
	 */
	public static Collection getRoots(RefPackage extent) {
		Collection roots = new java.util.ArrayList();
		addRoots(roots, extent);
		return roots;
	}

}