/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: OperationParameters.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.ast;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.ActionBase;
import ispuml.mdaTransformation.rules.xml.CompositeXmlAction;

/**
 * This action transforms an UML FeatureParamenter to a Java Parameter
 * (pdk_in, pdk_out, pdk_inout) or a TypeDescriptor (pdk_return).
 * 
 * @author Pierre Carpentier
 */
public class OperationParameters extends CompositeXmlAction {

    /**
     * Constructor.
     */
    public OperationParameters() {
        // The src property is not required. If it is not set, use the bean provided
        // as parameter.
        isSrcPropertyRequired = false;
        isDstPropertyRequired = false;
        action = new TransformAction();
    }

    /**
     * Inner class
     */
    class TransformAction extends ActionBase {
        /**
         * Transform the bean and return the result.
         * @param object
         * @return Object
         */
        public Object execute(Object bean, RuleContext request) throws TransformationException {
            org.omg.uml.core.Operation operation = (org.omg.uml.core.Operation) bean;
            CCMASTModelContext context;
            context = (CCMASTModelContext) request.engineContext.getModel("ccm-ast");
            org.objectweb.openccm.ast.api.OperationDecl op = (org.objectweb.openccm.ast.api.OperationDecl) request.getAttribute("declaration");

            java.util.Iterator parameters = operation.getParameter().iterator();
            while (parameters.hasNext()) {
                org.omg.uml.core.Parameter param = (org.omg.uml.core.Parameter) parameters.next();

                //org.objectweb.corba.ast.api.TypeRef typeParameter =
                //    (org.objectweb.corba.ast.api.TypeRef) request.engineContext.engine.transform(param, request);
                org.objectweb.openccm.ast.api.TypeRef typeParameter = getType(param.getType());

                if (param.getKind() == org.omg.uml.datatypes.ParameterDirectionKindEnum.PDK_IN) {
                    op.getParameterList().addInParam(param.getName(), typeParameter);
                } else if (param.getKind() == org.omg.uml.datatypes.ParameterDirectionKindEnum.PDK_OUT) {
                    op.getParameterList().addOutParam(param.getName(), typeParameter);
                } else if (param.getKind() == org.omg.uml.datatypes.ParameterDirectionKindEnum.PDK_INOUT) {
                    op.getParameterList().addInOutParam(param.getName(), typeParameter);
                } else if (param.getKind() == org.omg.uml.datatypes.ParameterDirectionKindEnum.PDK_RETURN) {
                    op.setType(typeParameter);
                }
            }

            return null;
        }

        /**
         * Get the CORBA type corresponding to an UML type. 
         * @param type The UML Type.
         * @return The CORBA Type.
         */
        private org.objectweb.openccm.ast.api.TypeRef getType(org.omg.uml.core.Classifier type) {
            String typeName = type.getName().toLowerCase();
            if (typeName.equals("short"))
                return CCMASTModelCreateUtils.getAst().getShortType();
            else if (typeName.equals("long"))
                return CCMASTModelCreateUtils.getAst().getLongType();
            else if (typeName.equals("long long"))
                return CCMASTModelCreateUtils.getAst().getLongLongType();
            else if (typeName.equals("double"))
                return CCMASTModelCreateUtils.getAst().getDoubleType();
            else if (typeName.equals("long double"))
                return CCMASTModelCreateUtils.getAst().getLongDoubleType();
            else if (typeName.equals("unsigned short"))
                return CCMASTModelCreateUtils.getAst().getUShortType();
            else if (typeName.equals("unsigned long"))
                return CCMASTModelCreateUtils.getAst().getULongType();
            else if (typeName.equals("unsigned long long"))
                return CCMASTModelCreateUtils.getAst().getULongLongType();
            else if (typeName.equals("any"))
                return CCMASTModelCreateUtils.getAst().getAnyType();
            else if (typeName.equals("boolean"))
                return CCMASTModelCreateUtils.getAst().getBooleanType();
            else if (typeName.equals("string"))
                return CCMASTModelCreateUtils.getAst().getStringType();
            else if (typeName.equals("octet"))
                return CCMASTModelCreateUtils.getAst().getOctetType();
            else if (typeName.equals("void"))
                return CCMASTModelCreateUtils.getAst().getVoidType();
            else if (typeName.equals("char"))
                return CCMASTModelCreateUtils.getAst().getCharType();
            else if (typeName.equals("wchar"))
                return CCMASTModelCreateUtils.getAst().getWCharType();
            else if (typeName.equals("float"))
                return CCMASTModelCreateUtils.getAst().getFloatType();
            else if (typeName.equals("wstring"))
                return CCMASTModelCreateUtils.getAst().getWStringType();
            else if (typeName.equals("typecode"))
                return CCMASTModelCreateUtils.getAst().getTypeCodeType();
            else if (typeName.equals("principal"))
                return CCMASTModelCreateUtils.getAst().getPrincipalType();
            else if (typeName.equals("object"))
                return CCMASTModelCreateUtils.getAst().getObjectType();
            else if (typeName.equals("valuebase"))
                return CCMASTModelCreateUtils.getAst().getValueBaseType();
            else
                System.out.println("Type " + type.getName() + " unrecognized !");
            return null;
        }

    } // end class
}
