/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: ValueMemberVisibility.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.ast;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.ActionBase;
import ispuml.mdaTransformation.rules.xml.CompositeXmlAction;


/**
 * This action sets the visibility of a ValueMember.
 */
public class ValueMemberVisibility extends CompositeXmlAction {

    /**
     * Constructor.
     */
    public ValueMemberVisibility() {
        // The src property is not required. If it is not set, use the bean provided
        // as parameter.
        isSrcPropertyRequired = false;
        isDstPropertyRequired = false;
        action = new TransformAction();
    }

    /**
     * Inner class
     */
    class TransformAction extends ActionBase {
        /**
         * Sets the visibility of the ValueMember (according to the visibility of the UML class).
         * @param bean The UML class representing the ValueMember.
         * @param request The context.
         * @return Object The ValueMember.
         */
        public Object execute(Object bean, RuleContext request) throws TransformationException {
            Object obj = request.getAttribute("visibility");
            org.omg.uml.datatypes.VisibilityKind visibility;
            visibility = (org.omg.uml.datatypes.VisibilityKind)obj;
            obj = request.getAttribute("declaration");
            org.objectweb.openccm.ast.api.ValueMemberDecl declaration;
            declaration = (org.objectweb.openccm.ast.api.ValueMemberDecl) obj;
            
            if (visibility.toString().equals("vk_public")) {
            	declaration.setPublic();
            } else if (visibility.toString().equals("vk_private")) {
            	declaration.setPrivate();
            }
            return declaration;
        }

    } // end class
}
