/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: WriteCidlFile.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.ast;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.ActionBase;
import ispuml.mdaTransformation.rules.xml.CompositeXmlAction;


/**
 * This action write the OMG CIDL file from the OpenCCM AST.
 */
public class WriteCidlFile extends CompositeXmlAction {

    /**
     * Constructor.
     */
    public WriteCidlFile() {
        // The src property is not required. If it is not set, use the bean provided
        // as parameter.
        isSrcPropertyRequired = false;
        isDstPropertyRequired = false;
        action = new TransformAction();
    }
    

    /**
     * Inner class
     */
    class TransformAction extends ActionBase {
        /**
         * Write the OMG CIDL File.
         * @param bean The current bean (not used).
         * @param request The context.
         * @return Object The CIDL Module.
         */
        public Object execute(Object bean, RuleContext request) throws TransformationException {
            Object obj = request.getAttribute("declaration");
            org.objectweb.openccm.ast.api.CidlModuleDecl cidlModule;
            cidlModule = (org.objectweb.openccm.ast.api.CidlModuleDecl) obj;
            String cidlModuleName = cidlModule.getName();
            
            // Generation of the CIDL file
            String outputdir = (String)request.engineContext.getAttribute("outputdir");
            String filename = outputdir + java.io.File.separator + cidlModuleName + ".cidl";
            org.objectweb.openccm.generator.cidl.lib.CIDLGenerator cidlGen;
            cidlGen = new org.objectweb.openccm.generator.cidl.lib.CIDLGenerator(CCMASTModelCreateUtils.getAst());
            cidlGen.initialize(filename, "XMItoCIDL");
            try {
                cidlGen.generate(cidlModule, org.objectweb.openccm.ast.api.DeclarationKind.dk_cidl);
            } catch(org.objectweb.openccm.generator.common.lib.GenerationException ex) {
                ex.printStackTrace();
            }
            
            return cidlModule;
        }

    } // end class
}
