/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: JmiModFactModelCreateUtils.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.modfact;

import ispuml.mdaTransformation.model.jmi.JmiModelCreateUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Collection;

import javax.jmi.reflect.RefPackage;


/**
 * Class containing utilities for a (ModFact) JMI Repository.
 * @author Cedric Dumoulin, Pierre Carpentier
 */

public class JmiModFactModelCreateUtils extends JmiModelCreateUtils {

	/**
	 * Create a new extent with the specified implementation.
	 * @param modelImplClassName The nsuml outermost package implementation.
	 * @return The new RefPackage.
	 */
	public static RefPackage createModel(String metaModelURL, String metamodelJmiClasses) {
		try {
			// Taylored
			Class metaModelClass = Class.forName(metamodelJmiClasses);
			Class parameterType [] = { String.class };
			java.lang.reflect.Method createMethod = metaModelClass.getMethod("create", parameterType);
			Object parameter [] = { metaModelURL };
			return (RefPackage)createMethod.invoke(null, parameter);
			// Reflexive
			//RefObject metaObject = RefPackageImpl.createMetaObject(metaModelName, metaModelURL);
			//return RefPackageImpl.getM1Repository(metaObject);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		return null;
	}

	/**
	 * Get the underlying implementation of the extent.
	 * @param extent The RefPackage
	 * @return The outermost package.
	 */
	public static RefPackage getModelImpl(RefPackage extent) {
		return extent.refOutermostPackage();
	}

	/**
	 * Read model from XMI.
     * @param file The XMI file which contains the model.
     * @param extent The outermost package.
     * @param importer The class which is able to import the XMI file.
	 */
	public static void readModel(File file, RefPackage extent, String importer) {
		try {
			readModel(new FileInputStream(file), extent, importer);
		} catch (java.lang.Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Read model from XMI.
     * @param input The InputStream of the XMI file.
     * @param extent The outermost package.
     * @param importer The class which is able to import the XMI file.
	 */
	public static void readModel(InputStream input, RefPackage extent, String importer) {
		try {
			// Taylored
            Class importerClass = Class.forName(importer);
            Object importerInstance = importerClass.newInstance();
            java.lang.reflect.Method [] methods = importerClass.getMethods();
            java.lang.reflect.Method initiateMethod = null;
            for (int i=0 ; i<methods.length ; i++) {
                if (methods[i].getName().equals("initiate")) {
                    initiateMethod = methods[i];
                }
            }
            if (initiateMethod == null)
                throw new Exception("Method initiate not found in " + importer + " class.");

            Object initParam [] = { extent };
            initiateMethod.invoke(importerInstance, initParam);
            Class parseParamType [] = { InputStream.class };
            java.lang.reflect.Method parseMethod = importerClass.getMethod("parse", parseParamType);
            Object parseParam [] = { input };
            parseMethod.invoke(importerInstance, parseParam);

            // Taylored - Only UML
			//org.omg.uml.xmi.UmlXMIImport xmiImport = new org.omg.uml.xmi.UmlXMIImport();
			//xmiImport.initiate((org.omg.uml.UmlPackage)extent);
			//org.omg.uml.UmlPackage _package = (org.omg.uml.UmlPackage) xmiImport.parse(input);
			
			// Reflexive
			//XmiReaderImpl reader = new XmiReaderImpl();
			//System.out.println("readModel(input, " + extent + ")");
			//reader.read(input, "", extent);

			//System.out.println("readModel(input, " + getModelImpl(extent) + ")");
			//reader.read(input, "", getModelImpl(extent));
			//MDFOutermostPackageImpl extentImpl = getModelImpl(extent);
			//extentImpl.getXMIReader( xmiVersion ).read( extentImpl, input );
		} catch (java.lang.Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Write model to XMI.
     * @param file The XMI file which will contain the model.
     * @param extent The outermost package.
     * @param exporter The class which is able to export the XMI file.
	 */
	public static void writeModel(File file, RefPackage extent, String exporter) {
		try {
            // Taylored - Only UML
            //org.omg.uml.xmi.UmlXMIExport xmiExport = new org.omg.uml.xmi.UmlXMIExport();
            //xmiExport.setOutput(new FileOutputStream(new File("output.xmi")));
            //xmiExport.parse((org.omg.uml.UmlPackage)extent);
            //org.omg.uml.UmlPackage _package = (org.omg.uml.UmlPackage) xmiImport.parse(input);
            org.objectweb.modfact.jmi.logging.ModFactLogger _logger;
            _logger = org.objectweb.modfact.jmi.logging.ModFactLogger.getLogger("ModelXMIExport");

            Class exporterClass = Class.forName(exporter);
            Class [] constructorParamType = { String.class, org.objectweb.modfact.jmi.logging.ModFactLogger.class };
            java.lang.reflect.Constructor constructor = exporterClass.getConstructor(constructorParamType);
            Object [] constructorParam = { new String(), _logger };
            Object exporterInstance = constructor.newInstance(constructorParam);
            //exporterClass.newInstance();
            
            FileOutputStream fileOutput = new FileOutputStream(file);
            Class outputParamType [] = { java.io.OutputStream.class };
            Object outputParam [] = { fileOutput };
            exporterClass.getMethod("setOutput", outputParamType).invoke(exporterInstance, outputParam);
            Class parseParamType [] = { Object.class };
            Object parseParam [] = { extent };
            exporterClass.getMethod("parse", parseParamType).invoke(exporterInstance, parseParam);
            
            // Reflexive
			//XmiWriterImpl writer = new XmiWriterImpl();
			//FileOutputStream fos = new FileOutputStream(new File(filename));
			//writer.write(fos, extent, "");
		} catch (java.lang.Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Get all roots from model.
	 * @param elementName
	 * @return The list of the roots.
	 */
	public static Collection getRoots(RefPackage extent) {
		Collection roots = new java.util.ArrayList();
		addRoots(roots, extent);
		return roots;
	}

}