/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: CopyAssociatedClass.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.xml;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.rules.xml.BeanProperty;
import ispuml.mdaTransformation.rules.xml.CopyXmlAction;

/**
 * Action to copy the associated class to a bean or a property.
 * 
 * @author Pierre Carpentier
 */
public class CopyAssociatedClass extends CopyXmlAction {
    
    /** The search stereotype of the association. */
    private String associatedClassWithStereotype;
    
    /** The stereotype of the class associated by the search association. */
    private String associatedClassHasStereotype;
    
    /** The type of the search association. */
    private String associatedClassWithType;

    /** The Util class which offers methods to retrieve the associated class. */
    private AssociatedClassUtils assoClassUtils;
    
    
    /**
     * Default Constructor
     */
    public CopyAssociatedClass () {
        super();
        assoClassUtils = new AssociatedClassUtils();
        srcProperty = new AssociatedBeanProperty();
    }


    /**
     * Sets the stereotype of the search association.
     * @param stereotype The stereotype.
     */
    public void setAssociatedClassWithStereotype(String stereotype) {
        associatedClassWithStereotype = stereotype;
    }


    /**
     * Sets the stereotype of the class associated by the search association.
     * @param stereotype The stereotype.
     */
    public void setAssociatedClassHasStereotype(String stereotype) {
        associatedClassHasStereotype = stereotype;
    }


    /**
     * Sets the type of the search association.
     * @param type The type.
     */
    public void setAssociatedClassWithType(String type) {
        associatedClassWithType = type;
    }
  
    
    private class AssociatedBeanProperty extends BeanProperty {
        /* (non-Javadoc)
         * @see ispuml.mdaTransformation.rules.xml.Property#getValue(java.lang.Object, ispuml.mdaTransformation.RuleContext)
         */
        public Object getValue(Object object, RuleContext context) throws TransformationException {
            Object bean = super.getValue(object, context);
            if (associatedClassWithType != null)
                return assoClassUtils.getAssociatedClass(bean, associatedClassWithType, context.engineContext.getModel(object));
            else
                return assoClassUtils.getAssociatedClass(bean, associatedClassWithStereotype, associatedClassHasStereotype, context.engineContext.getModel(object));
        }
    }

}
