/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: CopyEnumeration.java,v 1.2 2004/10/08 15:53:07 merle Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.xml;

import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.rules.xml.Rule;

import java.util.StringTokenizer;
import javax.jmi.reflect.RefEnum;

/**
 * Rule to copy an enumeration (source model) into an other
 * enumeration (in the destination model).
 * 
 * @author Pierre Carpentier
 */
public class CopyEnumeration extends Rule implements EnumerationRule {

    /** The possible value of the source enumeration. */
    private String srcLabels [];
    
    /** The possible value of the destination enumeration. */
    private String dstLabels [];
    
    /**
     * Parses the string representing the possible value of the enumeration
     * and converts it into an array of String.
     * @param labels The String representing the value of the enumeration (separated with '|')
     * @return The array of String (values of enumeration).
     */
    private String [] parseLabels(String labels) {
        StringTokenizer token = new StringTokenizer(labels, "|");
        String [] arrayLabels = new String[token.countTokens()];
        for (int i=0 ; token.hasMoreTokens() ; i++) {
            arrayLabels[i] = token.nextToken();
        }
        return arrayLabels;
    }

    /**
     * Sets the possible values of the source enumeration.
     * @param srcLabel The String representing the value of the source enumeration (separated with '|')
     */
    public void setMappingSrcLabel(String srcLabel) {
        srcLabels = parseLabels(srcLabel);
    }
    
    /**
     * Sets the possible values of the destination enumeration.
     * @param srcLabel The String representing the value of the destination enumeration (separated with '|')
     */
    public void setMappingDstLabel(String dstLabel) {
        dstLabels = parseLabels(dstLabel);
    }
    
    /**
     * Gets the equivalent value of the destination enumeration which 
     * corresponds to the value of the source enumeration.
     * @param object The source enumeration value.
     * @return The String representing the value of the destination enumeration.
     * @throws TransformationException
     */
    public String getEquivalentLabel(Object object) throws TransformationException {
        RefEnum refEnum = (RefEnum) object;
        if (srcLabels.length != dstLabels.length)
            throw new TransformationException("The mapping is incorrect.");
        for (int i=0 ; i<srcLabels.length ; i++) {
            //System.out.println(">>> Comparison between " + refEnum.toString() + " and " + srcLabels[i]);
            if (refEnum.toString().equals(srcLabels[i])) {
                 return dstLabels[i];
            }
        }
        throw new TransformationException("The equivalent of " + refEnum.toString() + " is not found.");
    }

}
