/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: ParameterTypeCondition.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.xml;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.rules.xml.Condition;

/**
 * Condition to test the type of a Parameter (according its DirectionKind)
 * 
 * @author Pierre Carpentier
 */
public class ParameterTypeCondition extends Condition {

    /** The direction kind to test. */
    private String directionKind;

    /**
     * Sets the direction kind to test.
     * @param directionKind The String representation of the direction kind.
     */
    public void setDirectionKind(String directionKind) {
        this.directionKind = directionKind;
    }
    
    /* (non-Javadoc)
     * @see ispuml.mdaTransformation.Guard#isAllowed(java.lang.Object, ispuml.mdaTransformation.RuleContext)
     */
    public boolean isAllowed(Object object, RuleContext context) throws TransformationException {
        if (org.omg.uml.core.Parameter.class.isAssignableFrom(object.getClass())) {
            org.omg.uml.core.Parameter param = (org.omg.uml.core.Parameter)object;
            if (param.getKind().toString().equals(directionKind))
                return true;
        }
        return false;
    }

}
