/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Pierre Carpentier.
Contributor(s): Philippe Merle.

---------------------------------------------------------------------
$Id: TransformAssociatedClass.java,v 1.1 2004/05/26 11:25:35 carpentier Exp $
====================================================================*/

package org.objectweb.openccm.uml.transformation.rules.xml;

import ispuml.mdaTransformation.RuleContext;
import ispuml.mdaTransformation.TransformationException;
import ispuml.mdaTransformation.rules.xml.BeanProperty;
import ispuml.mdaTransformation.rules.xml.RuleCallXmlAction;

/**
 * Class to transform a class which is associated with the
 * current bean according to a particular stereotype.
 * 
 * @author Pierre Carpentier 
 */
public class TransformAssociatedClass extends RuleCallXmlAction {

    /** The search stereotype of the association. */
    private String associatedClassWithStereotype;
    
    /** The Util class which offers methods to retrieve the associated class. */
    private AssociatedClassUtils assoClassUtils;
    
    
    /**
     * Default Constructor
     */
    public TransformAssociatedClass () {
        super();
        assoClassUtils = new AssociatedClassUtils();
    }
    

    /**
     * Sets the stereotype of the search association.
     * @param stereotype The stereotype.
     */
    public void setAssociatedClassWithStereotype(String stereotype) {
        associatedClassWithStereotype = stereotype;
    }
    

    /**
     * Sets the 'srcProperty' inherited attribute with a value corresponding
     * to the associated class.
     * @param bean
     * @param request
     * @throws TransformationException If no association with the search stereotype exists.
     */
    private void setSrcAssociatedClassWithStereotype(Object bean, RuleContext context) throws TransformationException {
        Object value = assoClassUtils.getAssociatedClass(bean, associatedClassWithStereotype, null, context.engineContext.getModel(bean));
        if (value != null) {
            if (srcProperty == null) {
                srcProperty = new BeanProperty();
            }
            ((BeanProperty) srcProperty).setBeanName(associatedClassWithStereotype);
            context.putAttribute(associatedClassWithStereotype, value);
        } else {
            throw new TransformationException("The Association stereotyped '" + associatedClassWithStereotype + "' is not present.");
        }
    }
    

    /**
     * Gets the source value.
     * This method overloadeds the method defined in CompositeXmlAction
     * in order to consider the associated class as the source bean.
     * @param bean The current bean.
     * @param request The context.
     * @return The source value.
     * @throws TransformationException
     */
    protected Object getSrcValue(Object bean, RuleContext request) throws TransformationException {
        setSrcAssociatedClassWithStereotype(bean, request);
        return super.getSrcValue(bean, request);
    }

}
