#include <CORBA_cpp_gene_main_servant_constr.h>
#include <debug_utils.h>

CORBA_cpp_gene_main_servant_constr::CORBA_cpp_gene_main_servant_constr( CORBA::ComponentIR::ComponentDescription* desc )
  : desc_( desc )
{
}

string CORBA_cpp_gene_main_servant_constr::generate_code_in_place_of()
{
  string to_be_returned;
  
 to_be_returned = 
   get_mccm_debug_trace_warning("COMPONENT_IMPLEM")
   + get_mccm_assert("the_orb")
   + get_mccm_assert("the_home")
   + get_mccm_assert("the_poa") +
   "   the_orb_ = the_orb;\n"
   "   the_poa_ = the_poa;\n"
   "   the_home_ = the_home;\n"
   "ctx_component_ = new CCM_" + string(desc_->name) + "_Context_impl( 255 );\n";

  CORBA::ComponentIR::UsesDescriptionSeq& the_receptacles 
    = desc_->used_interfaces;
  
  if( the_receptacles.length() > 0 )
    {
      to_be_returned += "const char* names_of_receptacles[ ] = { ";

      // parcours des receptacles et ajouter dans le tableau les noms des receptacles

      for( int ct = 0; ct < the_receptacles.length() - 1 ; ct++ )
	{
	  string name_receptacle( the_receptacles[ ct ].name );
	  
	  to_be_returned += "\"" + name_receptacle + "\"" + ",";
	}
      
      string name_receptacle( the_receptacles[ the_receptacles.length() - 1 ].name );
      to_be_returned += "\"" + name_receptacle + "\" };\n";

      to_be_returned += 
	"int size_names_of_receptacles = sizeof( names_of_receptacles ) / sizeof( const char* );\n"
	"for( int ct = size_names_of_receptacles - 1; ct >= 0; ct-- )\n"
	"{\n"
	"    refs_for_receptacles_[ names_of_receptacles[ ct ] ] = CORBA::Object::_nil();\n"
	"}\n"
	"\n";
    }

  CORBA::ComponentIR::ProvidesDescriptionSeq& the_facets
    = desc_->provided_interfaces;

  if( the_facets.length() > 0 )
    {
      to_be_returned += " const char* names_of_facets[ ] = { ";

      for( int ct = 0; ct < the_facets.length() - 1; ct++ )
	{
	  string name_facet( the_facets[ ct ].name );
	  to_be_returned += "\"" + name_facet + "\",";
	}
      string name_facet( the_facets[ the_facets.length() - 1 ].name );
      to_be_returned += "\"" + name_facet + "\" };\n";

      to_be_returned += 
	"int size_names_of_facets = sizeof( names_of_facets ) / sizeof( const char* );\n"
	"\n"
	"for( int ct = size_names_of_facets - 1; ct >= 0; ct-- )\n"
	"{\n"
	"   refs_for_facets_[ names_of_facets[ ct ] ] = CORBA::Object::_nil();\n"
	"}\n";

      if( desc_->publishes_events.length() > 0 )
	{
	  to_be_returned += string( "" ) 
	    + "ctx_component_->init_push_suppliers(the_orb_);\n"
	    "\n";
	}
    }

  // event publisher proxies
  CORBA::ComponentIR::EventPortDescriptionSeq& publishes_events
    = desc_->publishes_events;

  for(int ct_publisher = 0; ct_publisher < publishes_events.length(); ct_publisher++)
    {
      string name_of_publisher = string(publishes_events[ct_publisher].name);
      
      to_be_returned +=
	"proxy_" + name_of_publisher + "_ "
	+ "= new " + name_of_publisher + "_proxy(ctx_component_ );\n";
    }

  to_be_returned += "  ctx_component_->ptr_to_receptacles_ = &refs_for_receptacles_;\n";
  

  return to_be_returned;
}
