#include <CORBA_cpp_gene_main_servant_create_act.h>
#include <debug_utils.h>

CORBA_cpp_gene_main_servant_create_act::CORBA_cpp_gene_main_servant_create_act( CORBA::ComponentIR::ComponentDescription* desc,
										CORBA::ComponentIR::Repository_ptr ref_repo )
  : desc_( desc ), ref_repo_( ref_repo )
{
}

string CORBA_cpp_gene_main_servant_create_act::generate_code_in_place_of()
{
  string to_be_returned;

  CORBA::ComponentIR::ProvidesDescriptionSeq& provided_interfaces
    = desc_->provided_interfaces;

  for( int ct = 0; ct < provided_interfaces.length(); ct++ )
    {
      string name_of_port( provided_interfaces[ ct ].name );

      CORBA::Contained_ptr ct_ptr
	= ref_repo_->lookup_id( provided_interfaces[ ct ].interface_type );

      string interface_type( ct_ptr->name() );

      to_be_returned += string( "" ) + 
	get_mccm_debug_trace_warning("COMPONENT_IMPLEM")
	+ get_mccm_assert("the_poa_")
	+ get_mccm_assert("ctx_component_") +
	"obj_impl_" + name_of_port + "_\n"
	" = new _microccm_" + interface_type + "_impl( the_comp_instance_->get_" + name_of_port + "() );\n"
	"\n"
	"objid_" + name_of_port + "_ = the_poa_->activate_object( obj_impl_" + name_of_port + "_ );\n"
	"refs_for_facets_[ \"" + name_of_port + "\" ] = the_poa_->id_to_reference( objid_" + name_of_port + "_.in() );\n"
	"\n";
    }

  // activation of all  xxxxConsumer event sink and all  xxxxPushConsumer
  CORBA::ComponentIR::EventPortDescriptionSeq& consumes_events
    = desc_->consumes_events;
  string name_of_component = string(desc_->name);

  for(int ct_consumer = 0; ct_consumer < consumes_events.length(); ct_consumer++)
    {
      CORBA::ComponentIR::EventDef_var evt_def 
	= CORBA::ComponentIR::EventDef::_narrow(ref_repo_->lookup_id(consumes_events[ct_consumer].event));

      string name_of_event = string(evt_def->name());
      string name_of_sink = string(consumes_events[ct_consumer].name);
      string name_of_event_consumer = name_of_component + "__" + name_of_event + "Consumer";

      // activation of xxxxConsumer event sink
      to_be_returned += 
	"{\n"
	"  proxy_" + name_of_event +"Consumer" + "_" + 
	" = new " + name_of_event_consumer + "_impl" + "(the_comp_instance_, \"" + name_of_sink + "\" );\n"
	"  PortableServer::ObjectId_var " +  name_of_sink + "_oid" + " = the_poa_->activate_object( proxy_" + name_of_event + "Consumer" + "_);\n"
	"  CORBA::Object_var " + name_of_sink + "_ref" + "= the_poa_->id_to_reference(" + name_of_sink + "_oid.in());\n"
	"  " + name_of_sink + "_" + " = " + name_of_event + "Consumer" + "::_narrow(" + name_of_sink + "_ref.in());\n"
	"  if(CORBA::is_nil(" + name_of_sink + "_))\n" 
	"  {\n"
	"      cerr << \"the event sink isn't created.\" << endl;\n"
	"  }\n";


      // activation of xxxxPushConsumer
      string name_of_push_consumer = name_of_component + "__" + name_of_event + "PushConsumer";

      to_be_returned += 
	"   " + name_of_push_consumer + "*" + " pushconsumer = new " + name_of_push_consumer + "(proxy_" + name_of_event + "Consumer_);\n"
	"   PortableServer::ObjectId_var " +  name_of_event + "_pushconsumer_oid = the_poa_->activate_object(pushconsumer);\n"
	"   CORBA::Object_var " + name_of_event + "_pushconsumer_ref = the_poa_->id_to_reference(" + name_of_event + "_pushconsumer_oid.in());\n"
	"   proxy_" + name_of_event + "Consumer_" + "->cos_pushconsumer_ref_ =  CosEventComm::PushConsumer::_narrow(" + name_of_event + "_pushconsumer_ref.in());\n"
	"  if(CORBA::is_nil( proxy_" + name_of_event + "Consumer_->cos_pushconsumer_ref_))\n "
	"  {\n"
	"    cerr << \"cos_pushconsumer_ref__ is nil.\" << endl;\n"
	"  }\n"
	"   " + name_of_event + "PushConsumers_[ " + "\""+ name_of_sink + "_" + "\"" + " ] = pushconsumer;\n"
	"}\n";
    }

  to_be_returned += "the_service_integration_points_ = new MCCM_Service_Integration_Points;\n";
  
  to_be_returned += "MCCM_has_set_interceptor* casted; \n";

  for( int ct = 0; ct < provided_interfaces.length(); ct++ )
    {
      string name_of_port( provided_interfaces[ ct ].name );

      CORBA::Contained_ptr ct_ptr
	= ref_repo_->lookup_id( provided_interfaces[ ct ].interface_type );

      string interface_type( ct_ptr->name() );

      to_be_returned += 
      "casted = dynamic_cast< MCCM_has_set_interceptor* >( obj_impl_" + name_of_port + "_" + ");\n";
      to_be_returned += "if ( casted != NULL )\n { \n ";
      to_be_returned += "the_service_integration_points_->register_has_set_interceptor( casted, \"" + name_of_port + "\" );\n";
      to_be_returned += "\n";
      to_be_returned += "}\n";

      // add-on for internal interfaces
      to_be_returned += " the_service_integration_points_->register_ext_session_context( ctx_component_ );\n";

      to_be_returned += "\n";
    }

  return to_be_returned;

}

