#include <iostream>
#include <Cpp_class_gene.h>
#include <typeinfo>

using namespace std;

Cpp_class_gene::Cpp_class_gene( string name_of_proxy ) 
{
  name_of_proxy_ = name_of_proxy; 
}

string Cpp_class_gene::get_name_of_class()
{
  return name_of_proxy_;
}

void Cpp_class_gene::add_inherited_class( string name_of_class, 
					    string visibility,
					    bool virtual_or_not)
{
  Inheritance_spec inherit_spec;
  inherit_spec.visibility = visibility;
  inherit_spec.virtual_or_not = virtual_or_not;

  inherited_classes_[ name_of_class ] = inherit_spec;
}

void Cpp_class_gene::add_friend_class(string class_name)
{
  friend_classes_.push_back(class_name);
}

void Cpp_class_gene::add_method( Cpp_method the_cpp_method) 
{

  for( int ct = 0; ct < methods_.size(); ct++ )
    {
      cerr << " ***" << methods_[ ct ] << endl;

      if( methods_[ ct ].equals( the_cpp_method ) )
	{
	  cerr << "Trying to add multiple times a method of name " 
	       << the_cpp_method.get_name_of_method() << endl;
	  exit( 1 );
	}
    }

  methods_.push_back( the_cpp_method );
}

void Cpp_class_gene::add_member( string type_of_member, string name ) 
{
  members_[ name ] = type_of_member; 
}

void Cpp_class_gene::add_pre_post( string name_of_method, Pre_post_invocation* ppic ) 
{
  pre_post_interceptors_[name_of_method].push_back(ppic); 
}

void Cpp_class_gene::add_implem( string name_of_method,
				 In_place_of_invocation* ioi)
{
  cerr << "Cpp_class_gene : adding implementation for method " 
       << name_of_method << endl;

  // looks up if unique function name in the class.
  int counter=0;
  int where = -1;

  for( int ct=0; ct < methods_.size(); ct++)
    {
      if ( methods_[ct].get_name_of_method() == name_of_method )
	{
	  if (counter==0) {  where = ct; }
	  counter++;
	}
    }

  if( counter == 0 )
    {
      cerr << "Cpp_class_gene : trying to add implementation for undefined method" << endl;
      exit( 1 );
    }

  if ( counter != 1 )
    {
      cerr << __FILE__ << ":" << __LINE__ << endl;
      cerr << "Trying to set implementation for method " << name_of_method << 
	" but different methods have this name" << endl;
      exit(-1);
    }
  else
    {
      cerr << __FILE__ << ":" << __LINE__ << endl;
      in_place_of_interceptors_.push_back( pair< Cpp_method, In_place_of_invocation* >( methods_[where], ioi) );
    }
  
  return;
}

void Cpp_class_gene::add_implem( Cpp_method the_method,
				 In_place_of_invocation* ioi)
{
  cerr << "Adding implem for method ..." << the_method << endl;

  in_place_of_interceptors_.push_back( pair<Cpp_method, In_place_of_invocation*> ( the_method, ioi ) );
  return;
}

string Cpp_class_gene::Inheritance_spec::get_cpp_decl( string name_of_inherited_class )
{
  string to_be_returned;
  to_be_returned += visibility;
  to_be_returned += " ";
  if (virtual_or_not) to_be_returned += "virtual ";
  to_be_returned += name_of_inherited_class;

  return to_be_returned;
}

pair<string,string> Cpp_class_gene::generate_code()
{
  cerr << __FILE__ << ":" << __LINE__ << endl;

  pair<string,string> to_be_returned;

  string& h_gen = to_be_returned.first;
  string& cpp_gen = to_be_returned.second;

  string _1_TAB = " ";
  string _2_TAB = "  ";

  // ---------- H Generation ---------------------

  cerr << __FILE__ << ":" << __LINE__ << endl;

  // --- generation of #include < xyz.h > -----

  for(int ct=0; ct < required_includes_.size(); ct++)
    {
      h_gen += string("#include ") + required_includes_[ct] + "\n";
    }

  h_gen += string("class ") + name_of_proxy_;

  for(map<string,Inheritance_spec>::iterator iter = inherited_classes_.begin();
      iter != inherited_classes_.end();
      iter++)
    {
      (iter == inherited_classes_.begin()) ? h_gen += " : " : h_gen += " , ";
      h_gen += iter->second.get_cpp_decl( iter->first);
    }

  h_gen += "\n";
  h_gen += string("{") + "\n";
  h_gen += _1_TAB + "public:" + "\n";

  for (int i = 0; i < friend_classes_.size(); i++)
    h_gen += _2_TAB + "friend class " + friend_classes_[i] + ";\n";
  h_gen += "\n";
    
  for( vector< Cpp_method >::iterator iter = methods_.begin(); 
	 iter != methods_.end();
	 iter++)
      {
	h_gen += _2_TAB + iter->get_in_class_declaration();
	h_gen += "\n";
      }
  
  //  h_gen += "\n" + _1_TAB + "private:" + "\n";
  h_gen += "\n" + _1_TAB + "public:" + "\n"; // xxx a revoir ...

  for( map<string,string>::const_iterator iter = members_.begin();
       iter != members_.end();
       iter++)
    {
      h_gen += _2_TAB + iter->second + " " + iter->first + ";" + "\n";
    }

  h_gen += string("};") + "\n";

  // --------- CPP Generation ----------------

  cerr << __FILE__ << ":" << __LINE__ << endl;

  cpp_gen += string("") + " // code generated ---------" + "\n";

  for( vector< Cpp_method >::iterator iter = methods_.begin();
       iter != methods_.end();
       iter++)
    {
      if (iter->has_an_implementation() )
	{
	  cpp_gen += iter->get_outof_class_header( name_of_proxy_ ) + "\n";
	  cpp_gen += string("") + "{" + "\n";

	  cerr << __FILE__ << ":" << __LINE__ << endl;

	  Pre_post_invocation_context the_pre_post_invocation_context;
	  the_pre_post_invocation_context.set_current_method_name( iter->get_name_of_method() );
	  the_pre_post_invocation_context.set_current_class_name( name_of_proxy_ );

	  for(list<Pre_post_invocation*>::iterator the_interceptors_iter 
		= pre_post_interceptors_[iter->get_name_of_method()].begin();
	      the_interceptors_iter != pre_post_interceptors_[iter->get_name_of_method()].end();
	      the_interceptors_iter++)
	    {
	      cerr << __FILE__ << ":" << __LINE__ << endl;

	      cpp_gen += "\n";

	      (*the_interceptors_iter)->set_context( the_pre_post_invocation_context );

	      cpp_gen += (*the_interceptors_iter)->generate_pre_code() + "\n";
	    }

	  cerr << __FILE__ << ":" << __LINE__ << endl;

	  vector< pair<Cpp_method, In_place_of_invocation*> >::iterator the_iter 
	    = in_place_of_interceptors_.begin();

	  cerr << __FILE__ << ":" << __LINE__ << endl;
	  
	  while ( ! (the_iter->first.equals( *iter ) ) ) 
	    {
	      the_iter++; 
	      if (the_iter == in_place_of_interceptors_.end() )
	      {
		cerr << "You did not specify any implementation for method " 
		     << ( *iter ) << endl;
		
		exit(-1);
	      }
	    }

	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << " name_of_method == " 
	       << iter->get_name_of_method()  << endl;

	  // ---- in_place_of_interceptors_[ ... ]
	  cpp_gen += the_iter->second->generate_code_in_place_of();

	  cerr << __PRETTY_FUNCTION__ << ":" << __LINE__ << endl;

	  //    cpp_gen += string("") + ";" + "\n";

	  cpp_gen += string("") + "}" + "\n";
	  cpp_gen += "\n";
	} // method has an implementation ...
    }

  cerr << __FILE__ << ":" << __LINE__ << endl;

  return to_be_returned;
}

void Cpp_class_gene::add_required_include( string name_of_include )
{
  required_includes_.push_back( name_of_include );
  return;
}

int Cpp_class_gene::get_nb_methods()
{
  return methods_.size();
}

string Cpp_class_gene::get_name_of_method_nb( int nb )
{
  return methods_[ nb ].get_name_of_method();
}

Cpp_method& Cpp_class_gene::get_method( int index )
{
  return methods_[ index ];
}

void Cpp_class_gene::change_inherited_class( string name_of_orig_class, 
					       string new_name,  
					       string visibility, 
					       bool virtual_or_not )
{
  map< string, Inheritance_spec >::iterator found = inherited_classes_.end();
  
  for( map<string, Inheritance_spec>::iterator iter = inherited_classes_.begin();
       iter != inherited_classes_.end();
       iter++)
    {
      if (iter->first == name_of_orig_class) found=iter;
    }

  if (found == inherited_classes_.end() )
    {
      cerr << "You want to replace and inheritance to a class that is not inherited from" << endl;
      exit(-1);
    }

  inherited_classes_.erase( found );
  
  add_inherited_class( new_name, visibility, virtual_or_not );
  return;
}




