/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.runtime.common.lib;


/**
 * ConnectorBase is a base class for Connector.
 * It implements the org.objectweb.openccm.pss.runtime.api.Connector interface.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public abstract class ConnectorBase
              extends org.omg.CORBA.LocalObject
           implements org.objectweb.openccm.pss.runtime.common.api.Connector
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * The implementation id.
     */
    protected String _implementation_id;

    /**
     * Registered object factories.
     */
    protected java.util.Map _storage_object_factories;

    /**
     * Registered home factories.
     */
    protected java.util.Map _storage_home_factories;

     /**
     * Next free id for Catalogs.
     */
    protected int _free_id;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     *
     * @param id - The connector id.
     */
    public ConnectorBase(String id)
    {
        _implementation_id = id;
        _storage_object_factories = new java.util.HashMap();
        _storage_home_factories = new java.util.HashMap();
        _free_id = 0;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.Connector
    //
    // ==================================================================

    /**
     * Read accessor for implementation_id attribute
     *
     * @return the attribute value
     */
    public String implementation_id()
    {
        return _implementation_id;
    }

    /**
     * Get the PID of a Storage Object.
     */
    public byte[]
    get_pid(java.lang.Object obj)
    {
        return ((org.omg.CosPersistentState.StorageObject) obj).get_pid();
    }

    /**
     * Get the short PID of a Storage Object.
     */
    public byte[]
    get_short_pid(java.lang.Object obj)
    {
        return ((org.omg.CosPersistentState.StorageObject) obj).get_short_pid();
    }

    /**
     * Create a basic session.
     */
    public abstract org.omg.CosPersistentState.Session
    create_basic_session(short access_mode,
                         org.omg.CosPersistentState.Parameter[] additional_parameters);

    /**
     * Create a transactional session.
     */
    public abstract org.omg.CosPersistentState.TransactionalSession
    create_transactional_session(short access_mode,
                                 short default_isolation_level,
                                 org.omg.CosPersistentState.EndOfAssociationCallback callback,
                                 org.omg.CosPersistentState.Parameter[] additional_parameters);

    /**
     * Create a session pool.
     */
    public abstract org.omg.CosPersistentState.SessionPool
    create_session_pool(short access_mode,
                        short tx_policy,
                        org.omg.CosPersistentState.Parameter[] additional_parameters);

    /**
     * Get the current session.
     */
    public abstract org.omg.CosPersistentState.TransactionalSession
    current_session();

    /**
     * Get sessions.
     */
    public abstract org.omg.CosPersistentState.TransactionalSession[]
    sessions(org.omg.CosTransactions.Coordinator transaction);

    /**
     * Register a storage object factory.
     */
    public java.lang.Class
    register_storage_object_factory(String storage_type_name,
                                    java.lang.Class storage_object_factory)
    {
        java.lang.Class previous = null;

        previous = (Class) _storage_object_factories.get(storage_type_name);
        if ( previous != null )
            _storage_object_factories.remove(storage_type_name);

        _storage_object_factories.put(storage_type_name, storage_object_factory);
        return previous;
    }

    /**
     * Register a storage home factory
     */
    public java.lang.Class
    register_storage_home_factory(String storage_home_type_name,
                                  java.lang.Class storage_home_factory)
    {
        java.lang.Class previous = null;

        previous = (Class) _storage_home_factories.get(storage_home_type_name);
        if ( previous != null )
            _storage_home_factories.remove(storage_home_type_name);

        _storage_home_factories.put(storage_home_type_name, storage_home_factory);
        return previous;
    }

    /**
     * Register a session factory.
     */
    public java.lang.Class
    register_session_factory(java.lang.Class session_factory)
    {
        // Don't know how to implement this method
        return null;
    }

    /**
     * Register a session pool factory.
     */
    public java.lang.Class
    register_session_pool_factory(java.lang.Class session_pool_factory)
    {
        // Don't know how to implement this method
        return null;
    }

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.common.api.Connector
    //
    // ==================================================================

    /**
     * Get a storage object factory.
     */
    public java.lang.Class
    get_storage_object_factory(String storage_type_name)
    {
        return (Class) _storage_object_factories.get(storage_type_name);
    }

    /**
     * Get a storage home factory
     */
    public java.lang.Class
    get_storage_home_factory(String storage_home_type_name)
    {
        return (Class) _storage_home_factories.get(storage_home_type_name);
    }

    /**
     * Get the next free id for catalogs
     */
    public int
    get_free_id()
    {
        return _free_id++;
    }
}
