/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.runtime.common.lib;

/**
 * This is a base class for Storage Homes.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public abstract class StorageHomeBase
              extends org.omg.CORBA.LocalObject
           implements org.objectweb.openccm.pss.runtime.common.api.StorageHomeBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /**
     * The catalog.
     */
    private org.objectweb.openccm.pss.runtime.common.api.CatalogBase _catalog;

    /**
     * Managed storage type incarnations.
     */
    protected java.util.List _storage_type_incarnations;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StorageHomeBase()
    {
        _storage_type_incarnations = new java.util.ArrayList();
        _catalog = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for org.omg.CosPersistentState.StorageHomeBase
    //
    // ==================================================================

    /**
     * Get the catalog.
     */
    public org.omg.CosPersistentState.CatalogBase
    get_catalog()
    {
        return _catalog;
    }

    /**
     * Retrieve a storage object with its short pid.
     *
     * @param short_pid - The short pid to find.
     *
     * @throws org.omg.CosPersistentState.NotFound ifstorage object can't be found.
     *
     * @return The binding storage object.
     */
    public abstract java.lang.Object
    find_by_short_pid(byte[] short_pid)
    throws org.omg.CosPersistentState.NotFound;

    // ==================================================================
    //
    // Methods for org.objectweb.openccm.pss.runtime.common.api.StorageHomeBase
    //
    // ==================================================================

    /**
     * Initialize the storage home.
     *
     * @param catalog - The storage home catalog.
     */
    public void
    initialize(org.objectweb.openccm.pss.runtime.common.api.CatalogBase catalog)
    {
        _catalog = catalog;
    }

    /**
     * Register a storage type incarnation.
     */
    public void
    register_storage_type_incarnation(java.lang.Object st_incarnation)
    {
        _storage_type_incarnations.add(st_incarnation);
    }

    /**
     * Unregister a storage type incarnation.
     */
    public void
    unregister_storage_type_incarnation(byte[] pid)
    {
        java.util.Iterator it = _storage_type_incarnations.iterator();
        org.omg.CosPersistentState.StorageObject so = null;

        while(it.hasNext())
        {
            so = (org.omg.CosPersistentState.StorageObject) it.next();
            if ( java.util.Arrays.equals(so.get_pid(), pid) )
            {
                it.remove();
                return;
            }
        }
    }

    /**
     * Get the Repository ID of a storage home.
     *
     * @return The storage home RID.
     */
    public abstract String
    get_rid();

    /**
     * Get the Repository ID of the managed storage type.
     *
     * @return The storage type RID.
     */
    public abstract String
    get_st_rid();
}
